C
C           A T T E N T I O N !!!
C
C !!! THIS FILE EXIST IN source/fcm/ and in source/moldyn/ !!!
C !!! if you modify it do it in both locations             !!!

C $Header: /export/cvs/c34a1/source/fcm/HEAPMB.INC,v 1.1.1.1 2006/08/17 02:29:52 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
C
C     DEVELOPERS: Please modify/update this description as the details
C                 change so it will make sense to other people.
C     (added July 25, 1996)
C
C     HEAPMB is a step towards more efficient memory management in
C     MBO(N)D.  Many of the largest arrays that were originally 
C     declared with a fixed size upon compilation have been placed in
C     HEAPMB, which is a one dimensional vector.  This allows packed 
C     storage and therefore problem sizes can be much larger than before
C     for the same amount of allocated memory.  The single dimension 
C     packed storage also allows the sizes of those arrays to change if 
C     MBO(N)D is invoked more than once, with different configurations, 
C     constraint conditions, or number of modes.
C
C     The data stored in HEAPMB can be located with the use of indices
C     that are named to refer to the original array they represent.
C     These arrays of indices, ibh_ptr for example, are declared in 
C     the *.INC file that the original array was declared in (see below).  
C     The subroutines in the file memory.f (asnmem, chkmem, newptr) take 
C     care of initializing indices and handling space on HEAPMB.
C
C     HEAPMB takes advantage of the fact that Fortran stores each array
C     in one long column by stacking columns on top of each other. 
C     When a 'matrix' is passed to a subroutine, what is really passed is
C     the index to the first element on the 1-D array it is stored on 
C     (HEAPMB in this case) and its 'size', which is the leading column 
C     length. 
C
C     In order to continue to treat arrays that now reside on HEAPMB as 
C     arrays, some new subroutines called within original subroutines 
C     have been created (with the name ORIGINAL_1).  For example, bckwrd
C     has a DO loop that uses array variables like AMU, RMBAR, and BH.
C     In this loop, the indices to the section of HEAPMB that correspond
C     to those arrays are calculated and passed to the subroutine bckwrd_1 
C     (which is physically located within the file bckwrd.f).  bckwrd_1 
C     contains most of the original code of bckwrd and treats the HEAPMB 
C     locations of AMU, RMBAR, and BH as their original selves. This way, 
C     messy indices don't need to be calculated all the time and the code 
C     of the subroutines remains essentially the same.
C
C     The arrays on HEAPMB are still handled with their original names 
C     within many subroutines.  Their location on HEAPMB is passed into 
C     the subroutine and treated as the original array.  If it is not 
C     convenient to create a new subroutine or pass the correct section 
C     of HEAPMB, plain HEAPMB is used and the true identity of that section 
C     of HEAPMB is referenced.  Documentation exists in various places 
C     that defines or discusses the original arrays so it seems valuable 
C     to keep these names.
C
C     The HATS array is filled in modput1 in CHARMM before MBO(N)D is
C     called.  It is first on HEAPMB.  Other arrays that are on HEAPMB
C     are: AMU, AM, RMBAR, ACOF, BCOF, COF11, COF22, COF33, COF12, 
C     COF13, COF23, COFXY, COFXZ, COFYZ, and BH.  BH is at the end of
C     HEAPMB and its indices are calculated in bhgener.  (A detailed 
C     account of what is and is not stored in BH is in BHBSRD.INC.)
C     After all of the array indices are stored there is a check in 
C     bhgenr that the calculated size of all of the arrays will not exceed
C     the declared size of HEAPMB.  A small report is printed out to 
C     tell how large HEAPMB was declared and how much of it was used.
C     In the case that bhgenr is not called (i.e. there are no bodies), 
C     the last element is checked in chkmem and the final report is 
C     printed out there.
C
C     Here is a list of references and index names.
C
C      HATS      - ihtsptr      See modstrans.src and HANDS2.INC for details.
C            - ihthptr      See msmodf.f and HANDS2.INC for details.
C      AM      - iam_ptr      See memory.f and AMUBW.INC for details.
C      AMU      - iamuptr      See memory.f and INTGRL.INC for details.               
C      RMBAR      - irm_ptr      See memory.f and FIBKF2.INC for details.               
C      ACOF      - iacfptr      See memory.f and INTGRL.INC for details. 
C      BCOF      - ibcfptr      See memory.f and INTGRL.INC for details.
C      COF11      - ic11ptr      See memory.f and INTGRL.INC for details.
C      COF22      - ic22ptr      See memory.f and INTGRL.INC for details.
C      COF33      - ic33ptr      See memory.f and INTGRL.INC for details.
C      COF12      - ic12ptr      See memory.f and INTGRL.INC for details.
C      COF13      - ic13ptr      See memory.f and INTGRL.INC for details.
C      FREQ       - ifreqptr      See memory.f and CSTIFF.INC
C      AMDAMP     - iamdampptr    See memory.f and CSTIFF.INC
C      COF23      - ic23ptr      See memory.f and INTGRL.INC for details.
C       COFXY      - icxyptr      See memory.f and INTGRL.INC for details.
C      
C      FREQ       - ifreqptr     See memory.f and CSTIFF.INC
C      AMDAMP     - iamdampptr       See memory.f and CSTIFF.INC
C      PSI        - ipsiptr       See memory.f and F1BKF2.INC
C      GGS        - iggsptr       See memory.f and GGSAVE.INC
C      Y          - iyptr        See memory.f and VECTOR.INC
C      YDT        - iydtptr      See memory.f and VECTOR.INC
C      COFXZ      - icxzptr      See memory.f and INTGRL.INC for details.
C      COFYZ      - icyzptr      See memory.f and INTGRL.INC for details.
c      AM_langevin - ilangevinptr   See memory.f and CHARMM.INC
c      RF         - irfptr       See memory.f and CHARMM.INC
c      gamLng      - igamlngptr   See memory.f and CHARMM.INC
c      Gpos       - igpos         See memory.f and GCOMP.INC
c      Gpos_stage - igpos_stage   See memory.f and GCOMP.INC
c      G_chm      - ig_chmptr     See memory.f and GCOMP.INC
c      G_gyro     - ig_gyroptr    See memory.f and GCOMP.INC
c      G_deform   - ig_deformptr  See memory.f and GCOMP.INC
c      p          - ipptr         See memory.f and MOMENG.INC
c      pmom       - ipmomptr      See memory.f and MOMENG.INC
c      BTILDE     - ibtildeptr    See memory.f and CLOOP.INC
c      BICDST     - ibicdstptr    See memory.f and CLOOP.INC
c      SU         - isuptr        See memory.f and NOUTPT.INC
C      SBT        - isbtptr       See memory.f and NOUTPT.INC
C      BRN        - ibrnptr       See memory.f and NOUTPT.INC
c      GVEC       - igvec         See memory.f and CSTIFF.INC

C      BH      - ibh_ptr      See memory.f, bhgenr.f and BHBSRD.INC for details.
C
C Notes: HATS is not used if there are no modes or no bodies.  
C        BH is not used if there are no bodies.
C        None of the COFs are used if there are no modes.
C

C Stage II of memory management.
C IIa.
C   Dynamic memory allocation for HATS.  HATS is no longer stored on HEAP_MB.
C   Modify all the source codes that reference to HEAP_MB for HATS which include
C   the part of the codes that reference to HATS for BH.  From the previous HEAP_MB
C   modification, part of BH was no longer stored and reference to HATS for the data.  
C   Dimension of HATS is calculated in mbmodput(modtrans.src) when the modes have
C   changed. 
C IIb.
C   Dynamic memory allocation for BH.  BH is no longer stored on HEAP_MB.  Modify
C   all the source codes that reference to HEAP_MB for BH.  Dimension of BH is calculated
C   in bhgenr(bhgenr.f). 
C IIc.
C   Dynamic memory allocation for HEAP_MB.  Dimension of HEAP_MB is calculated in 
C   asnmem(memory.f).

C   Dynamic memory allocation for INTHEAP_MB.  Dimension is calculated in 
C   asnmem(memory.f).

C   Dynamic memory allocation for LOCALHEAP.  Dimension is calculated in initheap
C   (initHeap.f).  Local variables that use L values as dimensions are now store
C   int LOCALHEAP.

C   Dynamic memory allocation for INTLOCALHEAP.  Same as LOCALHEAP except
C   INTLOCALHEAP is an integer HEAP and LOCALHEAP is a double precision.
C   Set in the same place where LOCALHEAP is set.

C   Dynamic memory allocation for LOOPHEAP.  Dimension are determined in 
C   init_loopheap(initHeap.f).  LOOPHEAP stores global variables that 
C   use MXLOOP and related L values as dimensions.  

C   Dynamic memory allocation for INTLOOPHEAP.  Same as LOOPHEAP except 
C   INTLOOPHEAP is integer heap and LOOPHEAP is double precision.

C   Dynamic memory allocation for LOGICLOOPHEAP. Same as LOOPHEAP except 
C   LOGICLOOPHEAP is logical heap and LOOPHEAP is double precision.

c   NOTE: INTHEAP_MB,LOCALHEAP,INTLOCALHEAP,LOOPHEAP,INTLOOPHEAP,
C         and LOGICLOOPHEAP are not packed storage.  And part of
c         HEAP_MB is also not packed storage.
   
C Pointer has integer type by default.
C
#ifdef POINTER_KEYWORD
      POINTER (hatsptr, HATS)
      DOUBLE PRECISION HATS(*)

      POINTER (alptr, AINTGL)
      DOUBLE PRECISION AINTGL(*)

      POINTER (hpmbptr, HEAP_MB)      
      DOUBLE PRECISION HEAP_MB(*) 

c     integer heap_mb
      POINTER (ihpmbptr, intHEAP_MB)      
      INTEGER intHEAP_MB(*) 

      POINTER (bhptr, BH)
      DOUBLE PRECISION BH(*)

c     Heap for local variables
      POINTER (lhpptr,LocalHeap)
      DOUBLE PRECISION LocalHeap(*)

c     Heap for local variables
      POINTER (ilhpptr,intLocalHeap) 
      INTEGER intLocalHeap(*)

C     Heap for Local variables in mbdyna.f
      POINTER (mbdynaptr, mbdynaHeap)
      DOUBLE PRECISION mbdynaHeap(*)

C     Heap for Local variables in chkbon.f
      POINTER (chkbonPtr, chkbonHeap)
      INTEGER chkbonHeap(*)


      COMMON /ISVPTR/ alptr, hatsptr, hpmbptr, bhptr, chkbonPtr,
     &                lhpptr,ilhpptr,ihpmbptr, mbdynaptr
c
c     Heap for parameter mxloop

      POINTER (imlpptr,intLoopHeap)
      INTEGER intLoopHeap(*)
      POINTER (mlpptr, LoopHeap)
      DOUBLE PRECISION LoopHeap(*)
      POINTER (lmlpptr, logicLoopHeap)
      LOGICAL logicLoopHeap(*)

      COMMON /MLOOPHP/ imlpptr,mlpptr,lmlpptr

c     Heaps for cmpic2.f and dlespa.f
c     amat in cmpic2.f and dlespa.f
      POINTER(naa_p,pre_amat)
      DOUBLE PRECISION pre_amat(*)

c     dmd, glm, icrow, iccol and vecr in dlespa.f
      pointer(ndmd_p,pre_dmd)
      double precision pre_dmd(*)

      pointer(nlm_p,pre_glm)
      double precision pre_glm(*)

      pointer(icr_p,pre_icrow)
      integer pre_icrow(*)

      pointer(icc_p,pre_iccol)
      integer pre_iccol(*)

      pointer(nx_p,pre_vecr)
      double precision pre_vecr(*)


#else
C     Statically allocate these arrays.
      DOUBLE PRECISION HATS(MAX_HATS)

      DOUBLE PRECISION AINTGL(MAX_AINTGL)

      DOUBLE PRECISION HEAP_MB(MAX_HEAP_MB) 

      DOUBLE PRECISION intHEAP_MB(MAX_intHEAP_MB) 

      DOUBLE PRECISION BH(MAX_BH)

      DOUBLE PRECISION LocalHeap(MAX_LocalHeap)

      DOUBLE PRECISION intLocalHeap(MAX_intLocalHeap)

C     Heap for Local variables in mbdyna.f
      DOUBLE PRECISION mbdynaHeap(MAX_mbdynaHeap)

C     Heap for Local variables in chkbon.f
      INTEGER chkbonHeap(MAX_chkbonHeap)

c     Heap for parameter mxloop
      INTEGER intLoopHeap(MAX_intLoopHeap)
      DOUBLE PRECISION LoopHeap(MAX_LoopHeap)
      LOGICAL logicLoopHeap(MAX_logicLoopHeap)

      COMMON /STATIC_HEAPS/ HATS,AINTGL,HEAP_MB,intHEAP_MB,BH,
     &         LocalHeap,intLocalHeap,mbdynaHeap,chkbonHeap,
     &         intLoopHeap,LoopHeap,logicLoopHeap

c     Heap for cmpic2.f and dlespa.f
      INTEGER lsqfit_ptr
      INTEGER iptr_dmd, iptr_glm, iptr_icrow, iptr_iccol, iptr_vecr
      DOUBLE PRECISION heap_lsqfit
      COMMON /LSQFIT/ heap_lsqfit(MAX_cmpic2Heap),lsqfit_ptr,iptr_dmd,
     &     iptr_glm, iptr_icrow, iptr_iccol, iptr_vecr

#endif

   
c     pointers for local variables.  each file has it own 
c     common block.
c     See initHeap.f on pointer setup.
c
c     Pointers to localheap for msmodf.f local variables
      INTEGER Aptr,Bptr,Cptr,WSptr,UVECptr     
      COMMON /IMSMODF/ aptr,bptr,cptr,wsptr,uvecptr

c     Pointer to LocalHeap for local variable used in engmom.f
      INTEGER iwvptr_engmom
      COMMON /IENGMOM/ iwvptr_engmom 

c     Pointers to LocalHeap  (mgen.f pointers)
      INTEGER irwptr_mgen, icwptr_mgen
      COMMON /IMGEN/ irwptr_mgen, icwptr_mgen

c     torque.f
      INTEGER irwptr_torque, icwptr_torque, iv1ptr_torque
      COMMON /ITORQUE/ irwptr_torque,icwptr_torque,iv1ptr_torque
C
c     init_vcd.f
      INTEGER izptr_init_vcd,izdtptr_init_vcd,iysavptr_init_vcd,
     &        idysavptr_init_vcd,ix_tmpptr_init_vcd
      COMMON /IINITVCD/ izptr_init_vcd,izdtptr_init_vcd,
     &        iysavptr_init_vcd,
     &        idysavptr_init_vcd,ix_tmpptr_init_vcd

c     mdloop_CHARMM.f
      INTEGER izptr_mdloop_charmm,izdtptr_mdloop_charmm    
      COMMON /IMDLOOPCHARMM/ izptr_mdloop_charmm,
     *                       izdtptr_mdloop_charmm

c     multb.f
      INTEGER iwptr_multb
      COMMON /IMULTB/ iwptr_multb

c     rk4.f
      INTEGER iytptr_rk4,idytptr_rk4,idymptr_rk4
      COMMON /IRK4C/ iytptr_rk4,idytptr_rk4,idymptr_rk4

c     verlcd.f
      INTEGER iaccptr_verlcd
      COMMON /IVERLCD/ iaccptr_verlcd
c
c     bckwrd.f
      INTEGER irmttmp_bckwrd,ikpvtmptr_bckwrd,itempptr_bckwrd,
     *        itemp2ptr_bckwrd,izwkptr_bckwrd,irmttmp_bckwrd1,
     *        ikpvtmptr_bckwrd1,itempptr_bckwrd1,
     *        itemp2ptr_bckwrd1,izwkptr_bckwrd1,
     *        ibhinbptr_bckwrd1,ibhoutptr_bckwrd1
      COMMON /IBCKWRD/ irmttmp_bckwrd,ikpvtmptr_bckwrd,
     *                 itempptr_bckwrd,itemp2ptr_bckwrd,
     *                 izwkptr_bckwrd,irmttmp_bckwrd1,
     *                 ikpvtmptr_bckwrd1,itempptr_bckwrd1,
     *                 itemp2ptr_bckwrd1,izwkptr_bckwrd1,
     *                 ibhinbptr_bckwrd1,ibhoutptr_bckwrd1

c     bhgenr.f
      INTEGER ibhqptr_bhgenr, ibhpptr_bhgenr
      COMMON /IBHGENR/ ibhqptr_bhgenr, ibhpptr_bhgenr

c     cjmpls.f
      INTEGER ibhinbptr_cjmpls,idbetxiptr_cjmpls,
     *        itemp1ptr_cjmpls,idbetxiptr_cjmpls_1,
     *        itempptr_cjmpls_1,itemp1ptr_cjmpls_1
      COMMON /ICJMPLS/ ibhinbptr_cjmpls,idbetxiptr_cjmpls,
     *               itemp1ptr_cjmpls,idbetxiptr_cjmpls_1,
     *               itempptr_cjmpls_1,itemp1ptr_cjmpls_1


c     clbck1.f
      INTEGER ibhqptr_clbck1,ibhpptr_clbck1,ibhinbptr_clbck1,
     *        ibhoutptr_clbck1,irmttmpptr_clbck1,ia1ptr_clbck1,
     *        ia3ptr_clbck1,itempptr_clbck1,itempaptr_clbck1,
     *        itempa2ptr_clbck1,ikpvtmptr_clbck1
      COMMON /ICLBCK1/ ibhqptr_clbck1,ibhpptr_clbck1,ibhinbptr_clbck1,
     *             ibhoutptr_clbck1,irmttmpptr_clbck1,ia1ptr_clbck1,
     *             ia3ptr_clbck1,itempptr_clbck1,itempaptr_clbck1,
     *             itempa2ptr_clbck1,ikpvtmptr_clbck1

c     clbck2.f
      INTEGER ibhinbptr_clbck2,ibhoutptr_clbck2,ikpvtmptr_clbck2,
     *        irmttmpptr_clbck2
      COMMON /ICLBCK2/ ibhinbptr_clbck2,ibhoutptr_clbck2,
     *                 ikpvtmptr_clbck2,irmttmpptr_clbck2

c     clfor1.f
      INTEGER ibhqptr_clfor1,ibhpptr_clfor1
      COMMON /ICLFOR1/ ibhqptr_clfor1,ibhpptr_clfor1

c     clfor3.f
      INTEGER ikpvtptr_clfor3,ikpvtptr_clfor3_1,
     &        ibhoutptr_clfor3_1,itempptr_clfor3,
     &        itemp1ptr_clfor3,itempptr_clfor3_1,
     &        itemp1ptr_clfor3_1
      COMMON /ICLFOR3/ ikpvtptr_clfor3,ikpvtptr_clfor3_1,
     &                 ibhoutptr_clfor3_1,itempptr_clfor3,
     &                 itemp1ptr_clfor3,itempptr_clfor3_1,
     &                 itemp1ptr_clfor3_1

c     clfr2a.f
      INTEGER ibhinbptr_clfr2a,iacopyptr_clfr2a,izptr_clfr2a,
     *        itempvptr_clfr2a
      COMMON /ICLFR2A/ ibhinbptr_clfr2a,iacopyptr_clfr2a,
     *        izptr_clfr2a,itempvptr_clfr2a

c     clfr2b.f
      INTEGER ibhinbptr_clfr2b,itempptr_clfr2b,itemp1ptr_clfr2b,
     *        ibdtqptr_clfr2b,ibdtpptr_clfr2b,iustarptr_clfr2b,
     *        izetacptr_clfr2b
      COMMON /ICLFR2A/ ibhinbptr_clfr2b,itempptr_clfr2b,
     *                 itemp1ptr_clfr2b,ibdtqptr_clfr2b,
     *                 ibdtpptr_clfr2b,iustarptr_clfr2b,
     *                 izetacptr_clfr2b

c     cmpic2.f
      INTEGER icmatptr_cmpic2,icmat2ptr_cmpic2,ifmatptr_cmpic2,
     *        ikpvtmptr_cmpic2,igmxptr_cmpic2,ismptr_cmpic2,
     *        ikpvtalptr_cmpic2,ivsavptr_cmpic2,
     *        irotcoorptr,imodevctrptr,ixmassptr,
     *        itmpampptr,ierrorptr,ievalptr,
     *        iidblptr_cmpic2,iicblptr_cmpic2
      COMMON /ICMPIC2/ icmatptr_cmpic2,icmat2ptr_cmpic2,
     *                 ifmatptr_cmpic2,ikpvtmptr_cmpic2,
     *                 igmxptr_cmpic2,ismptr_cmpic2,
     *                 ikpvtalptr_cmpic2,ivsavptr_cmpic2,
     *                 irotcoorptr,imodevctrptr,ixmassptr,
     *                 itmpampptr,ierrorptr,ievalptr,
     *                 iidblptr_cmpic2,iicblptr_cmpic2

c     frwd1b.f
      INTEGER ibdoticptr_frwd1b,ibdotinptr_frwd1b
      COMMON /IFRWD1B/ ibdoticptr_frwd1b,ibdotinptr_frwd1b

c     frwrd2.f
      INTEGER ikpvtptr_frwrd2,itempptr_frwrd2,itemp1ptr_frwrd2,
     *        izwkptr_frwrd2
      INTEGER ikpvtptr_frwrd2_1,itempptr_frwrd2_1,itemp1ptr_frwrd2_1,
     *        izwkptr_frwrd2_1
      COMMON /IFRWRD2/ ikpvtptr_frwrd2,itempptr_frwrd2,
     *                 itemp1ptr_frwrd2,izwkptr_frwrd2,
     *                 ikpvtptr_frwrd2_1,itempptr_frwrd2_1,
     *                 itemp1ptr_frwrd2_1,izwkptr_frwrd2_1

c     ydot.f
      INTEGER iggvptr_ydot,iirhptr_ydot
      COMMON /IYDOT/ iggvptr_ydot,iirhptr_ydot

c     velrms.f
      INTEGER icmatptr_velrms
      COMMON /IVERLRMS/ icmatptr_velrms

c     ctinit1_CHARMM.f
      INTEGER iiopptr_ctinit1_CHARMM,iloopptr_ctinit1_CHARMM
      COMMON /CTINIT1/ iiopptr_ctinit1_CHARMM,iloopptr_ctinit1_CHARMM

c     rotdh1_CHARMM.f
      INTEGER iiopptr_rotdh1_CHARMM
      COMMON /ROTDH1/ iiopptr_rotdh1_CHARMM

#ifdef IBMRS
      INTEGER FMALLOC
      EXTERNAL FMALLOC
#endif

      CHARACTER*128  HEAPMB_RCSID
      DATA           HEAPMB_RCSID
     &/'$RCSfile: HEAPMB.INC,v $$Revision: 1.1.1.1 $'/




