C $Header: /export/cvs/c34a1/source/moldyn/amadji.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
      SUBROUTINE amadji(xloc,xvel,ami,x0,aam,n,job)
c
      IMPLICIT NONE

#include "TAPENO.INC"
#include "ERRHDL.INC"
c
C
c
c     Argument list
c
      INTEGER n                     !  Number of atoms in group
      INTEGER job                   !  0:  Compute aam only
                                    !  1:  Zero out angular momentum
      DOUBLE PRECISION xloc(3,n)    !  Input atomic coordinates
      DOUBLE PRECISION xvel(3,n)    !  Input/output atomic velocities
      DOUBLE PRECISION ami(n)       !  Atomic mass inverse
      DOUBLE PRECISION x0(3)        !  Input group center of mass
      DOUBLE PRECISION aam(3)       !  Angular momentum of group
c
c     Local Variables
c
      INTEGER i, j, k, i1           !  DO loop indices
      INTEGER info                  !  Error flag from dsifa
      INTEGER kpvt(3)               !  Pivot indices for lin. eq. soln.
      DOUBLE PRECISION amas         !  Atomic mass
      DOUBLE PRECISION omega(3)     !  Body angular velocity
      DOUBLE PRECISION r(3)         !  Coordinates relative to x0
      DOUBLE PRECISION r2           !  Squared dist. of atom from origin
      DOUBLE PRECISION amv(3)       !  Lin. momentum for single atom
      DOUBLE PRECISION amat(3,3)    !  Skew symmetric matrix
      DOUBLE PRECISION ainer(3,3)   !  Inertia matrix of group
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: amadji.f,v $$Revision: 1.1.1.1 $'/
c
c
c
c     Check input job options
c
      IF (job .LT. 0 .OR. job .GT. 1) THEN
         CALL CLEARERR()
         WRITE(ERRMSG,300) 'Job option out of range.'
         CALL ERRHDL(-5, '<AMADJI>')
      END IF
 300  FORMAT(1X,A)
c
c     Zero out aam and ainer
c
      CALL zero(aam,3,1,3)
      CALL zero(ainer,3,3,3)
c
c     Compute (mv) x r
c     where mv is made into a skew symmetric matrix and
c           r is relative to x0
c
      DO i = 1, n
         amas = 1.d0/ami(i)
         r2 = 0D0
         DO i1 = 1, 3
            r(i1) = xloc(i1,i) - x0(i1)
            r2 = r2 + r(i1)**2
            amv(i1) = amas*xvel(i1,i)
         END DO
         CALL skewv3(amv,amat,1,3)
CC@CC
CC@CC ...MULTAD amat, r, aam
CC@CC ...3, 3, 1
CC@CC ...3, 3, 3
      aam(1) = aam(1) + amat(1,1)*r(1) + amat(1,2)*r(2) + amat(1,3)*r(3)
      aam(2) = aam(2) + amat(2,1)*r(1) + amat(2,2)*r(2) + amat(2,3)*r(3)
      aam(3) = aam(3) + amat(3,1)*r(1) + amat(3,2)*r(2) + amat(3,3)*r(3)
CC@CC end of expanded call
CC@CC
c
c     Accumulate the inertia matrix
c
         IF (job .EQ. 1) THEN
            DO j = 1, 3
               DO k = j, 3
                  ainer(j,k) = ainer(j,k) - amas*r(j)*r(k)
               END DO
               ainer(j,j) = ainer(j,j) + amas*r2
            END DO
         END IF
c
      END DO
c
c     Take out angular motion of system
c
      IF (job .EQ. 1) THEN
         DO i = 1, 3
            omega(i) = aam(i)
         END DO
         CALL dsifa(ainer,3,3,kpvt,info)
         IF (info .NE. 0) THEN
            CALL CLEARERR()
            WRITE(ERRMSG,300 ) 'Inertia matrix singular.'
            CALL ERRHDL(-5,'<AMADJI>')
         END IF
         CALL dsisl(ainer,3,3,kpvt,omega)
         CALL skewv3(omega,amat,1,3)
         DO i = 1, n
            DO i1 = 1, 3
               r(i1) = xloc(i1,i) - x0(i1)
            END DO
CC@CC
CC@CC ......MULTAD amat, r, xvel(1,i)
CC@CC ......3, 3, 1
CC@CC ......3, 3, 3
      xvel(1,i) = xvel(1,i) + amat(1,1)*r(1) + amat(1,2)*r(2) + amat(1,3
     .)*r(3)
      xvel(2,i) = xvel(2,i) + amat(2,1)*r(1) + amat(2,2)*r(2) + amat(2,3
     .)*r(3)
      xvel(3,i) = xvel(3,i) + amat(3,1)*r(1) + amat(3,2)*r(2) + amat(3,3
     .)*r(3)
CC@CC end of expanded call
CC@CC
         END DO
      END IF
c
      RETURN
      END
