C $Header: /export/cvs/c34a1/source/moldyn/bckwrd.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
      SUBROUTINE BCKWRD (GGV,RMTTMP,KPVTM,TEMP,TEMP2,ZWK,ZETA,
     &                   IHTHPTR,IHPVT,PIN,ISEL,ISDOF,ITOPOL,
     &                   IBH_PTR)
C                                                       DEBUG #  75
CSE
      IMPLICIT NONE
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: bckwrd.f,v $$Revision: 1.1.1.1 $'/
CPS
C     PURPOSE
C             RECURSIVELY COMPUTE THE EQUIVALENT INERTIA AND EQUIVALENT
C             GENERALIZED FORCE VECTORS WHICH ACCOUNT FOR THE INERTIA AND
C             GENERALIZED FORCES FOR ALL THE OUTBOARD BODIES.  THE RECURSION
C             IS PERFORMED STARTING FROM THE END BODY (OR BODIES) AND ENDING
C             WITH THE BASE BODY
CPE
CAS
C                         BCKWRD USED TO COMPUTE
C
C          PSI = transformation matrix array, stored in /F1BKF2/
C    (PSI(I,J,k),I=1,3),J=1,3) = Upper right partition of the transformation
C                                matrix PSI(k) used to transform M(tilde) the
C                                equivalent inertia matrix for body k to
C                                M(bar) the transformed inertia matrix for
C                                body k.
C        RMBAR = transformed inertia matrices M-bar
C          GGV = vector of equivalent generalized force vectors
C
CAE
C
C     LOCAL VARIABLES:
C
C         IBODY - DO LOOP SEQUENCE NUMBER, NB,NB-1,...,1
C        ICBODY - OUTBOARD BODY NUMBER
C        INBODY - INBOARD BODY NUMBER
C        IHINGE - CONNECTING HINGE NUMBER
C         NFREE - NUMBER OF UNCONSTRAINED DOF's AT IHINGE
C         NCONS - NUMBER OF FIXED & RHEONOMIC CONSTRAINTS AT IHINGE
C                 SUCH THAT NCONS+NFREE=6
C            NM - NUMBER OF FLEXIBLE BODY MODES FOR BODY ICBODY
C           NMI - NUMBER OF FLEXIBLE BODY MODES FOR BODY INBODY
C           NMW - NUMBER OF VARIABLE SPEED WHEELS IN BODY ICBODY
C        MTSIZE - LENGTH OF RATE VARIABLES PARTITION FOR ICBODY IN Y
C        MTISIZ - LENGTH OF RATE VARIABLES PARTITION FOR INBODY IN Y
C        LOCUIC - STARTING LOCATION IN Y OF ICBODY RATE STATE VARIABLES
C        LOCUIN - STARTING LOCATION IN Y OF INBODY RATE STATE VARIABLES
C
C
C  ALGORITHM:
C
C  Develop elements of the recursion relations defined in section 4.5.2.2
C    The backward sweep is used to recursively compute the equivalent inertia
C    matrices and generalized force vectors which account for all the outboard
C    bodies.  The recursion is started by initializing M_tild_sub_i and
C    G_tild_sub_i to:
C                M_sub_i = AMU(I,J,ICBODY),I,J=1,...,MTSIZE
C                G_sub_i = GGV(LOCUIC+I),I=1,...,MTSIZE
C    Each step in the recursion involves computation of two temporary
C    variables and two accumulation operations.
C
C    For i=e,...,2
C        Define the tempory variables
C
C                         ~-1   T       ___
C                 A   =   M    B    cap_Phi
C                  1       i    Ii         Ii
C
C                            ___T        -1     ___T
C                 A   = (cap_Phi   B   A )  cap_Phi   B
C                  2            Ii  Ii  1          Ii  O,i-1
C
C        where the size of A_sub_1 is MTSIZExNCONS+NGW and
C              the size of A_sub_2 is NCONS+NGWxMTISIZ and
C              the order of the matrix inverse in A_sub_2 is NCONS+NGW
C
C        Define accumulation variables
C
C                ~        ~         T         ___
C                M    :=  M    +   B      cap_Phi   A
C                 i-1      i-1      O,i-1        Ii  2
C
C                ~        ~        T      ___T             T  ~
C                G    :=  G    +  A  (cap_Phi  cap_Xi  -  A   G )
C                 i-1      i-1     2         Ii      i     1   i
C
C  cap_Phi    = Selection matrix for the free hinge DOF's at the inboard
C         Ii    hinge of body i, defined in Section 4.2, equation 23.
C               Partition elements defined in section 3.1 equations 11-12
C               Size: 6+NGWxNFREE, where the 1,1 partition is 6xNFREE
C      ___
C  cap_Phi    = Selection matrix for the constrained hinge DOF's at the
C         Ii    inboard hinge of body i, defined Section 4,2, equation 24.
C               Partition elements defined in section 3.1 equations 11-12
C               Size: 6+NGWxNCONS+NGW, where the 1,1 partition is 6xNCONS
C
C                      +- ___      -+                 +-      -+
C               ___    |  Phi    0  |                 | Phi    |
C           cap_Phi  = |     Ii     |     cap_Phi  =  |    Ii  |
C                  Ii  |   0     1  |                 |   0    |
C                      +-          -+                 +-      -+
C
C      B      = Kinematics matrix for the directly inboard hinge on body i,
C       Ii      that is, body ICBODY.  Matrix is defined in section 4.2,
C               equation 23.  Partition elements b_sub_Ii are computed and
C               defined in the comments in subroutine BHGENR.
C               Size: NGWxMTSIZE, where the 1,1 partition is 6xLEN, the 2,1
C                     partition is NGWxLEN, and the 2,2 partition is NGWxNMW
C
C      B      = Kinematics matix for the directly outboard hinge on body i-1,
C       O,i-1   that is, body INBODY.  Matrix is defined in section 4.2,
C               equation 24.  Partition elements b_sub_O,i-1 are computed and
C               defined in the comments in subroutine BHGENR.
C               Size: NGWxMTISIZ, where the 1,1 partition is 6xILEN, the 2,1
C                     partition is NGWxILEN, and the 2,2 partition is NGWxNMWI
C
C                  +-             -+             +-                     -+
C                  |   b        0  |             |   b              0    |
C                  |    Ii         |             |    O,i-1              |
C         B    =   |   T        T  |    B      = |   T             T     |
C          Ii      |  P  b     W   |     O,i-1   |  P     b       W      |
C                  |   Ii Ii    Ii |             |   O,i-1 O,i-1   O,i-1 |
C                  +-             -+             +-                     -+
C
C       where    P_sub_Ii  is a gear ratio selection matrix which selects the
C                          geared DOF at the inboard hinge of body i and
C                          multiplies it by the gear ratio.  Size NGWx6
C                W_sub_Ii  is the selection matrix which selects the particular
C                          variable speed wheel in body i used for the rotor.
C                          Size NGWxNMW.
C
C
C    AMU(ICBODY) - From MGEN, mass matrix of order MTSIZE defined by equation
C                  II-109 in original DISCOS theory. wheel effects in last NMW
C                  row and columns.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
c     Include files
c
#include "PARAM.FOR"
c
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "CONSTR.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "JILFLG.INC"
#include "LDEBUG.INC"
#include "MISCNO.INC"
#include "MISCN2.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "PINRP.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
c
c     Passed
      DOUBLE PRECISION ZETA(6,cL02)
      DOUBLE PRECISION PIN(3,3,cL02)
      INTEGER IHTHPTR(2*cL02)
      INTEGER IHPVT(cL02)
      INTEGER ISEL(6*cL02)
      INTEGER ISDOF(9,cL02)
      INTEGER ITOPOL(2,cL02)
      INTEGER ibh_ptr(cL12+1)

c     Local Variables 
c
      INTEGER IBODY, ICBODY, INBODY, IHINGE
      INTEGER ITHISB,INBORD
      INTEGER NCONS, NM, inlen,mtsize
      INTEGER ISROW, LOCC, IELEM, IROW, ICOL
      INTEGER j, i, LOCUIC
      INTEGER iamu_ic,iamu_in,iptamu  ! Pointers for amu
      INTEGER irm_ic                  ! Pointer for rmbar
      INTEGER ibh_ic                  ! Pointer for bh
      LOGICAL LEQU,qot,useHATS      
      DOUBLE PRECISION RMTTMP(cL08,cL08), KPVTM(cL08)
      DOUBLE PRECISION TEMP(cL08), TEMP2(cL08)
      DOUBLE PRECISION RJTILD(6,6)
      INTEGER KPVTJ(6)
      DOUBLE PRECISION GGV(cL10)
      DOUBLE PRECISION ZWK(cL08)
      DOUBLE PRECISION RCOND, SUM
      DOUBLE PRECISION DDOT
      EQUIVALENCE (LEQU,DEBUG(75))
c
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE BCKWRD ')
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     PERFORM BACKWARD RECURSION
C
      DO 200 IBODY = NB, 1, -1
C
C        EXTRACT DATA ON THIS BODY AND ITS INBOARD HINGE
C
         ICBODY = ITOPW(2,IBODY)
         IHINGE = ITOPW(1,IBODY)
         NM     = IRGFLX(ICBODY)
         mtsize = LENU(ICBODY)

C
C        EXTRACT DATA ON THIS BODY AND ITS INBOARD HINGE
C
C        DETERMINE WHICH FRAME IS ON WHICH BODY
         IF (IHINGE .LT. 0) THEN      ! P FRAME IS ON THIS (OUTBOARD) BODY
            IHINGE = -IHINGE
            INBODY = ITOPOL(1,IHINGE)
            ITHISB = 2 * IHINGE - 1
            INBORD = 2 * IHINGE - 2
            qot = .FALSE.
         ELSE                         ! Q FRAME IS ON THIS (OUTBOARD) BODY
            INBODY = ITOPOL(2,IHINGE)
            ITHISB = 2 * IHINGE - 2
            INBORD = 2 * IHINGE - 1
C           THE ALGORITHM FOR THE OUTBOARD BODY IS DIFFERENT 
C           FOR BODY/HINGE 1
            IF (ITHISB .EQ. 0) ITHISB = 1
            qot = .TRUE.
         END IF
C
         IF (inbody .GE. 1) THEN
            inlen   = LENU(inbody)
            iamu_in = iamuptr(inbody) ! amu pointer
         ELSE
            iamu_in = BADADDR
         END IF
C
         iamu_ic = iamuptr(icbody) ! amu pointer
         irm_ic  = irm_ptr(icbody)  ! rmbar pointer 
c
c     if nm > 0, bh(1,7,ithisb) is used, get the proper pointer
c     to where it is stored
c
         useHATS = .FALSE.
         if (nm.GT.0 .AND. ihinge.NE.1) then
            if (qot) then
               if (ihpvt(ihinge) .EQ. 0) then
                  ibh_ic = ibh_ptr(ithisb) + 9
               else
                  ibh_ic = ibh_ptr(ithisb) + 18
               end if
            else
               if (ihpvt(ihinge) .EQ. 0) then
                  ibh_ic = ihthptr(ithisb-1) + 1 ! point to hats, this
                  useHATS = .TRUE.               ! is what is in bh here
               else      
                  ibh_ic = ibh_ptr(ithisb) + 18
               end if
            end if
         else
            ibh_ic = BADADDR
         end if
c
c     call bckwrd_1 to pass heap_mb variables as arrays
c
         if (useHATS) then
            call bckwrd_1(GGV,HEAP_MB(iamu_ic),HEAP_MB(iamu_in),
     &                 HEAP_MB(irm_ic),HATS(ibh_ic),mtsize,inlen,
     &                 icbody,inbody,inbord,qot,nm,ihinge,
     &                 HEAP_MB(ipsiptr),HEAP_MB(iyptr),
     $                 HEAP_MB(iydtptr),
     $                 LocalHeap(irmttmp_bckwrd1),LocalHeap(ikpvtm
     $                 ptr_bckwrd1),LocalHeap(itempptr_bckwrd1),
     $                 LocalHeap(itemp2ptr_bckwrd1),
     $                 LocalHeap(izwkptr_bckwrd1),
     $                 LocalHeap(ibhinbptr_bckwrd1),
     $                 LocalHeap(ibhoutptr_bckwrd1),
     $                 LoopHeap(irinvptr),LoopHeap(izetaptr),
     $                 intLoopHeap(iiselptr),intLoopHeap(iisdofptr))
         else
            call bckwrd_1(GGV,HEAP_MB(iamu_ic),HEAP_MB(iamu_in),
     &                 HEAP_MB(irm_ic),BH(ibh_ic),mtsize,inlen,
     &                 icbody,inbody,inbord,qot,nm,ihinge,
     &                 HEAP_MB(ipsiptr),HEAP_MB(iyptr),
     &		       HEAP_MB(iydtptr),
     &                 LocalHeap(irmttmp_bckwrd1),LocalHeap(ikpvtm
     $                 ptr_bckwrd1),LocalHeap(itempptr_bckwrd1),
     $                 LocalHeap(itemp2ptr_bckwrd1),
     $                 LocalHeap(izwkptr_bckwrd1),
     $                 LocalHeap(ibhinbptr_bckwrd1),
     $                 LocalHeap(ibhoutptr_bckwrd1),
     $                 LoopHeap(irinvptr),LoopHeap(izetaptr),
     $                 intLoopHeap(iiselptr),intLoopHeap(iisdofptr))
         end if
            


C
 200  CONTINUE
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     COMPUTE JTILDE FOR BODY NUMBER 1, IT IS NEEDED ONLY FOR THE
C     OPTIONAL LAMBDA CALCULATION.
C
      IF ((IPRNT.NE.NOPRNT) .OR. JIL .NE. 4) RETURN
C
C     EXTRACT DATA ON THIS BODY
      IBODY  = 1
      ICBODY = 1
      IHINGE = 1
C
      NM     = IRGFLX(ICBODY)
      MTSIZE = LENU(ICBODY)
C
C     Q FRAME IS ON THIS BODY
C
      ITHISB = 1
C
C     PULL OUT THIS HINGE'S SELECTION INFORMATION
      LOCC  = ISDOF(2,IHINGE) - 1
      NCONS = ISDOF(4,IHINGE)
      LOCUIC = LOCU(ICBODY) - 1
C
C        Since the backward recursion involves overwriting of AMU and GGV
C        for the inboard body by M and G tilde, the calculation of AMU and GGV
C        is complete for Body 1 at this point in the subroutine.
C        IF THE HINGE HAS CONSTRAINED DOFS, NOW COMPUTE J-TILDE.
C
      IF (NCONS .GT. 0) THEN
C
C        COPY RMTILD FOR THIS BODY TO A TEMP LOCATION.  IT WILL BE
C        OVERWRITTEN WHEN IT IS DECOMPOSED.
C
         DO ICOL = 1, MTSIZE
            DO IROW = 1, ICOL    ! only need upper triangle
               iptamu = iamuptr(icbody) + irow + mtsize*(icol-1)
               RMTTMP(IROW,ICOL) = HEAP_MB(iptamu) ! amu(irow,icol)
            END DO
         END DO
C
C        NOW DECOMPOSE THE COPIED ARRAY
         CALL DSICO (RMTTMP,cL08,MTSIZE,KPVTM,RCOND,ZWK)
         if (rcond+1d0 .eq. 1d0) then
            CALL CLEARERR()
            WRITE (ERRMSG,100) '3>> RCOND = ',rcond
            CALL ERRHDL(-5,'<BCKWRD>')
         end if
 100     FORMAT(1X,A15,E15.8E2)
C
         DO ICOL = 1, NCONS
            ISROW = ISEL(LOCC + ICOL)
C
c	inline use of BH(1)
            if (nbdy .EQ. 0) then   ! Only particles
               if (isrow .LE. 3) then
                  do ielem = 1, mtsize
                     temp(ielem) = 0.0d0
                  end do
                  temp(isrow) = 1.0d0
               end if
            else
               if (isrow .LE. 3) then
                  do ielem = 1,3
                     temp(ielem) = pin(isrow,ielem,1)
                  end do
                  DO IELEM = 4, MTSIZE
                     TEMP(IELEM) = 0.0d0
                  END DO
               else
                  do ielem = 1,3
                     temp(ielem) = 0.0d0
                     temp(ielem+3) = r0l(isrow-3,ielem,1)
                  end do
                  do ielem = 7,mtsize
                     temp(ielem) = 0.0d0
                  end do
               end if
            end if

            CALL DSISL (RMTTMP,cL08,MTSIZE,KPVTM,TEMP)
CC@CC
CC@CC ......MULT3 TEMP, GGV(LOCUIC+1), SUM
CC@CC ......1, MTSIZE, 1
CC@CC ......1, L08, 1
      SUM = DDOT (MTSIZE,TEMP,1,GGV(LOCUIC+1),1)
CC@CC end of expanded call
CC@CC
            TEMP2(ICOL) = ZETA(ISROW,IHINGE) - SUM
C
 300	    FORMAT(1X,A)
            DO IROW = 1, ICOL
               SUM = 0.0D0
               ISROW = ISEL(LOCC + IROW)

	       if (nbdy .EQ. 0) then   ! Only particles
                  if (isrow .LE. 3) then
                     sum = sum + temp(isrow)
                  else
                     CALL CLEARERR()
                     WRITE(ERRMSG,300) 'Bad size for hinge 1.'
                     CALL ERRHDL(-5,'<BCKWRD>')
                  end if
               else                    ! Body 1 is a body
                  if (isrow .LE. 3) then
                     if (mtsize .GE. 3) then
                        DO IELEM = 1, 3
                           SUM = SUM + pin(ISROW,IELEM,1)*TEMP(IELEM)
                        end do
                     else
                        CALL CLEARERR()
                        WRITE(ERRMSG,300) 'Bad size for hinge 1'
                        CALL ERRHDL(-5,'<BCKWRD>')
                     end if
                  else
                     if (mtsize .GE. 6) then
                        do ielem = 1,3
                           sum = sum + 
     &                           r0l(isrow-3,ielem,1)*temp(ielem+3)
                        end do
                     else
                        CALL CLEARERR()
                        WRITE(ERRMSG,300) 'Bad size for hinge 1.'
                        CALL ERRHDL(-5,'<BCKWRD>')
                     end if
                  end if
	       end if

               RJTILD(IROW,ICOL) = SUM
               RJTILD(ICOL,IROW) = SUM
            END DO
         END DO

         IF(LEQU) THEN
            WRITE (MBOUT,1050) 1
 1050       FORMAT (1X,' J TILDE FOR BODY ',I3)
            CALL WRITES(RJTILD,NCONS,NCONS,6)
            WRITE (MBOUT,1060) 1
 1060       FORMAT (1X,' VLAM FOR BODY ',I3)
            CALL WRITES(TEMP2,1,NCONS,1)
         ENDIF
C
C           J TILDE FOR THE CURRENT BODY HAS BEEN COMPUTED.
C           DECOMPOSE IT AND SAVE IT ALONG WITH TEMP2
C
         CALL DSICO (RJTILD,6,NCONS,KPVTJ,RCOND,ZWK)
         if (rcond+1d0 .eq. 1d0) then
            CALL CLEARERR()
            WRITE (ERRMSG,100) '4>> RCOND = ',rcond
            CALL ERRHDL(-5,'<BCKWRD>')
         end if
         DO J = 1, NCONS
            DO I = 1, J
               RJUPPR(I,J,ICBODY) = RJTILD(I,J)
            END DO
            JPVT(J,ICBODY) = KPVTJ(J)
            VLAM (J,ICBODY) = TEMP2(J)
         END DO
      END IF
C
      RETURN
      END     ! End body 1 for BCKWRD

CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     Beginning of BCKWRD_1
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      SUBROUTINE BCKWRD_1(GGV,amu_ic,amu_in,rmbar,bh17,iclen,inlen,
     &                    icbody,inbody,inbord,qot,nm,ihinge,
     &                    psi,y,ydt,rmttmp,kpvtm,temp,temp2,zwk,
     &                    bhinb,bhout,rinv,zeta,isel,isdof)
C
c     Include files
c
#include "PARAM.FOR"
c
#include "BHBSRD.INC"
#include "CONSTR.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "JILFLG.INC"
#include "LDEBUG.INC"
#include "HEAPMB.INC"
#include "MISCNO.INC"
#include "MISCN2.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "PINRP.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
c
c     Passed Variables
c
      INTEGER iclen,inlen,icbody,inbody,ihinge
      INTEGER inbord,nm
      INTEGER isel(6*cL02)
      INTEGER isdof(9,cL02)
      LOGICAL qot
      DOUBLE PRECISION amu_ic(iclen,iclen),amu_in(inlen,inlen)
      DOUBLE PRECISION rmbar(iclen,iclen),GGV(cL10) 
      DOUBLE PRECISION bh17(3,*)  ! passed location of bh(1,7,ithisb)
      DOUBLE PRECISION psi(6,cL06,MAXBOD)
      DOUBLE PRECISION y(cL07), ydt(cL07)
      DOUBLE PRECISION rinv(6,6,cL02)
      DOUBLE PRECISION zeta(6,cL02)
c
c     Local Variables 
c
      INTEGER NFREE
      INTEGER NCONS, NMI
      INTEGER LOCUIC, LOCUIN, LOCF
      INTEGER ISROW, LOCC, IELEM, IROW, ICOL, ISCOL
      INTEGER NDOF, MB
      INTEGER j, i
      DOUBLE PRECISION RMTTMP(cL08,cL08), KPVTM(cL08)
      DOUBLE PRECISION TEMP(cL08), TEMP2(cL08)
      DOUBLE PRECISION RJTILD(6,6)
      INTEGER KPVTJ(6)
      DOUBLE PRECISION ZWK(cL08)
      DOUBLE PRECISION RCOND, SUM
      DOUBLE PRECISION DDOT
      DOUBLE PRECISION bhout(6,cL08),bhinb(6,cL08)
      LOGICAL LEQU
      EQUIVALENCE (LEQU,DEBUG(75))
c
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE BCKWRD_1')
C         
C
C        PULL OUT THIS HINGE'S SELECTION INFORMATION
      LOCF  = ISDOF(1,IHINGE) - 1
      LOCC  = ISDOF(2,IHINGE) - 1
      NFREE = ISDOF(3,IHINGE)
      NCONS = ISDOF(4,IHINGE)
      NDOF  = NFREE + NCONS
C
      LOCUIC = LOCU(ICBODY) - 1
C
      if (ncons .GT. 0) then
         IF (inbody .GE. 1) nmi = irgflx(inbody)
         if (qot) then         ! Q-frame is outboard
            CALL getbh(ihinge,2,.TRUE.,bhout,icbody,nm,bhinb,inbody,nmi,
     *             intLoopHeap(iibhtypeptr),intLoopHeap(iihthptrptr),
     *             intLoopheap(iibh_ptrptr))
         else                  ! P-frame is outboard
            CALL getbh(ihinge,2,.TRUE.,bhinb,inbody,nmi,bhout,icbody,nm,
     *             intLoopHeap(iibhtypeptr),intLoopHeap(iihthptrptr),
     *             intLoopheap(iibh_ptrptr))
         end if
      end if
C
C        Since the backward recursion involves overwriting of AMU and GGV
C        for the inboard body by M and G tilde, the calculation of AMU and GGV
C        is complete for the current body at this point in the subroutine.
C        If the hinge has constrained DOFs now compute J-tilde and then this
C        body's contribution to the inboard body's M-tilde and G-tilde.
C
         IF (INBODY .NE. 0 .AND. NCONS .GT. 0) THEN
C
            LOCUIN  = LOCU(INBODY) - 1
            NMI     = IRGFLX(INBODY)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C           Copy M tilde for current body and decompose it for later use in
C           linear equation solution.
C
            DO ICOL = 1,ICLEN
               DO IROW = 1, ICOL ! only need upper triangle
                  RMTTMP(IROW,ICOL) = AMU_IC(irow,icol)
               END DO
            END DO
            CALL DSICO (RMTTMP,cL08,ICLEN,KPVTM,RCOND,ZWK)
            if (rcond+1d0 .eq. 1d0) then
               CALL CLEARERR()
               WRITE (ERRMSG,100) '1>> RCOND = ',rcond
               CALL ERRHDL(-5,'<BCKWRD_1>')
            end if
 100        FORMAT(1X,A15,E15.8E2)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C           Compute columns of the ICLENxNCONS matrix, A_sub_1.  Each
C           column is stored in vector TEMP of length ICLEN.  The column
C           vector is then used to build up the A_sub_2 matrix and G tilde
C           vector.  The vector TEMP is then overwritten by the next column
C           of A_sub_1.
C
C      Use ISEL contents to effect B(transpose)*cap_Phi multiplication
C      to obtain column ICOL of A1.  Store it in TEMP.
C
            DO 203 ICOL = 1, NCONS

C              Pick out selected column of B_sub_Ii_transpose.  Load in TEMP.
               ISROW = ISEL(LOCC + ICOL)
               DO 204 IELEM = 1, ICLEN
                  TEMP(IELEM) = BHOUT(ISROW,IELEM)
204            CONTINUE
C
C           Solve for column ICOL of A_sub_1
               CALL DSISL (RMTTMP,cL08,ICLEN,KPVTM,TEMP)
C
C           TEMP now contains the ICOL column of A_sub_1
C
C     Get the ICOL element of the associated zeta vector - projected
C     generalized torque.  Store result in TEMP2.
C
C                    ___T             T  ~
C                cap_Phi  cap_Xi  -  A   G
C                       Ii      i     1   i
C
CC@CC
CC@CC .........MULT3 TEMP, GGV(LOCUIC+1), SUM
CC@CC .........1, ICLEN, 1
CC@CC .........1, L08, 1
      SUM = DDOT (ICLEN,TEMP,1,GGV(LOCUIC+1),1)
CC@CC end of expanded call
CC@CC
               TEMP2(ICOL) = ZETA(ISROW,IHINGE) - SUM
C
C     Develop elements in upper left NCONSxNCONS partition of the order
C     NCONS symmetric matrix J_tilde associated with body i.  TEMP
C     contains the ICLEN elements of column ICOL of A_sub_1, ICOL=1,...,NCONS
C
C                                  ___T
C                J_tilde  =    cap_Phi   B   A
C                                     Ii  Ii  1
C
               DO 205 IROW = 1, ICOL
                  SUM = 0.0D0
                  ISROW = ISEL(LOCC + IROW)
                  DO 206 IELEM = 1, ICLEN
                     SUM = SUM + BHOUT(ISROW,IELEM) * TEMP(IELEM)
206               CONTINUE
                  RJTILD(IROW,ICOL) = SUM
                  RJTILD(ICOL,IROW) = SUM
205            CONTINUE
203         CONTINUE
C
C    Finished with first NCONS elements.
C
            IF(LEQU) THEN
              WRITE (MBOUT,1050) ICBODY
1050          FORMAT (1X,' J TILDE FOR BODY ',I3)
              CALL WRITES(RJTILD,NCONS,NCONS,6)
              WRITE (MBOUT,1060) ICBODY
1060          FORMAT (1X,' VLAM FOR BODY ',I3)
              CALL WRITES(TEMP2,1,NCONS,1)
            ENDIF
C
C  The calculation and application of the columns of A_sub_1 has been
C  completed.
C  J_tilde for body ICBODY has been computed.  It is of order NCONS.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C  Decompose RJTILD.  This is needed to compute A_sub_2 which is used for
C  the computation of the accumulation variables G_tilde and M_tilde.
C
            CALL DSICO (RJTILD,6,NCONS,KPVTJ,RCOND,ZWK)
            if (rcond+1d0 .eq. 1d0) then
               CALL CLEARERR()
               WRITE (ERRMSG,100) '2>> RCOND = ',rcond
               CALL ERRHDL(-5,'<BCKWRD_1>')
            end if
C
C  Save decomposition information for output and surface contact.
C
            IF ((IPRNT .EQ. NOPRNT) .AND. JIL .EQ. 4) THEN
               DO J = 1, NCONS
                  DO I = 1, J
                     RJUPPR(I,J,ICBODY) = RJTILD(I,J)
                  END DO
                  JPVT(J,ICBODY) = KPVTJ(J)
                  VLAM(J,ICBODY) = TEMP2(J)
               END DO
            END IF
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C           Compute columns of A_sub_2 and apply the result to the
C           accumulation of the inboard body's G tilde and M tilde.
C
C                ~        ~         T         ___
C                M    :=  M    +   B      cap_Phi   A
C                 i-1      i-1      O,i-1        Ii  2
C
C                ~        ~        T      ___T             T  ~
C                G    :=  G    +  A  (cap_Phi  cap_Xi  -  A   G )
C                 i-1      i-1     2         Ii      i     1   i
C
C                                                   ___
C      Use ISEL contents to effect B(transpose)*cap_Phi multiplication
C      Obtain column ICOL of order MTISIZ matrix M_tilde_sub_i-1
C

            DO 400 ICOL = 1, INLEN
C       Pick out NCONS+1 elements of B_sub_O,i-1_transpose, column ICOL
               DO 401 IELEM = 1, NCONS
                  TEMP(IELEM) = BHINB(ISEL(LOCC+IELEM),ICOL)
401            CONTINUE
C
C  Solve for column ICOL of A_sub_2 and store in TEMP, use it to get the
C  ICOL row element of the generalized force vector GGV(LOCUIN+ICOL)
C
               CALL DSISL (RJTILD,6,NCONS,KPVTJ,TEMP)
CC@CC
CC@CC .........MULTAD temp, temp2, ggv(locuin+icol)
CC@CC .........1, ncons, 1
CC@CC .........1, 7, 1
      ggv(locuin+icol) = ggv(locuin+icol) + DDOT (ncons,temp,1,temp2,1)
CC@CC end of expanded call
CC@CC
C
C  Use column ICOL of A_sub_2 in TEMP to obtain elements of M_tilde_sub_i-1
C  rows 1,...,ICOL, ICOL=1,...,INLEN.  Use symmetry to fill matrix

               DO 402 IROW = 1, ICOL
                  SUM = AMU_IN(irow,icol)
C  Now premultiply by the selected rows of of B_sub_O,i-1_transpose
                  DO 403 IELEM = 1, NCONS
                     SUM = SUM +
     +                     BHINB(ISEL(LOCC+IELEM),IROW)*TEMP(IELEM)
403               CONTINUE
                  AMU_IN(irow,icol) = SUM  
                  AMU_IN(icol,irow) = SUM  
402            CONTINUE
400         CONTINUE
C
C   Finished with all rigid and modal degrees of freedom for inbody.
C
            IF(LEQU) THEN
               WRITE (MBOUT,1010) INBODY
 1010          FORMAT (1X,' M TILDE FOR INBOARD BODY ',I3)
               CALL WRITES(AMU_IN,INLEN,INLEN,inlen) 
            ENDIF
C
C           The current body's contribution to the inboard body's
C           M and G tildes has been computed.
C
         END IF        ! End of IF(INBODY.NE.0.AND.NCONS.GT.0) test
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C        Now compute PSI (Only the upper right partition is stored).
C
C                  -1         |    -1
C                 R    PHI    |  -R    F
C        PSI  =    Ii     Ii  |    Ii   Ii
C           i     ____________|____________
C                             |
C                    0        |    I
C                             |     nm
C
         IF (NM .GT. 0) THEN   ! Not a particle, flex bodies only
CCC
CCC MULT3N rinv(1,1,ihinge),bh(1,7,ithisb),psi(1,1,icbody),ndof,ndof,nm,
CCC        6,6,6
CCC   (inline use of BH mode section)
CCC
            if (ndof .EQ. 6) then
               j = 4
            else
               j = 1
            end if 
            if (ihinge .GT. 1) then
CC@CC
CC@CC ...MULT3N rinv(1,j,ihinge), bh17, psi(1,1,icbody)
CC@CC ...ndof, 3, nm
CC@CC ...6, 3, 6
      CALL DGEMM ('N','N',ndof,nm,3,-1.0d0,rinv(1,j,ihinge),6,bh17,3,0.0
     .d0,psi(1,1,icbody),6)
CC@CC end of expanded call
CC@CC
            else   ! if it's the 1st hinge
               do j = 1, nm
                  do i = 1,ndof
                     psi(i,j,icbody) = 0.0d0
                  end do
               end do
            end if
c
            IF(LEQU) THEN
               WRITE (MBOUT,1030) ICBODY
 1030          FORMAT (1X,' PSI ARRAY FOR BODY ',I3)
               CALL WRITES(PSI(1,1,ICBODY),ndof,NM,6)
            END IF
         END IF

C
C        Use the whole PSI to compute RMBAR, recalling that only the 1,2
C        partition of PSI stored
C
C        _         T  ~
C        M   =  PSI   M   PSI
C         i        i   i     i
C
         DO 500 ICOL = 1, NFREE
            ISCOL = ISEL(LOCF + ICOL)
            DO 501 IROW = 1, ICLEN
               SUM = 0.0D0
               DO 502 IELEM = 1, ndof
                  SUM = SUM + AMU_IC(irow,ielem) * 
     *                 RINV(IELEM,ISCOL,IHINGE)        
 502           CONTINUE
               TEMP(IROW) = SUM
 501        CONTINUE
C     NOW MULTIPLY TEMP BY PSI**T, BY COLUMNS
C
C     1,1 BLOCK OF RMBAR
            DO 503 IROW = 1, ICOL
               ISROW = ISEL(LOCF + IROW)
               SUM = 0.0D0
               DO 504 IELEM = 1, ndof
                  SUM = SUM + RINV(IELEM,ISROW,IHINGE) * TEMP(IELEM)
 504           CONTINUE
               RMBAR(irow,icol)  = SUM
               RMBAR(icol,irow) = SUM
 503        CONTINUE
C
C           2,1 BLOCK OF RMBAR
            DO 505 IROW = 1, NM   ! Bodies only
               SUM = 0.0D0
               DO 506 IELEM = 1, ndof
                  SUM = SUM + PSI(IELEM,IROW,icbody) * TEMP(IELEM)
 506           CONTINUE
               RMBAR(nfree+irow,icol)  = SUM + TEMP(ndof+IROW)
               RMBAR(icol,nfree+irow) = RMBAR(nfree+irow,icol)
 505        CONTINUE
C
 500     CONTINUE
C
C        NOW DO THE NM COLUMNS OF PSI.
C
C  ( NM=0 for particle, if particle, goto 507)
C
         DO 507  ICOL = 1, NM   
            DO 508 IROW = 1, ICLEN
               SUM = AMU_IC(irow,ndof+icol)
               DO 509 IELEM = 1, ndof
                  SUM = SUM + AMU_IC(irow,ielem) * 
     &                       PSI(IELEM,ICOL,icbody)
 509           CONTINUE
               TEMP(IROW) = SUM
 508        CONTINUE
C
C           2,2 BLOCK OF RMBAR
            DO 512 IROW = 1, ICOL
               SUM = 0.0D0
               DO 513 IELEM = 1, ndof
                  SUM = SUM + PSI(IELEM,IROW,icbody) * TEMP(IELEM)
 513           CONTINUE
               RMBAR(nfree+irow,icol+nfree) = SUM + TEMP(ndof+IROW)
               RMBAR(icol+nfree,nfree+irow) = 
     &                                 RMBAR(nfree+irow,icol+nfree)
 512        CONTINUE
 507     CONTINUE
C
         IF(LEQU) THEN
            WRITE (MBOUT,1040) ICBODY
 1040       FORMAT (1X,' M-bar ARRAY RMBAR FOR BODY ',I3)
calisha ????            MB=NFREE+NM+MTSIZE-len (len=mtsize=iclen)
            MB=NFREE+NM
            CALL WRITES(RMBAR,MB,MB,ICLEN)
         ENDIF
C
C
      RETURN
      END      ! End of BKWRD_1


