C $Header: /export/cvs/c34a1/source/moldyn/bhgenr.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
C
      SUBROUTINE BHGENR(bhq,bhp,ibhtype,ihthptr,ihpvt,isdof,itopol,
     &                  ihdata,ibh_ptr)
CSE
C                                                          DEBUG #  45
CPS        PURPOSE:
C
C                   USED TO COMPUTE THE ELEMENTS OF THE BH ARRAY. THIS
C                   ARRAY DEFINES HINGE POINT KINEMATICS AND IS DEFINED
C                   IN VOL 1 BY EQUATIONS II-82 AND II-83, ON PAGE 36
C
C                  CORRECTION ADDED TO ACCOUNT FOR LINEAR VELOCITY OF
C                  Q-FRAME ORIGIN DUE TO ANGULAR VELOCITY OF P-FRAME
C
CPE
CAS
C
C         BHGENR - HINGE POINT KINEMATICS COEFFICIENTS
C         --------------------------------------------
C
C     BH     = VELOCITY TRANSFORMATION ARRAY. THE NON-ZERO PARTITIONS
C              OF THE VELOCITY TRANSFORMATION EQUATION II-84 ARE STORED
C              IN THIS ARRAY. THE ARRAY IS STORED IN /BSBSRD/ AND
C              DIMENSIONED (6,6+NMDBOD,2*NHMAX-1).
C              SET DEBUG(45) = .TRUE. TO PRINT
C         ((BH(I,J,1),I=1,6),J=1,6+IRGFLX(1)) = COMPONENTS OF THE (1,1)
C                             PARTITION OF THE COEFFICIENT MATRIX IN
C                             EQUATION II-84. RELATES TO THE Q-FRAME IN
C                             BODY 1 AT HINGE POINT 1, EVALUATED BY USE
C                             OF EQUATION II-83
C               AT HINGE POINT N0H, N0H=2,3...,NH LET
C                           LQ = 2*N0H-2
C                           LP = 2*N0H-1
C                            N = ITOPOL(1,N0H)  (Q-FRAME IN BODY N)
C                            M = ITOPOL(2,N0H)  (P-FRAME IN BODY M)
C                          NMQ = IRGFLX(N)      (NMQ MODES FOR BODY N)
C                          NMP = IRGFLX(M)      (NMP MODES FOR BODY M)
C         ((BH(I,J,LP),I=1,6),J=1,6+NMP) = COMPONENTS OF THE (N0H,M)
C                             PARTITION OF THE COEFFICIENT MATRIX IN
C                             EQUATION II-84. RELATES TO THE P-FRAME IN
C                             BODY M AT HINGE POINT N0H, EVALUATED BY
C                             USE OF EQUATION II-82
C         ((BH(I,J,LQ),I=1,6),J=1,6+NMQ) = COMPONENTS OF THE (N0H,N)
C                             PARTITION OF THE COEFFICIENT MATRIX IN
C                             EQUATION II-82. RELATES TO THE Q-FRAME IN
C                             BODY N AT HINGE POINT N0H, EVALUATED BY
C                             USE OF EQUATION II-82
C
C
CAE
      IMPLICIT NONE

      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: bhgenr.f,v $$Revision: 1.1.1.1 $'/
C
#include "PARAM.FOR"
C
#include "BHBSRD.INC"
#include "CHARMM.INC"
#include "ERRHDL.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "IFIRST.INC"
#include "INTGRL.INC"
#include "LDEBUG.INC"
#include "PARTCL.INC"
#include "PINRP.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
C
C     External variables   
#ifdef POINTER_KEYWORD
C subroutines mbfrePTR and mbalcPTR are declared in source/mbond/memory.f
      EXTERNAL mbfrePTR
      EXTERNAL mbalcPTR
#endif
c
c     passed
      INTEGER ibhtype(cL02)
      INTEGER ihthptr(2*cL02)
      INTEGER ihpvt(cL02)
      INTEGER isdof(9,cL02)
      INTEGER itopol(2,cL02)
      INTEGER ihdata(7,cL02)
      INTEGER ibh_ptr(cL12+1)
C
C     Local Variables
      INTEGER I1ST, K, ihinge
      INTEGER LP, LQ               ! hinge frame pointers
      INTEGER NOBP, NOBQ, NMP, NMQ
      INTEGER inc                  ! for initializing bh size
      INTEGER idim                 ! dimension of bh.
      INTEGER IP,IQ                ! bh pointers
      INTEGER ihatq,ihatp          ! hats pointers
      INTEGER ndof,i
      DOUBLE PRECISION dump        ! dummy arg for bh2
      DOUBLE PRECISION bhq(6,cL08),bhp(6,cL08) ! for printing
#ifdef POINTER_KEYWORD
      INTEGER nbytes               ! Dimension of BH * 8 bytes = nbytes of BH 
#endif
      INTEGER LEQU    
      INTEGER i1BH                ! Logical variable for BH
C
C     Equivalences
C      EQUIVALENCE (lequ,debug(45))
C
      EQUIVALENCE (i1BH,i1pass(8))
      EQUIVALENCE (i1st,i1pass(1))
C
      if (debug(45)) then
         lequ = 1
      else
         lequ = 0
      endif
C
C
      IF(LEQU.EQ.1) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE BHGENR ')
C
C     This section allocates space in the bh array.  Only non-zero
C     entries that aren't already stored somewhere else are calculated.
C     Particle information is either constant or the identity matrix
C     and is not saved.  GETBH or BHMULT called when BH needs to be used.
C     
C     ibh_ptr is common in BHBSRD, sized max # of frames (2*NH - 1).  
C            It stores the pointer to the beginning
C            of each stored section in BH.  Elements are 0 if nothing is
C            stored
C     ibhtype is common in BHBSRD, sized NH.  It is a code for what 
C            is stored in BH for later ease in manipulation.
c            (codes described in BHBSRD.INC)
C
c
      IF (i1st .EQ. 1) GO TO 100
      i1st = 1
C
C     Because BH is the last array on HEAPMB and its size is complex,
C     the indices of BH on HEAPMB are calculated here.  
C     isvptr is the next available space on HEAPMB (from memory.f).
C
C
C      ibh_ptr(1) = isvptr   ! Get pointer for first element of BH
C      INC        = isvptr
C
      ibh_ptr(1) = 1   
      INC        = 1
c
      IF (nbdy .EQ. 0) THEN ! Body 1 is a particle
         ibhtype(1) = 1
      ELSE
         ibhtype(1) = 0
      END IF
c
      DO ihinge = 2,NH
         NOBP = ITOPOL(2,ihinge)
         NOBQ = ITOPOL(1,ihinge)
         NMP  = IRGFLX(NOBP)
         NMQ  = IRGFLX(NOBQ)
         LQ   = 2*ihinge - 2
         LP   = LQ + 1
         IF (nobp .GT. nbdy) THEN      
            IF (nobq .GT. nbdy) THEN          ! Both particles
               IF (ihpvt(ihinge) .EQ. 0) THEN
                  ibhtype(ihinge) = 1
               ELSE
                  ibhtype(ihinge) = 2
               END IF
c
            ELSE                            ! 1 bdy/ 1 prtcl 
               IF (ihpvt(ihinge) .EQ. 0) THEN  ! No FBL
                  ibh_ptr(lq) = inc  
                  inc = inc + 3*(3 + NMQ)
c
                  IF (ihdata(1,ihinge) .EQ. 0) THEN
                     ibhtype(ihinge) = 3    ! Qframe inboard
                  ELSE
                     ibhtype(ihinge) = 4    ! Qframe outboard
                  END IF
c
               ELSE                         ! Yes FBL
                  ibh_ptr(lq) = inc  
                  inc = inc + 3*(6 + NMQ)
c
                  IF (ihdata(1,ihinge) .EQ. 0) THEN
                     ibhtype(ihinge) = 5    ! Qframe inboard
                  ELSE
                     ibhtype(ihinge) = 6    ! Qframe outboard
                  END IF
               END IF
            END IF
         ELSE                               ! Both are bodies
            IF (ihpvt(ihinge) .EQ. 0) THEN  ! No FBL
               ibh_ptr(lq) = inc
               inc = inc + 3*(3 + NMQ)
               ibh_ptr(lp) = inc
               inc = inc + 18
               ibhtype(ihinge) = 7
            ELSE                               ! Yes FBL
               ibh_ptr(lq) = inc
               inc = inc + 3*(6 + NMQ)
               ibh_ptr(lp) = inc
               inc = inc + 3*(6 + NMP)
               ibhtype(ihinge) = 8
            END IF
         END IF
      END DO
      ibh_ptr(2*nh) = inc               ! Save the end length
      idim = inc - 1
        
      IF (i1BH .EQ. 1) THEN
#ifdef POINTER_KEYWORD
        CALL mbfrePTR(bhptr)
#endif
        i1BH = 0
      ENDIF
#ifdef POINTER_KEYWORD
C dimensions of BH is inc and size of double precision is 8 bytes
      IF (idim .GT. 0) THEN
        nbytes = DBPRESZ * idim
        CALL mbalcPTR(bhptr, nbytes)
        IF (bhptr .LT. 1 ) THEN
         CALL CLEARERR()
         WRITE(ERRMSG,200)   
     +    'Do not have enough memory to allocate BH.'
         CALL ERRHDL(-5,'<BHGENR>')
        END IF   
        i1BH = 1
      ENDIF
 200  FORMAT(1X,A)
#else
      IF (idim .GT. MAX_BH) THEN 
         CALL CLEARERR()
         WRITE(MBOUT,*)
     +   'Do not have enough memory allocated for BH
     + in source/moldyn/PARAM.FOR'
         WRITE(ERRMSG,201)
     + 'Increase MAX_BH to > ', idim,
     +   ' in PARAM.FOR'
         CALL ERRHDL(-5,'<BHGENR>')
      END IF
 201  FORMAT(1X,A,I9,A)
#endif

C
C
      DO k = 1,idim             ! zero BH out
        BH(k) = 0.0d0
      END DO
c
  100 continue                          ! skip to here if not 1st time
C
      DO IHINGE=2,NH
         nobq = itopol(1,ihinge)
         nobp = itopol(2,ihinge)
c
         LQ = 2*ihinge - 2
         LP = LQ + 1
C
C        LQ = 2,4,6,...
C        LP = 3,5,7,...
C        BH(LQ) = EQUATION II-83, VOL 1
C        BH(LP) = EQUATION II-82, VOL 1
C
         iq    = ibh_ptr(LQ)        ! heap pointer to bh(q)
         ip    = ibh_ptr(LP)        ! heap pointer to bh(p)
         ihatq = ihthptr(lq-1) + 1  ! heap pointer to hats(q)
         ihatp = ihthptr(lp-1) + 1  ! heap pointer to hats(p)
c
c        iqsize = lenu(nobq)   length of q-frame
c        ipsize = lenu(nobp)   length of p-frame
c
         IF ((ip .EQ. 0) .AND. (iq .EQ. 0)) THEN
c        No need to call bh2....all constant or zero

         ELSE IF ((ip .EQ. 0) .AND. (iq .GT.0)) THEN
c        Nothing stored for p-frame, use dummy arg.
            CALL BH2(BH(iq),dump,HATS(ihatq),dump,ihinge,
     &               intLoopHeap(iihpvtptr),LoopHeap(ipinptr),
     &               intLoopHeap(iitopolptr),Loopheap(irhptr),
     &               Loopheap(idhptr))

         ELSE
c        Store something in both p and q frames (both bodies)
            CALL BH2(BH(iq),BH(ip),HATS(ihatq),
     &               HATS(ihatp),ihinge,intLoopHeap(iihpvtptr),
     &               LoopHeap(ipinptr),intLoopHeap(iitopolptr),
     &               Loopheap(irhptr),Loopheap(idhptr))

         END IF
C
C
      END DO
C
      IF (LEQU.NE.1) RETURN
      do i = 1, nh
         ndof = isdof(3,i) + isdof(4,i)
         lq = 2*i - 2
         lp = lq + 1
         nobq = itopol(1,i)
         nobp = itopol(2,i)
         if (i .gt. 1) then   
            call getbh(i,2,.TRUE.,bhq,nobq,irgflx(nobq),bhp,nobp,
     &               irgflx(nobp),intLoopHeap(iibhtypeptr),
     &               intLoopHeap(iihthptrptr),
     &               intLoopheap(iibh_ptrptr))
            write(mbout,1001) lp,i,nobp
            call writes(bhp,ndof,lenu(nobp),6)
         else    ! First hinge is only q-frame
            call getbh(i,1,.TRUE.,bhq,nobq,irgflx(nobq),bhp,0,0,
     *                 intLoopHeap(iibhtypeptr),
     *                 intLoopHeap(iihthptrptr),
     *                 intLoopheap(iibh_ptrptr))
         end if
         write(mbout,1001) lq,i,nobq
         call writes(bhq,ndof,lenu(nobq),6)
      end do
 1001 FORMAT(' BH(',I3,') FOR HINGE', I3, ' OF BODY', I3, ' IS :')

      RETURN
      END
