C $Header: /export/cvs/c34a1/source/moldyn/chkbon.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
c
      SUBROUTINE chkbon(iaconn,ibconn,ihconn,iconnh,ihpvt,itopol,stats)
c
c     Initialize the itopol array by checking with the covalent bonds between
c     bodies and connecting the resulting orphaned bodies.
c     Also, set up the iconnh, ihconn, and ibconn arrays.
c
      IMPLICIT NONE
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: chkbon.f,v $$Revision: 1.1.1.1 $'/
c
c     COMMON blocks
c
#include "PARAM.FOR"
c
#include "CSTIFF.INC"
#include "CPATHS.INC"
#include "ERRHDL.INC"
#include "HEAPMB.INC"
#include "PARTCL.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
c
c     passed
      integer stats
      integer iaconn(2,cL02)
      integer ibconn(2,cL02)
      integer ihconn(2,cL02)
      integer iconnh(cL02)
      integer ihpvt(cL02)
      integer itopol(2,cL02)
c     Local variables
c
      INTEGER i, j, i2   ! DO loop indices
      INTEGER ibond      ! Bond DO loop index
      INTEGER noh        ! Hinge DO loop index
      INTEGER ib1, ib2   ! Bodies associated with the bond atoms
      INTEGER iat, jat   ! Atom numbers of bonded atoms
      INTEGER lastIndx
#ifdef POINTER_KEYWORD
      INTEGER nbytes
#endif
c
      INTEGER       ntc,ntcc,ntf,ntid,ntn,ntnb,nsnb,ndfmin
        double precision tol,dtemp
      COMMON/mdcon/ ntc,ntcc,ntf,ntid,ntn,ntnb,nsnb,ndfmin,tol,dtemp
c
c
c     Initialize the first column of itopol to relate Body 1 to Inertial Frame
c
      itopol(1,1) = 1
      itopol(2,1) = 0
c
c     Set up the bonded connections between bodies by checking the bond
c     list to find which bonded atom pair has atoms on two different bodies.
c     Also identify this bonded pair as a hinge between the two bodies.
c
      DO i = 1, nconn
         iat = iaconn(1,i)
         jat = iaconn(2,i)
         itopol(1,i+1) = iftsmw(iat)
         itopol(2,i+1) = iftsmw(jat)
      END DO
      IF (nconn+1 .GT. cL02) THEN
         CALL CLEARERR()
         WRITE(ERRMSG,300) 
     +     ' Too many hinges or inter-body bonds'
         CALL ERRHDL(-5,'<CHKBON>')
      END IF
 300  FORMAT(1X,A)
      nh = nconn + 1
c
c     Find the bodies that are not yet in the itopol array and add them in.
c
      DO i = 1, nb
         DO j = 1, nh
            IF (itopol(1,j).EQ.i .OR. itopol(2,j).EQ.i) GO TO 105
         END DO
c
c     Body i is found to be 'isolated'.  Interface it to Body 1.
c
         nh = nh + 1
         IF (nh .GT. cL02) THEN
             stats = 0
             goto 1001
c            CALL CLEARERR()
c            WRITE(ERRMSG,300) 
c     +        ' Too many hinges or inter-body bonds'
c            CALL ERRHDL(-5,'<CHKBON>')
         END IF
         itopol(1,nh) = i
         itopol(2,nh) = 1
c
 105     CONTINUE
      END DO
c
c     Now traverse the system to see if there is a path from Body 1 to all 
c     bodies, and fix any orphaned bodies/groups.
c
      nctcon = 0
      lastIndx = 2 * cL02
#ifdef POINTER_KEYWORD
      nbytes = intgrsz * lastIndx
      CALL mbalcptr(chkbonPtr,nbytes)
      IF ( chkbonPtr .LE. 0 ) then
         CALL CLEARERR()
         WRITE(ERRMSG,200)
     *        'NOT ENOUGH MEMORY ALLOCATED FOR TEMPORARY
     *         IN CHKBON'
         CALL ERRHDL(-5,'<CHKBON>')
      end if
 200  FORMAT(1X,A)
#else
      if ( lastIndx .GT. MAX_chkbonHeap ) then
         WRITE(MBOUT,*)
     +     'Do not have enough memory allocated for chkbonHeap
     + in source/moldyn/PARAM.FOR'
         CALL CLEARERR()
         WRITE(ERRMSG,201)
     + 'Increase MAX_chkbonHeap to > ', lastIndx,
     +     ' in PARAM.FOR'

         CALL ERRHDL(-5,'<CHKBON>')
      end if
 201  FORMAT(1X,A,I9,A)
#endif

      CALL branch_out(itopol,chkbonHeap(1),stats)

#ifdef POINTER_KEYWORD
      CALL mbfreptr(chkbonPtr)
#endif
c
c     Establish body affiliation of each bonded pair of atoms
c
      do i = 1, nconn
         do i2 = 1, 2
            iat = iaconn(i2,i)
            do j = 1, nb
               if (iat.ge.iatoms(1,j) .and. iat.le.iatoms(2,j))
     *              ibconn(i2,i) = j
            end do
            if (ibconn(i2,i).eq.0) then
               CALL CLEARERR()
               WRITE(ERRMSG,300) 
     +           'Bond atom out of range.'
               CALL ERRHDL(-5,'<CHKBON>')
            end if
         end do
         if (ibconn(1,i) .eq. ibconn(2,i)) then
            CALL CLEARERR()
            WRITE(ERRMSG,300) 
     +        'Intra-fragment bonds are not allowed.'
            CALL ERRHDL(-5,'<CHKBON>')
         end if
      end do
      CALL writim(ibconn,2,nconn,'IBCONN - Body affiliation of bonds',2)
c
c     For particle simulations, set up the one hinge that is
c     affiliated with each bond
c
      do i = 1, cl02
         iconnh(i) = 0
      end do
      do 404 ibond = 1, nconn
         ib1 = ibconn(1,ibond)   ! Body associated with 1st bond atom
         ib2 = ibconn(2,ibond)   ! Body associated with 2nd bond atom
         do 403 noh = 2, nh
            if ((ib1 .eq. itopol(1,noh) .and. 
     +           ib2 .eq. itopol(2,noh)) .OR.
     +           (ib1 .eq. itopol(2,noh) .and.
     +           ib2 .eq. itopol(1,noh))) then  ! Two bodies of ibond match
                                                ! two bodies of hinge noh   
               do i = 1, ibond-1
                  if (noh .EQ. ihconn(1,i)) go to 403 ! Hinge noh already
                                                  ! used for another bond
               end do
               ihconn(1,ibond) = noh            ! Second row no longer used
               iconnh(noh)     = ibond
               ihpvt(noh)      = 1
               GO TO 404
            END IF
 403     CONTINUE
         WRITE(MBOUT,'(//A,I3,A,I3,A,I3//)')
     +        ' **** Creating new hinge for connection',
     +        ibond, ' between', ib1, ' and', ib2
         nh = nh + 1
         IF (nh .GT. cL02) THEN
            CALL CLEARERR()
            WRITE(ERRMSG,100) 'Too Many Hinges, NH =', nh
            CALL ERRHDL(-5,'<CHKBON>')
         END IF
         itopol(1,nh)    = ib1
         itopol(2,nh)    = ib2
         ihconn(1,ibond) = nh
         iconnh(nh)      = ibond
         ihpvt(nh)       = 1
 404  CONTINUE
      CALL writim(ihconn,1,nconn,
     .     'IHCONN - Hinge affiliation of bonds',2)
      CALL writim(iconnh,1,nh,
     .     'ICONNH - Bond affiliation of hinges',1)
c
 
 1001 RETURN
 100  FORMAT(1X,A25,I10)
 2050 FORMAT(//A,I3,A,I3,A,I3//)
      END

