C $Header: /export/cvs/c34a1/source/moldyn/clbck1.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
      SUBROUTINE CLBCK1(BTILDE,BHQ,BHP,IHTHPTR,IHPVT,ISEL,ISDOF,
     &                  ITOPOL,ibh_ptr,ihcutj)
C                                                  DEBUG #  84
CSE
      IMPLICIT NONE
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: clbck1.f,v $$Revision: 1.1.1.1 $'/
C
CPS   PURPOSE
C
C              Perform the first backward pass for closed loop configurations.
C     In addition to recursively computing the equivalent inertiae, this
C     subroutine also computes various coefficients for cut joint multipliers.
C     These are used in the solution of the multipliers and in the forward
C     recursion for the relative accelerations.  Some of the operations
C     performed are similar to what is in the subroutine BCKWRD.  The
C     recursion is performed starting from the end body (or bodies) and ending
C     with the base body.
C
CPE
CAS                           ARGUMENT LIST
C
C        BTILDE  =  Cut joint coefficients, these are partitions of the
C                   velocity transformation array "BH", recursion defined with
C                   equation 8, the modified kinetics equation.
C   ((BTILDE(I,J,IBODY,ICUT),I=1,6+IRGFLX(IBODY)),J=1,NCONS) = Elements
C                   extracted directly from BH array, (IBODY=1,...,NB),
C                   (ICUT=1,...,NCUT).  Used during the backward recursion
C                   pass to compute inertia, gyroscopic and constraint loads.
C           AMU  =  Equivalent inertia matrices M_tilda sub i, recursion
C                   defined with equation 8, the modified kinetics equation.
C         RMBAR  =  Transformed inertia matrices M_bar sub i, defined with
C                   equation 10, the intermediate body recursion equation for
C                   relative acceleration
C           SBT  =  Script B_sub ic transpose matrix, to be used on second
C                   forward loop for closed loop topologies, equation 19
C            RN  =  Cap N_sub i matrix, to be used on second forward loop for
C                   closed loop topologies, equation 20
C           PSI  =  Upper right partition of the transformation matrices
C                   "PSI", defined with equation 3
C
C  Reference equation numbers from Cambridge Research Memo: Hon Chun to
C    James D. Turner, "Preliminary Order (N) Algorithm for Multiple
C    Closed-Loop Topologies," 16 May 1989
C
CAE
C
#include "PARAM.FOR"
C
C     DISCOS COMMON BLOCKS
C
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "CLOOP.INC"
#include "CONSTR.INC"
#include "CPATHS.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "LDEBUG.INC"
#include "HEAPMB.INC"
#include "NOUTPT.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
C
c     passed
c
      DOUBLE PRECISION BTILDE(cL11,6,L01,cL63)
      INTEGER IHTHPTR(2*cL02)
      INTEGER IHPVT(cL02)
      INTEGER ISEL(6*cL02)
      INTEGER ISDOF(9,cL02)
      INTEGER ITOPOL(2,cL02)
      INTEGER ibh_ptr(cl12+1)
      INTEGER ihcutj(cL63)
c
C     External variables
C
C     LOCAL VARIABLES
C
      INTEGER iamu_ic,iamu_in,irm_ic,ibh_ic     ! heap pointers 
      INTEGER ICUT, NFREE,NCONS,NDOF,LOCC,IQ,IP
      INTEGER IHINGE, IBODY, IBODYQ,IBODYP
      INTEGER ICOL, ISCOL, IROW
      INTEGER ICBODY, INBODY, NM
      INTEGER INBORD,ITHISB
      INTEGER MTSIZE, MTISIZ
      LOGICAL LEQU,qot,useHATS
C
C     LOCAL ARRAYS
C
      DOUBLE PRECISION   BHQ(6,cL08), BHP(6,cL08)
C
C     Equivalences
C
      EQUIVALENCE (LEQU,DEBUG(84))
C
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE CLBCK1 ')
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     INITIALIZE BTILDE ARRAY
C
C       For the end bodies assoicated with each cut joint
C
C                 ~T          T  ___
C                 b    =    b    PHI
C                  i,c       O,i    c
C
C      for all other bodies BTILDE is zero.
C
      DO 103 ICUT = 1, NCUT
         IHINGE = IHCUTJ(ICUT)
         NFREE = ISDOF(3,IHINGE)
         NCONS = ISDOF(4,IHINGE)
         NDOF  = NFREE + NCONS
         LOCC   = ISDOF(2,IHINGE) - 1
         IQ     = 2*IHINGE - 2
         IP     = 2*IHINGE - 1
C
         IBODYQ = ITOPOL(1,IHINGE)
         IBODYP = ITOPOL(2,IHINGE)
C
         IF (NCONS .GT. 0) THEN
            CALL getbh(ihinge,2,.TRUE.,bhq,ibodyq,irgflx(ibodyq),bhp,
     &             ibodyp,irgflx(ibodyp),intLoopHeap(iibhtypeptr),
     &             intLoopHeap(iihthptrptr),intLoopheap(iibh_ptrptr))
         END IF
C
         DO 104 IBODY = 1, NB
            MTSIZE = LENU(IBODY)
            IF (IBODY .EQ. IBODYQ) THEN
               DO 107 ICOL = 1, NCONS
                  ISCOL = ISEL(LOCC + ICOL)
                  DO 108 IROW = 1, MTSIZE
                     BTILDE(IROW,ICOL,IBODY,ICUT) = BHQ(ISCOL,IROW)
 108              CONTINUE
 107           CONTINUE
            ELSE IF (IBODY .EQ. IBODYP) THEN
               DO 109 ICOL = 1, NCONS
                  ISCOL = ISEL(LOCC + ICOL)
                  DO 110 IROW = 1, MTSIZE
                     BTILDE(IROW,ICOL,IBODY,ICUT) = BHP(ISCOL,IROW)
 110              CONTINUE
 109           CONTINUE
            ELSE
               DO 105 ICOL = 1, NCONS
                  DO 106 IROW = 1, MTSIZE
                     BTILDE(IROW,ICOL,IBODY,ICUT) = 0.0D0
 106              CONTINUE
 105           CONTINUE
            END IF
 104     CONTINUE
 103  CONTINUE
C     
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     NOW FOR EACH BODY, COMPUTE M AND J TILDE
C
      DO 200 IBODY = NB, 1, -1
C
C
C         IBODY - DO LOOP SEQUENCE NUMBER, NB,NB-1,...,1
C        ICBODY - CURRENT BODY NUMBER
C            NM - NUMBER OF FLEXIBLE BODY MODES FOR BODY ICBODY
C        INBODY - INBOARD BODY NUMBER
C        IHINGE - CONNECTING HINGE NUMBER
C
C        EXTRACT INFORMATION ON THIS BODY
C
         ICBODY = ITOPW(2,IBODY)
         IHINGE = ITOPW(1,IBODY)
         NM     = IRGFLX(icbody)
C     
         IF (IHINGE .LT. 0) THEN    ! Pframe is on this body
            IHINGE = -IHINGE
            INBODY = ITOPOL(1,IHINGE)
            INBORD = 2 * IHINGE - 2
            ITHISB = INBORD + 1
            QOT    = .FALSE.
         ELSE                       ! Qframe is on this body
            INBODY = ITOPOL(2,IHINGE)
            ITHISB = 2 * IHINGE - 2
            INBORD = ITHISB + 1
C           THE ALGORITHM FOR ITHISB IS DIFFERENT FOR BODY/HINGE 1
            IF (ITHISB .EQ. 0) ITHISB = 1
            QOT    = .TRUE.
         END IF

         iamu_ic = iamuptr(icbody)     ! amu pointers
         iamu_in = iamuptr(inbody)
         irm_ic  = irm_ptr(icbody)     ! rmbar pointer
         mtsize  = lenu(icbody)        ! amu/rmbar size
         mtisiz  = lenu(inbody)
         useHATS = .FALSE.
c
c     get pointer to mode section of BH
c
         if (nm .GT. 0) then
            if (qot) then       ! q is outboard
               if (ihpvt(ihinge) .EQ. 0) then
                  ibh_ic = ibh_ptr(ithisb) + 9
               else
                  ibh_ic = ibh_ptr(ithisb) + 18
               end if
            else       ! p is outboard
               if (ihpvt(ihinge) .EQ. 0) then
c                 mode section of BH = -hats here
                  ibh_ic = ihthptr(ithisb-1) + 1   ! part of HATS store the same info. as BH.
                  useHATS = .TRUE.                 ! referencing to HATS for BH.   
               else
                  ibh_ic = ibh_ptr(ithisb) + 18
               end if
            end if
         else
            ibh_ic = BADADDR
         end if
c
c     Pass heap_mb location of amu,rmbar, and bh to clbck1_1
c
         if (useHATS) then
           call clbck1_1(HEAP_MB(iamu_ic),HEAP_MB(iamu_in),
     &                 HEAP_MB(irm_ic),HATS(ibh_ic),icbody,
     &                 inbody,mtsize,mtisiz,ithisb,ihinge,nm,qot,
     &                 HEAP_MB(ipsiptr),intHEAP_MB(ikmbpvtptr),
     &                 HEAP_MB(ibtildeptr),HEAP_MB(isbtptr),
     &                 HEAP_MB(ibrnptr),
     &                 LocalHeap(ibhinbptr_clbck1),
     &                 LocalHeap(ibhoutptr_clbck1),
     &                 LocalHeap(irmttmpptr_clbck1),
     &                 LocalHeap(ia1ptr_clbck1),
     &                 LocalHeap(ia3ptr_clbck1),
     &                 LocalHeap(itempptr_clbck1),
     &                 LocalHeap(itempaptr_clbck1),
     &                 LocalHeap(itempa2ptr_clbck1),
     &                 intLocalHeap(ikpvtmptr_clbck1),
     &                 logicLoopHeap(iihpathptr),
     &                 LoopHeap(irinvptr),intLoopheap(iiselptr),
     &                 intLoopHeap(iisdofptr),
     &                 intLoopheap(iihcutjptr))
         else
           call clbck1_1(HEAP_MB(iamu_ic),HEAP_MB(iamu_in),
     &                 HEAP_MB(irm_ic),BH(ibh_ic),icbody,
     &                 inbody,mtsize,mtisiz,ithisb,ihinge,nm,qot,
     &                 HEAP_MB(ipsiptr),intHEAP_MB(ikmbpvtptr),
     &                 HEAP_MB(ibtildeptr),HEAP_MB(isbtptr),
     &                 HEAP_MB(ibrnptr),
     &                 LocalHeap(ibhinbptr_clbck1),
     &                 LocalHeap(ibhoutptr_clbck1),
     &                 LocalHeap(irmttmpptr_clbck1),
     &                 LocalHeap(ia1ptr_clbck1),
     &                 LocalHeap(ia3ptr_clbck1),
     &                 LocalHeap(itempptr_clbck1),
     &                 LocalHeap(itempaptr_clbck1),
     &                 LocalHeap(itempa2ptr_clbck1),
     &                 intLocalHeap(ikpvtmptr_clbck1),
     &                 logicLoopHeap(iihpathptr),
     &                 LoopHeap(irinvptr),intLoopHeap(iiselptr),
     &                 intLoopHeap(iisdofptr),
     &                 intLoopheap(iihcutjptr))
         end if
C
 200  CONTINUE
C
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     Beginning of clbck1_1
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC 
      SUBROUTINE CLBCK1_1(amu_ic,amu_in,rmbar,bh17,icbody,inbody,mtsize,
     &                    mtisiz,ithisb,ihinge,nm,qot,psi,kmbpvt,
     &                    btilde,sbt,brn,bhinb,bhout,rmttmp,a1,a3,
     &                    temp,tempa,tempa2,kpvtm,ihpath,rinv,isel,
     &                    isdof,ihcutj)
C
C                                                  DEBUG #  84
CSE
      IMPLICIT NONE
C
#include "PARAM.FOR"
C
C     DISCOS COMMON BLOCKS
C
#include "BHBSRD.INC"
#include "CLOOP.INC"
#include "CONSTR.INC"
#include "CPATHS.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "LDEBUG.INC"
#include "NOUTPT.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
C     External variables
      EXTERNAL ERRHDL 
C
C
C     Passed Variables
C
      INTEGER mtsize,mtisiz,icbody,inbody,nm,ihinge,ithisb
      INTEGER isel(6*cL02)
      INTEGER isdof(9,cL02)
      INTEGER ihcutj(cL63)
      LOGICAL QOT
      LOGICAL IHPATH(cL63+1,cL02)
      DOUBLE PRECISION amu_ic(mtsize,mtsize),amu_in(mtisiz,mtisiz)
      DOUBLE PRECISION rmbar(mtsize,mtsize)
      DOUBLE PRECISION bh17(3,*)   ! mode section only
      DOUBLE PRECISION psi(6,cL06,MAXBOD)
      integer KMBPVT(cl08,l01)
      DOUBLE PRECISION BTILDE(cL11,6,L01,cL63)
      DOUBLE PRECISION SBT(cL11,6,L01,cL63)
      DOUBLE PRECISION BRN(cL11,6,MAXBOD)
      DOUBLE PRECISION RINV(6,6,cL02)

C
C     LOCAL VARIABLES
C
      INTEGER INFO
      INTEGER I, J, K
      INTEGER icprt,inprt            ! pointers for particle arrays
      INTEGER ICUT, IHINGC
      INTEGER ICOL, ISCOL, IROW, ISROW, IELEM
      INTEGER NCONS, NCONC, LOCC, LOCF
      INTEGER NFREE, NDOF
      LOGICAL LEQU
      DOUBLE PRECISION SUM
C
C     LOCAL ARRAYS
C
      DOUBLE PRECISION   RMTTMP(cL11,cL11)
      DOUBLE PRECISION   A1(cL11,6)
      DOUBLE PRECISION   A2(6,6)
      DOUBLE PRECISION   A3(6,cL11)
      DOUBLE PRECISION   A4(6,6)
      DOUBLE PRECISION   TEMP(cL11)
      DOUBLE PRECISION   TEMPA(cL11,6), TEMPA2(cL11,6)
      DOUBLE PRECISION   BHINB(6,cL08),BHOUT(6,cL08) 
      INTEGER   KPVTM(cL11),KPVTA2(6)
C
C     Equivalences
C
      EQUIVALENCE (LEQU,DEBUG(84))
C
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE CLBCK1 ')
C
C
C        DETERMINE WHICH FRAME IS ON WHICH BODY
C
c
C
C        PULL OUT THIS HINGE'S SELECTION INFORMATION
C
         LOCF  = ISDOF(1,IHINGE) - 1
         LOCC  = ISDOF(2,IHINGE) - 1
         NFREE = ISDOF(3,IHINGE)
         NCONS = ISDOF(4,IHINGE)
         NDOF  = NFREE + NCONS
C
         IF (NCONS .GT. 0) THEN
            IF (QOT) THEN
               CALL getbh(ihinge,2,.TRUE.,bhout,icbody,nm,bhinb,inbody,
     &              irgflx(inbody),intLoopHeap(iibhtypeptr),
     &              intLoopHeap(iihthptrptr),intLoopheap(iibh_ptrptr))
            ELSE
               CALL getbh(ihinge,2,.TRUE.,bhinb,inbody,irgflx(inbody),
     &              bhout,icbody,nm,intLoopHeap(iibhtypeptr),
     &              intLoopHeap(iihthptrptr),intLoopheap(iibh_ptrptr))
            END IF
         END IF
         icprt  = icbody - nbdy
         inprt  = inbody - nbdy
C
C        BECAUSE OF OVERWRITING OF AMU AND GGV BY M AND G TILDE, THE 
C        CALCULATION OF THESE VARIABLES FOR THE CURRENT BODY IS COMPLETE AS
C        OF THIS POINT IN THE CODE.  IF THE HINGE HAS
C        CONSTRAINED DOFS NOW COMPUTE J-TILDE AND THEN THIS BODY'S
C        CONTRIBUTION TO THE INBOARD BODY'S M-TILDE AND G-TILDE.
C
         IF (LEQU) THEN
            WRITE (MBOUT,800) ICBODY
 800        FORMAT (' EQUIVALENT INERTIA M-TILDE_i, i = ',I3)
            CALL WRITES (AMU_IC(1,1),MTSIZE,MTSIZE,MTSIZE)
         END IF
C
C           COPY RMTILD FOR THIS BODY TO A TEMP LOCATION, WHICH WILL BE
C           OVER-WRITTEN WHEN DECOMPOSED.
C
         DO 201 ICOL = 1, MTSIZE
            DO 202 IROW = 1, ICOL
               RMTTMP(IROW,ICOL) = AMU_IC(IROW,ICOL)
 202        CONTINUE
 201     CONTINUE
C
C           NOW DECOMPOSE THE COPIED ARRAY
C
         CALL DSIFA (RMTTMP,cL11,MTSIZE,KPVTM,INFO)
         IF (INFO .NE. 0) THEN
            CALL CLEARERR()
            WRITE(ERRMSG,300)      
     +        ' MATRIX RMTTMP SINGULAR (1)'
            CALL ERRHDL(-5,'<CLBCK1_1>')
         END IF
 300     FORMAT(1X,A)
C
         IF (NCONS .EQ. 0) GO TO 910
C
C                   ~-1  T  ___
C           A  =    M   b   PHI
C            1       i   Ii    Ii
C
C
C                   ___T         -1
C           A  =  [ PHI   b   A ]
C            2         Ii  Ii  1
C
C
C        USE ISEL CONTENTS TO EFFECT PHI, PHI(TRANSPOSE) MULTIPLICATION
C
         DO 203 ICOL = 1, NCONS
            ISROW = ISEL(LOCC + ICOL)
C
            DO 204 IELEM = 1, MTSIZE
               A1(IELEM,ICOL) = BHOUT(ISROW,IELEM)
 204        CONTINUE
C
            CALL DSISL (RMTTMP,cL11,MTSIZE,KPVTM,A1(1,ICOL))
C
            DO 205 IROW = 1, ICOL
               SUM = 0.0D0
               ISROW = ISEL(LOCC + IROW)
               DO 206 IELEM = 1, MTSIZE
                  SUM = SUM + BHOUT(ISROW,IELEM) * A1(IELEM,ICOL)
 206           CONTINUE
               A2(IROW,ICOL) = SUM
 205        CONTINUE
 203     CONTINUE
C
         IF (LEQU) THEN
            WRITE (MBOUT,1050) ICBODY
 1050       FORMAT (' A2 FOR BODY ',I3)
            CALL WRITES(A2,NCONS,NCONS,6)
         ENDIF
C
C     DECOMPOSE A2 AND SAVE
C
         CALL DSIFA (A2,6,NCONS,KPVTA2,INFO)
         IF (INFO .NE. 0) THEN
            CALL CLEARERR()
            WRITE(ERRMSG,300)
     +        ' A2 MATRIX IS SINGULAR (2)'
            CALL ERRHDL(-5,'<CLBCK1_1>')
         END IF
C
C                   N  =  A * A
C                    i     1   2
C

         IF (icbody .le. nbdy) THEN
            DO IROW = 1, MTSIZE
               DO ICOL = 1, NCONS
                  TEMP(ICOL) = A1(IROW,ICOL)
               END DO
               CALL DSISL (A2,6,NCONS,KPVTA2,TEMP)
               DO ICOL = 1, NCONS
                  BRN(IROW,ICOL,ICBODY) = TEMP(ICOL)
               END DO
            END DO
         ELSE
            IF (ncons .gt. 3) THEN
               CALL CLEARERR()
               WRITE(ERRMSG,100) '> NCONS for particle', icprt,
     $             ',  is greater than 3'
               CALL ERRHDL(-5,'<CLBCK1_1>')
            END IF
 100        FORMAT(1X,A,I9,A)
            DO IROW = 1, MTSIZE
               DO ICOL = 1, NCONS
                  TEMP(ICOL) = A1(IROW,ICOL)
               END DO
               CALL DSISL (A2,6,NCONS,KPVTA2,TEMP)
               DO ICOL = 1, NCONS
                  PRN(IROW,ICOL,ICPRT) = TEMP(ICOL)
               END DO
            END DO
         END IF
C
         IF (LEQU) THEN
            WRITE (MBOUT,902) ICBODY
 902        FORMAT (' N_i MATRIX RN - ICBODY = ',I3)
            IF (icbody .le. nbdy) THEN
               CALL WRITES(BRN(1,1,ICBODY),MTSIZE,NCONS,cL11)
            ELSE
               CALL WRITES(PRN(1,1,ICPRT),MTSIZE,NCONS,3)
            END IF
         ENDIF
C
         IF (INBODY .NE. 0) THEN
C
C                             ___ T
C                   A  = A  * PHI   * b
C                    3    2      Ii    O,i-1
C
            DO 904  ICOL = 1, MTISIZ
               DO 905  IROW = 1, NCONS
                  ISROW = ISEL(LOCC + IROW)
                  A3(IROW,ICOL) = BHINB(ISROW,ICOL)
 905           CONTINUE
               CALL DSISL (A2,6,NCONS,KPVTA2,A3(1,ICOL))
 904        CONTINUE
C
C              NOW COMPUTE THIS BODY'S CONTRIBUTIONS TO THE INBOARD BODY
C
C                   ~       ~       T     ___
C                   M    =  M    + b      PHI   * A
C                    i-1     i-1    O,i-1    Ii    3
C
            DO J = 1, MTISIZ
               DO I = 1, J
                  SUM = AMU_IN(J,I)
                  DO K = 1, NCONS
                     ISROW = ISEL(LOCC + K)
                     SUM   = SUM + BHINB(ISROW,J)*A3(K,I)
                  END DO
                  AMU_IN(J,I) = SUM
                  AMU_IN(I,J) = SUM
               END DO
            END DO
C
C              THE CURRENT BODY'S CONTRIBUTION TO THE INBOARD BODY'S
C              M TILDE HAS BEEN COMPUTED.
C
         END IF        ! (INBODY .NE. 0) test
C
 910     CONTINUE
C
C     NOW COMPUTE THE CUT JOINT DEPENDENT QUANTITIES.
C
         DO 907 ICUT = 1, NCUT
            IF (.NOT. IHPATH(ICUT,IHINGE)) GO TO 907
C
C        DETERMINE THIS CUT-JOINT'S HINGE NUMBER
C
            IHINGC = IHCUTJ(ICUT)
            NCONC  = ISDOF(4,IHINGC)
C
            IF (NCONC .EQ. 0) GO TO 907
            IF (NCONS .GT. 0) THEN
C
C                         T   ~T
C                   A  = A  * b
C                    4    1    ic
C
CC@CC
CC@CC .........MULT3T A1, BTILDE(1,1,ICBODY,ICUT), A4
CC@CC .........NCONS, MTSIZE, NCONC
CC@CC .........L11, L11, 6
      CALL DGEMM('T','N',NCONS,NCONC,MTSIZE,1.0d0,A1,cL11,BTILDE(1,1,ICB
     .ODY,ICUT),cL11,0.0d0,A4,6)
CC@CC end of expanded call
CC@CC
            END IF
C
C
C                    T     ~-1  ~ T     *
C                   B   =  M    b   -  M   v
C                    ic     i    ic         2c
C
            DO 908 ICOL = 1, NCONC
               DO 909  IELEM = 1, MTSIZE
                  SBT(IELEM,ICOL,ICBODY,ICUT) =
     +                 BTILDE(IELEM,ICOL,ICBODY,ICUT)
 909           CONTINUE
               CALL DSISL(RMTTMP,cL11,MTSIZE,KPVTM,
     +              SBT(1,ICOL,ICBODY,ICUT))
 908        CONTINUE
C
            IF (NCONS .GT. 0) THEN
               IF (icbody .le. nbdy) THEN
CC@CC
CC@CC .........MULT3 BRN(1,1,ICBODY), A4, TEMPA2
CC@CC .........MTSIZE, NCONS, NCONC
CC@CC .........L11, 6, L11
      CALL DGEMM('N','N',MTSIZE,NCONC,NCONS,1.0d0,BRN(1,1,ICBODY),cL11,A
     .4,6,0.0d0,TEMPA2,cL11)
CC@CC end of expanded call
CC@CC
               ELSE
CC@CC
CC@CC MULT3 PRN(1,1,ICPRT), A4, TEMPA2
CC@CC MTSIZE, NCONS, NCONC
CC@CC 3, 6, L11
      CALL DGEMM('N','N',MTSIZE,NCONC,NCONS,1.0d0,PRN(1,1,ICPRT),3,A4,6
     .,0.0d0,TEMPA2,cL11)
CC@CC end of expanded call
CC@CC
               END IF
               CALL MSUB(SBT(1,1,ICBODY,ICUT),TEMPA2,
     +              SBT(1,1,ICBODY,ICUT),MTSIZE,NCONC,cL11)
            END IF
C
            IF (INBODY.EQ.0 .OR. NCONS.EQ.0) GO TO 907
C
C                    ~ T       ~ T         T
C                    b      =  b       -  A   * A
C                     i-1,c     i-1,c      3     4
C
CC@CC
CC@CC ......MULT3T A3, A4, TEMPA
CC@CC ......MTISIZ, NCONS, NCONC
CC@CC ......6, 6, L11
      CALL DGEMM('T','N',MTISIZ,NCONC,NCONS,1.0d0,A3,6,A4,6,0.0d0,TEMPA
     .,cL11)
CC@CC end of expanded call
CC@CC
            CALL MSUB(BTILDE(1,1,INBODY,ICUT),TEMPA,
     +           BTILDE(1,1,INBODY,ICUT),MTISIZ,NCONC,cL11)
C
 907     CONTINUE
C
         IF (LEQU) THEN
            DO ICUT = 1, NCUT
               IHINGC = IHCUTJ(ICUT)
               NCONC  = ISDOF(4,IHINGC)
               WRITE (MBOUT,820) ICBODY,ICUT
 820           FORMAT (' SCRIPT B_ic, MATRIX SBT - ICBODY = ',I5,
     *           ', ICUT = ',I5)
               CALL WRITES (SBT(1,1,ICBODY,ICUT),MTSIZE,NCONC,cL11)
               WRITE (MBOUT,830) ICBODY,ICUT
 830           FORMAT (' b-TILDE_ic, MATRIX BTILDE - ICBODY =',I5,
     *              ', ICUT = ',I5)
               CALL WRITES(BTILDE(1,1,ICBODY,ICUT),MTSIZE,NCONC,cL11)
            END DO
         ENDIF
C
C
C        Now compute PSI (Only the upper right partition is stored).
C
C                  -1         |    -1
C                 R    PHI    |  -R    F
C        PSI  =    Ii     Ii  |    Ii   Ii
C           i     ____________|____________
C                             |
C                    0        |    I
C                             |     nm
C
C        COMPUTE AND STORE THE UPPER RIGHT PARTITION OF PSI.
C
         IF (NM .GT. 0 .AND. ITHISB .NE. 1) THEN
CCC
CCC MULT3N rinv(1,1,ihinge),bh(1,7,ithisb),psi(1,1,icbody),ndof,ndof,nm,
CCC        6,6,6
CCC
            if (ndof .EQ. 6) then
               j = 4
            else
               j = 1
            end if 
CC@CC
CC@CC ...MULT3N rinv(1,j,ihinge), bh17, psi(1,1,icbody)
CC@CC ...ndof, 3, nm
CC@CC ...6, 3, 6
      CALL DGEMM ('N','N',ndof,nm,3,-1.0d0,rinv(1,j,ihinge),6,bh17,3,0.0
     .d0,psi(1,1,icbody),6)
CC@CC end of expanded call
CC@CC
            IF (LEQU) THEN
               WRITE (MBOUT,1030) ICBODY
 1030          FORMAT (1X,' PSI ARRAY FOR BODY ',I3)
               CALL WRITES(PSI(1,1,ICBODY),NDOF,NM,6)
            ENDIF
         END IF
C
C        USE THE WHOLE PSI TO COMPUTE RMBAR, RECALLING THAT ONLY THE
C        1,2 PARTITION OF PSI IS STORED.
C
C        _         T  ~
C        M   =  PSI   M   PSI
C         i        i   i     i
C
         DO 500 ICOL = 1, NFREE
C
            ISCOL = ISEL(LOCF + ICOL)
C
            DO 501 IROW = 1, MTSIZE
               SUM = 0.0D0
               DO 502 IELEM = 1, NDOF
                  SUM = SUM + AMU_IC(IROW,IELEM) *
     *                 RINV(IELEM,ISCOL,IHINGE)
 502           CONTINUE
               TEMP(IROW) = SUM
 501        CONTINUE
C
C     NOW MULTIPLY TEMP BY PSI**T, BY COLUMNS
C
C           1,1 BLOCK OF RMBAR
C
            DO 503 IROW = 1, ICOL
               ISROW = ISEL(LOCF + IROW)
               SUM = 0.0D0
               DO 504 IELEM = 1, NDOF
                  SUM = SUM + RINV(IELEM,ISROW,IHINGE) * TEMP(IELEM)
 504           CONTINUE
               RMBAR(irow,icol) = SUM
 503        CONTINUE
C
 500     CONTINUE
C
C        NOW DO THE NM COLUMNS OF PSI.
C
C  ( NM = 0 for particle, goto 507 if particle)
C
         DO 507  ICOL = 1, NM
            DO 508 IROW = 1, MTSIZE
               SUM = AMU_IC(IROW,NDOF+ICOL)
               DO 509 IELEM = 1, NDOF
                  SUM = SUM + AMU_IC(IROW,IELEM) *
     *                 PSI(IELEM,ICOL,ICBODY)
 509           CONTINUE
               TEMP(IROW) = SUM
 508        CONTINUE
C
C           1,2 BLOCK OF RMBAR
C
            DO IROW = 1, NFREE
               ISROW = ISEL(LOCF + IROW)
               SUM = 0D0
               DO IELEM = 1, NDOF
                  SUM = SUM + RINV(IELEM,ISROW,IHINGE) * TEMP(IELEM)
               END DO
               RMBAR(irow,nfree+icol) = SUM
            END DO
C
C           2,2 BLOCK OF RMBAR
C
            DO 512 IROW = 1, ICOL
               SUM = TEMP(NDOF+IROW)
               DO 513 IELEM = 1, NDOF
                  SUM = SUM + PSI(IELEM,IROW,ICBODY) * TEMP(IELEM)
 513           CONTINUE
               RMBAR(nfree+irow,nfree+icol) = SUM
 512        CONTINUE
 507     CONTINUE
C
         IF (NFREE+NM .GT. 0) THEN
            IF (LEQU) THEN
               WRITE (MBOUT,1040) ICBODY
 1040          FORMAT (' M-bar ARRAY RMBAR FOR BODY ',I3)
               CALL WRITES(RMBAR,NFREE+NM,NFREE+NM,MTSIZE)
            END IF
C
C        DECOMPOSE THE M BAR MATRIX
C
            CALL DSIFA (RMBAR,MTSIZE,NFREE+NM,
     +              KMBPVT(1,ICBODY),INFO)
            IF (INFO .NE. 0) THEN
               CALL CLEARERR()
               WRITE(ERRMSG,300)
     +           ' MATRIX RMBAR SINGULAR IN (3)'
               CALL ERRHDL(-5,'<CLBCK1_1>')
            END IF
         END IF      ! (NFREE + NM > 0) test
C
C        SAVE THE DECOMPOSED A2 MATRIX FOR G TILDE CALCULATION IN CLBCK2
C
         DO J = 1, NCONS
            JPVT(J,ICBODY) = KPVTA2(J)
            DO I = 1, J
               RJUPPR(I,J,ICBODY) = A2(I,J)
            END DO
         END DO
C
C
      RETURN
      END     ! End of clbck1_1

