C $Header: /export/cvs/c34a1/source/moldyn/clbck2.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
      SUBROUTINE CLBCK2 (GGV,SU,SBT,BRN,BHINB,BHOUT,KPVTM,RMTTMP,
     &                   ZETA,ISEL,ISDOF,ITOPOL)
C                                                  DEBUG #  84
CSE
      IMPLICIT NONE
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: clbck2.f,v $$Revision: 1.1.1.1 $'/
C
CPS   PURPOSE
C
C              Perform the backward pass for closed loop configurations.
C     This subroutine computes the equivalent generalized force vectors.
C     The operations performed are similar to what is in the subroutine BCKWRD.
C     The recursion is performed starting from the end body (or bodies) and
C     ending with the base body.
C
CPE
CAS                           ARGUMENT LIST
C
C           AMU  =  Equivalent inertia matrices M_tilde sub i, recursion
C                   defined with equation 8, the modified kinetics equation.
C           GGV  =  Equivalent generalized force vectors G_tilde sub i,
C                   recursion defined with equation 8, the modified kinetics
C                   equation.
C            SU  =  Script U_sub i vector, to be used on second forward loop
C                   for closed loop topologies, equation 18
C           SBT  =  Script B_sub ic transpose matrix, to be used on second
C                   forward loop for closed loop topologies, equation 19
C
C  Reference equation numbers from Cambridge Research Memo: Hon Chun to
C    James D. Turner, "Preliminary Order (N) Algorithm for Multiple
C    Closed-Loop Topologies," 16 May 1989
C
CAE
C
#include "PARAM.FOR"
C
C     ARGUMENTS
C
      DOUBLE PRECISION GGV(cL10)
      DOUBLE PRECISION ZETA(6,cL02)
      INTEGER ISEL(6*cL02)
      INTEGER ISDOF(9,cL02)
      INTEGER ITOPOL(2,cL02)
C
C     DISCOS COMMON BLOCKS
C
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "CONSTR.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "LDEBUG.INC"
#include "MISCNO.INC"
#include "NOUTPT.INC"
#include "NUMBRS.INC"
#include "ORDERN.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
C
c     Passed
c
      DOUBLE PRECISION SU(L01*cL11)
      DOUBLE PRECISION SBT(cL11,6,L01,cL63)
      DOUBLE PRECISION BRN(cL11,6,MAXBOD)
c
C     LOCAL VARIABLES
C
      INTEGER INFO
      INTEGER IBODY, ICBODY, INBODY, IHINGE,inbord,ithisb
      INTEGER ICOL, IROW, ISROW, IELEM
      INTEGER LOCF, LOCC, NFREE, NCONS, NDOF, LOCUIC, LOCUIN
      INTEGER LEN, MTSIZE, MTISIZ
      INTEGER icprt           ! Pointer for particle arrays
      INTEGER icamu           ! Pointer to beg. of amu body
      LOGICAL LEQU
      DOUBLE PRECISION SUM
C
C     LOCAL ARRAYS
C
      INTEGER            KPVTM(cL11)
      DOUBLE PRECISION   RMTTMP(cL11,cL11)
      DOUBLE PRECISION   V(6)
      DOUBLE PRECISION   bhinb(6,cL08),bhout(6,cL08)
C
C     Equivalences
C
      EQUIVALENCE (LEQU,DEBUG(84))
C
C
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE CLBCK2 ')
C
C     FOR EACH BODY, COMPUTE G TILDE AND SCRIPT U
C
      DO 200 IBODY = NB, 1, -1
C
C         IBODY - DO LOOP SEQUENCE NUMBER, NB,NB-1,...,1
C        ICBODY - CURRENT BODY NUMBER
C        INBODY - INBOARD BODY NUMBER
C        IHINGE - CONNECTING HINGE NUMBER
C
C        EXTRACT INFORMATION ON THIS BODY
C
         ICBODY = ITOPW(2,IBODY)
         IHINGE = ITOPW(1,IBODY)
C
C        DETERMINE WHICH FRAME IS ON WHICH BODY
C
         IF (IHINGE .LT. 0) THEN   ! Pframe is on this body
            IHINGE = -IHINGE
            INBODY = ITOPOL(1,IHINGE)
            ithisb = 2*ihinge - 1
            inbord = 2*ihinge - 2
c
            NCONS  = ISDOF(4,IHINGE)
            IF (NCONS .GT. 0) THEN
               CALL getbh(ihinge,2,.TRUE.,bhinb,inbody,irgflx(inbody),
     &                    bhout,icbody,irgflx(icbody),
     &                 intLoopHeap(iibhtypeptr),
     &                 intLoopHeap(iihthptrptr),
     &                 intLoopheap(iibh_ptrptr))
            END IF
         ELSE                     ! Qframe is on this body
            INBODY = ITOPOL(2,IHINGE)
c
C           THE ALGORITHM FOR THE OUTBOARD BODY IS DIFFERENT 
C           FOR BODY/HINGE 1, bhout and bhinb are the same
            NCONS = ISDOF(4,IHINGE)
            ithisb = 2*ihinge - 2
            inbord = 2*ihinge - 1
            IF (NCONS .GT. 0) THEN
               if (inbody .EQ. 0) then ! icbody is q-frame
                  CALL getbh(ihinge,1,.TRUE.,bhout,icbody,
     &                 irgflx(icbody),bhinb,0,0,
     &                 intLoopHeap(iibhtypeptr),
     &                 intLoopHeap(iihthptrptr),
     &                 intLoopheap(iibh_ptrptr))
               else
                  CALL getbh(ihinge,2,.TRUE.,bhout,icbody,
     &                 irgflx(icbody),bhinb,inbody,irgflx(inbody),
     &                 intLoopHeap(iibhtypeptr),
     &                 intLoopHeap(iihthptrptr),
     &                 intLoopheap(iibh_ptrptr))
               end if
            END IF
         END IF
C
C        PULL OUT THIS HINGE'S SELECTION INFORMATION
C
         LOCF  = ISDOF(1,IHINGE) - 1
         LOCC  = ISDOF(2,IHINGE) - 1
         NFREE = ISDOF(3,IHINGE)
         NDOF  = NFREE + NCONS
C
         LOCUIC = LOCU(ICBODY)
         MTSIZE = LENU(ICBODY)
         icprt  = icbody - nbdy
         icamu  = iamuptr(icbody)-1
C
C        BECAUSE OF OVERWRITING OF GGV WITH G TILDE, AS OF THIS POINT IN THE
C        CODE THE CURRENT BODY'S GGV ARRAY HOLDS THE G TILDE VECTOR. IF THE
C        HINGE HAS CONSTRAINED DOFS, NOW COMPUTE THIS BODY'S CONTRIBUTION TO
C        THE INBOARD BODY'S G TILDE.
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C           COPY RMTILD FOR THIS BODY TO A TEMP LOCATION, WHICH WILL BE
C           OVER-WRITTEN WHEN DECOMPOSED.
C
         DO 201 ICOL = 1, MTSIZE
            DO 202 IROW = 1, MTSIZE   ! amu(irow,icol)
               RMTTMP(IROW,ICOL) = HEAP_MB(icamu+IROW+(ICOL-1)*mtsize)
 202        CONTINUE
 201     CONTINUE
C
C           NOW DECOMPOSE THE COPIED ARRAY
C
         CALL DSIFA (RMTTMP,cL11,MTSIZE,KPVTM,INFO)
         IF (INFO .NE. 0) THEN
            CALL CLEARERR()
            WRITE(ERRMSG,300) 
     +        'MATRIX RMTTMP SINGULAR'
            CALL ERRHDL(-5,'<CLBCK2>')
         END IF
 300     FORMAT(1X,A)
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     INITIALIZE THE SCRIPT U VECTOR:
C
C                         ~-1 ~
C                   U  =  M   G   +  N  V
C                    i     i   i      i
C
         DO 906  IELEM = 1, MTSIZE
            SU(LOCUIC+IELEM-1) = GGV(LOCUIC+IELEM-1)
 906     CONTINUE
         CALL DSISL(RMTTMP,cL11,MTSIZE,KPVTM,SU(LOCUIC))
C
         IF (NCONS .EQ. 0) GO TO 200
C
C     COMPUTE THE INTERMEDIATE V VECTOR FOR BOTH SCRIPT U AND G TILDE:
C
C                ___T             ~-1 ~
C           V =  phi  { zeta - b  M   G }
C                   Ii      i   Ii i   i
C
         LEN = MTSIZE
         DO IROW = 1, NCONS
            ISROW = ISEL(LOCC + IROW)
            SUM   = ZETA(ISROW,IHINGE)
            DO 204 IELEM = 1, LEN
               SUM = SUM - BHOUT(ISROW,IELEM)*SU(LOCUIC+IELEM-1)
 204        CONTINUE
            V(IROW)           = SUM
            VLAM(IROW,ICBODY) = SUM
         END DO
C
C     FINISH OFF THE SCRIPT U VECTOR:
C
C                         ~-1 ~
C                   U  =  M   G   +  N  V
C                    i     i   i      i
C
         IF (icbody .le. nbdy) THEN
CC@CC
CC@CC ...MULTAD BRN(1,1,ICBODY), V, SU(LOCUIC)
CC@CC ...MTSIZE, NCONS, 1
CC@CC ...L11, 6, L11
      CALL DGEMV('N',MTSIZE,NCONS,1.0d0,BRN(1,1,ICBODY),cL11,V,1,1.0d0,S
     .U(LOCUIC),1)
CC@CC end of expanded call
CC@CC
         ELSE
CC@CC
CC@CC MULTAD PRN(1,1,ICPRT), V, SU(LOCUIC)
CC@CC MTSIZE, NCONS, 1
CC@CC 3, 6, L11
      CALL DGEMV('N',MTSIZE,NCONS,1.0d0,PRN(1,1,ICPRT),3,V,1,1.0d0,SU(L
     .OCUIC),1)
CC@CC end of expanded call
CC@CC
         END IF
C
         IF (LEQU) THEN
            WRITE (MBOUT,903) ICBODY,LOCUIC,MTSIZE
            CALL WRITES (SU(LOCUIC),1,MTSIZE,1)
         ENDIF
 903     FORMAT (' SCRIPT Ui VECTOR SU - ICBODY =',I3,
     +        ', LOCUIC =', I3, ', MTSIZE = ',I3)
C
         IF (INBODY .NE. 0) THEN
            LOCUIN = LOCU(INBODY)
            MTISIZ = LENU(INBODY)
C
C     COMPUTE THE INBOARD BODY'S G TILDE VECTOR
C
C     ~      ~       T    ___
C     G   := G    + b     phi   A  V
C      i-1    i-1    O,i-1   Ii  2
C
            CALL DSISL(RJUPPR(1,1,ICBODY),6,NCONS,
     +           JPVT(1,ICBODY),V)
C
            DO 904 IROW = 1, MTISIZ
               SUM = GGV(LOCUIN+IROW-1)
               DO 905 ICOL = 1, NCONS
                  ISROW = ISEL(LOCC + ICOL)
                  SUM = SUM + BHINB(ISROW,IROW)*V(ICOL)
 905           CONTINUE
               GGV(LOCUIN+IROW-1) = SUM
 904        CONTINUE
C
            IF (LEQU) THEN
               WRITE (MBOUT,810) INBODY
               CALL WRITES (GGV(LOCUIN),1,MTISIZ,1)
            END IF
 810        FORMAT (' ACCUMULATED VECTOR G-TILDE SUB i, i = ',I3)
C
         END IF          ! (INBODY .NE. 0) loop       
C
 200  CONTINUE
C
      RETURN
      END
