C $Header: /export/cvs/c34a1/source/moldyn/compin.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
C                                                          DEBUG # 124
C     PURPOSE:
c
c     compin_1.f: 
c     Compute the total mass, center of mass of each of the bodies, 
c     static (first) moments of inertia of the system, and the 
c     center of mass of the entire system.
c
c     compin_2.f:
c     Compute the system inertia matrix about the system center of mass
c     in inertial coordinates.  Find the prinicpal moments of 
c     inertia and associated rotation transformation.
c
c
      SUBROUTINE compin_1
c
      IMPLICIT NONE
C
#include "PARAM.FOR"
c
c     COMMON Blocks
c
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "CSTIFF.INC"
#include "COMPOS.INC"
#include "HEAPMB.INC"
#include "LDEBUG.INC"
#include "NUMBRS.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#ifdef CHARMM
#include "CHARMM.INC"
#endif
c
c     Local Variables
c
      INTEGER i, n                !  DO loop index
      INTEGER iamu,len            !  Amu pointer/size
      LOGICAL lequ                !  Debug flag
      DOUBLE PRECISION am         !  Body/particle mass
      DOUBLE PRECISION sbar(3)    !  First moment of inertia
      DOUBLE PRECISION wv(3)      !  Temp. working vector
      DOUBLE PRECISION vec3(3)    !  Temp. working vector
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: compin.f,v $$Revision: 1.1.1.1 $'/
c
c     Equivalences
c
      EQUIVALENCE (LEQU,DEBUG(124))
c
c     FORMATS
c
 1000 FORMAT (10X,'Subroutine compin_1 entered')
 1001 FORMAT (5X,'Location of composite system center of mass, wrt',
     &           ' inertial reference ')
c
      IF (lequ) WRITE(MBOUT,1000)
c
c     Compute the total mass, center of mass of each of the bodies, and 
c     static (first) moments of inertia of the system
c
      amt = ZRO
      DO i = 1, 3
         sbar(i) = ZRO
      END DO
c
      DO n = 1, nb
         IF (n.GT.nbdy) THEN    ! Its a particle
#ifdef CHARMM
            IF (PrtFix(i) .EQ. 0) THEN
#endif
               DO i = 1,3
                  cgnom (i,n) = d0l(i,n)
               END DO
               am = HEAP_MB(iamuptr(n))
               DO  i = 1, 3
                  sbar(i) = sbar(i) + am*d0l(i,n)
               END DO
#ifdef CHARMM
            ELSE
               am = 0
            END IF
#endif
         ELSE                   ! Its a body
            iamu=iamuptr(n)-1
            len=lenu(n)
            am    = HEAP_MB(iamu+4+3*len)  ! amu(4,4)
            wv(1) = HEAP_MB(iamu+3+4*len)  ! amu(3,5)
            wv(2) = HEAP_MB(iamu+1+5*len)  ! amu(1,6)
            wv(3) = HEAP_MB(iamu+2+3*len)  ! amu(2,4)
CC@CC
CC@CC ......MULT3 r0l(1,1,n), wv, vec3
CC@CC ......3, 3, 1
CC@CC ......3, 3, 3
      vec3(1) = r0l(1,1,n)*wv(1) + r0l(1,2,n)*wv(2) + r0l(1,3,n)*wv(3)
      vec3(2) = r0l(2,1,n)*wv(1) + r0l(2,2,n)*wv(2) + r0l(2,3,n)*wv(3)
      vec3(3) = r0l(3,1,n)*wv(1) + r0l(3,2,n)*wv(2) + r0l(3,3,n)*wv(3)
CC@CC end of expanded call
CC@CC
            DO i = 1, 3
               cgnom(i,n) = vec3(i)/am + d0l(i,n)
               sbar(i) = sbar(i) + vec3(i) + am*d0l(i,n)
            END DO
         END IF
         amt = amt + am
      END DO
c
c     Compute the system center of mass in inertial coordinates
c
      DO i = 1, 3
         d0c(i) = sbar(i)/amt
      END DO
c
      IF (lequ) THEN
         WRITE (MBOUT,1001)
         CALL writes(d0c,1,3,1)
      END IF
c
      RETURN
      END
c
c
      SUBROUTINE compin_2
c                                                          DEBUG # 124
      IMPLICIT NONE
c
#include "PARAM.FOR"
c
c     COMMON Blocks
c
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "CSTIFF.INC"
#include "COMPOS.INC"
#include "HEAPMB.INC"
#include "LDEBUG.INC"
#include "NUMBRS.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#ifdef CHARMM
#include "CHARMM.INC"
#endif
c
c     Local Variables
c
      INTEGER i,j,k,n             !  DO loop index
      INTEGER iamu,len            !  Amu pointer/size
      LOGICAL lequ                !  Debug flag
      DOUBLE PRECISION sum        !  Temp. variable
      DOUBLE PRECISION am         !  Body/particle mass
      DOUBLE PRECISION wv(3)      !  Temp. working vector
      DOUBLE PRECISION vec3(3)    !  Temp. working vector
      DOUBLE PRECISION vec6(6)    !  Temp. working vector
      DOUBLE PRECISION tr(6,3)    !  Transformation matrix for inertias
      DOUBLE PRECISION gam(3,3)   !  Temp. working matrix
      DOUBLE PRECISION ws(6,6)    !  Temp. working matrix
c
c     Equivalences
c
      EQUIVALENCE (LEQU,DEBUG(124))
      EQUIVALENCE (wv(1),vec6(1))
      EQUIVALENCE (vec3(1),vec6(4))
c
c     FORMATS
c
 1000 FORMAT (10X,'Subroutine compin_2 entered')
 1001 FORMAT (5X,'AJC system inertia tensor')
 1002 FORMAT (5X,' AJVAL principal moments of inertia')
 1003 FORMAT (5X,' AJVEC rotation transformation')
c
      IF (lequ) WRITE(MBOUT,1000)
c
c     Compute the system inertia matrix about system center of mass
c     in inertial coordinates
c     Compute the upper triangular part and reflect to lower triangle
c
      DO j = 1, 3
         DO i = 1, j
            ajc(i,j) = ZRO
         END DO
      END DO
c
      DO n = 1, nb
         iamu = iamuptr(n)-1
         IF (n.GT.nbdy) THEN !Its a particle
#ifdef CHARMM
         IF (PrtFix(n) .EQ. 0 .and. PrtCvel(n) .eq. 0) THEN ! Ignore dummy atoms
#endif
            am = HEAP_MB(iamu+1)  ! amu(n)
            DO i = 1, 3
               wv(i) = d0l(i,n) - d0c(i)
            END DO
            sum = am * wv(1)**2
            ajc(2,2) = ajc(2,2) + sum
            ajc(3,3) = ajc(3,3) + sum
            sum = am * wv(2)**2
            ajc(1,1) = ajc(1,1) + sum
            ajc(3,3) = ajc(3,3) + sum
            sum = am * wv(3)**2
            ajc(1,1) = ajc(1,1) + sum
            ajc(2,2) = ajc(2,2) + sum
            sum = am * wv(1)*wv(2)
            ajc(1,2) = ajc(1,2) - sum
            sum = am * wv(1)*wv(3)
            ajc(1,3) = ajc(1,3) - sum
            sum = am * wv(2)*wv(3)
            ajc(2,3) = ajc(2,3) - sum
#ifdef CHARMM
         ENDIF
#endif
         ELSE                    ! Its a body
            DO i = 1, 3
               wv(i) = d0l(i,n) - d0c(i)
            END DO
            CALL skewv3(wv,ws(1,1),3,6)
            DO i = 1, 3
               DO j = 1, 3
                  gam(i,j) = r0l(j,i,n)
               END DO
            END DO
            DO j = 1, 3
               DO i = 1, 3
                  tr(i,j) = gam(i,j)
               END DO
            END DO
CC@CC
CC@CC ......MULT3 gam, ws(1,1), tr(4,1)
CC@CC ......3, 3, 3
CC@CC ......3, 6, 6
      tr(4,1) = gam(1,1)*ws(1,1) + gam(1,2)*ws(2,1) + gam(1,3)*ws(3,1)
      tr(5,1) = gam(2,1)*ws(1,1) + gam(2,2)*ws(2,1) + gam(2,3)*ws(3,1)
      tr(6,1) = gam(3,1)*ws(1,1) + gam(3,2)*ws(2,1) + gam(3,3)*ws(3,1)
      tr(4,2) = gam(1,1)*ws(1,2) + gam(1,2)*ws(2,2) + gam(1,3)*ws(3,2)
      tr(5,2) = gam(2,1)*ws(1,2) + gam(2,2)*ws(2,2) + gam(2,3)*ws(3,2)
      tr(6,2) = gam(3,1)*ws(1,2) + gam(3,2)*ws(2,2) + gam(3,3)*ws(3,2)
      tr(4,3) = gam(1,1)*ws(1,3) + gam(1,2)*ws(2,3) + gam(1,3)*ws(3,3)
      tr(5,3) = gam(2,1)*ws(1,3) + gam(2,2)*ws(2,3) + gam(2,3)*ws(3,3)
      tr(6,3) = gam(3,1)*ws(1,3) + gam(3,2)*ws(2,3) + gam(3,3)*ws(3,3)
CC@CC end of expanded call
CC@CC
c
            len  = lenu(n)
            DO i = 1, 6
               DO k = 1, 6
                  vec6(k) = HEAP_MB(iamu+i+(k-1)*len) ! amu(i,k)
               END DO
               DO j = 1, 3
                  sum = ZRO
                  DO k = 1, 6
                     sum = sum + vec6(k)*tr(k,j)
                  END DO
                  ws(i,j) = sum
               END DO
            END DO
c
            DO j = 1, 3
               DO i = 1, j
                  sum = ZRO
                  DO k = 1, 6
                     sum = sum + tr(k,i)*ws(k,j)
                  END DO
                  ajc(i,j) = ajc(i,j) + sum
               END DO
            END DO
         END IF
      END DO
c
c     Copy system inertia matrix to array GAM
c
      DO j = 1, 3
         DO i = 1, j
            sum = ajc(i,j)
            gam(i,j) = sum
            ajc(j,i) = sum
            gam(j,i) = sum
         END DO
      END DO
c
c     CALL EIGN1 TO FIND PRINCIPAL MOMENTS OF INERTIA AND
c     ASSOCIATED ROTATION TRANSFORMATION
c
      sum = -ONE
      CALL eign1(gam,ajval,ajvec,3,sum,3)
c
      IF (lequ) THEN
         WRITE (MBOUT,1001)
         CALL writes(ajc,3,3,3)
         WRITE (MBOUT,1002)
         CALL writes(ajval,1,3,1)
         WRITE (MBOUT,1003)
         CALL writes(ajvec,3,3,3)
      END IF
c
      RETURN
      END
