C $Header: /export/cvs/c34a1/source/moldyn/dlespa.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
C                  SUBROUTINE - DLESPA.F                        
C    
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
c
      subroutine dlespa(amat,gvec,naa,ng,nx,nb,nfm,nh,nconn,nlm,
     &nctype,ihconn,itopol,named,namev,imbl,idbl,icbl,gmx,sm,
     &kpvtm,kpvtal,lnc,lnf)
c
c     DLESPA code solves the system of linear equations with the 
c     particular sparse matrix A ('amat') for subroutine cmpic2
c     (the position and velocity fit):
c
c     [  M     D'  ] {   dx   }     {   g   }
c     [            ] {        }  =  {       }
c     [  D     0   ] { lambda }     { -L/2  }
c
c        amat       *  unknown   =    gvec
c
c     DLESPA code takes advantage of partition and decomposes solution in
c     two steps: 
c     1).  A system of linear equations is solved for lambda:
c
c          [D*M(-1)*D']*lambda = D*M(-1)*g + L/2
c
c     2).  Systems of linear equations (for each body) are solved separately
c          for the I.C. vector dx (positions or velocities):
c
c          M*dx = g - D*lambda'  (M is block dioganal with respect to bodies)
c     
c     Note:  In the case of velocity fit:
c            D' = E' = [ED(constraints)' EV(momentums)'].
c     Note:  Partition of D with respect to constrains (bonds, angles,
c            dihedrals) is used, but only to reduce the storage for D  
c            while the block D*M(-1)*D' is handeled as a full matrix.
c     NOTE(!):  
c            This version of DLESPA code takes advantage of 
c            the SECOND LEVEL OF SPARSITY, i.e. only not-zero elements
c            of D*M(-1)*D' are stored and involved in solution of LE
c            (dllt.f and dlef.f, as substitution for dsifa.f and dsisl.f,
c            are written to handle the sparse system of LE).
c
c     DLESPA code is adjustable to any version of constraints in MBO(N)D
c     (e.g. it is possible to use arrays with nctype = 2 
c     for old versions with bond constraints).
c 
c     DLESPA code follows after sizes_amat.f which provides all sizes to
c     characterize the structure of amat.
c
c     Subroutine calls:
c        factor_b.f, cbod.f, vmispa.f, zero.f, dllt.f, dlef.f
c
c     The inputs are:
c     amat    Not-zero elements of matrix A stored in array (1 x naa) 
c             (see cmpic2.f).  The order of blocks is the following:
c             the body-blocks of M (taking into account their symmetry or
c             diagonality), not-zero body-blocks of D (ED),
c             the block EV (if any).
c     gvec    Vector [g' -L'/2]', size (1 x ng).
c     naa     "Storage" dimension of amat (number of not-zero elements).
c              Note:  While allocating amat, naa can be estimated given 
c                    a molecular structure and constraints.
c     ng      Dimension of gvec, (ng = nx + nlm).
c     nx      Size of I.C. vector.
c     nb      Number of bodies.
c     nfm     Number of parameters to be fitted for the most "flexible" body.
c     nh      Number of hinges.
c     nconn   Number of all constraints (bonds, angles, dihedrals).
c     nlm     == nconn for position fit, == nconn + 6 for velocity fit.
c     nctype  Type of constraints in allocation of arrays (2 - only bonds,
c             3 - angles are included,4 - dihedrals are included).  
c     ihconn  Matrix of hinge #s for each constraint, size (nctype x nconn).
c     itopol  Matrix of pairs of bodies' #s making a hinge, size (2 x nh).
c     named   "External" pointer for block D (or ED) in amat
c             Note:  "External pointer" sets the index of block's 
c                    first element in amat.
c     namev   "External" pointer for block EV in amat.
c     imbl    Auxiliary matrix to store sizes of body-blocks of M, (3 x nb),
c             1st row:  Size of each body-block (length of body-block of dx),
c             2nd row:  #s of first elements in body-blocks of dx,
c             3rd row:  amat's #s of first elements in body-blocks of M
c                       (Note:  for bodies - only triangular matrix is stored,
c                               for particles - only diagonal (3 x 1)).
c     idbl    Matrix for sizes of constraint-blocks of D, (3 x nconn)
c             1st row:  Number of bodies involved in constraint 
c                       (1 or 2 or 3 or 4),
c             2nd row:  Length of each not-zero constraint-block 
c                       (for all body-blocks involved in constraint),
c             3rd row:  amat's # of first element in not-zero 
c                       constraint-block of D.   
c     icbl    Auxiliary matrix to formalize the sparsity of D*M(-1)*D',
c             size (4 x nlm), (see dllt.f).
c     gmx     Auxiliary vector associated with body-blocks of M, (1 x nfm).
c     sm      Auxiliary matrix to operate with body-blocks of M, (nfm x nfm).
c     kpvtm,kpvtal
c             Auxiliary vectors of pivot indeces,
c             sizes (1 x nfm) and (1 x nx).
c     lnc, lnf
c             Maximal sizes for nconn and nfm set in the calling program.
c
c     The outputs are:
c     gvec    Solution to the linear system of equations [positions (or
c             velocities), modal coordinates (or velocities), as well as
c             Lagrange multipliers], size (1 x ng).
c
c     The internal variables are:
c     icob1,icob2
c             Vectors of #s of bodies involved in constraint 
c             (for two, "row's" and "column's", constraints).
c     ndmd    Number of distinct not-zero elements of D*M(-1)*D'.
c     inot0   Counter for distinct not-zero elements in D*M(-1)*D'.
c     inot0r  Counter for not-zero elements in a row of D*M(-1)*D'.
c     ifstr   Counter for the first element of constraint-block in dmd.
c
      IMPLICIT NONE
c
c     Include files
c
#include "PARAM.FOR"
#include "ERRHDL.INC"
#include "HEAPMB.INC"
#include "TAPENO.INC"
c
c     Declare variables:
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: dlespa.f,v $$Revision: 1.1.1.1 $'/
      integer naa,ng,nx,nb,nfm,nh,nconn,nlm,nctype     ! Inputs
      integer named,namev,lnc,lnf                      ! Inputs
      double precision amat(naa)                       ! Inputs
      double precision gvec(ng)                        ! Inputs/Outputs
      double precision gmx(nfm),sm(lnf,lnf)            ! Inputs
      double precision righ6(6)
      double precision left,left6,right,righ66
      double precision glmic, glm6
      integer ihconn(nctype,nconn),itopol(2,nh)        ! Inputs
      integer imbl(3,nb),idbl(3,nconn),icbl(4,nlm)     ! Inputs
      integer kpvtm(nfm),kpvtal(nx)                    ! Inputs
      integer icob1(4),icob2(4),icob(4)
      integer ifb,ic,ic1,ic2,icb,ix,i,i1,i2,i6
      integer ig,ii,jj,imbody,imbm
      integer ib1,ib2,ka,kaf,im,im1,im2,id,idb,kbb
      integer nbs,nbs1,nbs2,kboc,kbc
      integer iv,ivc,ivc1,ivc2,i61,i62,info
      integer ndmd,inform,nbytes
      integer inot0,inot0r,ifstr,n6ij
      integer iamdbx
c
c
c     1).  Perform dynamical allocation due to the structure of sparsity
c
c     Move declarations for pointers to HEAPMB.INC?
c
c     1.2).  Calculate parameter ndmd for dynamical allocation
c     (the number of not-zero elements in D*M(-1)*D')
      ndmd = 0
c
c_____1.2.1).  Organize cycles to overlap body-blocks of two constraints
c     Note:  Cycles are organazided only to compute ndmd and icbl(3,*)
c
      do ic = 1, nconn          
         icbl(3,ic) = 0
      end do
      do ic1 = 1, nconn         ! Skip if NO constraints have been set 
                                !  For "row's" constraint  
c
c        Define #s of different bodies involved in 'ic1'-th constraint
c        (#s are in the growing order, not more than 4 bodies)
         call cbod(ic1,nh,nconn,ihconn,itopol,nctype,icob1)
         nbs1 = idbl(1,ic1)
c         
         do ic2 = ic1, nconn    !  For "column's" constraint
c
c           Define #s of different bodies involved in 'ic2'-th constraint
c           (#s are in the growing order, not more than 4 bodies)
            call cbod(ic2,nh,nconn,ihconn,itopol,nctype,icob2)
c
c           Overlap not-zero body-blocks in 'ic1'-th and 'ic2'-th rows
            nbs2 = idbl(1,ic2)
            ka = 0
            do ii = 1, nbs1
               ib1 = icob1(ii)
               do jj = 1, nbs2
                  ib2 = icob2(jj)
                  if (ib1 .eq. ib2) then
                     ka = ka + 1
                     icob(ka) = ib1
c           Detect if first body in icob is not first in icob2  
                     if (ka .eq. 1) then
                        kaf = jj
                     end if
                  end if
               end do
            end do
            if (ka .ne. 0) then ! scip when "no overlap of body-blocks"
               ndmd = ndmd + 1
               icbl(3,ic2) = icbl(3,ic2) + 1
            end if
         end do
      end do
c
c     1.2.2).  Add momentum's not-zero elements (if any)
      if (namev .ne. 0) then
         ndmd = ndmd + 6*nconn + 21
         do i6 = 1, 6
            icbl(3,nconn+i6) = i6 + nconn
         end do
      end if
c
c     1.2.3).  Find icbl(4,*) and set counters to pack D*M(-1)*D'
      if (nlm .ne. 0) then
         icbl(4,1) = 1
         do ic = 2, nlm
            icbl(4,ic) = icbl(4,ic-1) + icbl(3,ic-1)
         end do
         inot0 = 1       !  Counter for not-zero elements in D*M(-1)*D'
         ifstr = 1       !  Counter for the first element of c-b in dmd.
      end if
c
c     1.3).  Allocate
#ifdef POINTER_KEYWORD
      nbytes = DBPRESZ*ndmd
      CALL mbAlcPtr(ndmd_p, nbytes)     ! dmd
      nbytes = DBPRESZ*nlm
      CALL mbAlcPtr(nlm_p, nbytes)      ! glm
      nbytes = INTGRSZ*ndmd
      CALL mbAlcPtr(icr_p, nbytes)      ! icrow
      nbytes = INTGRSZ*ndmd
      CALL mbAlcPtr(icc_p, nbytes)      ! iccol
      nbytes = DBPRESZ*nx
      CALL mbAlcPtr(nx_p, nbytes)       ! vecr
      CALL dlespa_1(pre_dmd,pre_glm,pre_icrow,pre_iccol,pre_vecr,
     &     amat,
     &     naa,ng,nx,nb,nfm,nh,nconn,nlm,nctype, ! Inputs
     &     named,namev,lnc,lnf,  ! Inputs
     &     gvec,   ! Inputs/Outputs
     &     gmx,sm, ! Inputs
     &     righ6,
     &     left,left6,right,righ66,
     &     glmic, glm6,
     &     ihconn,itopol, ! Inputs
     &     imbl,idbl,icbl, ! Inputs
     &     kpvtm,kpvtal, ! Inputs
     &     icob1,icob2,icob,
     &     ifb,ic,ic1,ic2,icb,ix,i,i1,i2,i6,
     &     ig,ii,jj,imbody,imbm,
     &     ib1,ib2,ka,kaf,im,im1,im2,id,idb,kbb,
     &     nbs,nbs1,nbs2,kboc,kbc,
     &     iv,ivc,ivc1,ivc2,i61,i62,info,
     &     ndmd,inform,nbytes,
     &     inot0,inot0r,ifstr,n6ij,
     &     iamdbx)

c     6).  Free dynamically allocated memory
      CALL mbFrePtr(ndmd_p)
      CALL mbFrePtr(nlm_p)
      CALL mbFrePtr(icr_p)
      CALL mbFrePtr(icc_p)
      CALL mbFrePtr(nx_p)

#else

      iptr_dmd   = lsqfit_ptr + 1
      iptr_glm   = iptr_dmd   + ndmd
      iptr_icrow = iptr_glm   + nlm
      iptr_iccol = iptr_icrow + (ndmd/2) + 1
      iptr_vecr  = iptr_iccol + (ndmd/2) + 1
      
      IF (iptr_vecr+nx .GT. MAX_cmpic2Heap) THEN
         WRITE(MBOUT,*)
     +     'Do not have enough memory allocated for cmpic2Heap
     +  in source/moldyn/PARAM.FOR'
         CALL CLEARERR()
         WRITE(ERRMSG,'(1X,A,I9,A)')
     + 'Increase MAX_cmpic2Heap to > ', iptr_vecr+nx,
     +     ' in PARAM.FOR'
         CALL ERRHDL(-5,'<DLESPA>')
      ENDIF

      CALL dlespa_1(heap_lsqfit(iptr_dmd),heap_lsqfit(iptr_glm),
     &     heap_lsqfit(iptr_icrow),heap_lsqfit(iptr_iccol),
     &     heap_lsqfit(iptr_vecr),heap_lsqfit(1),
     &     naa,ng,nx,nb,nfm,nh,nconn,nlm,nctype, ! Inputs
     &     named,namev,lnc,lnf,  ! Inputs
     &     gvec,   ! Inputs/Outputs
     &     gmx,sm, ! Inputs
     &     righ6,
     &     left,left6,right,righ66,
     &     glmic, glm6,
     &     ihconn,itopol, ! Inputs
     &     imbl,idbl,icbl, ! Inputs
     &     kpvtm,kpvtal, ! Inputs
     &     icob1,icob2,icob,
     &     ifb,ic,ic1,ic2,icb,ix,i,i1,i2,i6,
     &     ig,ii,jj,imbody,imbm,
     &     ib1,ib2,ka,kaf,im,im1,im2,id,idb,kbb,
     &     nbs,nbs1,nbs2,kboc,kbc,
     &     iv,ivc,ivc1,ivc2,i61,i62,info,
     &     ndmd,inform,nbytes,
     &     inot0,inot0r,ifstr,n6ij,
     &     iamdbx)

#endif

      RETURN
      END



C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::

      subroutine dlespa_1 (dmd,glm,icrow,iccol,vecr,amat,
     &   naa,ng,nx,nb,nfm,nh,nconn,nlm,nctype,
     &   named,namev,lnc,lnf,
     &   gvec,
     &   gmx,sm,
     &   righ6,
     &   left,left6,right,righ66,
     &   glmic, glm6,
     &   ihconn,itopol,
     &   imbl,idbl,icbl,
     &   kpvtm,kpvtal,
     &   icob1,icob2,icob,
     &   ifb,ic,ic1,ic2,icb,ix,i,i1,i2,i6,
     &   ig,ii,jj,imbody,imbm,
     &   ib1,ib2,ka,kaf,im,im1,im2,id,idb,kbb,
     &   nbs,nbs1,nbs2,kboc,kbc,
     &   iv,ivc,ivc1,ivc2,i61,i62,info,
     &   ndmd,inform,nbytes,
     &   inot0,inot0r,ifstr,n6ij,
     &   iamdbx)

c
      IMPLICIT NONE
c
c     Include files
c
#include "PARAM.FOR"
#include "ERRHDL.INC"
#include "HEAPMB.INC"
#include "TAPENO.INC"
c
c     Declare variables:
c
      integer naa,ng,nx,nb,nfm,nh,nconn,nlm,nctype     ! Inputs
      integer named,namev,lnc,lnf                      ! Inputs
      double precision gvec(ng)                        ! Inputs/Outputs
      double precision gmx(nfm),sm(lnf,lnf)            ! Inputs
      double precision righ6(6)
      double precision left,left6,right,righ66
      double precision glmic, glm6
      integer ihconn(nctype,nconn),itopol(2,nh)        ! Inputs
      integer imbl(3,nb),idbl(3,nconn),icbl(4,nlm)     ! Inputs
      integer kpvtm(nfm),kpvtal(nx)                    ! Inputs
      integer icob1(4),icob2(4),icob(4)
      integer ifb,ic,ic1,ic2,icb,ix,i,i1,i2,i6
      integer ig,ii,jj,imbody,imbm
      integer ib1,ib2,ka,kaf,im,im1,im2,id,idb,kbb
      integer nbs,nbs1,nbs2,kboc,kbc
      integer iv,ivc,ivc1,ivc2,i61,i62,info
      integer ndmd,inform,nbytes
      integer inot0,inot0r,ifstr,n6ij
      integer iamdbx


      double precision amat(*)
      double precision dmd(ndmd),glm(nlm),vecr(nx)
      integer icrow(ndmd),iccol(ndmd)

c
c_____2).  Perform factorization of body-blocks of M by dsifa.f 
c     (beforehand, since they will be used for solution of various LE systems)
      call factor_b(amat,naa,nx,nb,imbl,nfm,sm,kpvtm,kpvtal,lnf)
c
c_____3).  Form right-hand vector D*M(-1)*g + L/2 
c     and left-hand matrix D*M(-1)*D' for step #1 (LE for lambda)
c     Note:  D can be interpreted as D or [E(D),E(V)] or E(V)
c
c_____3.1).  Form not-zero elements of vector and matrix in LE for lambda,
c            associated with constraints 
c
c_____3.1.1).  Form initial arrays for right and left parts of LE
      do ic = 1, nlm
         glm(ic) = -gvec(nx+ic)
      end do
      call zero(dmd,ndmd,1,ndmd)
      call zero(vecr,nx,1,nx)
c
c_____3.1.2).  Organize cycles to overlap body-blocks of two constraints
c     Note:  Unlike the above cycles which were performed to compute
c            only the number ndmd of not-zero elements, now all not-zero
c            elements are calculated and are stored in dynamically
c            allocated memory.  Also, the structure of sparsity is
c            stored in dynamically allocated memory.
c
      do ic = 1, nconn           
         icbl(3,ic) = 0
      end do
      do ic1 = 1, nconn         ! Skip if NO constraints have been set
                                !  For "row's" constraint  
c
c        Define #s of different bodies involved in 'ic1'-th constraint
c        (#s are in the growing order, not more than 4 bodies)
         call cbod(ic1,nh,nconn,ihconn,itopol,nctype,icob1)         
c
c________3.1.3).  Perform sparse operations on "internal" body-blocks     
c 
c________3.1.3.1).  Compute z = D('ic1'-th row)*inv(M)
c        Note:  Invertion of M is avoided through solution of LE for z
c               taking into accout sparseness of M and D
c            
c        Form row of D (ED) for 'ic1'-th constraint
         im = idbl(3,ic1) - 1   ! pointer(-1) for 'ic1'-th row in amat 
         nbs1 = idbl(1,ic1)     ! number of not-zero b-b in 'ic1'-th row
         do ii = 1, nbs1
            ifb = icob1(ii)     ! Body's #
            imbody = imbl(1,ifb) ! Number of parameters in b-b
            kbb = imbl(2,ifb) - 1 ! Pointer(-1) for 'ifb'-th b-b in vecr
            do jj = 1, imbody
               vecr(kbb+jj) = amat(im+jj)
            end do 
            im = im + imbody    ! Move pointer(-1)
c                               ! to (ii+1)-th b-b in amat (D)
         end do   
c
c        Solve LE
         call vmispa(amat,vecr,naa,nx,nb,nbs1,nfm,imbl,
     &nctype,icob1,gmx,sm,kpvtm,kpvtal,lnf)        ! vecr is z now
            
c________3.1.3.2).  Compute L/2 + z*g
         right = glm(ic1)
         do ii = 1, nbs1
            ifb = icob1(ii)
            imbody = imbl(1,ifb)
            kbb = imbl(2,ifb) - 1
            do jj = 1, imbody
               right = right + vecr(kbb+jj)*gvec(kbb+jj)
            end do
         end do
c
c________3.1.3.3).  Compute z(body1)*EV'(body1) (if any)
         if (namev .ne. 0) then
            call zero(righ6,6,1,6) 
            do ii = 1, nbs1
               ifb = icob1(ii)
               imbody = imbl(1,ifb)
               kbb = imbl(2,ifb) - 1
               iv = namev + kbb - 1
c                         ! Pointer(-1) for 'ifb'-th b-b in amat (EV)
               do i6 = 1, 6
                  ivc = iv + nx*(i6-1) ! Tune pointer to current momentum
                  do jj = 1, imbody
                     righ6(i6) = righ6(i6) + vecr(kbb+jj)*
     &                    amat(ivc+jj)
                  end do
               end do
            end do
         end if
c
         inot0r = 0        !  Counter for not-zero elements in a row
c                          !  of D*M(-1)*D' from diagonal (including) 
         do ic2 = ic1, nconn    !  For "column's" constraint
c
c           Define #s of different bodies involved in 'ic2'-th constraint
c           (#s are in the growing order, not more than 4 bodies)
            call cbod(ic2,nh,nconn,ihconn,itopol,nctype,icob2)
c
c           Overlap not-zero body-blocks in 'ic1'-th and 'ic2'-th rows
            nbs2 = idbl(1,ic2)
            ka = 0
            do ii = 1, nbs1
               ib1 = icob1(ii)
               do jj = 1, nbs2
                  ib2 = icob2(jj)
                  if (ib1 .eq. ib2) then
                     ka = ka + 1
                     icob(ka) = ib1
c                    Detect if first body in icob is not first in icob2  
                     if (ka .eq. 1) then
                        kaf = jj
                     end if
                  end if
               end do
            end do
c
c___________3.1.3.4).  Compute z*D'('ic2'-th row)
c                      ! z for 'ic1'-th constraint
c           Run over all overlapped body-blocks in 
c           'ic1'-th and 'ic2'-th rows
            left = 0.0d 0
            if (ka .ne. 0) then ! scip when "no overlap of body-blocks"
               im2 = idbl(3,ic2) - 1
c              Move 'im2' forward if the first body of 'icob' is not
c              the first one into 'icob2'
               if (kaf .ne. 1) then
                  do jj = 1,kaf-1
                     im2 = im2 + imbl(1,icob2(jj))
                  end do
               end if
               do ii = 1, ka
                  ifb = icob(ii)
                  imbody = imbl(1,ifb)
                  kbb = imbl(2,ifb) - 1
                  do jj = 1, imbody
                     left = left + vecr(kbb+jj)*amat(im2+jj)
                  end do
                  im2 = im2 + imbody
               end do
c___________3.1.3.5).  Store blocks (left and right) for LE associated with
c                      constraints (6 x 6 and 1 x 6 blocks are computed next)
               dmd(inot0) = left  
c                         ! only upper triangular is needed for LE
               icrow(inot0) = ic2
               inot0r = inot0r + 1
               inot0 = inot0 + 1
               icbl(3,ic2) = icbl(3,ic2) + 1
               iccol(icbl(4,ic2) - 1 + icbl(3,ic2)) = ic1
            end if
         end do     ! ic2
         glm(ic1) = right
         if (namev .ne. 0) then
            do i6 = 1, 6
               dmd(inot0) = righ6(i6)
c                         ! only upper triangular is needed for LE
               icrow(inot0) = nconn + i6
               inot0 = inot0 + 1
            end do
            inot0r = inot0r + 6
         end if
         icbl(1,ic1) = inot0r
         icbl(2,ic1) = ifstr
         ifstr = ifstr + inot0r
      end do     ! ic1
c
c_____3.2).  Compute right (1 x 6) and left (6 X 6) blocks for momentums,
c            i.e. EV*M(-1)*h and EV*M(-1)*EV' (if this is a velocity fit)
      if (namev. ne. 0) then
c
c        Run over all 6 "first" and "second" momentums
         ivc1 = namev - 1      ! "First" pointer(-1) for block EV (to move)
         do i61 = 1, 6       ! over "first" momentums
c           Form 'i61'-th row of the block EV
            do ii = 1, nx
               vecr(ii) = amat(ivc1+ii)
            end do
c
c           Compute z = EV*M(-1) through LE
            call vmispa(amat,vecr,naa,nx,nb,nb,nfm,imbl,
     &nctype,icob1,gmx,sm,kpvtm,kpvtal,lnf)        ! vecr is z now
c
c           Compute z*h
            left6 = -gvec(ng - 6 + i61)
            do ii = 1, nx
               left6 = left6 + vecr(ii)*gvec(ii)
            end do
c
            n6ij = 7 - i61
            icbl(1,nconn+i61) = n6ij
            icbl(2,nconn+i61) = ifstr
            ifstr = ifstr + n6ij
            do i = 1, nconn+i61
               iccol(icbl(4,nconn+i61)-1+i) = i
            end do
            ivc2 = namev - 1 + (i61-1)*nx
                                  ! "Second" pointer(-1) for block EV (to move)
            do i62 = i61, 6       ! over "second" momentums
               righ66 = 0.0d 0
               do ii = 1, nx
                  righ66 = righ66 + vecr(ii)*amat(ivc2+ii)
               end do
               dmd(inot0) = righ66
               icrow(inot0) = nconn + i62
               inot0 = inot0 + 1
               ivc2 = ivc2 + nx      ! Move pointer to next "second" momentum
            end do   ! i62
            glm(nconn+i61) = left6
            ivc1 = ivc1 + nx      ! Move pointer to next "first" momentum
         end do   ! i61
      end if
c
c     Free gvec for other use
      do ii = 1, nx
         vecr(ii) = gvec(ii)
      end do
c
c_____4).  Perform step #1, i.e. solve LE for lambda (if any constraints)
      if (nlm .ne. 0) then
c        Perform factorization L*L' (for the tree topology)
         call dllt(dmd,ndmd,nlm,icbl,icrow,iccol,inform)            
         if (inform .ne. 0) then
            CALL CLEARERR()
            WRITE(ERRMSG,200) 
     +       'Matrix D*M(-1)*D**T cannot be factorized'
            CALL ERRHDL(-5,'<DLESPA>')
         end if
 200     FORMAT(1X,A)
c        Perform forward and backward recursion to solve (L*L')*lambda = g
c        Use memory from gvec to store right-hand part of LE
         do ic = 1, nlm
            gvec(ic) = glm(ic) 
         end do
         call dlef(dmd,glm,ndmd,nlm,icbl,icrow,iccol) 
c                                      ! glm is vector of LM (lambda) now
c        Perform correction of lambda due to closed-loop (if any)
c        by iterative Gauss-Seidel algorithm
         call dloop(dmd,gvec,glm,ndmd,nlm,icbl,icrow,iccol)
      end if
c
c_____5).  Perform step #2, i.e. solve a sequence
c     of systems of LE for I.C. of each body 
c
c_____5.1).  Form right-hand part of LE
c     Account for "constraints'" LM (due to block D or E(D)),
c     i.e. subtract E(D)'*lambda(D) from gvec
c     Note:  Cycles are performed in the order: constraints, bodies 
c            in order to handle D' (instead of D)
      do ic = 1, nconn
         call cbod(ic,nh,nconn,ihconn,itopol,nctype,icob)
         im1 = idbl(3,ic) - 1   ! Pointer(-1) for 'ic'-th row of D (E(D)) 
                                ! (stored in 1-dimensional amat)
         kboc = idbl(1,ic)      ! # of bodies involved in 'ic'-th constraint
         glmic = glm(ic)
         do ifb = 1, nb
c           Determine if 'ifb'-th body is associated with 'ic'-th constraint
            ig = 0
            do ii = 1, kboc
               if (ifb .eq. icob(ii)) then
                  ig = 1
                  kbc = ii      ! body's # in 'ic'-th row of D or E(D)
                  go to 20
               end if
            end do
 20         continue
c           Multiply 'ifb'-th body-block with corresponding block of LM
            if (ig .eq. 1) then
               imbody = imbl(1,ifb)
               kbb = imbl(2,ifb) - 1
c                 Move pointer(-1) in 'ic'-th row to b-b for 'ifb'-th body
               im = im1
               if (kbc .ne. 1) then
                  do jj = 1, kbc-1
                     im = im + imbl(1,icob(jj))
                  end do
               end if
               do jj = 1, imbody
                  vecr(kbb+jj) = vecr(kbb+jj) - amat(im+jj)*glmic
               end do
            end if
         end do
      end do
c 
c_____5.1.1).  Account for "momentums'" LM (due to block EV, if any)
c              i.e. subtract E(V)'*lambda(V) from gvec (in addition to 
c              constraints' part) 
      if (namev .ne. 0) then
         do ix = 1, nx
            glm6 = vecr(ix)
            do i = 1, 6
               ivc2 = namev - 1 + (i-1)*nx   ! Pointer(-1) for i-th row in EV 
               glm6 = glm6 - amat(ivc2+ix)*glm(nconn+i)
            end do
            vecr(ix) = glm6
         end do
      end if
c
c_____5.2).  Solve subsystem of LE for each body
      call vmispa(amat,vecr,naa,nx,nb,nb,nfm,imbl,
     &nctype,icob1,gmx,sm,kpvtm,kpvtal,lnf)     ! vecr is vector of I.C. now  
c
c_____5.3).  Assemble I.C. and LM vectors into the ouput gvec
      do ii = 1, nx
         gvec(ii) = vecr(ii)
      end do
      do ic = 1, nlm
         gvec(nx + ic) = glm(ic)
      end do
c
      return
      end









