C $Header: /export/cvs/c34a1/source/moldyn/eign1.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
      SUBROUTINE EIGN1  (A,VAL,VEC,NIN,FODIN,KR)
CSE
CPS        PURPOSE:
C
C                   TO COMPUTE THE EIGENVALUES AND ASSOCIATED
C                   EIGENVECTORS OF ANY REAL SYMMETRIC MATRIX USING
C                   METHOD OF JACOBI, THRESHOLD VERSION WITH
C                   PIVOTING. COMPUTED EIGENVALUES AND EIGENVECTORS
C                   ARE REAL.
C
C                   REQUIRED BY COMPIN TO COMPUTE SYSTEM PRINCIPAL
C                   MOMENTS OF INERTIA AND ASSOCIATED ROTATION
C                   TRANSFORMATION MATRIX.
C
CPE
      IMPLICIT NONE
 
C                                                          DEBUG # 125
C
c     Include files
c
#include "ERRHDL.INC"
#include "NUMBRS.INC"
#include "TAPENO.INC"
#include "PARAM.FOR"
c
C     External Variables
C
c     Local Variables
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: eign1.f,v $$Revision: 1.1.1.1 $'/
      INTEGER I,J,N         ! Loop counters
      INTEGER IP,IP1,IPM1,IPP1
      INTEGER JP,JPM1,JPP1
      INTEGER KR,NIN
      INTEGER IABS, NM1,IREDO
      DOUBLE PRECISION FOD,FODIN,DEL,RAD
      DOUBLE PRECISION A(KR,*), VAL(*), VEC(KR,*)
      DOUBLE PRECISION AIIP,AIPI,AIPIP,AJPJP
      DOUBLE PRECISION THRESH, TRACE,VECIIP
      DOUBLE PRECISION ASC,CS,CS2,SN,SN2,TN
C
C  CALCULATE EIGENVALUES / EIGENVECTORS OF (A)(VEC) = (VEC)(-VAL-).
C  JACOBI METHOD, THRESHOLD VERSION. PROGRESS FROM PIVOT ELEMENT
C  (IPIVOT,JPIVOT) TO ELEMENT (IPIVOT,JPIVOT+1) AFTER A PIVOT.
C  THE (A) MATRIX SHOULD BE REAL, SYMMETRIC. UPPER HALF IS USED.
C  CODED BY  RL WOHLEN.  APRIL 1969.
C  LAST REVISION BY HP FRISCH. APRIL 1985.
C
CAS         ***  ARGUMENT LIST  ***
C
C  A     = INPUT  MATRIX TO BE DIAGONALIZED. SIZE(N,N). *DESTROYED*
C  VAL   = OUTPUT VECTOR OF EIGENVALUES. SIZE(N).
C  VEC   = OUTPUT MATRIX OF EIGENVECTORS. SIZE(N,N).
C  NIN   = INPUT  DABS(NIN)=N IS THE SIZE OF MATRICES A,VEC, VECTOR VAL.
C                 IF NIN IS NEGATIVE, INITIAL VEC MATRIX IS ASSUMED TO
C                 BE SUPPLIED FROM ARGUMENT.
C  FODIN = INPUT  FINAL OFF-DIAGONAL VALUE FOR DIAGONALIZED A.
C                 IF FODIN .LE. 0., FOD=TRACE(A)*EPS_MACH WILL BE USED.
C  KR    = INPUT  ROW DIMENSION OF A,VEC IN CALLING PROGRAM.
C
CAE
C
      N = IABS(NIN)
      IF (NIN .LT. 0) GO TO 10
C  SET INITIAL VEC MATRIX TO UNITY.
      DO 6 I=1,N
      DO 5 J=1,N
    5 VEC(I,J) = 0.0 D0
    6 VEC(I,I) = 1.0 D0
C
   10 IF (N .EQ. 1) GO TO 60
C
C  FIND LARGEST OFF-DIAGONAL ELEMENT (THRESH) OF A.
C  CALCULATE SUM OF DIAGONALS (TRACE) OF A.
      TRACE = 0. D0
      THRESH = DABS(A(1,2))
      NM1 = N-1
      DO 15 I=1,NM1
      TRACE = TRACE + A(I,I)
      IP1 = I+1
      DO 15 J=IP1,N
   15 IF (DABS(A(I,J)) .GT. THRESH) THRESH=DABS(A(I,J))
      TRACE = TRACE + A(N,N)
      FOD = FODIN
      IF (FODIN .LE. 0. D0) FOD=TRACE*EPS_MACH
      IF (FOD .LE. 0. D0) GO TO 999
      IF (THRESH .LE. FOD) GO TO 60
C
C  SCAN UPPER OFF-DIAGONAL ELEMENTS OF MATRIX A BY ROWS UNTIL A VALUE
C  GREATER THAN THRESH IS FOUND. PIVOT ON THIS ELEMENT (IP,JP).
   20 THRESH = THRESH/10. D0
      IF (THRESH .LT. FOD) THRESH=FOD
   22 IREDO = 0
      DO 41 IP=1,NM1
      IPM1 = IP-1
      IPP1 = IP+1
      DO 40 JP=IPP1,N
      IF (DABS(A(IP,JP)) .LT. THRESH) GO TO 40
      IREDO = 1
C  CALCULATE ROTATION VALUES.
      DEL = A(IP,IP) - A(JP,JP)
      RAD = DSQRT (DEL**2 + 4. D0*A(IP,JP)**2)
      IF (DEL .LT. 0. D0) RAD=-RAD
      TN = (2. D0 * A(IP,JP)) / (DEL + RAD)
      CS = 1. D0 / DSQRT (1. D0 + TN**2)
      SN = TN * CS
C  DIAGONALIZE MATRIX (A). ONLY UPPER HALF IS USED.
      JPM1 = JP-1
      JPP1 = JP+1
      IF (IP .EQ. 1) GO TO 33
      DO 32 I=1,IPM1
      AIIP    = A(I,IP)*CS + A(I,JP)*SN
      A(I,JP) =-A(I,IP)*SN + A(I,JP)*CS
   32 A(I,IP) = AIIP
   33 IF (IPP1 .EQ. JP) GO TO 35
      DO 34 I=IPP1,JPM1
      AIPI    = A(IP,I)*CS + A(I,JP)*SN
      A(I,JP) =-A(IP,I)*SN + A(I,JP)*CS
   34 A(IP,I) = AIPI
   35 IF (JP .EQ. N) GO TO 37
      DO 36 I=JPP1,N
      AIPI    = A(IP,I)*CS + A(JP,I)*SN
      A(JP,I) =-A(IP,I)*SN + A(JP,I)*CS
   36 A(IP,I) = AIPI
   37 AIPIP = A(IP,IP)
      AJPJP = A(JP,JP)
      CS2 = CS**2
      SN2 = SN**2
      ASC = 2. D0*A(IP,JP)*SN*CS
      A(IP,IP) = AIPIP*CS2 + ASC + AJPJP*SN2
      A(JP,JP) = AIPIP*SN2 - ASC + AJPJP*CS2
      A(IP,JP) = 0.0 D0
C  CALCULATE EIGENVECTORS.
      DO 38 I=1,N
      VECIIP    = VEC(I,IP)*CS + VEC(I,JP)*SN
      VEC(I,JP) =-VEC(I,IP)*SN + VEC(I,JP)*CS
   38 VEC(I,IP) = VECIIP
   40 CONTINUE
   41 CONTINUE
      IF (IREDO .EQ. 1) GO TO 22
      IF (THRESH .GT. FOD) GO TO 20
C
C  PLACE DIAGONAL FROM A INTO VAL (EIGENVALUES).
   60 DO 61 I=1,N
   61 VAL(I) = A(I,I)
      RETURN

 999  CALL CLEARERR()
      WRITE (ERRMSG,200) 'Error in subroutine.'
      CALL ERRHDL(-5,'<EIGN1>')
 200  FORMAT(1X,A)
C
      END
