C $Header: /export/cvs/c34a1/source/moldyn/extor_CHARMM.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1994 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
#ifdef ANALYZE_FORCE
      SUBROUTINE EXTOR_CHARMM(GPOS,GPOS_STAGE,G_chm,p)
#else
      SUBROUTINE EXTOR_CHARMM(GPOS,GPOS_STAGE,p)
#endif 
CSE
C                                                          DEBUG # 108
CPS        PURPOSE:
C
C                   USER DEFINED SUBROUTINE USED TO DEFINE ALL
C                   EXTERNAL FORCE/TORQUE LOADS. 
C
C                   USERS MAY AVOID INSERTION OF CODE HEREIN BY MAKING
C                   USE OF DISCOS OPTION CAPABILITIES:
C
CPE
CIS
C
C         SUBROUTINE EXTOR    EXTERNAL TORQUES AND FORCES
C         -----------------------------------------------
C
C     ALL EXTERNAL TORQUES AND FORCES ACTING ON THE SYSTEM MUST BE
C         DEFINED IN SUBROUTINE EXTOR, SEE PAGE 12, VOL II. EXTOR IS
C         CALLED FROM SUBROUTINE TORQUE IMMEDIATELY AFTER RETURN FROM
C         CONTRL. UPON ENTRY INTO EXTOR THE STATE VECTOR Y CONTAINS THE
C         INSTANTANEOUS VALUES OF ALL PLANT AND CONTROL SYSTEM STATE
C         VARIABLES. THE USER MAY USE ANY OF THESE STATE VARIABLES OR
C         ANY OTHER SYSTEM PARAMETERS STORED IN COMMON TO CREATE THE
C         EXTERNAL FORCE/TORQUE VECTORS REQUIRED BY EQUATION II-74 OF
C         VOL I.
C
CIE
      IMPLICIT NONE
C
#include "PARAM.FOR"
C
#include "CHARMM.INC"
#include "COREA.INC"
#include "GCOMP.INC"
#include "JILFLG.INC"
#include "MAXMUM.INC"
#include "MISCNO.INC"
#include "MOMENG.INC"
#include "NUMBRS.INC"
#include "SPECIF.INC"
C
C     COMMON BLOCKS FOR MOLECULAR DYNAMICS
C
      INTEGER
     +  M05,M10,M15,M20,M25,M30,M35,M40,M45,M50,
     +  M55,M60,M65,M70,M75,M80,M85,M90,M95,M96,M97,M100
      COMMON/MEMLB/ 
     +  M05,M10,M15,M20,M25,M30,M35,M40,M45,M50,
     +  M55,M60,M65,M70,M75,M80,M85,M90,M95,M96,M97,M100
C
c     Passed variables
c
      DOUBLE PRECISION P(cL10)
      DOUBLE PRECISION GPOS(cL10), GPOS_STAGE(cL10)
#ifdef ANALYZE_FORCE
      DOUBLE PRECISION G_chm(cL10)
#endif
c
C     Local variables
C
      INTEGER i,istage,mrt
C
C
C     *********  modifications for MTS begin here   ****************** 
C
C     Initialize gpos 
      DO i = 1, locu(nb+1)-1
         GPOS(i) = ZRO
      END DO
C
C     Calculate combined forces in the case of not-MTS
C     or fast forces in the case of MTS 
      iparam(17) = 1
      CALL MBForce(nspt,x(m30),x(m35),iparam)
C
C     Transform to body forces
      CALL LFORCE(GPOS)
C
C     Use the n-bin MTSL version 
C     Note:  Non-MTS Lobatto automatically scips the "n-bin" part
C            (since nstages = 1) 
C
C     Compute forces for each MTS stage (after the fast one) 
C     and sum them up in the resulting forces for the MTSL integrator 
C     using the weights which are the ratios of the MTS stages:
C
C     GPOS = GPOS_1 + MRATIO(2)*GPOS_2 + MRATIO(3)*GPOS_3 + ... (etc.)
C
      DO istage = 2, nstages
C     Note:  Stage #1 is taken out of the cycle to avoid multiplications
C            1 * GPOS_STAGE(i)
         mrt = mratio(istage)
C     Compute the forces at the current MTS stage but the fast one
C     Note:  The computation takes place if the current time
C            is a multiple of the stage's ratio or is initial time
         IF (mod(icur_step,mrt) .EQ. 0) THEN
C
C     Initialize gpos_stage 
            DO i = 1, locu(nb+1)-1
               GPOS_STAGE(i) = ZRO
            END DO
C
C     Calculate the forces
            iparam(17) = istage 
            CALL MBForce(nspt,x(m30),x(m35),iparam)
C
C     Transform to body forces
            CALL LFORCE(GPOS_STAGE)
C 
C     Sum up the forces at the current MTS stage with the forces 
C     at previous (faster) MTS stages.  Use the stage's ratio as an
C     amplifying gain
            DO i = 1, locu(nb+1)-1
               GPOS(i) = GPOS(i) + mrt*GPOS_STAGE(i)
            END DO
         END IF                 ! Multiple of stage's ratio
      END DO                    ! MTS stages
C
#ifdef ANALYZE_FORCE
      DO i = 1, locu(nb+1)-1
         G_chm(i) = GPOS(i)
      END DO
#endif
C
C-----------------------------------------------------------------------
C
C     Note:  The next two lines are for electrostatic energy 
C            which is not accounted so far.  When accounted for, 
C            this part of the code should be adjusted for MTSL. 
      totpe  = totpe  + rparam(9)
      toteng = toteng + rparam(9)
C
      RETURN
      END



