C $Header: /export/cvs/c34a1/source/moldyn/factor_b.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
C                  SUBROUTINE - factor_b.f                        
C    
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
c
      subroutine factor_b(amat,naa,nx,nb,imbl,nfm,sm,kpvtm,kpvtal,lnf)
c
c     FACTOR_B code performes a factorization of body-blocks of matrix M
c     for position or velocity fit of I.C.  Factorization is performed 
c     by dsifa.f for each body-block.  
c
c     Subroutine calls:
c        dsifa.f
c
c     The inputs are:
c     amat    Not-zero elements of matrix A stored in array (1 x naa), 
c             only the body-blocks of M will be used
c             (taking into account their symmetry or diagonality).
c     naa     "Storage" dimension of amat (number of not-zero elements).
c     nx      Size of I.C. vector.
c     nb      Number of bodies      
c     imbl    Matrix to store sizes of body-blocks of M, (3 x nb),
c             1st row:  Size of each body-block (length of body-block of dx),
c             2nd row:  amat's # of first element in body-block of dx,
c             3rd row:  amat's # of first element in body-block of M
c                       (Note:  for bodies - only triangular matrix is stored,
c                               for particles - only diagonal (3 x 1)).
c     nfm     Maximal size of body-block 
c             (number of parameters to be fitted in the most "flexible" body).
c     sm      Auxiliary matrix to operate with body-blocks of M, (nfm x nfm).
c     kpvtm   Vector of pivot indeces for one body-block, size (1 x nfm).
c     kpvtal  Vector of pivot indeces for all body-blocks, size (1 x nx).
c             Note:  Stored for multiple usage in dsisl.f.
c     lnf     Maximal size for nfm set in the calling program. 
c
c     The ouputs are:
c     amat    Factorized body-blocks of M stored in the array
c             with the same structure as in iput's amat.   
c
      IMPLICIT NONE
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: factor_b.f,v $$Revision: 1.1.1.1 $'/
c
c     COMMON Blocks
c
#include "ERRHDL.INC"
#include "TAPENO.INC"
c
c     Argument List
c
      integer naa,nx,nb,nfm,lnf                  ! Inputs
      double precision amat(naa)                 ! Inputs/Outputs
      double precision sm(lnf,lnf)               ! Inputs      
      integer imbl(3,nb),kpvtm(nfm)              ! Inputs
      integer kpvtal(nx)                         ! Inputs
c
c     Local Variables
c
      integer ifb,imbody,kast,i,ii,jj,ka,kbb
      integer info
c
c     zero writes to a double, kpvtal is an integer!
c     call zero(kpvtal,nx,1,nx)
      DO i = 1, nx
         kpvtal(i) = 0
      ENDDO
c     Run over all body-blocks
      do ifb = 1, nb
c
c     Extract each-body block of M from the packed amat
         imbody = imbl(1,ifb)
         kast = imbl(3,ifb) - 1
         if (imbody .ne. 3) then        ! Scip particles 
            do ii = 1, imbody
               do jj = ii, imbody
                  ka = (jj-1)*jj/2 + ii
                  sm(ii,jj) = amat(kast+ka)
               end do    
            end do
c           Perform factorization
            call dsifa(sm,lnf,imbody,kpvtm,info)
            if (info .ne. 0) then
               CALL CLEARERR()
               WRITE(ERRMSG,200)
     +          'Body-block of M is singular.'
               CALL ERRHDL(-5,'<FACTOR_B>')
            end if
 200        FORMAT(1X,A)
c           Pack body-blocks back in amat
            do ii = 1, imbody
               do jj = ii, imbody
                  ka = (jj-1)*jj/2 + ii
                  amat(kast+ka) = sm(ii,jj)  ! only upper triangular is needed
               end do
            end do
c           Store pivot indeces for multiple usage in dsisl
            kbb = imbl(2,ifb) - 1
            do ii = 1, imbody
               kpvtal(kbb+ii) = kpvtm(ii)
            end do
         end if
      end do
      return
      end




