C $Header: /export/cvs/c34a1/source/moldyn/kabsch.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C






C-----------------------------------------------------------------------
      SUBROUTINE kabsch(mbout,    !i:  unit number of output file
     &                  np,       !i:  number of points
     &                  w,        !i:  weights (np)
     &                  rot,      !io: coor set to rotate  (3*np)
     &                  ref,      !i:  coor reference set  (3*np)
     &                  com_diff, !o:  COM difference: rot-ref  (3)
     &                  u,        !o:  rotation matrix (3*3)
     &                  status)   !io: completion status
C-----------------------------------------------------------------------
C
C Routine to overlap set ROT on set REF in such a way that 
C TARGET = sum(n) w(n)*(U*rot(n) - ref(n))**2 is minimized
C
C Wolfgang Kabsch "A solution for the best rotation
C to relate two sets of vectors" Acta Cryst. (1976) A32, 922
C
C
      IMPLICIT NONE

#include "STATUS.INC"

      integer mbout, np
      double precision rot(3,np),ref(3,np),w(np)
      double precision com_diff(3)
      double precision u(3,3)     ! rotation matrix
      integer status
      double precision com_rot(3) ! center of mass of rot set
      double precision com_ref(3) ! center of mass of ref set
      double precision r(3,3)     ! "cross set inertia tensor"
      double precision a(3,3)     ! eigenvectors of r(T)*r
      double precision eval(3)    ! eigenvalues  of r(T)*r
      double precision b(3,3)     ! b vectors
      integer i,j,k,n
      double precision eps, f
      parameter (eps = 1.0D-9)
      double precision det33
      external det33
#if DEBUG
      character*4 debug_str
#endif
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: kabsch.f,v $$Revision: 1.1.1.1 $'/

      call get_com(w,ref,mbout,np,com_ref,status)
      call get_com(w,rot,mbout,np,com_rot,status)

#if DEBUG
      CALL getenv('DEBUG_KABSCH',debug_str)
c23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
      if(debug_str.ne.'') then
      write(mbout,'(a)') 'kabsch> DEBUG: initial values'
      write(mbout,*)
      write(mbout,'(a,3f9.3)') 'kabsch> com_ref =',(com_ref(i),i=1,3)
      do j=1,np
         write(mbout,'(a,4f9.3)')
     &         'kabsch> ref,w   =',(ref(i,j),i=1,3),w(j)
      enddo
      write(mbout,*)
      write(mbout,'(a,3f9.3)') 'kabsch> com_rot =',(com_rot(i),i=1,3)
      do j=1,np
         write(mbout,'(a,4f9.3)') 
     &         'kabsch> rot,w   =',(rot(i,j),i=1,3),w(j)
      enddo
      endif
#endif

      do i=1,3
         com_diff(i) = com_rot(i) - com_ref(i)
         do j=1,3
            r(i,j) = 0
            a(i,j) = 0
         enddo
      enddo

      do n=1,np                      !........ "cross set inertia tensor"
         do i=1,3
            do j=1,3
               r(i,j) = r(i,j) + 
     &         w(n)*(ref(i,n) - com_ref(i))*(rot(j,n) - com_rot(j))
            enddo
         enddo
      enddo

#if DEBUG
      if(debug_str.ne.'') then
         write(mbout,*) 'kabsch> DEBUG: com_ref and r matrix ='
         call matout2(U,com_ref,3,3,3,6)
      endif
#endif

      do i=1,3                       !............. a = r(Transpose) * r 
         do j=1,3
            do k=1,3
               a(i,j) = a(i,j) + r(k,i)*r(k,j)
            enddo
         enddo
      enddo

#if DEBUG
      if(debug_str.ne.'') then
         write(mbout,*) 'kabsch> DEBUG: com_ref and r matrix ='
         call matout2(r,com_ref,3,3,3,6)
         write(mbout,*) 
     &         'kabsch> DEBUG: com_ref and r (=r^T * r) matrix ='
         call matout2(a,com_ref,3,3,3,6)
      endif
#endif

      call house(a,3,3,eval,b,status) !..........diagonalize a = r^T * r

C.....on output eigenvectors are in matrix a
#if DEBUG
      if(debug_str.ne.'') then
         write(mbout,*) 
     &         'kabsch> DEBUG: eval and eigenvalues of a matrix ='
         call matout2(a,eval,3,3,3,mbout)
      endif
#endif

      do k=1,3                        !.............Construct b vectors
         if(k.ne.3 .and. eval(k).gt.eps) then
            f = 1/sqrt(eval(k))
            do i=1,3
               b(i,k) = f*(r(i,1)*a(1,k)+r(i,2)*a(2,k)+r(i,3)*a(3,k))
            enddo
         else
            if(k.eq.3) then
               call vcross(b(1,1),b(1,2),b(1,3))
               call vcross(a(1,1),a(1,2),a(1,3))
            else
               write(mbout,'(a,i2,3e12.4)') 
     &               'kabsch> error: k,eval=',k,eval
c23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
               write(mbout,'(a)') 
     &               'kabsch> all points are either in line'
               write(mbout,'(a)') 'kabsch> or collapsed into one point'
               write(mbout,'(a)') 'kabsch> no overlap performed'
               status = FAILURE
               return
            endif
         endif
      enddo

      do i=1,3           !................Construct orthonormal U matrix
         do j=1,3
            u(i,j) = 0
            do k=1,3
               u(i,j) = u(i,j) + b(i,k)*a(j,k)
            enddo
         enddo
      enddo

cif DEBUG
c     if(debug_str.ne.'') then
         call check_ortn(u,mbout,3,status)
         if(status.ne.SUCCESS) then
            write(mbout,'(a)') 'kabsch> error: u is not orthonormal'
            return
         endif
c     endif
cendif

      if(det33(u).lt.0) then
         write(mbout,'(a)') 'kabsch> error: det33(U) less then zero'
         write(mbout,'(a)') 'kabsch> this should never happen !'
         write(mbout,'(a,e12.4)') 'kabsch> det33(U) =',det33(U)
         status = FAILURE
         return
      endif

      do n=1,np         !........................ overlap ROT set on REF
         do i=1,3
            eval(i)= u(i,1)*(rot(1,n) - com_rot(1)) +
     &               u(i,2)*(rot(2,n) - com_rot(2)) +
     &               u(i,3)*(rot(3,n) - com_rot(3))
         enddo
         rot(1,n) = com_ref(1) + eval(1)
         rot(2,n) = com_ref(2) + eval(2)
         rot(3,n) = com_ref(3) + eval(3)
      enddo

      status = SUCCESS

      return
      end


C-----------------------------------------------------------------------
      SUBROUTINE MATOUT2(A,B,NC,NR,NDIM,IOUT)
C-----------------------------------------------------------------------
C
C     IMPLICIT REAL*8(A-H,O-Z)
C     DIMENSION A(NDIM,NDIM),B(NDIM)
      real*8 A(NDIM,NDIM),B(NDIM)
C *** WISH TO OUTPUT THE MATRIX A(NC,NR)
      KA=1
      KC=10
1     KB=MIN0(KC,NC)
      WRITE(IOUT,600) (I,I=KA,KB)
      WRITE(IOUT,601) (B(I),I=KA,KB)
      WRITE(IOUT,602)
      LA=1
      LC=40
2     LB=MIN0(LC,NR)
      N=0
      DO 3 I=LA,LB
      WRITE(IOUT,603) I,(A(I,J),J=KA,KB)
      N=N+1
      IF(N.LT.10) GOTO 3
      WRITE(IOUT,602)
      N=0
3     CONTINUE
      IF(LB.EQ.NR) GOTO 4
      LA=LC+1
      LC=LC+40
      WRITE(IOUT,604)
      GOTO 2
4     IF(KB.EQ.NC) RETURN
      KA=KC+1
      KC=KC+10
      IF(NR.GT.25) WRITE(IOUT,604)
      GOTO 1
600   FORMAT(//9H ROOT NO.,I5,9I12)
c01   FORMAT(/5X,10F12.4)
601   FORMAT(/4X,10E12.4)
602   FORMAT(2H  )
603   FORMAT(I5,10F12.4)
c03   FORMAT(I5,10E12.4)
604   FORMAT(1H1)
      END
C
C-----------------------------------------------------------------------
      subroutine check_ortn(U,mbout,N,status) ! check if U is orthonormal
C-----------------------------------------------------------------------
      implicit none

#include "STATUS.INC"

      integer mbout, N, status
      double precision U(N,N)
      double precision eps
      parameter (eps = 1.0D-6)
      double precision dot1, dot2
      double precision e1,e2,err1,err2
      integer i,j,k

      err1 = 0.0
      err2 = 0.0

#if DEBUG
      write(mbout,'(/a,e12.4)') 'check_ortn> DEBUG: U, eps =',eps
      call matout2(U,U,3,3,3,6)
#endif

      do i=1,N
         do j=1,N
            dot1 = 0
            dot2 = 0
            do k=1,N
               dot1 = dot1 + U(k,i)*U(k,j)
               dot2 = dot2 + U(i,k)*U(j,k)
            enddo
            if(i.ne.j) then
               e1 = abs(dot1)
               e2 = abs(dot2)
            else
               e1 = abs(dot1-1)
               e2 = abs(dot2-1)
            endif
            err1 = err1 + e1
            err2 = err2 + e2
#if DEBUG
            if(e1.gt.eps .or. e2.gt.eps) then
               write(mbout,'(a,3e12.4)')
     &         'check_ortn> error: e1,e2,eps = ',e1,e2,eps
            endif
#endif
         enddo
      enddo
      if(err1.gt.eps .or. err2.gt.eps) then
         write(mbout,'(a,3e12.4)')
     &   'check_ortn> error: err1,err2,eps = ',err1,err2,eps
         status = FAILURE
      else
         status = SUCCESS
#if DEBUG
         write(mbout,'(a,3e12.4)')
     &   'check_ortn> success: err1,err2,eps = ',err1,err2,eps
#endif
      endif
      return
      end


C-----------------------------------------------------------------------
      double precision function det33(U)
C-----------------------------------------------------------------------
      double precision U(3,3)
      det33 =  U(1,1)*(U(2,2)*U(3,3)-U(3,2)*U(2,3))
     &        -U(2,1)*(U(1,2)*U(3,3)-U(1,3)*U(3,2))
     &        +U(3,1)*(U(1,2)*U(2,3)-U(1,3)*U(2,2))
      return
      end


C-----------------------------------------------------------------------
      subroutine get_com(w,coor,mbout,np,com,status)
C-----------------------------------------------------------------------
      implicit none

#include "STATUS.INC"

      integer mbout, np
      double precision w(np),coor(3,np)
      double precision com(3), tot
      integer n, status
      double precision eps
      parameter (eps = 1.0D-12)

      com(1) = 0
      com(2) = 0
      com(3) = 0
      tot = 0

      do n=1,np
         com(1) = com(1) + w(n)*coor(1,n)
         com(2) = com(2) + w(n)*coor(2,n)
         com(3) = com(3) + w(n)*coor(3,n)
         tot = tot + w(n)
      enddo

      if(tot.lt.eps) then
         status = FAILURE
         write(mbout,'(a,2e12.4)')'get_com> error: tot,eps=',tot,eps
      else
         status = SUCCESS
         com(1) = com(1)/tot
         com(2) = com(2)/tot
         com(3) = com(3)/tot
      endif

      return
      end

