C $Header: /export/cvs/c34a1/source/moldyn/lobto4.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1995 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
      SUBROUTINE lobto4(x,dx,d2x,n,t,dt,derivs,vdt2,p)
c
c     This code implements the fourth order lobatto integrator with iteration to solve 
c     for the velocity-dependent accelerations at the half-step
c
      IMPLICIT NONE
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: lobto4.f,v $$Revision: 1.1.1.1 $'/
c
#include "PARAM.FOR"
c
#include "CHARMM.INC"
#include "CPATHS.INC"
#include "HEAPMB.INC"
#include "IVCONS.INC"
#include "JILFLG.INC"
#include "MISCNO.INC"
#include "MOMENG.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
c
c     Arguments
c
      INTEGER n                      ! Number of coordinates
      DOUBLE PRECISION x(n)          ! Positions
      DOUBLE PRECISION dx(n)         ! Velocities
      DOUBLE PRECISION d2x(n)        ! Accelerations
      DOUBLE PRECISION t             ! Current time
      DOUBLE PRECISION dt            ! Time step
      DOUBLE PRECISION vdt2(n)       ! Extrapolated velocities at half dt
      DOUBLE PRECISION p(cL10)
      EXTERNAL DERIVS                ! Subroutine for derivative calc.
c
c     MBO(N)D COMMON blocks
c
c
c     Local Variables
c
      INTEGER i                      ! DO loop variable
      INTEGER iter                   ! Iterate counter 
      INTEGER iextor                 ! Indicator of call for external torque 
      DOUBLE PRECISION dt2           ! dt / 2
      DOUBLE PRECISION eng_old       ! Old value for total energy
      DOUBLE PRECISION delta_eng     ! Change in total energy between iterates

c     specific lobto4 variables
      integer nlseq,ilseq
      double precision alpha,beta,ucon,dt1
c
 2000 FORMAT(1X,A,I5)
c
      nlseq = 5
      alpha = 0.4144907717943758d0
c     alpha = 1.351207191959657d0
      beta  = 1.d0 -4*alpha
c     beta  = 1.d0 -2*alpha
      do ilseq=1,nlseq
      if(ilseq.eq.3)then
      ucon=beta
      else
      ucon=alpha
      endif
c
c     This is the regular lobatto piece which has to be repeated to construct
c     the lobatto4
c     write(*,*)'lobstep nownow is ',ilseq
      dt1 = ucon*dt
      dt2 = dt1 / 2D0
c
c
c     Propagate the velocity states by half delta t 
c     (to initialize iterative procedure for velocities)
      DO i = 1, n
         vdt2(i) = dx(i) + d2x(i)*dt2
      END DO
c
      CALL engmom_ke(HEAP_MB(iyptr),HEAP_MB(ipptr),
     &               HEAP_MB(ipmomptr))
      eng_old = totke

c     Begin iterating on velocities
c
      iter = 0     ! Only for counting iterations (not DERIVS' itflag)
      iextor = 0   ! Use external torque from previous call 
 100  CONTINUE
      iter = iter + 1   ! Counting iterations
      CALL DERIVS(t,x,vdt2,d2x,1,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intLoopHeap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
                   ! Note:  in the call to DERIVS iter = itflag is set as 1
                   ! (position-dependent arrays are not recomputed)
c     Propagate the velocity states by half delta t 
c     (using updated accelerations)
      DO i = 1, n
         vdt2(i) = dx(i) + d2x(i)*dt2
      END DO
      CALL ENGMOM_KE(HEAP_MB(iyptr),HEAP_MB(ipptr),
     &               HEAP_MB(ipmomptr))
      delta_eng = totke - eng_old
      eng_old = totke
      IF (iter .LE. itmax) THEN
         IF (ABS(delta_eng/totke) .GT. ERAT_VV) GO TO 100
      ELSE IF (mbplvl .GE. 9) THEN
         write(MBOUT,*) 'More than', itmax,
     +        ' iterations over velocity-dependent forces'
      END IF
c     End of iteration
c
c     Propagate the position states by delta t
      DO i = 1, n
         x(i) = x(i) + vdt2(i)*dt1
      END DO
c
c     Compute accelerations at delta t
      iter = 0    ! Update position-dependent arrays for multibody dynamics
      iextor = 1  ! Call to external torque
      CALL DERIVS(t,x,vdt2,d2x,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            Loopheap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
c
c     Propogate the velocity states by delta t
      DO i = 1, n
         dx(i) = vdt2(i) + d2x(i)*dt2
      END DO
c     This is the end of the regular lobatto cycle
c
      enddo
c
#ifdef CHARMM
C     If using Langevin dynamics call GASRND to populate RF, a 1xNU 
C     vector, with random values (forces)
      IF (LANGEVIN) CALL GASRND(mbdynaHeap(ilangevinptr),
     &                          mbdynaHeap(irfptr),
     &                          HEAP_MB(ipptr))
      iparam(16) = iparam(16) + 1
#endif
c
c     Re-calculate accelerations to start the next step to allow:
c     a)  Euler sequence or pivot type changing, and
c     b)  Velocity re-scaling, if use
      jil = 4
      iter = itmax + 1
      iextor = 0   !   Use external torque from previous call
      CALL DERIVS(t,x,dx,d2x,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      seqflg = .FALSE.
c
#ifdef CHARMM
      IF ((iparam(20).EQ.1) .AND. (iparam(1).NE.icur_step)) THEN
c     Recompute accelerations for the start of the next step 
c     if re-binning in the MTS took place and
c     if the current step is not the last step
         iter = 1     ! Skip computing position-dependent arrays
                      ! for multibody dynamics 
                      ! (re-binning effects only CHARMM forces)
         iextor = 1   ! Call to external torque (CHARMM forces)
         CALL DERIVS(t,x,dx,d2x,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      END IF
c     Increment the step counter for MTSL
      icur_step = icur_step + 1     
#endif
c
      RETURN
      END




      SUBROUTINE GASRND (AM_langevin,RF,p)
C
C     returns, RF, a 1xNU vector of normally distributed 
C     deviates with zero mean and unit variance.
C     Uses Box-Muller algorithm 
C
      IMPLICIT NONE

#include "PARAM.FOR"

#include "CHARMM.INC"
#include "SPECIF.INC"

       integer le, I
       integer leHlf
       integer iBod, iPtr
       DOUBLE PRECISION v1, v2, randum
       DOUBLE PRECISION RAND
       logical flagRD
       DOUBLE PRECISION AM_langevin(cL10)
       DOUBLE PRECISION RF(cL10)
       DOUBLE PRECISION P(cL10)

       DATA flagRD/.FALSE./
       DATA ranstore/0D0/

       SAVE flagRD, ranstore

C      Generate random force vector, RF, for bodies
       DO iBod = 1,NBDY
C         Determine the number of DOF's
          le = LENU(iBod)
C         Half the number of DOF's because algorithm generates two
C         random numbers at once.
          leHlf = INT(le/2.0)-1
C         Determine the location in the Y vector.
          iPtr = LOCU(iBod)

          DO I = 0, leHlf
             v1 = DSQRT(-2.D+0 * DLOG(RAND(iseed)))
             v2 = 6.2831853D+0 * RAND(iseed)
             RF(iPtr) = v1 * cos(v2) * AM_langevin(iPtr)
             iPtr = iPtr + 1
             RF(iPtr) = v1 * sin(v2) * AM_langevin(iPtr)
             iPtr = iPtr + 1
          ENDDO

C         If le is an odd number get one more random number.
          IF ( (2*(leHlf+1)) .LT. le) then
             IF (flagRD) then
                flagRD = .FALSE.
                RF(iPtr) = randum * AM_langevin(iPtr)
             ELSE
                flagRD = .TRUE.
                v1 = DSQRT(-2.D+0 * DLOG(RAND(iseed)))
                v2 = 6.2831853D+0 * RAND(iseed)
                RF(iPtr) = v1 * cos(v2) * AM_langevin(iPtr)
                randum=v1 * sin(v2)
             ENDIF
          ENDIF
       ENDDO


C      Generate random force vector, RF, for atoms
       DO iBod = (NBDY+1),NB
          iPtr = LOCU(iBod)
          v1 = DSQRT(-2.D+0 * DLOG(RAND(iseed)))
          v2 = 6.2831853D+0 * RAND(iseed)
          RF(iPtr) = v1 * cos(v2) * AM_langevin(iPtr)
          iPtr = iPtr + 1
          RF(iPtr) = v1 * sin(v2) * AM_langevin(iPtr)

          iPtr = iPtr + 1 
          IF (flagRD) then
             flagRD = .FALSE.
             RF(iPtr) = randum * AM_langevin(iPtr)
          ELSE
             flagRD = .TRUE.
             v1 = DSQRT(-2.D+0 * DLOG(RAND(iseed)))
             v2 = 6.2831853D+0 * RAND(iseed)
             RF(iPtr) = v1 * cos(v2) * AM_langevin(iPtr)
             randum = v1 * sin(v2)
          ENDIF
       ENDDO

       RETURN
       END
          
