C $Header: /export/cvs/c34a1/source/moldyn/rotdh.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
C
      SUBROUTINE ROTDH(Y,ISCUT,IHTHPTR,PIMAT,IHPVT,PIN,RP2,RP3,
     *                 IBETDT,BETAH,ITOPOL,IHDATA,RH,DH,IHCUTJ)
CSE
C                                                          DEBUG #  44
      IMPLICIT NONE
c
CPS        PURPOSE:
C
C                   TO COMPUTE HINGE FRAME TRANSFORMATIONS AND VELOCITY
C                   TRANSFORMATION MATRICES. THE ELEMENTS OF THE ARRAYS
C                   ITOPW,PIMAT,DH,RH,PIN,R0L AND D0L ARE COMPUTED HERE
C
C                   USED TO COMPUTE ARRAYS R0L AND D0L SEE VOL II,PAGE 18
C                   INTERMEDIATE ROTATIONS STORED IN RH
C                   VELOCITY TRANSFORMATIONS STORED IN PIN AND PIMAT
C
C                   WHEN ACCOUNTING FOR FLEXIBILITY IN THE DETERMINATION
C                   OF ORIENTATION OF SENSOR FRAMES WRT THE BODY FIXED
C                   FRAMES USE ROTTRQ
C         REASON:
C                   QUATERNION TECHNIQUE IS MORE EFFICIENT THAN EULER
C                   TECHNIQUE DUE TO SMALL ANGLE ELASTIC DEFORMATION
C                   ALSO EULER METHOD CAN LEAD TO ERRORS
C
CPE
CAS
C
C         ROTDH - SEQUENCE TOPOLOGY DATA FOR FORWARD AND BACKWARD SWEEPING
C                 IN THE RECURSIVE FORMULATION
C
C     ITOPW  = CONTAINS THE SEQUENCED TOPOLOGY DATA, INDICATING FOR EACH BODY
C              WHICH HINGE IS THE INBOARD HINGE, AND WHETHER THE ASSOCIATED
C              HINGE FRAME IS A P-FRAME OR A Q-FRAME.  HINGE DATA SEQUENCED
C              TO BEGIN FROM BODY 1 AND TO PROGRESS OUTWARD UNTIL THE END
C              BODY OF EACH BRANCH IS REACHED.  SEQUENCING IS NOT UNIQUE;
C              HOWEVER, THE PATH FROM THE HINGE/BODY PAIR DEFINED IN COLUMN I
C              OF ITOPW TO BODY 1 MUST BE DEFINED IN COLUMNS 1,..,I-1 OF
C              ITOPW.  ITOPW IS STORED IN /ORDERN/.  IT IS NEEDED FOR FORWARD
C              AND BACKWARD SWEEPING IN RECURSIVE ALGORITHM.  THE ITOPOL
C              ARRAY IS USED TO FIND THE INBOARD BODY NUMBER.
C                  ITOPW(1,I) = INBOARD HINGE NUMBER FOR BODY ITOPW(2,I)
C                               NEGATIVE IF P-FRAME ON OUTBOARD BODY
C                               POSITIVE IF Q-FRAME ON OUTBOARD BODY
C                  ITOPW(2,I) = OUTBOARD BODY NUMBER FOR HINGE ITOPW(1,I)
C
C         FOR TOPOLOGICAL TREES ITOPW IS INTERNALLY COMPUTED.
C
C         FOR TOPOLOGICAL LOOPS ITOPW IS USER SUPPLIED VIA NAMELIST INPUT.
C             ####  THIS IS NOT NECESSARY, IF USER DEFINES THE CUT JOINTS,
C             ####  THEN ITOPOL ARRAY DATA SUFFICIENT TO COMPUTE ITOPW
C             ####  IHPATH DATA CAN ALSO BE COMPUTED.  PUT IN CTINIT
C
C
C         ROTDH - TRANSFORMATION MATRICES COMPUTED THEREIN
C         ------------------------------------------------
C     THE FOLLOWING ARRAYS ARE FUNCTIONS OF TIME. ELEMENTS OF THE STATE
C        VECTOR Y ARE USED IN ROTDH TO COMPUTE THE KINEMATIC
C        TRANSFORMATION MATRICES ASSOCIATED WITH HINGE POINT AND BODY
C        FIXED REFERENCE FRAMES, SET DEBUG(44) = .TRUE. TO PRINT ARRAYS
C
C     BETAH  = EULER ANGLES UPDATED FROM STATE VECTOR Y AT TIME T IN
C              SUBROUTINE ROTDH, STORED IN /SPECIF/
C     DH     = REFERENCE POINT, HINGE POINT POSITION VECTOR ARRAY
C              DIMENSIONED (3,7*(NHMAX-1)), STORED IN /SPECIF/. ACROSS
C              HINGE POINT N0H, N0H=2,3,...,NH THE FOLLOWING POSITION
C              VECTORS ARE COMPUTED. LET
C                        LD1 = 7*(N0H-2) + 1
C              THEN
C          (DH(J,LD1+0),J=1,3) = 3 COORDINATES OF THE ORIGIN OF THE
C                                Q-FRAME ON BODY N=ITOPOL(1,N0H) WRT THE
C                                BODY N REFERENCE IN THE UNDEFORMED
C                                STATE (MSMODL OR MSMODC)
C          (DH(J,LD1+1),J=1,3) = 3 COORDINATES OF THE ORIGIN OF THE
C                                P-FRAME ON BODY M=ITOPOL(2,N0H) WRT THE
C                                BODY M REFERENCE IN THE UNDEFORMED
C                                STATE (MSMODL OR MSMODC)
C          (DH(J,LD1+2),J=1,3) = 3 COORDINATES OF THE ORIGIN OF THE
C                                Q-FRAME ON BODY N=ITOPOL(1,N0H) WRT THE
C                                BODY N REFERENCE, DEFORMED STATE
C          (DH(J,LD1+3),J=1,3) = 3 COORDINATES OF THE ORIGIN OF THE
C                                P-FRAME ON BODY M=ITOPOL(2,N0H) WRT THE
C                                BODY M REFERENCE DEFORMED STATE
C          (DH(J,LD1+4),J=1,3) = 3 COORDINATES OF THE ORIGIN OF THE
C                                Q-FRAME WRT THE P-FRAME AT HINGE
C                                POINT N0H
C          (DH(J,LD1+5),J=1,3) = 3 COORDINATES OF THE ORIGIN OF THE
C                                BODY N=ITOPOL(1,N0H) FIXED REFERENCE
C                                WRT THE BODY M=ITOPOL(2,N0H) FIXED
C                                REFERENCE
C          (DH(J,LD1+6),J=1,3) = 3 COORDINATES OF THE POSITION VECTOR
C                                FROM THE ORIGIN OF THE BODY
C                                M=ITOPOL(2,N0H) FIXED REFERENCE TO THE
C                                ORIGIN OF THE BODY N=ITOPOL(1,N0H)
C                                FIXED REFERENCE WRT THE INERTIAL
C                                REFERENCE FRAME
C     RH     = BODY FIXED REFERENCE FRAME, HINGE POINT REFERENCE FRAME
C              TRANSFORMATION MATRICES, DIMENSIONED (3,3,6*(NHMAX-1)),
C              STORED IN /SPECIF/. ACROSS HINGE POINT N0H, N0H=2,...,NH
C              THE FOLLOWING COORDINATE TRANSFORMATION MATRICES ARE
C              COMPUTED. LET
C                        LR1 = 6*(N0H-2) + 1
C              THEN
C        ((RH(I,J,LR1+0),I=1,3),J=1,3) = 9 ELEMENTS OF TRANSFORMATION
C                                MATRIX USED TO TAKE VECTORS FROM
C                                Q-FRAME COORDINATES AT HINGE POINT N0H
C                                TO BODY N=ITOPOL(1,N0H) FIXED REFERENCE
C                                FRAME COORDINATES IN BODY N UNDEFORMED
C                                STATE
C        ((RH(I,J,LR1+1),I=1,3),J=1,3) = 9 ELEMENTS OF TRANSFORMATION
C                                MATRIX USED TO TAKE VECTORS FROM
C                                P-FRAME COORDINATES AT HINGE POINT N0H
C                                TO BODY M=ITOPOL(2,N0H) FIXED REFERENCE
C                                FRAME COORDINATES IN BODY M UNDEFORMED
C                                STATE
C        ((RH(I,J,LR1+2),I=1,3),J=1,3) = TRANSFORMATION MATRIX Q-FRAME
C                                COORDINATES AT HINGE POINT N0H TO
C                                BODY N=ITOPOL(1,N0H) FIXED COORDINATES
C                                IN BODY N DEFORMED STATE
C        ((RH(I,J,LR1+3),I=1,3),J=1,3) = TRANSFORMATION MATRIX P-FRAME
C                                COORDINATES AT HINGE POINT N0H TO
C                                BODY M=ITOPOL(1,N0H) FIXED COORDINATES
C                                IN BODY M DEFORMED STATE
C        ((RH(I,J,LR1+4),I=1,3),J=1,3) = TRANSFORMATION MATRIX Q-FRAME
C                                COORDINATES TO P-FRAME COORDINATES AT
C                                HINGE POINT N0H
C        ((RH(I,J,LR1+5),I=1,3),J=1,3) = TRANSFORMATION MATRIX FROM
C                                BODY N=ITOPOL(1,N0H) FIXED COORDINATES
C                                TO BODY M=ITOPOL(2,N0H) FIXED
C                                COORDINATES IN THE DEFORMED STATE OF
C                                BODIES N AND M
C     R0L    = BODY FIXED TO INERTIAL FRAME TRANSFORMATION MATRIX ARRAYS
C              DIMENSIONED (3,3,NBMAX), STORED IN /BHBSRD/
C         ((R0L(I,J,N),I=1,3),J=1,3) = 9 ELEMENTS OF THE TRANSFORMATION
C                                      MATRIX USED TO TRANSFORM VECTORS
C                                      FROM BODY N FIXED REFERENCE FRAME
C                                      COORDINATES TO THE INERTIAL FIXED
C                                      REFERENCE FRAME
C     D0L    = POSITION VECTOR ARRAY, INERTIAL ORIGIN TO BODY FIXED
C              FRAME ORIGIN. DIMENSIONED (3,NBMAX), STORED IN /BHBSRD/
C                   (D0L(I,N),I=1,3) = 3 COMPONENTS OF THE POSITION
C                                      VECTOR FROM THE INERTIAL ORIGIN
C                                      TO THE ORIGIN OF THE BODY N FIXED
C                                      REFERENCE WRT THE INERTIALLY
C                                      FIXED REFERENCE
C     PIN    = PI INVERSE TRANSFORMATIONS FOR HINGE POINT KINEMATICS
C              DIMENSIONED (3,3,NHMAX), STORED IN /PINRP/,
C              SEE APPENDIX B VOL 1, EQUATION B-14
C         ((PIN(I,J,N0H),I=1,3),J=1,3) = PI INVERSE TRANSFORMATION USED
C                                  TO RELATE THE EULER RATES ABOUT THE
C                                  SKEWED AXES AT HINGE POINT N0H TO THE
C                                  COMPONENTS OF THE RELATIVE ANGULAR
C                                  VELOCTY VECTOR OF THE Q-FRAME WRT THE
C                                  P-FRAME AT HINGE POINT N0H.
C     PIMAT  = PI TRANSFORMATIONS REQUIRED IN THE DEVELOPMENT OF THE
C              R_SUB(I,i) TRANSFORMATION USED IN THE FIRST FORWARD PASS
C              THROUGH THE RECURSIVE ORDER N ALGORITHM,  SEE ORDER N
C              DISCOS REFERENCE.  DIMENSIONED (3,3,NHMAX), STORED IN /ORDERN/,
C              SEE APPENDIX B VOL 1, EQUATION B-13
C        ((PIMAT(I,J,NOH)I=1,3),J=1,3) = PI TRANSFORMATION USED
C                                  TO RELATE THE EULER RATES ABOUT THE
C                                  SKEWED AXES AT HINGE POINT N0H TO THE
C                                  COMPONENTS OF THE RELATIVE ANGULAR
C                                  VELOCTY VECTOR OF THE Q-FRAME WRT THE
C                                  P-FRAME AT HINGE POINT N0H.
C     RP2    = ARRAYS USED TO STORE MATRICES REQUIRED FOR THE
C     RP3    = COMPUTATION OF THE TIME DERIVATIVE OF THE PI INVERSE
C              MATRIX, BOTH DIMENSIONED (3,3,NHMAX) AND STORED
C              IN /PINPR/
C         ((RP2(I,J,NOH),I=1,3),J=1,3) = 3 BY 3 COEFFICIENT MATRIX FOR
C                                  THETA SUB 2 DOT IN EQ C-16 VOL 1 USED
C                                  TO GET PI INVERSE DOT AT HINGE N0H
C         ((RP3(I,J,NOH),I=1,3),J=1,3) = 3 BY 3 COEFFICIENT MATRIX FOR
C                                  THETA SUB 3 DOT IN EQ C-16 VOL 1 USED
C                                  TO GET PI INVERSE DOT AT HINGE N0H
C
CAE
C
#include "PARAM.FOR"
C
#include "BHBSRD.INC"
#include "CHARMM.INC"
#include "CPATHS.INC"
#include "ERRHDL.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "JILFLG.INC"
#include "LDEBUG.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "PINRP.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "TIMESS.INC"
#include "VECTOR.INC"
#include "MISCNO.INC"
C
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: rotdh.f,v $$Revision: 1.1.1.1 $'/
c
c     Passed variables
c
      DOUBLE PRECISION Y(cL07)
      DOUBLE PRECISION PIMAT(3,3,cL02)
      DOUBLE PRECISION PIN(3,3,cL02)
      DOUBLE PRECISION RP2(3,3,cL02)
      DOUBLE PRECISION RP3(3,3,cL02)
      DOUBLE PRECISION BETAH(6,cL02)
      DOUBLE PRECISION RH(3,3,cL15)
      DOUBLE PRECISION DH(3,cL16)
      LOGICAL ISCUT(cL02)
      INTEGER IHTHPTR(2*cL02)
      INTEGER IHPVT(cL02) 
      INTEGER IBETDT(cL02)
      INTEGER ITOPOL(2,cL02)
      INTEGER IHDATA(7,cL02)
      INTEGER IHCUTJ(cL63)
c
C     Local Variables
C
      INTEGER I,J,I1,I2,I3,L
      INTEGER NBET,IDOF1,IBT,NOBQ,NOBP
      INTEGER LR1,LR2,LR3,LR4,LR5,LR6,LR0,LR0J
      INTEGER LD1,LD2,LD3,LD4,LD5,LD6,LD7,LD0,LD0J
      INTEGER NMQ,NMP,LU,LHS,IT,JP3,NOH
      INTEGER ICUT,ICOUNT
      DOUBLE PRECISION DF(3),ANGF(3),RW(3,3)
      DOUBLE PRECISION BLEN,BLENSQ,DUM,TIME
C
      LOGICAL LEQU
      EQUIVALENCE (LEQU,DEBUG(44))
C
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE ROTDH ')
 2019 FORMAT(A,I3,A,F10.5)
C
C        LOAD BETAH WITH UPDATED ANGLES FROM Y STATE VECTOR
C
      NBET = LOCU(2*NB + 1) - 1
      DO J = 1, NH
         if (iscut(j)) go to 108
         if (ihdata(1,J) .EQ. 0) then
            idof1 = 4
         else
            idof1 = 1
         end if
         ibt = nbet
         DO I = idof1, 6
            I1 = I + 1
            IF (IHDATA(I1,J) .NE. 1) then
               ibt = ibt + 1
               BETAH(I,J) = Y(ibt)
            END IF
         end do
         NBET = IBT
 108     continue
      end do
c
c     Re-sequence the singular Euler sequences if singular
c
      IF (jil .EQ. 4) CALL check_sng1(.NOT. verlet,
     &                     logicLoopHeap(iiscutptr),
     &                     intLoopHeap(iihpvtptr),
     &                     LoopHeap(ibetahptr),
     &                     intLoopHeap(iihdataptr))
C
      DO 10 I=2,NH
         if (iscut(i)) go to 10
      NOBQ = ITOPOL(1,I)
      NOBP = ITOPOL(2,I)
C        LR1 = 1,7,13,19,...
C        LD1 = 1,8,15,22,...
      LR1 = 1 + 6*(I-2)
      LD1 = 1 + 7*(I-2)
      LR2 = LR1 + 1
      LR3 = LR1 + 2
      LR4 = LR1 + 3
      LR5 = LR1 + 4
      LR6 = LR1 + 5
      LD2 = LD1 + 1
      LD3 = LD1 + 2
      LD4 = LD1 + 3
      LD5 = LD1 + 4
      LD6 = LD1 + 5
      LD7 = LD1 + 6
      NMQ = IRGFLX(NOBQ)
      NMP = IRGFLX(NOBP)
C
C     GO TO 15 IF BODY CONTAINING Q FRAME RIGID
      IF (NMQ .EQ. 0) GO TO 15
C
C     ACCOUNT FOR BODY FLEXIBILITY TO GET POSITION DH(LD3) AND
C        ORIENTATION RH(LR3) OF Q FRAME AT HINGE I WRT ITS BODY FRAME
      LU = LOCU(NOBQ+NB)
      LHS = 2*I - 3
C Original: MULT3 HATS(hthptr(LHS)+1),Y(LU),DF,3,NMQ,1,3,NMQ,3
CC@CC
CC@CC MULT3 HATS(ihthptr(LHS)+1), Y(LU), DF
CC@CC 3, NMQ, 1
CC@CC 3, NMQ, 3
      CALL DGEMV ('N',3,NMQ,1.0d0,HATS(ihthptr(LHS)+1),3,Y(LU),1,0.0d0,D
     .F,1)
CC@CC end of expanded call
CC@CC
      DO 12 J=1,3
   12 DH(J,LD3) = DH(J,LD1) + DF(J)
C
C     GO TO 20 IF BODY CONTAINING P FRAME RIGID
   15 IF (NMP .EQ. 0) GO TO 20
C
C     ACCOUNT FOR BODY FLEXIBILITY TO GET POSITION DH(LD4) AND
C        ORIENTATION RH(LR4) OF P FRAME AT HINGE I WRT ITS BODY FRAME
      LU = LOCU(NOBP+NB)
      LHS = 2*I - 2
C Original: MULT3 HATS(hthptr(LHS)+1),Y(LU),DF,3,NMP,1,3,NMP,3
CC@CC
CC@CC MULT3 HATS(ihthptr(LHS)+1), Y(LU), DF
CC@CC 3, NMP, 1
CC@CC 3, NMP, 3
      CALL DGEMV ('N',3,NMP,1.0d0,HATS(ihthptr(LHS)+1),3,Y(LU),1,0.0d0,D
     .F,1)
CC@CC end of expanded call
CC@CC
      DO 17 J=1,3
   17 DH(J,LD4) = DH(J,LD2) + DF(J)
C
   20 continue
c
      if (ihpvt(I) .ne. 0) then
c                 beta (r,theta,gamma) for translational dofs
         blen = betah(4,i)
         call rotran(i,intLoopHeap(iihpvtptr),
     *               LoopHeap(ibetahptr),Loopheap(irhptr))
         dh(1,ld5) = rh(1,1,LR2) * blen
         dh(2,ld5) = rh(2,1,LR2) * blen
         dh(3,ld5) = rh(3,1,LR2) * blen
      else      !   beta (x,y,z) for translational dofs, which are all free
         DO J = 1, 3
            DH(J,LD5) = BETAH(J+3,I)
         end do
      end if
C
C     ACCOUNT FOR RELATIVE MOTION BETWEEN P AND Q FRAMES AT HINGE I
C        RH(LR5) USE INPUTTED BETAH ANGLES
      IT = IHDATA(1,I)
      if (IT .ne. 0) then
         CALL ROTTR(3,IT,betah(1,i),RH(1,1,LR5),DUM,DUM)
         if (nobp .GT. nbdy) then
            r0l(1,1,nobq) = rh(1,1,lr5)
            r0l(2,1,nobq) = rh(2,1,lr5)
            r0l(3,1,nobq) = rh(3,1,lr5)
            r0l(1,2,nobq) = rh(1,2,lr5)
            r0l(2,2,nobq) = rh(2,2,lr5)
            r0l(3,2,nobq) = rh(3,2,lr5)
            r0l(1,3,nobq) = rh(1,3,lr5)
            r0l(2,3,nobq) = rh(2,3,lr5)
            r0l(3,3,nobq) = rh(3,3,lr5)
         end if
C
C     GET PI AND PI INVERSE MATRIX OF EQUATION II-19 VOL 1 AND APPENDIX B
C             PIN(I) = INVERSE ROTATION TRANSFORMATION MATRIX RELATING BODY
C                      FIXED ANGULAR RATES TO EULER RATES BETWEEN P AND
C                      Q FRAME AT HINGE I(NON-ORTHONORMAL)
C           PIMAT(I) = ROTATION TRANSFORMATION MATRIX RELATING BODY
C                      FIXED ANGULAR RATES TO EULER RATES BETWEEN P AND
C                      Q FRAME AT HINGE I(NON-ORTHONORMAL)
         CALL ROTTR (1,IT,betah(1,i),PIN(1,1,I),RP2(1,1,I),
     +        RP3(1,1,I))
         CALL ROTTR (-1,IT,betah(1,i),PIMAT(1,1,I),DUM,DUM)
C
         rh(1,1,lr6) = rh(1,1,lr5)
         rh(2,1,lr6) = rh(2,1,lr5)
         rh(3,1,lr6) = rh(3,1,lr5)
         rh(1,2,lr6) = rh(1,2,lr5)
         rh(2,2,lr6) = rh(2,2,lr5)
         rh(3,2,lr6) = rh(3,2,lr5)
         rh(1,3,lr6) = rh(1,3,lr5)
         rh(2,3,lr6) = rh(2,3,lr5)
         rh(3,3,lr6) = rh(3,3,lr5)
      end if
C
   10 CONTINUE
C
C        CHECK ELEVATION GAMMA AT END OF EACH STEP TO PREVENT SINGULARITY
C
      IF (jil .EQ. 4) CALL check_sng2(.NOT. verlet,
     &                    HEAP_MB(iyptr),
     &                    logicLoopHeap(iiscutptr),
     &                    intLoopHeap(iichpvtptr),
     &                    intLoopHeap(iihpvtptr),
     &                    intLoopheap(iibetdtptr),
     &                    LoopHeap(ibetahptr),
     &                    intLoopheap(iihdataptr),
     &                    LoopHeap(irhptr),
     &                    LoopHeap(idhptr))
c
C        MOST OF THE RELATIVE POSITION AND ORIENTATION DATA OF P-Q FRAME AT
C        EACH HINGE POINT COMPUTED AND STORED IN DH AND RH ARRAYS
C
C        PROCEED TO COMPUTE
C             R0L(K) ROTATION TRANSFORMATION MATRIX BODY K FIXED AXIS
C                    TO INERTIAL AXIS FOR BODIES NOT DIRECTLY HINGED
C                    TO THE INERTIAL FRAME (PARTICLES ARE INERTIALLY ORIENTED)
C             D0L(K) POSITION VECTOR FROM INERTIAL ORIGIN TO BODY
C                    K FIXED REFERENCE
C             PIN, AND PIMAT FOR INERTIAL FRAME
      DO 45 J = 1, 3
         JP3 = J + 3
         D0L(J,1) = BETAH(JP3,1)
   45 continue
      if (nbdy .GT. 0) then  ! Body 1 is a body
         do j = 1, 3
            ANGF(J) = BETAH(J,1)
         end do
         IT = IHDATA(1,1)
         CALL ROTTR  (3,IT,ANGF,R0L(1,1,1),DUM,DUM)
         CALL ROTTR (1,IT,ANGF,PIN(1,1,1),RP2(1,1,1),RP3(1,1,1))
         CALL ROTTR (-1,IT,ANGF,PIMAT(1,1,1),DUM,DUM)
      end if
C
      DO 50 J=2,NB
      NOH  = ITOPW(1,J)
      LR0J = ITOPW(2,J)
      IF (NOH .LT. 0) GO TO 52
      LR6 = 6*(NOH - 1)
      LR0 = ITOPOL(2,NOH)
      if (lr0 .LE. nbdy .and. lr0j .LE. nbdy) then
c         both bodies
CC@CC
CC@CC ...MULT3 R0L(1,1,LR0), RH(1,1,LR6), R0L(1,1,LR0J)
CC@CC ...3, 3, 3
CC@CC ...3, 3, 3
      R0L(1,1,LR0J) = R0L(1,1,LR0)*RH(1,1,LR6) + R0L(1,2,LR0)*RH(2,1,LR6
     .) + R0L(1,3,LR0)*RH(3,1,LR6)
      R0L(2,1,LR0J) = R0L(2,1,LR0)*RH(1,1,LR6) + R0L(2,2,LR0)*RH(2,1,LR6
     .) + R0L(2,3,LR0)*RH(3,1,LR6)
      R0L(3,1,LR0J) = R0L(3,1,LR0)*RH(1,1,LR6) + R0L(3,2,LR0)*RH(2,1,LR6
     .) + R0L(3,3,LR0)*RH(3,1,LR6)
      R0L(1,2,LR0J) = R0L(1,1,LR0)*RH(1,2,LR6) + R0L(1,2,LR0)*RH(2,2,LR6
     .) + R0L(1,3,LR0)*RH(3,2,LR6)
      R0L(2,2,LR0J) = R0L(2,1,LR0)*RH(1,2,LR6) + R0L(2,2,LR0)*RH(2,2,LR6
     .) + R0L(2,3,LR0)*RH(3,2,LR6)
      R0L(3,2,LR0J) = R0L(3,1,LR0)*RH(1,2,LR6) + R0L(3,2,LR0)*RH(2,2,LR6
     .) + R0L(3,3,LR0)*RH(3,2,LR6)
      R0L(1,3,LR0J) = R0L(1,1,LR0)*RH(1,3,LR6) + R0L(1,2,LR0)*RH(2,3,LR6
     .) + R0L(1,3,LR0)*RH(3,3,LR6)
      R0L(2,3,LR0J) = R0L(2,1,LR0)*RH(1,3,LR6) + R0L(2,2,LR0)*RH(2,3,LR6
     .) + R0L(2,3,LR0)*RH(3,3,LR6)
      R0L(3,3,LR0J) = R0L(3,1,LR0)*RH(1,3,LR6) + R0L(3,2,LR0)*RH(2,3,LR6
     .) + R0L(3,3,LR0)*RH(3,3,LR6)
CC@CC end of expanded call
CC@CC
      end if
      GO TO 50
   52 NOH = -NOH
      LR6 = 6*(NOH - 1)
      LR0 = ITOPOL(1,NOH)
      if (lr0 .LE. nbdy .and. lr0j .LE. nbdy) then
c          both bodies
         DO 53 I=1,3
         DO 53 L=1,3
   53    RW(I,L) = RH(L,I,LR6)
CC@CC
CC@CC ...MULT3 R0L(1,1,LR0), RW, R0L(1,1,LR0J)
CC@CC ...3, 3, 3
CC@CC ...3, 3, 3
      R0L(1,1,LR0J) = R0L(1,1,LR0)*RW(1,1) + R0L(1,2,LR0)*RW(2,1) + R0L(
     .1,3,LR0)*RW(3,1)
      R0L(2,1,LR0J) = R0L(2,1,LR0)*RW(1,1) + R0L(2,2,LR0)*RW(2,1) + R0L(
     .2,3,LR0)*RW(3,1)
      R0L(3,1,LR0J) = R0L(3,1,LR0)*RW(1,1) + R0L(3,2,LR0)*RW(2,1) + R0L(
     .3,3,LR0)*RW(3,1)
      R0L(1,2,LR0J) = R0L(1,1,LR0)*RW(1,2) + R0L(1,2,LR0)*RW(2,2) + R0L(
     .1,3,LR0)*RW(3,2)
      R0L(2,2,LR0J) = R0L(2,1,LR0)*RW(1,2) + R0L(2,2,LR0)*RW(2,2) + R0L(
     .2,3,LR0)*RW(3,2)
      R0L(3,2,LR0J) = R0L(3,1,LR0)*RW(1,2) + R0L(3,2,LR0)*RW(2,2) + R0L(
     .3,3,LR0)*RW(3,2)
      R0L(1,3,LR0J) = R0L(1,1,LR0)*RW(1,3) + R0L(1,2,LR0)*RW(2,3) + R0L(
     .1,3,LR0)*RW(3,3)
      R0L(2,3,LR0J) = R0L(2,1,LR0)*RW(1,3) + R0L(2,2,LR0)*RW(2,3) + R0L(
     .2,3,LR0)*RW(3,3)
      R0L(3,3,LR0J) = R0L(3,1,LR0)*RW(1,3) + R0L(3,2,LR0)*RW(2,3) + R0L(
     .3,3,LR0)*RW(3,3)
CC@CC end of expanded call
CC@CC
      end if
   50 CONTINUE
C
c     Vector from m-origin to n-origin is transformed to inertial coordinates
c
      DO 60 J=2,NH
         if (iscut(j)) go to 60
         nobp = ITOPOL(2,J)
         nobq = itopol(1,j)
         LD3  = 3 + 7*(J-2)
         ld4  = ld3 + 1
         ld5  = ld3 + 2
         LD6  = ld3 + 3
         LD7  = LD3 + 4
C
C             DH(LD6) = VECTOR BETWEEN BODY FIXED REFERENCE FRAMES OF
C                       BODIES CONTIGUOUS AT HINGE J
         if (nobq .LE. nbdy) then
            if (ihdata(1,J) .ne. 0) then
               LR6 = 6*(J - 1)
CC@CC
CC@CC .........MULT3 RH(1,1,LR6), DH(1,LD3), ANGF
CC@CC .........3, 3, 1
CC@CC .........3, 3, 3
      ANGF(1) = RH(1,1,LR6)*DH(1,LD3) + RH(1,2,LR6)*DH(2,LD3) + RH(1,3,L
     .R6)*DH(3,LD3)
      ANGF(2) = RH(2,1,LR6)*DH(1,LD3) + RH(2,2,LR6)*DH(2,LD3) + RH(2,3,L
     .R6)*DH(3,LD3)
      ANGF(3) = RH(3,1,LR6)*DH(1,LD3) + RH(3,2,LR6)*DH(2,LD3) + RH(3,3,L
     .R6)*DH(3,LD3)
CC@CC end of expanded call
CC@CC
            else
CC@CC
CC@CC .........MULT3 r0l(1,1,nobq), dh(1,ld3), angf
CC@CC .........3, 3, 1
CC@CC .........3, 3, 3
      angf(1) = r0l(1,1,nobq)*dh(1,ld3) + r0l(1,2,nobq)*dh(2,ld3) + r0l(
     .1,3,nobq)*dh(3,ld3)
      angf(2) = r0l(2,1,nobq)*dh(1,ld3) + r0l(2,2,nobq)*dh(2,ld3) + r0l(
     .2,3,nobq)*dh(3,ld3)
      angf(3) = r0l(3,1,nobq)*dh(1,ld3) + r0l(3,2,nobq)*dh(2,ld3) + r0l(
     .3,3,nobq)*dh(3,ld3)
CC@CC end of expanded call
CC@CC
            end if
            if (nobp .LE. nbdy) then ! Both objects are rigid/flex bodies
               DO I = 1, 3
                  DH(I,LD6) = DH(I,LD4) + dh(I,ld5) - ANGF(I)
               end do
            else      ! Object with p-frame is particle, with q-frame is body
               DO I = 1, 3
                  DH(I,LD6) = DH(I,LD5) - ANGF(I)
               end do
            end if
         else                   ! Both objects are particles
            do i = 1, 3
               dh(i,ld6) = dh(i,ld5)
            end do
         end if
         if (nobp .GT. nbdy) then  ! P is a particle
            dh(1,ld7) = dh(1,ld6)
            dh(2,ld7) = dh(2,ld6)
            dh(3,ld7) = dh(3,ld6)
         else
CC@CC
CC@CC ......MULT3 R0L(1,1,NOBP), DH(1,LD6), DH(1,LD7)
CC@CC ......3, 3, 1
CC@CC ......3, 3, 3
      DH(1,LD7) = R0L(1,1,NOBP)*DH(1,LD6) + R0L(1,2,NOBP)*DH(2,LD6) + R0
     .L(1,3,NOBP)*DH(3,LD6)
      DH(2,LD7) = R0L(2,1,NOBP)*DH(1,LD6) + R0L(2,2,NOBP)*DH(2,LD6) + R0
     .L(2,3,NOBP)*DH(3,LD6)
      DH(3,LD7) = R0L(3,1,NOBP)*DH(1,LD6) + R0L(3,2,NOBP)*DH(2,LD6) + R0
     .L(3,3,NOBP)*DH(3,LD6)
CC@CC end of expanded call
CC@CC
         end if
   60 CONTINUE
C
c     Accumulate the d0l vectors
c
      DO 70 J = 2, NB
         NOH  = ITOPW(1,J)
         LD0J = ITOPW(2,J)
         IF (NOH .gt. 0) then
            LD0 = ITOPOL(2,NOH)
            LD7  = 7*(NOH - 1)
            DO I = 1, 3
               D0L(I,LD0J) = D0L(I,LD0) + DH(I,LD7)
            end do
         else
            NOH = - NOH
            LD0 = ITOPOL(1,NOH)
            LD7  = 7*(NOH - 1)
            DO I = 1, 3
               D0L(I,LD0J) = D0L(I,LD0) - DH(I,LD7)
            end do
         end if
   70 CONTINUE
C
C     COMPUTE THE CUT-JOINT HINGE DATA
C
      DO ICUT = 1, NCUT
         NOH  = IHCUTJ(ICUT)
         NOBP = ITOPOL(2,NOH)
         NOBQ = ITOPOL(1,NOH)
         NMP  = IRGFLX(NOBP)
         NMQ  = IRGFLX(NOBQ)
         LD1  = 7*(NOH-2) + 1
         LD2  = LD1 + 1
         LD3  = LD1 + 2
         LD4  = LD1 + 3
         LD5  = LD1 + 4
         LD6  = LD1 + 5
         LD7  = LD1 + 6
         LR1  = 1 + 6*(NOH-2)
         LR5  = LR1 + 4
C
C     COMPUTE DH(*,LD7), VECTOR FROM M-ORIGIN TO N-ORIGIN, IN INERTIAL
C     COORDINATES
C
         DH(1,LD7) = D0L(1,NOBQ) - D0L(1,NOBP)
         DH(2,LD7) = D0L(2,NOBQ) - D0L(2,NOBP)
         DH(3,LD7) = D0L(3,NOBQ) - D0L(3,NOBP)
C
         IF (NOBQ .LE. nbdy) THEN  ! q is a body
C
C     COMPUTE RH(LR5), TRANSFORMATION FROM Q-FRAME TO P-FRAME
C
            IF (NOBP .LE. nbdy) THEN  ! p is a body
CC@CC
CC@CC .........MULT3T R0L(1,1,NOBP), R0L(1,1,NOBQ), RH(1,1,LR5)
CC@CC .........3, 3, 3
CC@CC .........3, 3, 3
      RH(1,1,LR5) = R0L(1,1,NOBP)*R0L(1,1,NOBQ) + R0L(2,1,NOBP)*R0L(2,1,
     .NOBQ) + R0L(3,1,NOBP)*R0L(3,1,NOBQ)
      RH(2,1,LR5) = R0L(1,2,NOBP)*R0L(1,1,NOBQ) + R0L(2,2,NOBP)*R0L(2,1,
     .NOBQ) + R0L(3,2,NOBP)*R0L(3,1,NOBQ)
      RH(3,1,LR5) = R0L(1,3,NOBP)*R0L(1,1,NOBQ) + R0L(2,3,NOBP)*R0L(2,1,
     .NOBQ) + R0L(3,3,NOBP)*R0L(3,1,NOBQ)
      RH(1,2,LR5) = R0L(1,1,NOBP)*R0L(1,2,NOBQ) + R0L(2,1,NOBP)*R0L(2,2,
     .NOBQ) + R0L(3,1,NOBP)*R0L(3,2,NOBQ)
      RH(2,2,LR5) = R0L(1,2,NOBP)*R0L(1,2,NOBQ) + R0L(2,2,NOBP)*R0L(2,2,
     .NOBQ) + R0L(3,2,NOBP)*R0L(3,2,NOBQ)
      RH(3,2,LR5) = R0L(1,3,NOBP)*R0L(1,2,NOBQ) + R0L(2,3,NOBP)*R0L(2,2,
     .NOBQ) + R0L(3,3,NOBP)*R0L(3,2,NOBQ)
      RH(1,3,LR5) = R0L(1,1,NOBP)*R0L(1,3,NOBQ) + R0L(2,1,NOBP)*R0L(2,3,
     .NOBQ) + R0L(3,1,NOBP)*R0L(3,3,NOBQ)
      RH(2,3,LR5) = R0L(1,2,NOBP)*R0L(1,3,NOBQ) + R0L(2,2,NOBP)*R0L(2,3,
     .NOBQ) + R0L(3,2,NOBP)*R0L(3,3,NOBQ)
      RH(3,3,LR5) = R0L(1,3,NOBP)*R0L(1,3,NOBQ) + R0L(2,3,NOBP)*R0L(2,3,
     .NOBQ) + R0L(3,3,NOBP)*R0L(3,3,NOBQ)
CC@CC end of expanded call
CC@CC
            ELSE
               RH(1,1,LR5) = R0L(1,1,NOBQ)
               RH(2,1,LR5) = R0L(2,1,NOBQ)
               RH(3,1,LR5) = R0L(3,1,NOBQ)
               RH(1,2,LR5) = R0L(1,2,NOBQ)
               RH(2,2,LR5) = R0L(2,2,NOBQ)
               RH(3,2,LR5) = R0L(3,2,NOBQ)
               RH(1,3,LR5) = R0L(1,3,NOBQ)
               RH(2,3,LR5) = R0L(2,3,NOBQ)
               RH(3,3,LR5) = R0L(3,3,NOBQ)
            END IF
c
c     Extract cut-joint Euler angles
c
 208        CONTINUE
            IT = IHDATA(1,NOH)
            if (it .eq. 1) then !  1-2-3
               IF (abs(rh(1,3,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 2
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,1,lr5)**2 + rh(1,2,lr5)**2)
               betah(1,noh) = atan2(-rh(2,3,lr5), rh(3,3,lr5))
               betah(2,noh) = atan2( rh(1,3,lr5), dum)
               betah(3,noh) = atan2(-rh(1,2,lr5), rh(1,1,lr5))
c     
            else if (it .eq. 2) then !  1-2-1
               if (abs(rh(1,1,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 3
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,2,lr5)**2 + rh(1,3,lr5)**2)
               betah(1,noh) = atan2( rh(2,1,lr5),-rh(3,1,lr5))
               betah(2,noh) = atan2( dum,         rh(1,1,lr5))
               betah(3,noh) = atan2( rh(1,2,lr5), rh(1,3,lr5))
c     
            else if (it .eq. 3) then !  1-3-1
               if (abs(rh(1,1,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 4
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,2,lr5)**2 + rh(1,3,lr5)**2)
               betah(1,noh) = atan2( rh(3,1,lr5), rh(2,1,lr5))
               betah(2,noh) = atan2( dum,         rh(1,1,lr5))
               betah(3,noh) = atan2( rh(1,3,lr5),-rh(1,2,lr5))
c     
            else if (it .eq. 4) then !  1-3-2
               if (abs(rh(1,2,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 5
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,1,lr5)**2 + rh(1,3,lr5)**2)
               betah(1,noh) = atan2( rh(3,2,lr5), rh(2,2,lr5))
               betah(2,noh) = atan2(-rh(1,2,lr5), dum)
               betah(3,noh) = atan2( rh(1,3,lr5), rh(1,1,lr5))
c     
            else if (it .eq. 5) then !  2-3-1
               if (abs(rh(2,1,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 6
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,1,lr5)**2 + rh(3,1,lr5)**2)
               betah(1,noh) = atan2(-rh(3,1,lr5), rh(1,1,lr5))
               betah(2,noh) = atan2( rh(2,1,lr5), dum)
               betah(3,noh) = atan2(-rh(2,3,lr5), rh(2,2,lr5))
c
            else if (it .eq. 6) then !  2-3-2
               if (abs(rh(2,2,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 7
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,2,lr5)**2 + rh(3,2,lr5)**2)
               betah(1,noh) = atan2( rh(3,2,lr5),-rh(1,2,lr5))
               betah(2,noh) = atan2( dum,         rh(2,2,lr5))
               betah(3,noh) = atan2( rh(2,3,lr5), rh(2,1,lr5))
c     
            else if (it .eq. 7) then !  2-1-2
               if (abs(rh(2,2,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 8
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,2,lr5)**2 + rh(3,2,lr5)**2)
               betah(1,noh) = atan2( rh(1,2,lr5), rh(3,2,lr5))
               betah(2,noh) = atan2( dum,         rh(2,2,lr5))
               betah(3,noh) = atan2( rh(2,1,lr5),-rh(2,3,lr5))
c
            else if (it .eq. 8) then !  2-1-3
               if (abs(rh(2,3,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 9
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,3,lr5)**2 + rh(3,3,lr5)**2)
               betah(1,noh) = atan2( rh(1,3,lr5), rh(3,3,lr5))
               betah(2,noh) = atan2(-rh(2,3,lr5), dum)
               betah(3,noh) = atan2( rh(2,1,lr5), rh(2,2,lr5))
c
            else if (it .eq. 9) then !  3-1-2
               if (abs(rh(3,2,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 10
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,2,lr5)**2 + rh(2,2,lr5)**2)
               betah(1,noh) = atan2(-rh(1,2,lr5), rh(2,2,lr5))
               betah(2,noh) = atan2( rh(3,2,lr5), dum)
               betah(3,noh) = atan2(-rh(3,1,lr5), rh(3,3,lr5))
c
            else if (it .eq. 10) then !  3-1-3
               if (abs(rh(3,3,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 11
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,3,lr5)**2 + rh(2,3,lr5)**2)
               betah(1,noh) = atan2( rh(1,3,lr5),-rh(2,3,lr5))
               betah(2,noh) = atan2( dum,         rh(3,3,lr5))
               betah(3,noh) = atan2( rh(3,1,lr5), rh(3,2,lr5))
c
            else if (it .eq. 11) then !  3-2-3
               if (abs(rh(3,3,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 12
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,3,lr5)**2 + rh(2,3,lr5)**2)
               betah(1,noh) = atan2( rh(2,3,lr5), rh(1,3,lr5))
               betah(2,noh) = atan2( dum,         rh(3,3,lr5))
               betah(3,noh) = atan2( rh(3,2,lr5),-rh(3,1,lr5))
c
            else if (it .eq. 12) then !  3-2-1
               if (abs(rh(3,1,lr5)) .gt. 0.95d0) THEN
                  ihdata(1,noh) = 1
                  GOTO 208
               ENDIF
               dum = DSQRT(rh(1,1,lr5)**2 + rh(2,1,lr5)**2)
               betah(1,noh) = atan2( rh(2,1,lr5), rh(1,1,lr5))
               betah(2,noh) = atan2(-rh(3,1,lr5), dum)
               betah(3,noh) = atan2( rh(3,2,lr5), rh(3,3,lr5))
c
            end if
c
c     COMPUTE PIN, RP2, AND RP3.
c
            do j = 1, 3
               ANGF(J) = BETAH(J,NOH)
            end do
            CALL ROTTR(1,IT,ANGF,PIN(1,1,NOH),RP2(1,1,NOH),RP3(1,1,NOH))
C
         END IF
C
         IF (NOBP .GT. nbdy) THEN  ! p is a particle
C
C     COMPUTE DH(LD6), VECTOR FROM M-ORIGIN TO N-ORIGIN, IN M-FRAME
C     COORDINATES
C
            DH(1,LD6) = DH(1,LD7)
            DH(2,LD6) = DH(2,LD7)
            DH(3,LD6) = DH(3,LD7)
C
         ELSE      ! BOTH OBJECTS ARE RIGID/FLEX BODIES
C
C     COMPUTE DH(LD6), VECTOR FROM M-ORIGIN TO N-ORIGIN, IN M-FRAME
C     COORDINATES
C
CC@CC
CC@CC ......MULT3T R0L(1,1,NOBP), DH(1,LD7), DH(1,LD6)
CC@CC ......3, 3, 1
CC@CC ......3, 3, 3
      DH(1,LD6) = R0L(1,1,NOBP)*DH(1,LD7) + R0L(2,1,NOBP)*DH(2,LD7) + R0
     .L(3,1,NOBP)*DH(3,LD7)
      DH(2,LD6) = R0L(1,2,NOBP)*DH(1,LD7) + R0L(2,2,NOBP)*DH(2,LD7) + R0
     .L(3,2,NOBP)*DH(3,LD7)
      DH(3,LD6) = R0L(1,3,NOBP)*DH(1,LD7) + R0L(2,3,NOBP)*DH(2,LD7) + R0
     .L(3,3,NOBP)*DH(3,LD7)
CC@CC end of expanded call
CC@CC
C
         END IF
C
C     COMPUTE DH(LD5), VECTOR FROM P-ORIGIN TO Q-ORIGIN, IN P-FRAME
C     COORDINATES
C
         DH(1,LD5) = DH(1,LD6)
         DH(2,LD5) = DH(2,LD6)
         DH(3,LD5) = DH(3,LD6)
C
         IF (NOBQ .LE. nbdy) THEN 
            IF (NMQ .NE. 0) THEN
C
C     ACCOUNT FOR BODY FLEXIBILITY TO GET POSITION DH(LD3)
C
               LU = LOCU(NOBQ+NB)
               LHS = 2*NOH - 3
C Original: MULT3 HATS(hthptr(LHS)+1),Y(LU),DF,3,NMQ,1,3,NMQ,3
CC@CC
CC@CC .........MULT3 HATS(ihthptr(LHS)+1), Y(LU), DF
CC@CC .........3, NMQ, 1
CC@CC .........3, NMQ, 3
      CALL DGEMV ('N',3,NMQ,1.0d0,HATS(ihthptr(LHS)+1),3,Y(LU),1,0.0d0,D
     .F,1)
CC@CC end of expanded call
CC@CC
               DH(1,LD3) = DH(1,LD1) + DF(1)
               DH(2,LD3) = DH(2,LD1) + DF(2)
               DH(3,LD3) = DH(3,LD1) + DF(3)
            END IF
CC@CC
CC@CC ......MULTAD RH(1,1,LR5), DH(1,LD3), DH(1,LD5)
CC@CC ......3, 3, 1
CC@CC ......3, 3, 3
      DH(1,LD5) = DH(1,LD5) + RH(1,1,LR5)*DH(1,LD3) + RH(1,2,LR5)*DH(2,L
     .D3) + RH(1,3,LR5)*DH(3,LD3)
      DH(2,LD5) = DH(2,LD5) + RH(2,1,LR5)*DH(1,LD3) + RH(2,2,LR5)*DH(2,L
     .D3) + RH(2,3,LR5)*DH(3,LD3)
      DH(3,LD5) = DH(3,LD5) + RH(3,1,LR5)*DH(1,LD3) + RH(3,2,LR5)*DH(2,L
     .D3) + RH(3,3,LR5)*DH(3,LD3)
CC@CC end of expanded call
CC@CC
         END IF
C
         IF (NOBP .LE. nbdy) THEN
            IF (NMP .NE. 0) THEN
C
C     ACCOUNT FOR BODY FLEXIBILITY TO GET POSITION DH(LD4)
C
               LU = LOCU(NOBP+NB)
               LHS = 2*NOH - 2
C Original: MULT3 HATS(hthptr(LHS)+1),Y(LU),DF,3,NMP,1,,3,NMP,3
CC@CC
CC@CC .........MULT3 HATS(ihthptr(LHS)+1), Y(LU), DF
CC@CC .........3, NMP, 1
CC@CC .........3, NMP, 3
      CALL DGEMV ('N',3,NMP,1.0d0,HATS(ihthptr(LHS)+1),3,Y(LU),1,0.0d0,D
     .F,1)
CC@CC end of expanded call
CC@CC
               DH(1,LD4) = DH(1,LD2) + DF(1)
               DH(2,LD4) = DH(2,LD2) + DF(2)
               DH(3,LD4) = DH(3,LD2) + DF(3)
            END IF
            DH(1,LD5) = DH(1,LD5) - DH(1,LD4)
            DH(2,LD5) = DH(2,LD5) - DH(2,LD4)
            DH(3,LD5) = DH(3,LD5) - DH(3,LD4)
         END IF
C
C     COMPUTE THE TRANSLATIONAL BETA'S AT THE CUT JOINT
C
         IF (IHPVT(NOH) .NE. 0) THEN
C
            BLENSQ = DH(1,LD5)**2 + DH(2,LD5)**2 + DH(3,LD5)**2
            BETAH(4,NOH) = SQRT(BLENSQ)
            ICOUNT = 0
 205        CONTINUE
            ICOUNT = ICOUNT + 1
            IF (ICOUNT .GT. 2) THEN
               CALL CLEARERR()
               WRITE(ERRMSG,200)
     +           ' ERROR IN CUT-JOINT PIVOT TYPE LOOP'
               CALL ERRHDL(-5,'<ROTDH>')
            END IF
 200        FORMAT(1X,A)
            IF (IHPVT(NOH) .EQ. 1) THEN
               I1 = 1
               I2 = 2
               I3 = 3
            ELSE
               I1 = 2
               I2 = 3
               I3 = 1
            END IF
            BETAH(5,NOH) = ATAN2(DH(I2,LD5),DH(I1,LD5))
            BETAH(6,NOH) = ASIN(DH(I3,LD5)/BETAH(4,NOH))
C
C        CHECK ELEVATION GAMMA AT END OF EACH STEP TO PREVENT SINGULARITY
C
C23456789 123456789 123456789 123456789 123456789 123456789 123456789 12
            IF (JIL. EQ. 4 .AND.
     &          ABS(SIN(BETAH(6,NOH))).GT.EULER_THRESHOLD) THEN

CRCZ970801 attempt to use BETAHD*DELTAT leads to instability
C
C    &         (ABS(SIN(BETAH(6,NOH))).GT.EULER_THRESHOLD .OR.
C    &         ABS(SIN(BETAHD(6,NOH)*DELTAT)).GT.EULER_THRESHOLD)) THEN
c              Change the hinge pivoting Euler sequence
               IF (IHPVT(NOH) .EQ. 1) THEN
                  IHPVT(NOH) = 2
               ELSE
                  IHPVT(NOH) = 1
               END IF
               time = (t-startt)/20.455d0 + startt
               IF (mbplvl .eq. 9) WRITE (MBOUT,2010) 
     &              ' PIVOT TYPE CHANGED TO ', IHPVT(NOH),
     &              ' FOR CUT-JOINT HINGE', NOH, ' AT TIME = ', time
               GO TO 205
            END IF
C
C     COMPUTE RH(LR2) AND RH(LR4) FOR PIVOTING HINGE
C
            CALL ROTRAN(NOH,intLoopHeap(iihpvtptr),
     &              LoopHeap(ibetahptr),Loopheap(irhptr))
C
         ELSE
C
            BETAH(4,NOH) = DH(1,LD5)
            BETAH(5,NOH) = DH(2,LD5)
            BETAH(6,NOH) = DH(3,LD5)
C
         END IF
C
      END DO
C
      IF(.NOT.LEQU) RETURN
      I1 = 6*(NH-1)
      I2 = 7*(NH-1)
      WRITE (MBOUT,1002) (I,((RH(J,L,I),J=1,3),L=1,3),I=1,I1)
      WRITE (MBOUT,1003) (I,(DH(J,I),J=1,3),I=1,I2)
      WRITE (MBOUT,1004) (I,((PIN(J,L,I),J=1,3),L=1,3),I=1,NH)
      WRITE (MBOUT,1009) (I,((PIMAT(J,L,I),J=1,3),L=1,3),I=1,NH)
      WRITE (MBOUT,1005) (I,((R0L(J,L,I),J=1,3),L=1,3),I=1,NH)
      WRITE (MBOUT,1006) (I,(D0L(J,I),J=1,3),I=1,NH)
      WRITE (MBOUT,1007) (I,((RP2(J,L,I),J=1,3),L=1,3),I=1,NH)
      WRITE (MBOUT,1008) (I,((RP3(J,L,I),J=1,3),L=1,3),I=1,NH)
 1002 FORMAT (1X,'    RH(',I3,') =',1P9D11.3)
 1003 FORMAT (1X,'    DH(',I3,') =',1P3D11.3)
 1004 FORMAT (1X,'   PIN(',I3,') =',1P9D11.3)
 1005 FORMAT (1X,'   R0L(',I3,') =',1P9D11.3)
 1006 FORMAT (1X,'   D0L(',I3,') =',1P3D11.3)
 1007 FORMAT (1X,'   RP2(',I3,') =',1P9D11.3)
 1008 FORMAT (1X,'   RP3(',I3,') =',1P9D11.3)
 1009 FORMAT (1X,' PIMAT(',I3,') =',1P9D11.3)
      RETURN
c
 2010 FORMAT(A,I3,/,A,I3,A,F10.5)
C
      END
