C $Header: /export/cvs/c34a1/source/moldyn/rtiter.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
      SUBROUTINE rtiter(P,Y,FTOL,xloc,dsloc,ami,n,ibod)
c
C     Find the transformation to minimize the weighted least squares
C     fit of the body coordinates to the problem coordinates
C
      IMPLICIT NONE

#include "ERRHDL.INC"
#include "TAPENO.INC"
c
c     Argument List
c
      INTEGER n                       ! Number of atoms in body ibod
      INTEGER ibod                    ! Body number
      DOUBLE PRECISION ftol           ! Tolerance
      DOUBLE PRECISION p(4,3)
      DOUBLE PRECISION y(4)
      DOUBLE PRECISION dsloc(3,2*n)   ! Nominal coords in body frame
      DOUBLE PRECISION xloc(3,n)      ! Problem coords in inertial frame
      DOUBLE PRECISION ami(n)         ! Inverse masses for weighting
c
c     Local Parameters
c
      INTEGER          itmax
      DOUBLE PRECISION alpha, beta, gamma
      PARAMETER (ALPHA=1.0, BETA=0.5, GAMMA=2.0, ITMAX=500)
c
c     Local Variables
c
      INTEGER i, j                    ! DO loop index
      INTEGER iter                    ! Iteration count
      INTEGER ilo, ihi, inhi          ! Indices
      DOUBLE PRECISION rtol           ! Actual tolerance
      DOUBLE PRECISION ypr, yprr
      DOUBLE PRECISION pr(3), prr(3), pbar(3)
c
c     Function Declarations
c
      DOUBLE PRECISION rtcst2
c 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: rtiter.f,v $$Revision: 1.1.1.1 $'/
c
      ITER = 0
 1    ILO = 1
      IF (Y(1) .GT. Y(2)) THEN
         IHI = 1
         INHI = 2
      ELSE
         IHI = 2
         INHI = 1
      ENDIF
C
      DO I = 1, 4
         IF (Y(I) .LT. Y(ILO)) ILO=I
         IF (Y(I) .GT. Y(IHI)) THEN
            INHI = IHI
            IHI = I
         ELSE IF (Y(I) .GT. Y(INHI)) THEN
            IF (I.NE.IHI) INHI = I
         ENDIF
      END DO
C
      RTOL = ABS(Y(IHI)) + ABS(Y(ILO))
      IF (RTOL .LT. 1D-08) RETURN
      RTOL = 2D0 * ABS(Y(IHI)-Y(ILO)) / RTOL
      IF (RTOL .LT. FTOL) RETURN
      IF (ITER .EQ. ITMAX) THEN
         CALL CLEARERR()
         WRITE(ERRMSG,200) 
     +    'Exceeding maximum iterations.'
         CALL ERRHDL(-5,'<RTITER>')
      END IF
 200  FORMAT(1X,A)
c
      ITER = ITER + 1
      DO J = 1, 3
         PBAR(J) = 0.
      end do
      DO I = 1, 4
         IF (I .NE. IHI) THEN
            DO J = 1, 3
               PBAR(J) = PBAR(J) + P(I,J)
            end do
         ENDIF
      end do
      DO J = 1, 3
         PBAR(J) = PBAR(J)/3
         PR(J) = (1.+ALPHA)*PBAR(J) - ALPHA*P(IHI,J)
      end do
      YPR = rtcst2(PR,xloc,dsloc,ami,n,ibod)
      IF (YPR .LE. Y(ILO)) THEN
         DO J = 1, 3
            PRR(J) = GAMMA*PR(J) + (1.-GAMMA)*PBAR(J)
         end do
         YPRR = rtcst2(PRR,xloc,dsloc,ami,n,ibod)
         IF (YPRR .LT. Y(ILO)) THEN
            DO J = 1, 3
               P(IHI,J) = PRR(J)
            end do
            Y(IHI) = YPRR
         ELSE
            DO J = 1, 3
               P(IHI,J) = PR(J)
            end do
            Y(IHI) = YPR
         ENDIF
      ELSE IF (YPR .GE. Y(INHI)) THEN
         IF (YPR .LT. Y(IHI)) THEN
            DO J = 1, 3
               P(IHI,J) = PR(J)
            end do
            Y(IHI) = YPR
         ENDIF
         DO J = 1, 3
            PRR(J) = BETA*P(IHI,J) + (1.-BETA)*PBAR(J)
         end do
         YPRR = rtcst2(PRR,xloc,dsloc,ami,n,ibod)
         IF (YPRR .LT. Y(IHI)) THEN
            DO J = 1, 3
               P(IHI,J) = PRR(J)
            end do
            Y(IHI) = YPRR
         ELSE
            DO I = 1, 4
               IF (I .NE. ILO) THEN
                  DO J = 1, 3
                     PR(J) = 0.5*(P(I,J) + P(ILO,J))
                     P(I,J) = PR(J)
                  end do
                  Y(I) = rtcst2(PR,xloc,dsloc,ami,n,ibod)
               ENDIF
            end do
         ENDIF
      ELSE
         DO J = 1, 3
            P(IHI,J) = PR(J)
         end do
         Y(IHI) = YPR
      ENDIF
      GO TO 1
      END
