C $Header: /export/cvs/c34a1/source/moldyn/sizes_amat.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
C                    SUBROUTINE - SIZES_AMAT.F
C
C:::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
c
      subroutine sizes_amat(ifit,nb,nh,nconn,nctype,IRGFLX,iatoms,
     &ihconn,itopol,naa,ng,nx,nfm,named,namev,imbl,idbl)
c
c     SIZES_AMAT code provides information on sizes of sub-blocks
c     (structure of sparseness) in the matrix amat emerging in position/
c     velocity fit of I.C.
c
c     The code accounts for two situations: 'with constraints' and
c                                           'without constraints'
c
c     The inputs are:
c     ifit    0 - for position fit, 1 - for velocity fit. 
c     nb      Number of bodies.
c     nh      Number of hinges.
c     nconn   Number of all constraints (bonds, angles, dihedrals).
c     nctype  Type of constraints in allocation of arrays (2 - only bonds,
c             3 - angles are included,4 - dihedrals are included).
c     IRGFLX  Vector of numbers of flexible modes for each bogy, size (1 x nb).
c     iatoms  Matrix of atoms' # and #s of atoms for each fragment,
c             size (3 x nb):
c             1st row - # of first atom in fragment,
c             2nd row - # of last atom in fragment,
c             3rd row - # of atoms in fragment.
c     ihconn  Matrix of hinge #s for each constraint, size (nctype x nconn).
c     itopol  Matrix of pairs of bodies' #s making a hinge, size (2,nh).   
c
c     The outputs are:
c     naa    "Storage" dimension of amat (number of not-zero elements).
c             Note:  While allocating amat, naa can be estimated given 
c                    a molecular structure and constraints.
c     ng      Dimension of gvec, (ng = nx + nconn + 6(if momentums)).
c     nx      Size of I.C. vector.
c     nfm     Number of parameters to be fitted for the most "flexible" body.
c     named   "External" pointer for block D (or ED) in amat
c             Note:  "External pointer" sets the index of block's 
c                    first element in amat.
c     namev   "External" pointer for block ED in amat.
c     imbl    Auxiliary matrix to store sizes of body-blocks of M, (3 x nb),
c             1st row:  Size of each body-block (length of body-block of dx),
c             2nd row:  #s of first elements in body-blocks of dx,
c             3rd row:  amat's #s of first elements in body-blocks of M
c                       (Note:  for bodies - only triangular matrix is stored,
c                               for particles - only diagonal (3 x 1)).
c     idbl    Matrix for sizes of constraint-blocks of D, (3 x nconn)
c             1st row:  Number of bodies involved in constraint 
c                       (1 or 2 or 3 or 4),
c             2nd row:  Length of each not-zero constraint-block 
c                       (for all body-blocks involved in constraint),
c             3rd row:  amat's # of first element in not-zero 
c                       constraint-block of D.
c
      IMPLICIT NONE

#include "PARAM.FOR"

c
c     Declare variables:
c
      integer ifit,nb,nh,nconn,nctype                 ! Inputs
      integer IRGFLX(nb),iatoms(3,nb)                 ! Inputs
      integer ihconn(nctype,nconn), itopol(2,nh)      ! Inputs
      integer imbl(3,nb),idbl(3,nconn)                ! Outputs
      integer icob(4)                         
      integer naa,ng,nx,nfm,named,namev               ! Outputs
      integer ifb,iat,imbody,imbm,ic,idb,id,ib,i
      integer imaxfl,ibody
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: sizes_amat.f,v $$Revision: 1.1.1.1 $'/
c
c     Form array of sizes for body-blocks of M

      imbody = BADADDR
      imbm = BADADDR

      imaxfl = 0
      ibody = 0
      imbl(2,1) = 1
      imbl(3,1) = 1
      do ifb = 1, nb
         iat = iatoms(3,ifb)
         if (iat .eq. 1) then
            imbody = 3
            imbm = 3
         else
            imbody = 6 + IRGFLX(ifb)
            imbm = imbody*(imbody+1)/2
            ibody = 1
            if (IRGFLX(ifb) .gt. imaxfl) then
               imaxfl = IRGFLX(ifb)
            end if
         end if
         imbl(1,ifb) = imbody
         if (ifb .ne. nb) then
            imbl(2,ifb+1) = imbl(2,ifb) + imbody
            imbl(3,ifb+1) = imbl(3,ifb) + imbm
         end if
      end do
c
c     Determine the size of I.C. vector and maximal size of body-block
      nx = imbl(2,nb) + imbody - 1          ! imbody corresponds to nb
      if (ibody .eq. 0) then
         nfm = 3
      else if (ibody .eq. 1) then
         nfm = 6 + imaxfl
      end if
c
c     Consider case when constraints have been set
      if (nconn .gt. 0) then        
c        Determine array of sizes for constraint-blocks of D
         idbl(3,1) = imbl(3,nb) + imbm     ! imbm corresponds to nb 
         do ic = 1, nconn
            call cbod(ic,nh,nconn,ihconn,itopol,nctype,icob)
            idb = 1
            do i = 2, nctype
               if (icob(i) .ne. 0) then
                  idb = idb + 1
               end if
            end do
            idbl(1,ic) = idb
            id = 0
            do i = 1, idb
               ib = icob(i)
               id = id + imbl(1,ib)
            end do
            idbl(2,ic) = id
            if (ic .ne. nconn) then
               idbl(3,ic+1) = idbl(3,ic) + id
            end if
         end do
c
c        Determine "external" pointers in amat to separate
c        M, D (or ED), and E(V)
         named = idbl(3,1)
         if (ifit .eq. 0) then         ! position fit
            namev = 0 
         else if (ifit .eq. 1) then    ! velocity fit 
            namev = idbl(3,nconn) + idbl(2,nconn)  
         end if
c
c        Determine the size of vector gvec and the size of array to store 
c        not-zero element of matrix amat
         if (ifit .eq. 0) then         ! position fit
            ng = nx + nconn
            naa = idbl(3,nconn) - 1 + idbl(2,nconn)
         else if (ifit .eq. 1) then    ! velocity fit
            ng = nx + 6 + nconn
            naa = namev - 1 + 6*nx
         end if
c
c     Consider case when NO constraints have been set
      else if (nconn .eq. 0) then
c        Determine "external" pointers in amat to separate
c        M and E(V)
         named = 0
         if (ifit .eq. 0) then  ! position fit
            namev = 0 
         else if (ifit .eq. 1) then ! velocity fit 
            namev = imbl(3,nb) + imbm  
         end if
c
c        Determine the size of vector gvec and the size of array to store 
c        not-zero element of matrix amat
         if (ifit .eq. 0) then  ! position fit
            ng = nx 
            naa = imbl(3,nb) - 1 + imbm
         else if (ifit .eq. 1) then ! velocity fit
            ng = nx + 6 
            naa = namev - 1 + 6*nx
         end if
      end if
c
      return
      end








