C $Header: /export/cvs/c34a1/source/moldyn/vel_verl.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
      SUBROUTINE vel_verl(x,dx,d2x,n,t,dt,derivs,vdt2,p)
c
c     This code implements velocity verlet with iteration to solve for the
c     velocity-dependent accelerations
c
      IMPLICIT NONE
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: vel_verl.f,v $$Revision: 1.1.1.1 $'/
c
#include "PARAM.FOR"
c
#ifdef CHARMM
#include "CHARMM.INC"
#endif
#include "CPATHS.INC"
#include "ERRHDL.INC"
#include "HEAPMB.INC"
#include "IVCONS.INC"
#include "JILFLG.INC"
#include "MISCNO.INC"
#include "MOMENG.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
#include "SELECT.INC"
#include "SPECIF.INC"

c
c     Arguments
c
      INTEGER n                      ! Number of coordinates
      DOUBLE PRECISION x(n)          ! Positions
      DOUBLE PRECISION dx(n)         ! Velocities
      DOUBLE PRECISION d2x(n)        ! Accelerations
      DOUBLE PRECISION t             ! Current time
      DOUBLE PRECISION dt            ! Time step
      DOUBLE PRECISION vdt2(n)       ! Extrapolated velocities at half dt
      DOUBLE PRECISION p(cL10)
      EXTERNAL DERIVS                ! Subroutine for derivative calc.
c
c     MBO(N)D COMMON blocks
c
c
c     Local Variables
c
      INTEGER i                      ! DO loop variable
      INTEGER iter                   ! Iterate counter
      INTEGER iextor                 ! Indicator of call for external torque 
      DOUBLE PRECISION dt2           ! dt / 2
      DOUBLE PRECISION eng_old       ! Old value for total energy
      DOUBLE PRECISION delta_eng     ! Change in total energy between iterates
c
 2000 FORMAT(1X,A,I5)
c
      dt2 = dt / 2D0
c
c
      DO i = 1, n
c
c     Propagate the velocity states by half delta t & by full delta t
c
         vdt2(i) = dx(i) + d2x(i)*dt2
         dx(i)   = dx(i) + d2x(i)*dt
c
c     Propagate the positional states by delta t
c
         x(i)    = x(i)  + vdt2(i)*dt
      END DO
c
c     Initialize iteration on the solution for velocity at delta t
c
      CALL ENGMOM_KE(HEAP_MB(iyptr),HEAP_MB(ipptr),
     &               HEAP_MB(ipmomptr))
      eng_old = totke
      iter = 0
c
c     Begin iteration loop
c
#ifdef CHARMM
      iparam(16) = iparam(16) + 1
#endif
 100  CONTINUE
      IF (iter .EQ. 0) THEN
         iextor = 1   ! Call to external torque
      ELSE
         iextor = 0   ! Scip call to external torque
      END IF
      CALL DERIVS(t,x,dx,d2x,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      DO i = 1, n
         dx(i) = vdt2(i) + dt2*d2x(i)
      END DO
      iter = iter + 1
      CALL ENGMOM_KE(HEAP_MB(iyptr),HEAP_MB(ipptr),
     &               HEAP_MB(ipmomptr))
      delta_eng = totke - eng_old
      eng_old = totke
      IF (iter .GT. itmax) THEN
         CALL CLEARERR()
         WRITE(MBOUT,200) 
     +       'Too many iterations over v-dependent forces.',
     +       iter, abs(delta_eng/totke)*100.0
         WRITE(ERRMSG,'(1X,A)') 'Error in subroutine.'
         CALL ERRHDL(-5,'<VEL_VERL>')
      END IF
 200  FORMAT(A,/,I9,E15.8)
      IF (ABS(delta_eng/totke) .GT. ERAT_VV) GO TO 100
c
c     End of iteration
c
c     Re-calculate acceleration to start the next step to allow:
c     a)  Euler sequence or pivot type changing, and
c     b)  Velocity re-scaling, is used
c
      jil = 4
      iter = itmax + 1
      iextor = 0   ! Skip call to external torque
      CALL DERIVS(t,x,dx,d2x,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      seqflg = .FALSE.
c
      RETURN
      END
