C $Header: /export/cvs/c34a1/source/moldyn/verlcd.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
      SUBROUTINE verlcd(x1,x2,x3,vel,n,time,dt,derivs,p,acc)
c
c     This code implements the Verlet central difference integrator with 
c     iteration to solve for the velocity-dependent accelerations
c
C     INPUTS:  x1 contains the positions at t
C              x2 contains the positions at t - dt
C              x3 contains the positions at t - 2*dt
C
C     OUTPUTS:  x1 contains the positions at t + dt
C               x2 contains the positions at t
C               x3 contains the positions at t - dt
C               vel contains the velocities at t
C
C
      IMPLICIT NONE
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: verlcd.f,v $$Revision: 1.1.1.1 $'/
c
c
c     MBO(N)D COMMON blocks
c
#include "PARAM.FOR"
#ifdef CHARMM
#include "CHARMM.INC"
#endif
#include "CPATHS.INC"
#include "ERRHDL.INC"
#include "IFIRST.INC"
#include "IVCONS.INC"
#include "JILFLG.INC"
#include "MISCN2.INC"
#include "MISCNO.INC"
#include "MOMENG.INC"
#include "NUMBRS.INC"
#include "TAPENO.INC"
#include "HEAPMB.INC"
#include "VECTOR.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
c
c     Arguments
c
      INTEGER n                      ! Number of coordinates
      DOUBLE PRECISION x1(n)         ! See notes above
      DOUBLE PRECISION x2(n)         ! See notes above
      DOUBLE PRECISION x3(n)         ! See notes above
      DOUBLE PRECISION vel(n)        ! See notes above
      DOUBLE PRECISION time          ! Current time
      DOUBLE PRECISION dt            ! Time step
      DOUBLE PRECISION p(cL10)
      EXTERNAL DERIVS                ! Subroutine for derivative calc.
c
c     Local Variables
c
      INTEGER i                      ! DO loop variable
      INTEGER iter                   ! Iterate counter
      INTEGER iextor                 ! Indicator of call for external torque 
      INTEGER i1st                   ! First pass indicator
      DOUBLE PRECISION dtx2          ! 1 / (2 * dt)
      DOUBLE PRECISION dt6           ! dt / 6
      DOUBLE PRECISION dtsq          ! dt**2
      DOUBLE PRECISION dtsqx2        !  2  * dt**2
      DOUBLE PRECISION hdt2          ! 0.5 * dt**2
      DOUBLE PRECISION acc(cL07/3)    ! Accelerations at 
      DOUBLE PRECISION eng_old       ! Old value for total energy
      DOUBLE PRECISION delta_eng     ! Change in total energy between iterates
c
c     Equivalences
c
      EQUIVALENCE (i1st,i1pass(4))
      SAVE dtx2,dt6,dtsq,dtsqx2,hdt2
c
c     Initialize constants
c
      IF (i1st .EQ. 0) THEN
         i1st   = 1
         dtx2   = 0.5D0 / dt
         dt6    = 1D0 / (6D0*dt)
         dtsq   = dt**2
         dtsqx2 = TWO * dtsq
         hdt2   = 0.5D0 * dtsq
         IF (n .GT. cL07/3) THEN
            CALL CLEARERR()
            WRITE(ERRMSG,210) ' Dimension error in subroutine.'
            CALL ERRHDL(-5,'<VERLCD>')
         END IF
      END IF
 210  FORMAT(1X,A)
c
c     Evaluate an initial estimate of velocities
c
      DO i = 1, n
         vel(i) = (TRES*x1(i) - 4D0*x2(i) + x3(i)) * dtx2
      END DO
c
c     Compute accelerations based on first guess for velocity at t
c
#ifdef CHARMM
      iparam(16) = iparam(16) + 1
#endif
      iter = 0
      iextor = 1   ! Call to external torque
      CALL DERIVS(time,x1,vel,acc,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      CALL ENGMOM_KE(HEAP_MB(iyptr),HEAP_MB(ipptr),
     &               HEAP_MB(ipmomptr))
      eng_old = totke
c
c     Iterate on the solution for velocity at t
c
 100  CONTINUE
      iter = iter + 1
      iextor = 0   ! Scip call to external torque
      DO i = 1, n
         vel(i) = (7D0*x1(i) - 8D0*x2(i) + x3(i) + dtsqx2*acc(i)) * dt6
      END DO
      CALL DERIVS(time,x1,vel,acc,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      CALL ENGMOM_KE(HEAP_MB(iyptr),HEAP_MB(ipptr),
     &               HEAP_MB(ipmomptr))
      delta_eng = totke - eng_old
      eng_old = totke
      IF (iter .EQ. itmax) THEN
         CALL CLEARERR()
         WRITE(MBOUT,200) ' Too many iterations in subroutine.',
     +        ' iter = ',iter, ' Percent error = ', 
     +        ABS(delta_eng/totke)*100.0
         WRITE(ERRMSG,210) 'Error in subroutine.'
         CALL ERRHDL(-5,'<VERLCD>') 
      END IF
 200  FORMAT(A,/,A,I9,/,A,E15.8)
      IF (ABS(delta_eng/totke) .GT. ERAT_VV) GO TO 100
C
C     Final call for velocity output and velocity scaling
C
      jil  = 4
      iter = itmax + 1
      CALL DERIVS(time,x2,vel,acc,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggvptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
c
c     Calculate the new position, and shift all the x registers
c
      DO i = 1, n
         x3(i) = x2(i)
         x2(i) = x1(i)
         x1(i) = TWO*x2(i) - x3(i) + dtsq*acc(i)
      END DO
c
c     Re-define the positions if Euler sequence has changed
c
      IF (seqflg) THEN
c
c     Note that the arguments in the following call assume that x1, x2, and 
c     x3 are partitions of the z and zdt arrays defined in mdloop.
c
#ifdef CHARMM
         CALL init_vcd(i,time,x1,x3,HEAP_MB(iyptr),
     *                 HEAP_MB(iydtptr),LocalHeap(iysavptr_init_vcd),
     *                 LocalHeap(idysavptr_init_vcd),
     *                 LocalHeap(ix_tmpptr_init_vcd))
#else
         IF (iprnt .EQ. noprnt) THEN
            DO i = 1, n
               acc(i) = vel(i)
            END DO
         END IF
c
c     Note that the arguments in the following call assume that x1, x2, and 
c     x3 are partitions of the z and zdt arrays defined in mdloop.
c
         CALL init_vcd(i,time,x1,x3,HEAP_MB(iyptr),
     *                 HEAP_MB(iydtptr),LocalHeap(iysavptr_init_vcd),
     *                 LocalHeap(idysavptr_init_vcd),
     *                 LocalHeap(ix_tmpptr_init_vcd))
c
c     Extra call to get proper potential energies if we are printing
c     immediately upon return from vcdint
c
         IF (iprnt .EQ. noprnt) THEN
            DO i = 1, n
               vel(i) = acc(i)
            END DO
            iter = 0
            iextor = 1   ! Call to external torque
            CALL DERIVS(time,x2,vel,acc,iter,iextor,HEAP_MB(iyptr),
     &            HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &            LocalHeap(iggptr_ydot),
     &            logicLoopHeap(iiscutptr),
     &            intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &            LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &            intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
         END IF
#endif
c
c     The following line has been commented out because seqflg is already 
c     set to FALSE by the RK4 call to YDOT.
c
c         seqflg = .FALSE.
      END IF
c
      RETURN
      END


