C $Header: /export/cvs/c34a1/source/moldyn/vmispa.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
C                  SUBROUTINE - VMISPA.F                        
C    
C::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
c
      subroutine vmispa(amat,vecr,naa,nx,nb,nbs,nfm,imbl,
     &nctype,icob,gmx,sm,kpvtm,kpvtal,lnf)
c
c     VMISPA code computes a vector VL = VR*M(-1) with pre-factored 
c     (beforehand) matrix M.
c
c     Matrix M (nx x nx) has a physial block-diagonal structure for
c     position or velocity fit, i.e. M is arranged in body-blocks:
c     (3 x 3) - for particles, (6 x 6) - for rigid bodies
c     ((6+N_flex) x (6+N_flex)) - for flexible bodies.
c 
c     Vector VR ("right" vector) (1 x nx) can formalize different situations: 
c     a row of blocks D (ED), EV, vector g, etc. (see dlespa.f).
c
c     In all operations sparseness of M and VR is accounted for in
c     the following way: in body-blocks for particles only diagonals are
c     stored (due to diagonality), in body-blocks for rigid and flexible
c     bodies only triangular matrices are stored (due to symmetry).
c
c     Subroutine calls:
c        dsisl.f
c
c     The inputs are:
c     amat    Not-zero elements of matrix A stored in array (1 x naa), 
c             only the body-blocks of M will be used
c             (taking into account their symmetry or diagonality).
c             Note:  Body-blocks of M are already factored in factor_b.f
c                    by dsifa.f.   
c     vecr    Not-zero body-blocks of "right" vector VR, size (1 x nx).
c     naa     "Storage" dimension of amat (number of not-zero elements).
c     nx      Length of I.C. vector.  
c     nb      Number of bodies.
c     nbs     Number of not-zero body-blocks in VR.
c             Note:  if nbs = nb then VR is treated as a full vector.
c     nfm     Number of parameters to be fitted for the most "flexible" body.
c     imbl    Auxiliary matrix to store sizes of body-blocks of M, (3 x nb),
c             1st row:  Size of each body-block (length of body-block of dx),
c             2nd row:  amat's # of first element in body-block of dx,
c             3rd row:  amat's # of first element in body-block of M
c                       (Note:  for bodies - only triangular matrix is stored,
c                               for particles - only diagonal (3 x 1)).
c     nctype  Type of constraints in allocation of arrays (2 - only bonds,
c             3 - angles are included, 4 - dihedrals are included).
c     icob    Vector of #s of bodies which have not zero blocks in VR,
c             (in the growing order, size (1 x nctype) but not more than 4).
c             Note:  icob is not needed if VR is a full vector, but if VR is
c                    sparse (nbs < nb) then icob must be consistent with nbs.
c     gmx     Auxiliary vector associated with body-blocks of M, (1 x nfm).
c     sm      Auxiliary matrix to operate with body-blocks of M, (nfm x nfm).
c     kpvtm   Vector of pivot indeces for one body-block, size (1 x nfm).
c     kpvtal  Vector of pivot indeces for all body-blocks, size (1 x nx).     
c     lnf     Maximal size for nfm set in the calling program. 
c
c     The ouputs are:
c     vecr    "Left" vector VL = VR = VR*M(-1).
c
      IMPLICIT NONE
c
c     Declare variables:
c
      integer naa,nx,nb,nbs,nfm,nctype,lnf       ! Inputs
      double precision amat(naa)                 ! Inputs
      double precision vecr(nx)                  ! Inputs/Outputs
      double precision gmx(nfm),sm(lnf,lnf)      ! Inputs
      integer imbl(3,nb),icob(nctype)            ! Inputs
      integer kpvtm(lnf),kpvtal(nx)              ! Inputs
      integer ka,ii,jj,ib,ifb,imbody,kast,kbb
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: vmispa.f,v $$Revision: 1.1.1.1 $'/
c
c     Run over all not-zero body-blocks
      do ib = 1, nbs
c     Determine actual # of body
         if (nbs .eq. nb) then
            ifb = ib
         else
            ifb = icob(ib)
         end if
c
c        Form body-block of M:
         imbody = imbl(1,ifb)
         kast = imbl(3,ifb) - 1
         if (imbody .ne. 3) then             ! Scip particles
            do ii = 1, imbody
               do jj = ii, imbody
                  ka = (jj-1)*jj/2 + ii
                  sm(ii,jj) = amat(kast+ka)  ! only upper triangular is needed
               end do    
            end do
         end if 
c
c        Solve for VL(body):  VL(body)*M(body) = VR(body)
c        Note:  Invertion of body-block M(body) is avoided through solution
c               of linear system of equations.
c
c        Form right-hand part of LE
         kbb = imbl(2,ifb) - 1
         do ii = 1, imbody
            gmx(ii) = vecr(kbb+ii)
         end do
c
c        Solve LE 
         if (imbody .ne. 3) then    ! For bodies
c           Note:  factorization of body-blocks of M (i.e. of matrices sm)
c                  is performed beforehand in factor_b.f
c           Use pivot indeces for the current body-block from storage  
            do ii = 1, imbody
               kpvtm(ii) = kpvtal(kbb+ii)
            end do
            call dsisl(sm,lnf,imbody,kpvtm,gmx)    ! gmx is VL(body) now
         else if (imbody .eq. 3) then              ! For particles       
            do ii = 1, 3
               gmx(ii) = gmx(ii)/amat(kast+ii)     ! gmx is VL(body) now
            end do
         end if
c
c        Put VL(body) into VL
         do ii = 1, imbody
            vecr(kbb+ii) = gmx(ii)  
         end do
      end do   ! ib
      return 
      end













