c     SUBROUTINE EGLCAO
c     =================
c
c     Copyright 1997 by Peter Blaudeck, Dirk Porezag, Michael Haugk,
c                       Joachim Elsner
c
c *********************************************************************
c
c     PROGRAM CHARACTERISTICS
c     -----------------------
c
c eglcao calculates energy and gradient for dylcao as shown by Seifert. 
c The determination of the occupation numbers has been changed to be
c also valid for metallic systems.
c
c PARAMETERS:
c n3      i  3 * number of atoms
c telec   r  electronic temperature (for Fermi distribution)
c x       r  coordinates (n3)
c scftol  r  convergence criterion for scf (if negative, no scf)
c atomic  l  should there be an atomic energy calculation
c e       r  total energy
c eel     r  electronic energy
c gr      r  gradient (n3)
c miter   i  number of scf-iterations performed
c evector l  prints out the eigenvectors
c lgrad   l  should gradients be calculated?
c
c *********************************************************************
c
       subroutine eglcao(n3,telec,x,scftol,atomic,e,eel,gr,
     &                  miter,evector,qmat,lgrad,
     &                  qsccewc,qsccews,qscqewc,qscqews,
c added for GHO ... PJ 7/2004
     &                  pho,fao,sao,who,fhb,shb,chb,cao)
c      =================
c
c maxima definitions
c
       implicit REAL*8 (A-H,O-Z)
       include 'maxima.inc'
c
       logical doscf, period, atomic, evector,constr2,lgrad
c      QC: UW0406 add ewald related
       real*8  qsccewc(*),qsccews(*)
       real*8  qscqewc(*),qscqews(*)
       integer izp(NNDIM), ind(NNDIM+1),lmax(MAXTYP)
       integer indj,indk,indj1,indk1,nstart, nlat
       integer conat(NNDIM+1),cai
       integer numint(MAXTYP,MAXTYP)
       real*8 x(3,*), gr(3,*), hgrad(3,3*NNDIM), scftol
       real*8 coeff(6,MAXINT,MAXTYP,MAXTYP)
       real*8 xr(2,MAXINT,MAXTYP,MAXTYP)
       real*8 efkt(3,MAXTYP,MAXTYP),cutoff(MAXTYP,MAXTYP)
       real*8 eel, ecoul, e, efermi,convec(3,NNDIM)
       real*8 au(LDIM,LDIM), bu(LDIM,LDIM), nel
       real*8 a(MDIM,MDIM), b(MDIM,MDIM), spro
       real*8 gammamat(NNDIM,NNDIM),derivx(NNDIM,NNDIM)
       real*8 derivy(NNDIM,NNDIM),derivz(NNDIM,NNDIM)
       real*8 h(MDIM), ev(MDIM), occ(MDIM)
       real*8 qmulli(MDIM), qmold(NNDIM), qmat(NNDIM)
       real*8 Qdiff(NNDIM),dipol(3),dipabs
       real*8 hamil(MDIM,MDIM), overl(MDIM,MDIM), shift(NNDIM)
       logical lerep
cccccccccccccccc  cc-korrection
        real*8 ecorr
cccccccccccccc CHARMM+ external charges
       integer MXCHMPT
       parameter(MXCHMPT=25120)
       real*8 xE(3,MXCHMPT) ,ZE(MXCHMPT),shiftE(NNDIM)
       integer nE
       character*2 EXT
       integer mysccid
ccccccccccccccc
       external skhpar,skspar
c     DISERSION
        logical dispers
        real*8 Edis
        common /disper/ Edis,dispers 
        integer writeout
        common /eglext/ writeout
c     DISPERSION END
cccccccccccccc CHARMM
       common /extchr/ xE, ZE, nE, EXT ,mysccid
cccccccccc
cccc correction for C-C bond
       common /correct/ nco, nc(NNDIM)
cccccccccccc
       common /muell/ a,b
c      only for not taking space from stack (FORTRAN problem)
       common /concgr/ convec, constr2, conat
c      constrains
       common /lmax/ lmax
c              s : 1, p : 2, u.s.w. for all atom types
C      QC: izp-->izpc
       common /izpc/ izp, nel, nbeweg
c              atom types, number of electrons, number of movable atoms
C     QC_UW_06: hbond correction
      logical lscchb
      real*8 kl1
      common /scchb/ lscchb,kl1

C      QC: UW_06 Haibo adds Gaussian SRP
      real*8 gbump(3,maxtyp,maxtyp),ronsw(maxtyp,maxtyp)
      real*8 deltar(maxtyp,maxtyp)
      logical lgausw

      common /spltab/ coeff,xr,efkt,cutoff,numint,gbump,ronsw,deltar,
     &       lgausw  

c              spline data for repulsiv force
       common /box/ boxsiz(3,3), xinvbox(3,3), xnullvec(3), period, nlat(3)
c              data fo supercell
       common /machine/ racc
c              machine accuracy
       common /mcharge/ qzeroscc(MAXTYP), uhubb(MAXTYP)
c
       common /varhelp/ Qdiff

C      QC: GSBP Common blocks
       INTEGER MMDIM
       PARAMETER(MMDIM=NNDIM*(NNDIM+1)/2) 
       REAL*8 GAMAGS(MMDIM)
       REAL*8 OMEGAGS(NNDIM)
       LOGICAL QGSBPSCC
       COMMON /SCCGS/ GAMAGS,OMEGAGS,QGSBPSCC

C      QC: UW_05 Hubbard derivatives (on-site version)
       real*8  uhder
       logical luhder,luhgau
       common /uhq/ uhder(MAXTYP),luhder,luhgau
       real*8 v0_hgau,alp_hgau,q0_hgau
       common /uhgau/ v0_hgau,alp_hgau,q0_hgau 
C      Local variables
       real*8  eqhu,qdif(NNDIM),uhu2,uhuq(NNDIM),uhuqd(NNDIM)
C      --------------------------------------------------
c
c GHO common block and variables ... PJ 7/2004
c
c ---------------------- GHO common block ---------------------------
c
      LOGICAL QLINK
      INTEGER NATQM,NQMLNK,IQLINK,JQLINK,KQLINK,MAXQMLINK,MXQM16
      REAL*8  BT,BTM,QMATMQ,DBTMMM

      PARAMETER (MAXQMLINK=5,MXQM16=16*MAXQMLINK)
      COMMON/QLINKI/NATQM,NQMLNK,IQLINK(MAXQMLINK),JQLINK(3,MAXQMLINK),
     &              KQLINK(MAXQMLINK)
      COMMON/QLINKL/QLINK
      COMMON/QLINKF/BT(MXQM16),BTM(MXQM16),QMATMQ(MAXQMLINK),
     &              DBTMMM(3,3,MXQM16)

      INTEGER  NORBG
      COMMON/QLINKN/NORBG
C
C label of GHO atoms and QM atom directly linked to GHO atoms in
C the QM domain, similiar to IQLINK and KQLINK
C
      INTEGER MXNN
      PARAMETER (MXNN=650)

      INTEGER IGHOSL
      COMMON /GHOSEL/ IGHOSL(MXNN)

      INTEGER IGLNK,KGLNK
      COMMON/QLINKQ/ IGLNK(MAXQMLINK),KGLNK(MAXQMLINK)
c
c large 1-d arrays have been allocated dynamically
c
      real*8  pho(*),fao(*),sao(*),who(*),
     &        fhb(*),shb(*),chb(*),cao(*)
c
c arraies for working space related to GHO 
c
      integer nfirst, nlast
      common /gorbind/ nfirst(NNDIM+1), nlast(NNDIM+1)
      real*8 ahb(MDIM,MDIM), bhb(MDIM,MDIM)
      real*8 eauxhb(3,NNDIM)
c
c Variables for QM-boundary atoms
c
      integer k, norbhb, naos, linao
      real*8 nelhb
c---------------------------------------------------------------------
c
       real*8 factor
       factor = 0.529177249D0
c
c setup
c

c no gradients in mode 6
c      if (atomic) lgrad= .false.
       lerep = .false.
       if (scftol .lt. 0.0) then
         doscf = .false. 
       else
         doscf = .true.
       endif
       nn = n3 / 3
C      QC_UW04: specify nbeweg as well - which is ALWAYS nn for CHARMM
       nbeweg=nn
C      QC_UW04
       
       
c calculation of indices for matrices H and S 
c
       ind(1) = 0
       do j = 1,nn 
         izpj = izp(j)
C QC: debug
         ind(j+1) = ind(j)+lmax(izpj)**2
       end do
c
c actual dimension of matrix
       ndim = ind(nn+1)      
       
       if (ndim .gt. MDIM) then
         write(*,*) ' eglcao: ndim > ', MDIM
         stop
       endif
c
c construct GHO related dimension and number active electrons
c                                                  ... PJ 7/2004
c
      if(qlink) then
         norbs = ndim
         norbg = norbs
         norbhb = norbs - 3*nqmlnk
         naos = norbhb-nqmlnk
         linao = (naos*(naos+1))/2
         do i = 1, nn
            nfirst(i) = ind(i)+1
            nlast(i) = ind(i+1)
         end do
         nlast(nn) = ndim
         nelhb = nel - 3*nqmlnk
      endif
       
c setup of charge-independent part of H and S
C          ^^^^^^^^^^^^^^^^^^
c
       do j = 1,nn 
         do k = 1,j 
         
           call slkmatrices(j,k,x,au,bu)
           
           do n = 1,ind(k+1)-ind(k) 
             do m = 1,ind(j+1)-ind(j)
               hamil(ind(j)+m,ind(k)+n) = au(m,n)
               hamil(ind(k)+n,ind(j)+m) = au(m,n)       
               overl(ind(j)+m,ind(k)+n) = bu(m,n)
               overl(ind(k)+n,ind(j)+m) = bu(m,n)
             end do
           end do
         end do
       end do
c for external charges : extra shift shiftE 
      if ((EXT.eq.'CH') .or. (EXT.eq.'EF')) then
       call externalshift(nn,x,izp,shiftE,qsccewc,qsccews)
      endif
C QC-GSBP: The charge independent part for GSBP-SCC is OMEGAGS, which
C has already been computed in gsbpscc.src. Simply use it whereever
C shiftE appears
c setup for SCF cycle (why freeze maxiter?!:-) QC.
C related to broden_mixing scheme?!
c
       if (doscf) then
c        if (mysccid.eq.0) then 
c          open(70,file='scfsum',form='formatted',status='unknown') 
c          close(70, status='delete')
c        endif
         almix = 0.2
         maxiter = 200 
         eelold = 1.0d10
       else
          maxiter = 1
       endif
       
c  *************************
c  SCF cycle starts here
c  *************************
       
       do niter = 1,maxiter 
         miter = niter
       
c save old charges
c
         if (doscf) then
           do i = 1,nn 
             qmold(i) = qmat(i)
           end do
         endif
       
       
c charge-independent part of H and S
c
       do j = 1,nn 
         indj  = ind(j)
         indj1 = ind(j+1)
         do k = 1,j 
           indk  = ind(k)
           indk1 = ind(k+1)
           
           do n = 1,indk1-indk 
             do m = 1,indj1-indj 
               a(indj+m,indk+n) = hamil(indj+m,indk+n)
               b(indj+m,indk+n) = overl(indj+m,indk+n)
             end do
           end do
         end do
       end do
       
c zero shift 
c
         do i = 1,nn
           shift(i) = 0.0
         end do
       
c add charge dependent terms (Hubbard, etc. )
c start by defining shift 
c
         if (doscf) then
           call HAMILSHIFT(nn,x,izp,boxsiz,period,uhubb,qmat,qzeroscc,
     &                     miter,gammamat,shift)
ccc add external charges! CHARMM
          if ((EXT.eq.'CH').or. (EXT.eq.'EF'))then 
           do i = 1,nn
           shift(i) = shift(i) - shiftE(i)
c minus, weil Z positiv sind!, positive DeltaQ meinen aber negative Ladung
           end do
          endif 

C Add in the Hubbard derivative term@@@Will add the fitted Gaussian para later @@@
c      QC_UW_08: Charge-dependent on-site Hubbard 
       if (luhder) then
c        write(*,*) "charge-derivative"
         uhu2 = 0.15
         do i = 1, nn 
          qdif(i) = (qmat(i)-qzeroscc(izp(i)))
C         QC_UW0305: additional charge-dependence term
          if (luhgau) then
            qtmp=exp(-alp_hgau*(qdif(i)-q0_hgau)*(qdif(i)-q0_hgau))
          else
            qtmp=0.0
          endif
          uhuq(i) = uhder(izp(i)) + v0_hgau*qtmp
          uhuqd(i) = -v0_hgau*qtmp*2.d0*alp_hgau*(qdif(i)-q0_hgau)

COLD      qtmp=exp(-25.0*(qdif(i)-1.0)*(qdif(i)-1.0))
COLD      uhuq(i) = uhder(izp(i)) - 0.06*qtmp
COLD      uhuqd(i) = 0.06*qtmp*2.d0*25.d0*(qdif(i)-1.0)
         enddo    
c        write(27,'(10F12.4)') (uhuq(i),i=1,nn)
       endif   


C >>>> QC_GSBP: add GSBP contributions to the matrix elements
           IF (QGSBPSCC) 
     $       CALL GSBPSHIFT(nn,izp,qmat,qzeroscc,gamags,omegags,shift)
c
c update hamiltonian matrix 
c
      do i = 1,nn 
       do li = 1,lmax(izp(i))**2 
        do j = 1,i
         do lj = 1,lmax(izp(j))**2 
              a(ind(i)+li,ind(j)+lj) = a(ind(i)+li,ind(j)+lj) 
     &        +0.5*overl(ind(i)+li,ind(j)+lj)*(shift(i)+shift(j))
         end do
        end do
       end do
      end do

      endif ! doscf

C QC: UW_0305: Include charge dependent Hubbard
       if (luhder) then
c       write(*,*) "update Hamiltonian -charge "
         do i = 1,nn
           do li= 1,lmax(izp(i))**2
             do j= 1,i
               do lj= 1,lmax(izp(j))**2
                 a(ind(i)+li,ind(j)+lj)=a(ind(i)+li,ind(j)+lj)
     &           +0.25*overl(ind(i)+li,ind(j)+lj)*
     &             ( uhuq(i)*(qmat(i)-qzeroscc(izp(i)))**2 +
     &               uhuq(j)*(qmat(j)-qzeroscc(izp(j)))**2 +
     $              uhuqd(i)*(qmat(i)-qzeroscc(izp(i)))**3/3.0 +
     $              uhuqd(j)*(qmat(j)-qzeroscc(izp(j)))**3/3.0 )

               end do  
             end do  
           end do  
         end do   
       endif    
            
C QC_UW08: end

c
c GHO transformation of H and S matrices ... PJ 7/2004
c
      if (qlink) then
c
c pack H, and S to lower triangle form
c
        k = 0
        do i = 1, norbs
           do j = 1, i
              k = k + 1
              fao(k) = a(i,j)
              sao(k) = b(i,j)
           end do
        end do
c
c transform S and H into hb basis for QM link atom, also
c get the auxiliary orbital energies as diagonal elements
c of the H matrix in the hybrid basis. Note that eauxhb
c for the S transform is only dummy parameters, but for
c the H transformation, it contains the auxililary orbital
c energies, which will be included in the TB band structure
c energies.
c
        call ftofhb(sao,shb,bt,natqm,nqmlnk,norbs,nfirst,nlast,eauxhb)
        call ftofhb(fao,fhb,bt,natqm,nqmlnk,norbs,nfirst,nlast,eauxhb)
c
c tranfer H and S in HB basis to 2-d arries in working space
c
        k = 0
        do i = 1, norbhb
           do j = 1, i
              k = k + 1
              ahb(i,j) = fhb(k)
              bhb(i,j) = shb(k)
           end do
        end do
c
c GHO diagonalization, solve HC=SCE in active hybrid basis
c
        call ewevge(mdim,mdim,norbhb,ahb,bhb,ev,h,1,-1,ier)
c
c append and transform auxiliary hybrid orbitals to AO basis
c
        k = 0
        do j = 1, norbhb
           do i = 1, norbhb
              k = k + 1
              chb(k) =  ahb(i,j)
           end do
        enddo
        call ctrasf(chb, cao, norbs)
        k = 0
        do j = 1, norbs
           do i = 1, norbs
              k = k + 1
              a(i,j) = cao(k)
           end do
        enddo
c
c update the corresponding auxiliary orbital energies
c
         k = norbhb
         do i = 1, nqmlnk
            do j = 1, 3
               k = k + 1
               ev(k) = eauxhb(j,i)
            end do
         end do
      else
         
c solution of eigenvalue problem 
c
C QC: Use ewevge here.
         call ewevge(MDIM,MDIM,ndim,a,b,ev,h,1,-1,ier)
c        call dsygv(1,'V','L',ndim,a,MDIM,b,MDIM,ev,h,34*MDIM,ier)

c ... PJ 7/2004
      endif
       
         if (ier .ne. 0) then 
           print *,' ewevge: ier =',ier
C QC: exit gracefully
C          stop
           call WRNDIE(-5,'<EGLCAO>','SCCDFTB not converged.')
         endif
c
c Machine accuracy 
c
         dacc = 4*racc
c
c form the GHO density matrix ... PJ 7/2004
c
         if (qlink) then
c
c add the fractional occupations for GHO auxililary hybrid orbitals
c this equals to an evenly distribution of the MM charge of GHO
c boundary B over three auxiliary orbitals
c
             call FERMI(nelhb,telec,norbhb,ev,dacc,occ,efermi)
             k = norbhb
             do i = 1, nqmlnk
                do j = 1, 3
                   k = k + 1
                   occ(k) = 1.0-qmatmq(i)/3.0
                end do
             end do
c
c form the total density matrix and energy weighted density
c matrix in HB basis for GHO gradient
c
             call cexpd(chb,norbs,nqmlnk)
             call pwho(pho,who,chb,occ,ev,norbs)
         else
c
c Calculate occupation (occ) and fermi energy (efermi)
c using fermi-distribution
         
         call FERMI(nel,telec,ndim,ev,dacc,occ,efermi)
c
c ... PJ 7/2004
         endif 
         
c sum of occupied eigenvalues
c
         eel = 0.0
         do i = 1,ndim 
c
c for GHO, we have to loop over all orbitals, becuase
c auxiliary orbitals are also occupied ... PJ 7/2004
c
           if (.not. qlink) then
              if (occ(i) .lt. dacc) goto 75
           end if
c
           eel = eel + occ(i)*ev(i)
         end do
75       continue
c
c Lowest unoccupied level
c
         nstart = i

c        QC: debug
c        write(*,*) "EGLCAO: UNOCC",nstart
       
c
c determine mulliken charges , the charge of the whole system
c and the mulliken 
c
         if(doscf .or. atomic) then
          call MULLIKEN(nn,x,izp,qmat,qzeroscc,qmulli,qtot,ndim,dacc,
     $                  occ,a,
     &                  overl,lmax,ind,dipol,dipabs)
         endif
       
c output spektra, occupation numbers, mulliken charges 
c
C QC: Disable for the moment 
         if (mysccid.eq.0.and.atomic) 
     &     call outspec(nn,nbeweg,ndim,ind,lmax,izp,ev,occ,efermi,
     &                qmat,qmulli,qtot,dipol,dipabs)
       
         if (evector) then
           call outeigenvectors(a,ev,occ,ind,nn)
         endif
        
       
c complete calculation of electronic energy:
c charge-dependent energy contribution
c warning: this will only lead to the right result if convergence
c has been reached
c
         if (doscf) then
           ecoul = 0.0
           eext = 0.0
           do i = 1,nn
            ecoul = ecoul + shift(i)*(qmat(i)+qzeroscc(izp(i)))
           end do
ccccccc CHARMM
           if((EXT.eq.'CH').or. (EXT.eq.'EF'))then
           do i = 1,nn
           eext = eext + shiftE(i)*(qzeroscc(izp(i))-qmat(i))
           end do
           endif
ccccccc
C >>>>     QC_GSBP contribution. The GAMAGS terms are already included
C >>>>     in the eel and ecoul terms, similar to the gamma term from 
C >>>>     sccdftb
C >>>>     But the OMEGAGS terms need to be modified similar to 
C >>>>     the CHARMM contribution
           egsbpscc=0.0
           IF (QGSBPSCC) THEN
             do i=1,nn
               egsbpscc = egsbpscc + 
     $                    OMEGAGS(i)*(qzeroscc(izp(i))-qmat(i))
     $                                         *factor
             enddo
           ENDIF

C          UW_0605: Hubbard derivative contribution@@
           eqhu = 0.0
           if (luhder) then
             do i = 1,nn
              eqhu = eqhu  +
     &                  uhuq(i)*(qmat(i)-qzeroscc(izp(i)))**3/6.d0
     &            - 0.5*uhuq(i)*(qmat(i)-qzeroscc(izp(i)))**2
     &                         * qmat(i)
     $            -    uhuqd(i)*(qmat(i)-qzeroscc(izp(i)))**3/6.d0
     $                         * qmat(i)

             end do
            endif
            
C          eel = eel-0.5*ecoul + 0.5*eext
           eel = eel-0.5*ecoul + 0.5*eext + 0.5*egsbpscc + eqhu
c          write(*,*) "EEL>",eel,ecoul,eext,egsbpscc,eqhu
C >>>>     QC_GSBP end
         endif
       
c print energy, check convergence, call broyden mixing
c
         if (doscf) then
c          if (mysccid.eq.0) then 
c            open(70,file='scfsum',form='formatted',status='unknown')
c            write(70,*) 'iter: ',niter,', E= ',eel
c            close(70) 
c          endif
           if (abs(eel-eelold) .lt. scftol) goto 70
           if (mysccid.eq.0) then
            if ( writeout .ge. 1 ) then
             write(72,*) atomic,' iter: ',niter,', E= ',eel
            endif
            if ( writeout .ge. 2 ) then
             write(73,*) ' iter: ',niter 
             write(73,'(10F12.6)') (qmat(i),i=1,nn)
             write(74,*) ' iter: ',niter 
             write(74,'(10F12.6)') (occ(i),ev(i),i=1,ndim)
            endif
           endif
           eelold = eel
c
c   broyden mixing
c
           call broyden(niter,almix,nn,qmold,qmat)
           do i = 1,nn
             qmat(i)= qmold(i)
           end do
         end if
       end do
70     continue

c   QC: give at least a warning if maxit reached

       if (niter.ge.maxiter) write(*,*) "EGLCAO> Maxiter Reached."
       
       if (doscf) then
         call HAMILSHIFT(nn,x,izp,boxsiz,period,uhubb,qmat,qzeroscc,
     &                   miter,gammamat,shift)
       endif
       
c end of SCF
c
ccc add EXTERNAL CHARGES CHARMM
          if ((EXT.eq.'CH').or. (EXT.eq.'EF'))then
           do i = 1,nn
           shift(i) = shift(i) - shiftE(i)
           enddo
          endif
c minus, weil Z positiv sind!, positive DeltaQ meinen aber negative Ladung  
C >>>> QC_GSBP: add GSBP contributions to the matrix elements
           IF (QGSBPSCC) 
     $       CALL GSBPSHIFT(nn,izp,qmat,qzeroscc,gamags,omegags,shift)

C      if (atomic) then       
       
c
c calculation of atomic energies
c 
C      call atomenerg(nn,ndim,ind,izp,shift,
C    &      x,hamil,a,occ,doscf,period,boxsiz,xinvbox,nlat)
C      endif
       
c start with repulsive energy
       erep = 0.0
       
c
c Calculate repulsive energy
c
       call repulsive(nn,izp,period,x,boxsiz,xinvbox,nlat,erep)
       
c total energy 
c
       e = eel + erep
c      write(*,*) "Repulsive",eel,erep

       if (niter.ge.maxiter) then
        write(*,*) 'iter: ',niter,', E= ',e,', Eel= ',eel
     c   ,'SCCDFTB NOT CONVERGED: maxiter',maxiter
c       else
c       if (mysccid.eq.0) then
c       write(71,*) 'iter: ',niter,', E= ',e,', Eel= ',eel
c       endif
       endif

c start with gradient
        
       do j = 1,nn 
         do i = 1,3 
           gr(i,j) = 0.0
         end do
       end do
c DISERPSION
       if (dispers) then
        call  dispersion_egr(nn,x,hgrad,boxsiz,nlat,period)
        do j=1,nn
         do i=1,3
           gr(i,j) = gr(i,j) + hgrad(i,j)
         end do
        end do
c   add dispersion energy to total:
        e = e + Edis
       endif
c END DISPERSION       
c BEGIN EXTERNAL CHARGES CHARMM
      if((EXT.eq.'CH').or. (EXT.eq.'EF'))then
       call externalchgrad(nn,x,izp,hgrad,qmat,
     $                     qsccewc,qsccews,qscqewc,qscqews)
        do j=1,nn
         do i=1,3
           gr(i,j) = gr(i,j) + hgrad(i,j)
         end do
        end do
      endif
c END EXTERNAL CHRARGES
       if(lgrad) then
c
c Calculate gradient of the repulsive energy
c
       call repulsivegrd(nn,nbeweg,izp,period,x,boxsiz,
     &                   xinvbox,nlat,hgrad)
       do j=1,nn 
         do i=1,3 
           gr(i,j) = gr(i,j) + hgrad(i,j)
         end do
       end do
       
      if (luhder) then
        do i=1,nn
         shift(i) = shift(i) + 0.5*uhuq(i)*
     &     (qmat(i) - qzeroscc(izp(i)))**2
     $     + uhuqd(i)*(qmat(i) - qzeroscc(izp(i)))**3/6.d0
        enddo
      endif

c gradient 
c
       call usualgrd(nn,nbeweg,ndim,izp,lmax,ind,period,doscf,
     &              x,ev,a,b,occ,shift,dacc,boxsiz,xinvbox,hgrad)
                    
       do j=1,nn 
        do i=1,3
         gr(i,j) = gr(i,j) + hgrad(i,j)
        end do
       end do
       
c here are the contributions due to gamma if in scf mode
       if (doscf) then
         call GAMMAGRAD(nn,nbeweg,x,izp,boxsiz,period,uhubb,
     &                  qmat,qzeroscc,derivx,derivy,derivz,hgrad)
           do j=1,nn 
            do i=1,3 
              gr(i,j) = gr(i,j) + hgrad(i,j)
            end do
           end do
       endif

c >>>> QC_GSBP: the GSBP_SCC contribution to the gradident
C      those involve overlap derivatives (i.e., terms
C      depend on charge variations) are already handled in
C      usualgrd because GSBP components have been included in SHIFT.
C      Those depend on FIXED QM Mulliken charges(and GSBP basis vectors)
C      are handled conveniently in misc/gsbpscc.src/sccgsbp4 following
C      very closely the way that MM GSBP component works. 
C <<<< QC_GSBP: done.
c
c constraints?
c
       if(constr2) then
        do i=1,conat(1)
         cai=conat(i+1)
         spro=convec(1,i)*gr(1,cai) 
         spro=spro+convec(2,i)*gr(2,cai) 
         spro=spro+convec(3,i)*gr(3,cai) 
         gr(1,cai)=gr(1,cai)-spro*convec(1,i)
         gr(2,cai)=gr(2,cai)-spro*convec(2,i)
         gr(3,cai)=gr(3,cai)-spro*convec(3,i)
        end do
       endif
c
c End of gradient calculation , only done when not in mode 6
c
       endif
       
       end

