! 1) contains common conversion factors
!--------- CONSTANTS AND CONVERSION FACTORS ---------------
! Conversion from Bohr to AU
_REAL_, parameter :: BOHRS_TO_A = 0.529177249D0   ! Bohrs * this = angstroms - Same constants as used in dynamo v2.
                                                  ! 0.529177249D0 : dynamo v2.
                                                  ! 0.52917706D0  : Gaussian 98
                                                  ! 0.529177D0    : Mopac6 hcore.f
_REAL_, parameter :: A_TO_BOHRS   = one/BOHRS_TO_A
                                                  ! 1.0d0 / BOHRS_TO_A
                                                  ! 1.88976D0      : used in Mopac6 gover.f
_REAL_, parameter :: A2_TO_BOHRS2 = A_TO_BOHRS*A_TO_BOHRS
                                                  ! 3.5711928576D0 : used in Mopac6 gover.f
_REAL_, parameter :: A3_TO_BOHRS3 = A2_TO_BOHRS2*A_TO_BOHRS
_REAL_, parameter :: A4_TO_BOHRS4 = A2_TO_BOHRS2*A2_TO_BOHRS2

! Conversion from AU to EV
_REAL_, parameter :: AU_TO_EV = 27.21d0           ! AU to EV
                                                  ! - dynamo v2 and Gaussian 98
                                                  ! - Mopac6: 27.21D0 in calpar.f, delri.f and repp.f
                                                  !           27.2107 in ffhpol.f
                                                  !           27.211  in manual
                                                  ! - more precise would be: 1 a.u. 27.211396 eV
_REAL_, parameter :: HALF_AU_TO_EV   = AU_TO_EV * half
_REAL_, parameter :: FOURTH_AU_TO_EV = AU_TO_EV * fourth
_REAL_, parameter :: EIGHTH_AU_TO_EV = AU_TO_EV * eighth
_REAL_, parameter :: SXNTH_AU_TO_EV  = EIGHTH_AU_TO_EV*half

! Converstion from EV to Kcal/mol
_REAL_, parameter :: EV_TO_KCAL = 23.061d0        ! Conversion from EV to KCAL/MOL : 23.060362D0
                                                  ! 23.061d0 : Dynamo
                                                  ! 23.061   : Mopac6
_REAL_, parameter :: KCAL_TO_EV = one / EV_TO_KCAL

! Conversion from AU to Kcal/mol
_REAL_, parameter :: AU_TO_KCAL            = AU_TO_EV*EV_TO_KCAL
_REAL_, parameter :: A2_TO_BOHRS2xAU_TO_EV = A2_TO_BOHRS2*AU_TO_EV

!
!***Note***
! Added by namkh, since it is used in routine DQLINK4 for GHO correction
! CCELEC constants in CHARMM
! CCELEC is 1/ (4 pi eps ) in AKMA units, conversion from SI
! units: CCELEC = e*e*Na / (4*pi*eps*1Kcal*1A)
! 
! for details. refer consta.fcm
_REAL_, parameter :: CCELEC = 332.0716D0          ! this old CHARMM value is kept for compatibility reasons
                                                  ! use compatibility with other electrostatics
                                                  ! 332.0522173D0 : AMBER
                                                  ! 332.054D0     : DISCOVER
                                                  ! 332.0636D0    : old value of dubious origin
                                                  ! 331.843D0     : value from 1986-1987 CRC Handbook 
                                                  !                          of Chemistry and Physics

! It should be moved some other places..
! Exponential decay factor for the MM atom in the core-core interaction
_REAL_, parameter :: ALPH_MM = 5.0d0
_REAL_, parameter :: AXIS_TOL = 1.0d-8            ! Tolerance at which to define a vector is along the axis.
_REAL_, parameter :: OVERLAP_CUTOFF = 100.0d0*A2_TO_BOHRS2
                                                  ! Distance^2 in bohrs at which to assume
                                                  ! Gaussian overlap is zero.
_REAL_, parameter :: EXPONENTIAL_CUTOFF = 30.0d0  ! Value of x at which to assume Exp(-x) = zero.


! 2) Common constants and definitions   
! ---- Common constants and definitions ----

_REAL_, parameter  :: DEGS_PER_RADIAN = 180.0d0 / PI 
                                                  ! Degrees in a radian

INTEGER, PARAMETER :: nelements = 86              ! Total number of elements
                                                  ! Refer parameters.h
INTEGER, PARAMETER :: MAX_VALENCE_ORBITALS =4
INTEGER, PARAMETER :: MAX_VALENCE_DIMENSION=MAX_VALENCE_ORBITALS*(MAX_VALENCE_ORBITALS+1)/2
                                                  ! Locks maximum valence orbitals at 4 = S,P
                                                  ! Not yet with D or F orbitals

integer, parameter :: PM3 = 1
integer, parameter :: AM1 = 2
integer, parameter :: MNDO = 3
integer, parameter :: PDDGPM3 = 4
integer, parameter :: PDDGMNDO = 5
integer, parameter :: PM3CARB1 = 6

_REAL_, parameter :: ONE_SQRT2 = 0.7071067811865475244D0 
                                                  ! 1.0D0/sqrt(2.0D0)
!------------------End of Conversion Factors-----------------------------------------!
