CHARMM Element tool/prefx.src 1.2
      PROGRAM PREFLX
C=======================================================================
C
C Preprocess CHARMM source to handle machine control
C    and INCLUDE files in a consistant manner.
C
C  compiler keywords read from file "preflx.dat" or "pref.dat"
C
C processed control commands (examples)
C
C-----------------------------------------------------------------------
C  ##IF    SCALAR         - process following code if "scalar" on list.
C  ##IFN   VAX SUN IRIS   - process following code if none on list.
C  ##ELIF  APOLLO         - after and IF, alternate processing.
C  ##ELSE                 - after and IF or IFN, process the rest.
C  ##ENDIF                - terminates IF, IFN, ELSE, or ELIF.
C-----------------------------------------------------------------------
C   - Any flag may be preceeded by the 5 characters ".not." (lower case)
C     to specify the complement of the flag (e.g. ".not.REPLICA").
C-----------------------------------------------------------------------
C  ##ERROR 'text'         - indicates error within IF section
C                     (usually used after an ##ELSE condition)
C-----------------------------------------------------------------------
C  ##KEYWORDS LIST unit   - inserts a fortran write lines of all current
C                           keywords to the selected write unit.
C                           "unit" may be a variable name or a number
C                           but it is limited to 8 characters maximum.
C  ##KEYWORDS FILL count array  - fills integer variable count with the
C                           number of active keys and a character*12
C                           array in the program with the current keys.
C-----------------------------------------------------------------------
C  ##INCLUDE '~/charmm_fcm/psf.fcm'   - includes a file inline
C   (some parsing of name may be done and the directory may be modified)
C              (parsing of name is performed on non-UNIX machines)
C
C-----------------------------------------------------------------------
C  ##EXPAND flag1 flag2 ...  .when.  flagc1 flagc2 ... (identifier)
C                         - setup a section for code expansion 1
C     Expand subcommands (immediately following the ##EXPAND):
C           ##PASS1 flag1 flag2 ...
C           ##PASS2 flag1 flag2 ...
C           ##PASS3 ...   - code sections and conditions for each pass
C           ##PASS[n] ...
C           ##EXFIN       - code section for the termination of the expand section
C           ##EXEND       - end of expansion specification
C  ##ENDEX   (identifier)
C     (the identifier is required and must match the corresponding ##EXPAND).
C      For each pass, the specified flags are temporarily set (or .not. set)
C      as requested.  If all of the conditions for the code expansion (flags
C      specified after the .when. construct) are not set, then all flags from
C      the ##EXPAND line (before the .when.) are temporarily set and no code
C      expansion is processed.
C-----------------------------------------------------------------------
C  Inline substitution:  {V*} is replaced by the most recent two letter
C  keyword starting with the letter "V".  This only works with any two
C  letter keywords and issues an error if none is found.
C-----------------------------------------------------------------------
C  ##SET   flag1 flag2 .. - set a flag at the current expand level
C-----------------------------------------------------------------------
C nesting of IF's is allowed, parsing is allowed in included files.
C nesting of include files is allowed.
C-----------------------------------------------------------------------
C   - Allows the construct " !## flag1 flag2 ..." at the end of a fortran line
C     for conditional processing.
C     For example, the following constructs are equivalent:
C           Standard format:
C                 ##IF LONGLINE
C                       QLONGL=.TRUE.
C                 ##ELSE
C                       QLONGL=.FALSE.
C                 ##ENDIF
C           Compact format:
C                       QLONGL=.TRUE.     !##LONGLINE        ! comments
C                       QLONGL=.FALSE.    !##.not.LONGLINE   ! comments
C     Both "and" and "or" conditions can be used:
C           !##PERT  !##PARALLEL   - An "AND" conditional compile
C           !##PERT PARALLEL       - AN "OR" conditional compile
C
C   - Allows "!" comments following valid FORTRAN lines
C-----------------------------------------------------------------------
C   - Allows control line identifiers.  For example:
C         ##IF PERT (pertprint)
C            ...
C         ##ELSE    (pertprint)
C            ...
C         ##ENDIF   (pertprint)
C     This construct checks the identifiers and prints a fatal
C     error message if the identifiers do not match.
C-----------------------------------------------------------------------
C   - Allows the include directory to be specified in preflx.dat
C         FCMDIR=directory_name ! Use the specified directory
C         FCMDIR=LOCAL          ! Use the local directory
C         FCMDIR=CURRENT        ! Use the one in the ##INCLUDE line
C-----------------------------------------------------------------------
C  Keyword Rules
C
C   - Keyword may not exceed 12 characters in length.
C   - Global keywords should be all uppercase
C   - Local  keywords should be all lowercase
C   - Keywords should otherwiswe follow fortran standards for naming
C   - Recommendation: Avoid one and two letter keywords
C-----------------------------------------------------------------------
C  Reserved Keywords when Running PREFLX
C
C   SINGLE        - will invoke a conversion to SP.
C   PUTFCM        - include files are explicitly copied to the source.
C   VMS           - Use VAX VMS directory names are
C   REMIMPNON     - Remove all "IMPLICIT NONE" lines
C   UPPERCASE     - Convert all source code (not text) to uppercase
C   INCOMMENTS ** - copy comments from all included files.
C   FCMDIR        - (See above)
C   END           - Stop reading keywords (any following keywords are ignored)
C
C      ** - May be specified by "##SET" or "##SET .not." lines in the source
C-----------------------------------------------------------------------
C   - Tabs in the source code are fatal errors.
C-----------------------------------------------------------------------
C   - Checks for fortran line lengths exceeding 72 characters.
C-----------------------------------------------------------------------
C   - For CHARMM usage, the following conventions are used:
C         Global flags (set in the file pref.dat or preflx.dat):
C                Upper case (e.g. "PERT")
C         Local flags (defined by ##SET or ##EXPAND or ##PASSSn):
C                Lower case (e.g. "nopert")
C         Local non-expand flags (##EXPAND line)
C                Single character uppercase (e.g. "P")
C-----------------------------------------------------------------------
C
C WARNING:: This source file has PREFLX constructs.  To use this
C     without a previous version of PREF, PREFLX, or PREFX,
C     some hand editing is required (search for "##").
C
C================================================================
C  Original version: Bernard R. Brooks  - October 31, 1989
C  Contributions from Youngdo Won and Ryszard Czerminski
C  Overhauled November 4, 1993 - BRB
C  Code expansion added October 21, 1996 - BRB
C  List option added July 29, 2003 - BRB
C
      CALL DRIVER
      END
C
      SUBROUTINE DRIVER
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
C
C NSTRM    - current stream number for main file or include file
C MXSTRM   - maximum file unit number for nested include files
C OUTU     - output unit for fortran source file
C LSTRM(*) - The ## structure level at entry for each include file.
C            This prevents ## structures from starting
C            and ending in different files.
      INTEGER NSTRM,MXSTRM,OUTU
      PARAMETER (MXSTRM=80)
      INTEGER LSTRM(MXSTRM)
C
C NLEV     - The current ## structure level (how many nested IFs)
C MXLEV    - The maximum number of nested IFs
C ILEV(*)  - Level processing code:
C               -1 = do not export any code for the rest of this construct
C                0 = do not export code, but do so on a subsequent ##ELIF
C                    where conditions are true, or on a subsequent ##ELSE.
C                1 = export code
C LEVID(*) - Character data used for checking for ID mismatches.
C MATCHID  - local variable for the current ID
      INTEGER NLEV,MXLEV
      PARAMETER (MXLEV=40)
      INTEGER ILEV(MXLEV)
      CHARACTER*120 LEVID(MXLEV),MATCHID
C
C MXKEYS   - The maximum number of active conditional keys
C MXEXPL   - The maximum number of expand levels
C NKEYS(*) - The number of active keys (used for conditional compiles)
C KEYS(*)  - The text for searching a given key
C IEXPL    - The current expand level
C QEXP     - The flag for each include file indicating an expanded section
      INTEGER MXKEYS,MXEXPL
      PARAMETER (MXKEYS=300, MXEXPL=20)
      INTEGER NKEYS(MXEXPL),IEXPL
      CHARACTER*12 KEYS(MXKEYS,MXEXPL)
      LOGICAL QEXP(MXEXPL+21)
      INTEGER LNKEYS(MXKEYS)
C
C
C ISTRM      - Input stream for expand temp file
C MAXCLN     - Maximum number of control lines for each expand structure
C EXPCLN(*,*)- Saved expand structure control lines
C IEXPPT(*)  - pointer to next control line for each expand structure
C EXPNAM(*)  - names of temporary expand files
C
      INTEGER ISTRM
      INTEGER MAXCLN
      PARAMETER (MAXCLN=50)
      CHARACTER*120 EXPCLN(MAXCLN,MXEXPL)
      INTEGER IEXPPT(MXEXPL)
      CHARACTER*10 EXPNAM(MXEXPL)
C
C
C LLENA    - The length of the current input line
C LINE     - The current input line
C CFEED    - A comment line full of hyphens
C FCMDIR   - Name of directory in which to seek include files
      INTEGER LLENA,LLENB
      CHARACTER*120 LINEA,LINEB
      CHARACTER*72 CFEED
      CHARACTER*60 FCMDIR,FCMD1
C
C just temporary variables
      INTEGER I,IPT,JPT,KPT,IKEY
      CHARACTER*8 UNITNAME
C
C MATCH    - Logical flag indicting that a keyname match was found
C TABCH    - Logical flag to check for TABs
C LBLANK   - Logical flag indicating a ' ' has been found
C LPAREN   - Logical flag indicating a '(' has been found
C PUTFCM   - Logical flag to copy and process include files
C SINGLE   - Logical flag to convert to single precision
C QVMS     - Logical flag to create VAX like directory names
C QRIMP    - Logical flag to remove IMPLICIT NONE statements in the code
C QUPPRC   - Logical flag to convert code to upper case before processing
C QINCOM   - Logical flag to include comments in included files
C
      LOGICAL MATCH,TABCH,PUTFCM,SINGLE,QVMS
      LOGICAL QRIMP,QUPPRC,QINCOM
C
C
C external functions
      INTEGER  LINLEN
      EXTERNAL LINLEN
C
C.##IF MACINTOSH
C.##ENDIF
C
C STIN     - unit number for initial input
C STOUT    - unit number for output
C MSGU     - unit number for error messages
      INTEGER STIN,STOUT,MSGU
C.##IF APOLLO MACINTOSH
C.##ELSE
      PARAMETER (STIN=5,STOUT=6,MSGU=3)
C.##ENDIF
      SAVE
C
C Check for tabs?
      TABCH =.TRUE.
C
C start of code
C
C On Apollo, read from unit 20(+) and write to unit 19.
      OUTU=STOUT
      NSTRM=STIN
C
C create CFEED
      CFEED(1:1)='C'
      DO I=2,72
         CFEED(I:I)='-'
      ENDDO
C
C open the input and output files if appropriate (machine dependent)
      CALL OPENF(NSTRM,OUTU,MSGU)
C
C get keynames for this run
      CALL SETKYS(NKEYS(1),MXKEYS,KEYS,FCMDIR,MSGU)
C
C Check to see if conversion to single precision is warranted.
C !!!Note: Only the "SINGLE" key will cause conversion to SP. - BRB !!!
      SINGLE=.FALSE.
      DO IKEY=1,NKEYS(1)
         IF(KEYS(IKEY,1).EQ.'SINGLE') SINGLE=.TRUE.
      ENDDO
C
C Copy include files?  We may need to change this...
C.##IF APOLLO
C.##ELSE
      PUTFCM=.FALSE.
      DO IKEY=1,NKEYS(1)
         IF(KEYS(IKEY,1).EQ.'PUTFCM') PUTFCM=.TRUE.
      ENDDO
C.##ENDIF
C
C Is it a VAX?
      QVMS=.FALSE.
      DO IKEY=1,NKEYS(1)
         IF(KEYS(IKEY,1).EQ.'VMS') QVMS=.TRUE.
      ENDDO
C
C Remove IMPLICIT NONE?
      QRIMP=.FALSE.
      DO IKEY=1,NKEYS(1)
         IF(KEYS(IKEY,1).EQ.'REMIMPNON') QRIMP=.TRUE.
      ENDDO
C
C Convert code to uppercase?
      QUPPRC=.FALSE.
      DO IKEY=1,NKEYS(1)
         IF(KEYS(IKEY,1).EQ.'UPPERCASE') QUPPRC=.TRUE.
      ENDDO
C
C Convert code to uppercase?
      QINCOM=.FALSE.
      DO IKEY=1,NKEYS(1)
         IF(KEYS(IKEY,1).EQ.'INCOMMENTS') QINCOM=.TRUE.
      ENDDO
C
C some Macintosh code so that it can do all of the files in a single run.
C It gets the files from a list found on the file "preflx.list".
C.##IF MACINTOSH
C.##ENDIF
C
C
C setup values to start a new file for processing
      NLEV=1
      ILEV(1)=1
      LSTRM(1)=1
      LEVID(1)='LEVEL 1'
      IEXPL=1
C
C=======================================================================
C
C MAIN LOOP START
C
 100  CONTINUE
C read the next line
      READ(NSTRM,20,END=200) LINEA
  20  FORMAT(20A)
C See if there are any "!" on this line and remove flags not set
      CALL CHKLINE(LINEA,LLENA,NKEYS,MXKEYS,KEYS,IEXPL,MSGU,
     &             TABCH,QUPPRC)
C search for and comment out 'IMPLICIT NONE' lines
      IF(QRIMP) THEN
         I=INDEX(LINEA,'IMPLICIT NONE')
         IF (I.GT.0) LINEA(1:1)='C'
      ENDIF
C
C=======================================================================
C process non control lines
      IF(LINEA(1:2).NE.'##') THEN
         IF(ILEV(NLEV).EQ.1) THEN
            CALL OUTLINE(LINEA,OUTU,NSTRM,STIN,MSGU,SINGLE,QINCOM)
         ENDIF
C=======================================================================
C process control line
      ELSE IF(LINEA(1:5).EQ.'##IF ') THEN
C process-if-command
C   check-for-keyname-match
         CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
C add a new level
         NLEV=NLEV+1
         IF(NLEV.GT.MXLEV) STOP 'FATAL: TOO MANY LEVELS'
         LEVID(NLEV)=MATCHID
C write out auto indented comment
         IF(ILEV(NLEV-1).EQ.1) WRITE(OUTU,20) 'C',('.',I=1,NLEV-1),
     &                         LINEA(1:LLENA)
C set the correct ILEV value for this new ##IF construct
         IF(ILEV(NLEV-1).EQ.1) THEN
            IF(MATCH) THEN
               ILEV(NLEV)=1
            ELSE
               ILEV(NLEV)=0
            ENDIF
         ELSE
            ILEV(NLEV)=-1
         ENDIF
C=======================================================================
      ELSE IF(LINEA(1:6).EQ.'##IFN ') THEN
C process-ifn-command
C   check-for-keyname-match
         CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
C add a new level
         NLEV=NLEV+1
         IF(NLEV.GT.MXLEV) STOP 'FATAL: TOO MANY LEVELS'
         LEVID(NLEV)=MATCHID
C write out auto indented comment
         IF(ILEV(NLEV-1).EQ.1) WRITE(OUTU,20) 'C',('.',I=1,NLEV-1),
     &                         LINEA(1:LLENA)
C set the correct ILEV value for this new ##IFN construct
         IF(ILEV(NLEV-1).EQ.1) THEN
            IF(MATCH) THEN
               ILEV(NLEV)=0
            ELSE
               ILEV(NLEV)=1
            ENDIF
         ELSE
            ILEV(NLEV)=-1
         ENDIF
C=======================================================================
      ELSE IF((LINEA(1:7).EQ.'##ELIF ') .OR.
     &        (LINEA(1:9).EQ.'##ELSEIF ')) THEN
C process-elif-command
C write out auto indented comment
         IF(ILEV(NLEV-1).EQ.1) WRITE(OUTU,20) 'C',('.',I=1,NLEV-1),
     &                         LINEA(1:LLENA)
C   check-for-keyname-match
         CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
         IF(LEVID(NLEV).NE.MATCHID) THEN
C oops, as bad ID...
            WRITE(MSGU,20) 'ERROR: ID MISMATCH:'
            WRITE(MSGU,20) '    PREVIOUS: ',LEVID(NLEV)
            WRITE(MSGU,20) '    CURRENT:  ',MATCHID
            STOP  'ERROR: ID MISMATCH:'
         ENDIF
C modify the current ILEV value for this ##IF construct
         IF(ILEV(NLEV).EQ.1) ILEV(NLEV)=-1
         IF(ILEV(NLEV).EQ.0) THEN
            IF(MATCH) ILEV(NLEV)=1
         ENDIF
C=======================================================================
      ELSE IF(LINEA(1:7).EQ.'##ELSE ') THEN
C process-else-command
C   check-for-keyname-match
         CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
         IF(LEVID(NLEV).NE.MATCHID) THEN
C oops, as bad ID...
            WRITE(MSGU,20) 'ERROR: ID MISMATCH:'
            WRITE(MSGU,20) '    PREVIOUS: ',LEVID(NLEV)
            WRITE(MSGU,20) '    CURRENT:  ',MATCHID
            STOP  'ERROR: ID MISMATCH:'
         ENDIF
C write out auto indented comment
         IF(ILEV(NLEV-1).EQ.1) WRITE(OUTU,20) 'C',('.',I=1,NLEV-1),
     &                         LINEA(1:LLENA)
C modify the current ILEV value for this ##IF construct
         IF(ILEV(NLEV).EQ.1) ILEV(NLEV)=-1
         IF(ILEV(NLEV).EQ.0) ILEV(NLEV)=1
C=======================================================================
      ELSE IF(LINEA(1:8).EQ.'##ENDIF ') THEN
C process-endif-command
C   check-for-keyname-match
         CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
         IF(LEVID(NLEV).NE.MATCHID) THEN
            WRITE(MSGU,20) 'ERROR: ID MISMATCH:'
            WRITE(MSGU,20) '    PREVIOUS: ',LEVID(NLEV)
            WRITE(MSGU,20) '    CURRENT:  ',MATCHID
            STOP  'ERROR: ID MISMATCH:'
         ENDIF
C write out auto indented comment
         IF(ILEV(NLEV-1).EQ.1) WRITE(OUTU,20) 'C',('.',I=1,NLEV-1),
     &                         LINEA(1:LLENA)
C remove this current level (done with ##IF construct)
         NLEV=NLEV-1
C Oops, too many ##ENDIFs
         IF(NLEV.EQ.0) STOP 'ERROR with ENDIF COUNT'
C=======================================================================
      ELSE IF(LINEA(1:8).EQ.'##ERROR ') THEN
C process-error-command
         IF(ILEV(NLEV).EQ.1) THEN
            WRITE(OUTU,20) 'C',('.',I=1,NLEV-1),LINEA
            WRITE(MSGU,20) ' ERROR: Bad selection in parsing IF block'
            WRITE(MSGU,20) 'C',('.',I=1,NLEV-1),LINEA
            STOP  'ERROR: Programmed ERROR encountered'
         ENDIF
C=======================================================================
      ELSE IF(LINEA(1:6).EQ.'##SET ') THEN
C process-set-command
         IF(ILEV(NLEV).EQ.1) THEN
C add a temporary flag to the current level
            WRITE(OUTU,20) 'C:::',LINEA(1:LLENA)
            CALL ADDKYS(LINEA,MSGU,
     &                  NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL),QINCOM)
         ENDIF
C=======================================================================
      ELSE IF(LINEA(1:16).EQ.'##KEYWORDS LIST ') THEN
C process-keywords-list-command
C
C LIST mode: Create code that looks something like:
C00000000111111111122222222223333333333444444444455555555556666666666777
C23456789012345678901234567890123456789012345678901234567890123456789012
C     WRITE(OUTU    ,'(1X)')
C     WRITE(OUTU    ,'(1X,A)') 'KEYWORD LIST: Number of keys: xxx.'
C     WRITE(OUTU    ,'(6X,A)')
C    &'  KEY1 KEY2 KEY3 ...                                            '
C     WRITE(OUTU    ,'(6X,A)')
C    &'  KEY12 KEY13 KEY14 ...                                         '
C     WRITE(OUTU    ,'(1X)')
C     WRITE(OUTU    ,'(1X)')
C
         IF(ILEV(NLEV).EQ.1) THEN
           WRITE(OUTU,20) 'C:::',LINEA(1:LLENA)
C first get unit name
           LINEA(1:16)=' '
           CALL TRIMA(LINEA,LLENA)
           IF(LLENA.LE.0) STOP 'ERROR: LIST command syntax error.'
           JPT=LLENA
           DO IPT=LLENA,2,-1
             IF(LINEA(IPT:IPT).EQ.' ') JPT=IPT-1
           ENDDO
           IF(JPT.GT.8) STOP 'ERROR: LIST command file name too long.'
           UNITNAME=LINEA(1:JPT)
C write a blank line
           LINEA="      WRITE(        ,'(1X)')"
           LINEA(13:20)=UNITNAME
           LLENA=LINLEN(LINEA)
           WRITE(OUTU,20) LINEA(1:LLENA)
C write out number of active keys
           LINEA=' '
           LINEA( 1:36)="      WRITE(xxxxxxxx,'(1X,A)') 'KEYW"
           LINEA(37:72)="ORD LIST: Number of keys: xxx.'     "
           LINEA(13:20)=UNITNAME
           WRITE(LINEA(62:65),'(I4)') NKEYS(IEXPL)
           LLENA=LINLEN(LINEA)
           WRITE(OUTU,20) LINEA(1:LLENA)
C get key lengths
           DO IPT=1,NKEYS(IEXPL)
             LNKEYS(IPT)=12
             CALL TRIMA(KEYS(IPT,IEXPL),LNKEYS(IPT))
           ENDDO
C now write out the keys
           LINEA="      WRITE(        ,'(6X,A)')"
           LINEA(13:20)=UNITNAME
           LLENA=LINLEN(LINEA)
           JPT=8
           LINEB=' '
           LINEB( 1:36)="     &'                             "
           LINEB(37:72)="                                   '"
           LLENB=72
           DO IPT=1,NKEYS(IEXPL)
             IF(JPT+LNKEYS(IPT).GT.72) THEN
               WRITE(OUTU,20) LINEA(1:LLENA)
               WRITE(OUTU,20) LINEB(1:LLENB)
               JPT=8
               LINEB( 1:36)="     &'                             "
               LINEB(37:72)="                                   '"
             ENDIF
             LINEB(JPT:JPT+LNKEYS(IPT)-1)=KEYS(IPT,IEXPL)
             JPT=JPT+LNKEYS(IPT)+1
           ENDDO
           IF(JPT.GT.8) THEN
             WRITE(OUTU,20) LINEA(1:LLENA)
             WRITE(OUTU,20) LINEB(1:LLENB)
           ENDIF
C write two blank lines
           LINEA="      WRITE(        ,'(1X)')"
           LINEA(13:20)=UNITNAME
           LLENA=LINLEN(LINEA)
           WRITE(OUTU,20) LINEA(1:LLENA)
           WRITE(OUTU,20) 'C:::'
         ENDIF
C=======================================================================
      ELSE IF(LINEA(1:16).EQ.'##KEYWORDS FILL ') THEN
C process-keywords-fill-command
C
C FILL mode: Create code that looks something like:
C
C00000000111111111122222222223333333333444444444455555555556666666666777
C23456789012345678901234567890123456789012345678901234567890123456789012
C     COUNT    = xxxx
C     ARRAY    (   1) = 'KEY1        '
C     ARRAY    (   2) = 'KEY2        '
C
         IF(ILEV(NLEV).EQ.1) THEN
C
C first get count name
           WRITE(OUTU,20) 'C:::',LINEA(1:LLENA)
           LINEA(1:16)=' '
           CALL TRIMA(LINEA,LLENA)
           IF(LLENA.LE.0) STOP 'ERROR: FILL command syntax error.'
           JPT=LLENA
           DO IPT=LLENA,2,-1
             IF(LINEA(IPT:IPT).EQ.' ') JPT=IPT-1
           ENDDO
           IF(JPT.GT.8) STOP 'ERROR: FILL command count name too long.'
           UNITNAME=LINEA(1:JPT)
C write the count
           LINEB='                ='
           LINEB(7:16)=UNITNAME
           WRITE(LINEB(19:22),'(I4)') NKEYS(IEXPL)
           LLENB=22
           WRITE(OUTU,20) LINEB(1:LLENB)
C now get array name
           LINEA(1:JPT)=' '
           CALL TRIMA(LINEA,LLENA)
           IF(LLENA.LE.0) STOP 'ERROR: FILL command syntax error.'
           JPT=LLENA
           DO IPT=LLENA,2,-1
             IF(LINEA(IPT:IPT).EQ.' ') JPT=IPT-1
           ENDDO
           IF(JPT.GT.8) STOP 'ERROR: FILL command count name too long.'
           UNITNAME=LINEA(1:JPT)
C write the array
           DO IPT=1,NKEYS(IEXPL)
             LINEA="               (    ) = '            '"
             LINEA(7:14)=UNITNAME
             WRITE(LINEA(17:20),'(I4)') IPT
             LINEA(26:37)=KEYS(IPT,IEXPL)
             LLENA=38
             WRITE(OUTU,20) LINEA(1:LLENA)
           ENDDO
           WRITE(OUTU,20) 'C:::'
         ENDIF
C=======================================================================
      ELSE IF(LINEA(1:10).EQ.'##INCLUDE ') THEN
C process-include-command
         IF(ILEV(NLEV).EQ.1) THEN
            IF (PUTFCM) THEN
               WRITE(OUTU,20) CFEED
               WRITE(OUTU,20) 'C:::',LINEA(1:LLENA)
               LSTRM(NSTRM)=NLEV
               IF(NSTRM.EQ.STIN) THEN
                  NSTRM=21
               ELSE
                  NSTRM=NSTRM+1
               ENDIF
               QEXP(NSTRM)=.FALSE.
               IF(NSTRM.GT.MXSTRM) STOP 'ERROR: Too many include levels'
               LLENA=120
            ENDIF
            CALL TRIMA(LINEA,LLENA)
C handle machine specific include processing (directory name,...)
            CALL INCLDE(NSTRM,OUTU,LINEA,LLENA,PUTFCM,QVMS,FCMDIR)
         ENDIF
C=======================================================================
      ELSE IF(LINEA(1:9).EQ.'##EXPAND ') THEN
C process-expand-command
         I=MAX(INDEX(LINEA,'.WHEN.'),INDEX(LINEA,'.when.'))
         IF(ILEV(NLEV).EQ.1) THEN
            WRITE(OUTU,20) CFEED
            WRITE(OUTU,20) 'C:::',LINEA(1:LLENA)
C determine if the expand will be active.
            IF(I.GT.0) THEN
               LINEB=LINEA(I:)
               CALL KEYMATCH(LINEB,LLENB,MATCH,MATCHID,.TRUE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
            ELSE
               MATCH=.TRUE.
            ENDIF
         ELSE
            MATCH=.FALSE.
         ENDIF
C
            IF(MATCH) THEN
C Process this expand section
               ISTRM=NSTRM
               LSTRM(NSTRM)=NLEV
               IF(NSTRM.EQ.STIN) THEN
                  NSTRM=21
               ELSE
                  NSTRM=NSTRM+1
               ENDIF
               IF(NSTRM.GT.MXSTRM) STOP 'ERROR: Too many include levels'
               QEXP(NSTRM)=.TRUE.
C
C get MATCHID for this strucutre
               CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
               IF(MATCHID.EQ.' ') STOP '##EXPAND with no match id'
C copy current flags for the new expand level
               IF(IEXPL.GE.MXEXPL) STOP 'FATAL: TOO MANY EXPAND LEVELS'
               NKEYS(IEXPL+1)=NKEYS(IEXPL)
               DO I=1,NKEYS(IEXPL)
                  KEYS(I,IEXPL+1)=KEYS(I,IEXPL)
               ENDDO
               IEXPL=IEXPL+1
C create temporary include file for expanded code
               WRITE(EXPNAM(IEXPL),445) NSTRM
 445           FORMAT('FOR0',I2,'.DAT')
C.##IF APOLLO
C.##ELSE
               OPEN(UNIT=NSTRM,FORM='FORMATTED',FILE=EXPNAM(IEXPL),
     &              ACCESS='SEQUENTIAL',STATUS='UNKNOWN')
C.##ENDIF
C get the control lines for this expand construct
               IPT=1
               GOTO 500
 490           STOP 'End of file found in an expand control section'
 495           STOP 'End of file found in an expand section body'
 500           CONTINUE
               READ(ISTRM,20,END=490) LINEA
               LLENA=LINLEN(LINEA)
               IF(LLENA.LE.0) GOTO 500
               EXPCLN(IPT,IEXPL)=LINEA
               IPT=IPT+1
               IF(IPT.GT.MAXCLN) STOP 'Too many expand control lines'
               IF(LINEA(1:7).NE.'##EXEND') GOTO 500
C copy the body of the expand section into a temporary file
 520           CONTINUE
               READ(ISTRM,20,END=495) LINEA
               LLENA=LINLEN(LINEA)
               IF(LINEA(1:7).NE.'##ENDEX') THEN
                  IF(LLENA.GT.0) WRITE(NSTRM,20) LINEA(1:LLENA)
                  GOTO 520
               ENDIF
C ##ENDEX found - check to see if it is the current one.
               CALL KEYMATCH(LINEA,LLENA,MATCH,LINEB,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
               IF(LINEB.NE.MATCHID) THEN
                  IF(LLENA.GT.0) WRITE(NSTRM,20) LINEA(1:LLENA)
                  GOTO 520
               ENDIF
C match found, save temp file for subsequent read.
               CLOSE(UNIT=NSTRM)
C--------------------------
C OK, control section copied and temp file created - process first pass.
C     (there must be at least one pass.)
               IEXPPT(IEXPL)=1
               LINEA=EXPCLN(1,IEXPL)
               IF(LINEA(1:6).NE.'##PASS')
     &                   STOP 'Expand section has no first pass'
C              Process this first pass in an expanded section
               GOTO 700
C--------------------------
            ELSE
C Do not process this expand section (treat as an ##IF)
               CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
C add a new level
               NLEV=NLEV+1
               IF(NLEV.GT.MXLEV) STOP 'FATAL: TOO MANY LEVELS'
               LEVID(NLEV)=MATCHID
               ILEV(NLEV)=ILEV(NLEV-1)
               IF(IEXPL.GE.MXEXPL) STOP 'FATAL: TOO MANY EXPAND LEVELS'
               IEXPL=IEXPL+1
C turn on temporary flags
               IF(ILEV(NLEV).EQ.1) THEN
                  IF(I.GT.0) LINEA(I:)=' '
                  NKEYS(IEXPL)=NKEYS(IEXPL-1)
                  DO I=1,NKEYS(IEXPL)
                     KEYS(I,IEXPL)=KEYS(I,IEXPL-1)
                  ENDDO
                  CALL ADDKYS(LINEA,MSGU,
     &                NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL),QINCOM)
               ENDIF
C write out auto indented comment
               IF(ILEV(NLEV-1).EQ.1) WRITE(OUTU,20)'C',('.',I=1,NLEV-1),
     &                         LINEA(1:LLENA)
               GOTO 560
 559           STOP 'End of file found in an expand control section'
C remove the control lines for this unused expand construct
 560           CONTINUE
               READ(NSTRM,20,END=559) LINEA
               LLENA=LINLEN(LINEA)
               IF(LLENA.LE.0) GOTO 560
               IF(LINEA(1:7).NE.'##EXEND') GOTO 560
            ENDIF
C=======================================================================
      ELSE IF(LINEA(1:8).EQ.'##ENDEX ') THEN
C process-endex-command
C  if we get here, then the expand must be inactive.  Just treat
C  this as if it was an ##endif if the code is active.
         CALL KEYMATCH(LINEA,LLENA,MATCH,MATCHID,.FALSE.,
     &             NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
         IF(LEVID(NLEV).NE.MATCHID) THEN
            WRITE(MSGU,20) 'ERROR: ID MISMATCH:'
            WRITE(MSGU,20) '    PREVIOUS: ',LEVID(NLEV)
            WRITE(MSGU,20) '    CURRENT:  ',MATCHID
            STOP  'ERROR: ID MISMATCH:'
         ENDIF
C write out auto indented comment
         IF(ILEV(NLEV-1).EQ.1) WRITE(OUTU,20) 'C',('.',I=1,NLEV-1),
     &                      LINEA(1:LLENA)
C remove this current level (done with ##EXPAND construct)
         NLEV=NLEV-1
         IEXPL=IEXPL-1
C Oops, too many ##ENDEXs
         IF(NLEV.EQ.0 .OR. IEXPL.EQ.0) STOP 'ERROR with ENDEX COUNT'
C=======================================================================
      ELSE IF(LINEA(1:10).EQ.'##FORTRAN ') THEN
         WRITE(OUTU,20) 'C'//LINEA(1:LLENA)
C
C=======================================================================
      ELSE IF(LINEA(1:8).EQ.'##FLECS ') THEN
         WRITE(OUTU,20) 'C'//LINEA(1:LLENA)
         STOP 'ERROR: FLECS is no longer supported.'
C=======================================================================
      ELSE
C pass through as comments unrecognized commands, and issue warning
         WRITE(MSGU,35) LINEA(1:LLENA)
  35     FORMAT(' WARNING::: unrecognized command line: ',A)
         IF(ILEV(NLEV).EQ.1) WRITE(OUTU,20) 'C'//LINEA(1:LLENA)
         STOP 'ERROR: Unrecognized ## construct encountered'
C=======================================================================
      ENDIF
C=======================================================================
      GOTO 100
C=======================================================================
C
C process end of file encountered
 200  CONTINUE
      CLOSE(UNIT=NSTRM)
C Is this the main file?
      IF(NSTRM.EQ.STIN) GOTO 800
      IF(NSTRM.EQ.21) THEN
         ISTRM=STIN
      ELSE
         ISTRM=NSTRM-1
      ENDIF
C
      WRITE(OUTU,20) CFEED
C check to see if the level is correct.  Do not allow ##IF constructs
C to start and end in different files.
      IF(NLEV.GT.LSTRM(ISTRM))
     &    STOP 'ERROR, end of file found in IF block in expand'
      IF(NLEV.LT.LSTRM(ISTRM))
     &    STOP 'ERROR, too many ENDIFs in expand file'
C-------------------------
      IF(.NOT.QEXP(NSTRM)) THEN
         NSTRM=ISTRM
         GOTO 100
      ENDIF
C=======================================================================
 600  CONTINUE
C This was an expanded section of code.  Process next pass, or
C proces termination of the expanded section.
      LINEA=EXPCLN(IEXPPT(IEXPL),IEXPL)
      IF(LINEA(1:6).EQ.'##PASS') THEN
C copy keys for next pass
         NKEYS(IEXPL)=NKEYS(IEXPL-1)
         DO I=1,NKEYS(IEXPL-1)
            KEYS(I,IEXPL)=KEYS(I,IEXPL-1)
         ENDDO
C process next pass
         GOTO 700
      ELSE IF(LINEA(1:7).EQ.'##EXFIN') THEN
C process section finish
 660     CONTINUE
         IEXPPT(IEXPL)=IEXPPT(IEXPL)+1
         LINEA=EXPCLN(IEXPPT(IEXPL),IEXPL)
         IF(LINEA(1:2).NE.'##') THEN
            CALL CHKLINE(LINEA,LLENA,NKEYS,MXKEYS,KEYS,IEXPL,MSGU,
     &                   TABCH,QUPPRC)
            CALL OUTLINE(LINEA,OUTU,NSTRM,STIN,MSGU,SINGLE,QINCOM)
            GOTO 660
         ELSE
            IF(LINEA(1:7).NE.'##EXEND')
     &         STOP '##EXEND not found where expected'
         ENDIF
      ELSE IF(LINEA(1:7).EQ.'##EXEND') THEN
C terminate expand section
      ELSE IF(LINEA(1:2).EQ.'##') THEN
         STOP 'Unrecognized expand section ## subcommand'
      ELSE
         STOP 'No expand section ## subcommand where expected'
      ENDIF
C done with this expand section...
      IEXPL=IEXPL-1
C-------------------------
C Go back to the previous file.
      NSTRM=ISTRM
      GOTO 100
C
C=======================================================================
 700  CONTINUE
C Process another expand pass (or do first) in an expanded section
C   (assume key from previous level are already copied)
C
C Determine if the pass will be active
      I=MAX(INDEX(LINEA,'.WHEN.'),INDEX(LINEA,'.when.'))
      IF(I.GT.0) THEN
         LINEB=LINEA(I:)
         CALL KEYMATCH(LINEB,LLENB,MATCH,MATCHID,.TRUE.,
     &          NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
         LINEA(I:)=' '
      ELSE
         MATCH=.TRUE.
      ENDIF
C
      IF (MATCH) THEN
C process this pass
         CALL ADDKYS(LINEA,MSGU,
     &          NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL),QINCOM)
 750     CONTINUE
         IEXPPT(IEXPL)=IEXPPT(IEXPL)+1
         LINEA=EXPCLN(IEXPPT(IEXPL),IEXPL)
         IF(LINEA(1:2).NE.'##') THEN
            CALL CHKLINE(LINEA,LLENA,NKEYS,MXKEYS,KEYS,IEXPL,MSGU,
     &                   TABCH,QUPPRC)
            CALL OUTLINE(LINEA,OUTU,NSTRM,STIN,MSGU,SINGLE,QINCOM)
            GOTO 750
         ENDIF
C invoke the tempory file to get the body of the expanded section
         LINEA=''''//EXPNAM(IEXPL)//''''
         LLENA=120
         CALL TRIMA(LINEA,LLENA)
         FCMD1='LOCAL'
         CALL INCLDE(NSTRM,OUTU,LINEA,LLENA,PUTFCM,QVMS,FCMD1)
         GOTO 100
      ELSE
C ignore this pass
 760     CONTINUE
         IEXPPT(IEXPL)=IEXPPT(IEXPL)+1
         LINEA=EXPCLN(IEXPPT(IEXPL),IEXPL)
         IF(LINEA(1:2).NE.'##') GOTO 760
         GOTO 600
      ENDIF
C
C=======================================================================
 800  CONTINUE
C Now we are done processing this file.  Do a level check.
      IF(NLEV.NE.1) STOP 'ERROR, end of file found in IF block'
      IF(IEXPL.NE.1) STOP 'ERROR, end of file found in EXPAND block'
C
C.##IF MACINTOSH
C.##ENDIF
C
      STOP
      END
C=======================================================================
C=======================================================================
      SUBROUTINE OUTLINE(LINEA,OUTU,NSTRM,STIN,MSGU,SINGLE,QINCOM)
C
C Write out the current line after some processing and checking
C
C     LINEA - line to write out
C     OUTU  - unit number for current output
C     NSTRM - unit number for current  input
C     STIN  - unit number for standard input
C     MSGU  - unit number for error message (e.g. line too long)
C     SINGLE- logical flag for single precision conversion check
C     QINCOM- logical flag for writing comment liness in include files
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      CHARACTER*(*) LINEA
      INTEGER OUTU,NSTRM,STIN,MSGU
      LOGICAL SINGLE,QINCOM
C
      INTEGER LLENA
C external functions
      INTEGER  LINLEN
      EXTERNAL LINLEN
      SAVE
C
      IF(LINEA.EQ.' ') RETURN
      LLENA=LINLEN(LINEA)
C
      IF(LINEA(1:1).EQ.'C'.OR.LINEA(1:1).EQ.'c') THEN
C This is a comment line, write it out if not in an include file
         IF(NSTRM.EQ.STIN .OR. QINCOM) WRITE(OUTU,20) LINEA(1:LLENA)
      ELSE IF(ICHAR(LINEA(1:1)).EQ.12) THEN
C This line starts with a form feed, ignore it
         LINEA(1:1)='C'
      ELSE
C This line must be code, write it out, but first check for 72 limit
         IF(LLENA.GT.72) WRITE(MSGU,76) LINEA(1:LLENA)
  76        FORMAT('  ERROR:: Line longer than 72 characters:'/,
     &               '   "',A,'"')
         IF(SINGLE) CALL SINGLES(LINEA)
         WRITE(OUTU,20) LINEA(1:LLENA)
  20     FORMAT(20A)
      ENDIF
      RETURN
      END
C=======================================================================
C=======================================================================
      SUBROUTINE CHKLINE(LINEA,LLENA,NKEYS,MXKEYS,KEYS,IEXPL,MSGU,
     &                   TABCH,QUPPRC)
C
C Check the current line for !## flags and for ! in general.
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      CHARACTER*(*) LINEA
      INTEGER LLENA,MSGU,IEXPL
      INTEGER NKEYS(*),MXKEYS
      CHARACTER*(*) KEYS(MXKEYS,*)
      LOGICAL TABCH,QUPPRC
C
      INTEGER I,J,LLENB
      LOGICAL SQF,MATCH
      CHARACTER*120 MATCHID,LINEB
      CHARACTER*12  WRD
      INTEGER  LINLEN
      EXTERNAL LINLEN
      SAVE
C
      LLENA=LINLEN(LINEA)
C see if the line has any nasty tabs in it
      IF(TABCH) THEN
         DO I=1,LLENA
            IF(ICHAR(LINEA(I:I)).EQ.9)
     &              STOP 'FATAL ERROR: TABS IN THIS FILE!'
         ENDDO
      ENDIF
C
      IF(LLENA.GT.100) WRITE(MSGU,76) LINEA(1:LLENA)
  76  FORMAT('  WARNING:: Line longer than 100 characters:',
     &         ' Errors possible.'/,'   "',A,'"')
C
      IF(LINEA(1:1).EQ.'C') GOTO 220
      IF(LINEA(1:1).EQ.'c') GOTO 220
C See if there are any "{" on this line for substituions of the
C type {V*} => V0
  80  CONTINUE
      DO I=2,LLENA-3
        IF(LINEA(I:I).EQ.'{') THEN
          IF(LINEA(I+2:I+3).EQ.'*}') THEN
C           found a substitution...replace it
            DO J=NKEYS(IEXPL),1,-1
              WRD=KEYS(J,IEXPL)
              IF(WRD(1:1).EQ.LINEA(I+1:I+1)) THEN
                IF(WRD(3:).EQ.' ') THEN
C                 found a match...
                  LINEB(1:I-1)=LINEA(1:I-1)
                  LINEB(I:I+1)=WRD(1:2)
                  LINEB(I+2:)=LINEA(I+4:)
                  LINEA=LINEB
                  LLENA=LLENA-2
                  GOTO 80
                ENDIF
              ENDIF
            ENDDO
C           found no match
            STOP 'ERROR: NO MATCH OF TWO LETTER SUBSTITUTION KEYWORD'
          ENDIF
        ENDIF
      ENDDO
C
C See if there are any "!" on this line
      DO I=1,LLENA
         IF(LINEA(I:I).EQ.'!') THEN
            IF(QUPPRC) THEN
               SQF=.FALSE.
               DO J=1,I
                  IF(LINEA(J:J).EQ.'''') SQF=.NOT.SQF
                  IF(.NOT.SQF) CALL UPPERCASE(LINEA(J:J),1)
               ENDDO
            ENDIF
            GOTO 210
         ENDIF
      ENDDO
C
C No exclamation mark, now check uppercase flag
      IF(QUPPRC) THEN
         SQF=.FALSE.
         DO I=1,LLENA
            IF(LINEA(I:I).EQ.'''') SQF=.NOT.SQF
C           dont uppercase text in quotes.
            IF(.NOT.SQF) CALL UPPERCASE(LINEA(I:I),1)
         ENDDO
      ENDIF
C
      RETURN
 210  CONTINUE
C we found at least one, now lets process the line
      SQF=.FALSE.
      DO I=1,LLENA
         IF(LINEA(I:I).EQ.'''') SQF=.NOT.SQF
         IF(LINEA(I:I).EQ.'!') THEN
           IF(.NOT.SQF) THEN
C            here's a ! comment, remove it
             J=I+2
C            check for inline option parsing
             IF(LINEA(I:J).EQ.'!##') THEN
                LINEB=LINEA(I:)
                LLENB=LLENA-I+1
                DO J=2,LLENB
                   IF(LINEB(J:J).EQ.'!') LINEB(J:)=' '
                ENDDO
                LINEB(3:3)=' '
                CALL KEYMATCH(LINEB,LLENB,MATCH,MATCHID,.FALSE.,
     &             NKEYS(IEXPL),MXKEYS,KEYS(1,IEXPL))
                IF(MATCH) THEN
                   LINEA(I:I)=' '
                   DO J=I+1,LLENA
                      IF(LINEA(J:J).EQ.'!') GOTO 215
                      LINEA(J:J)=' '
                   ENDDO
                ELSE
                   LINEA(1:2)='C:'
                   GOTO 220
                ENDIF
             ELSE
C               it's just a simple comment - remove.
                LINEA(I:I)=' '
                DO J=I+1,LLENA
                   IF(LINEA(J:J).EQ.'!') GOTO 215
                   LINEA(J:J)=' '
                ENDDO
             ENDIF
           ENDIF
         ENDIF
 215     CONTINUE
      ENDDO
 220  CONTINUE
      LLENA=LINLEN(LINEA)
      RETURN
      END
C=======================================================================
C=======================================================================
      SUBROUTINE UPPERCASE(ST,STLEN)
C
C This routine converts a string to upper case.
C This routine is ASCII code dependant.
C
      INTEGER STLEN
      CHARACTER*(*) ST
C
      INTEGER ILITA,ILITZ,ICHUCA,IBIGA,ICHR,I
      SAVE
C
      ICHUCA=ICHAR('A')
      ILITA=ICHAR('a')
      ILITZ=ICHAR('z')
      IBIGA=ICHUCA-ILITA
C
      DO I=1,STLEN
        ICHR=ICHAR(ST(I:I))
        IF(ICHR.GE.ILITA.AND.ICHR.LE.ILITZ) ST(I:I)=CHAR(ICHR+IBIGA)
      ENDDO
C
      RETURN
      END
C=======================================================================
C=======================================================================
      SUBROUTINE KEYMATCH(LINE,LLEN,MATCH,MATCHID,QALL,
     &                    NKEYS,MXKEYS,KEYS)
C
C  Determine if flags on the line match the current key set.
C to check-for-keyname-match
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
C
      INTEGER LLEN
      CHARACTER*(*) LINE
      LOGICAL MATCH
      CHARACTER*(*) MATCHID
      LOGICAL QALL
      INTEGER NKEYS,MXKEYS
      CHARACTER*(*) KEYS(MXKEYS)
C
      LOGICAL LBLANK,LPAREN,QM
      INTEGER IPT,JPT,I
      SAVE
C
      LLEN=LEN(LINE)
      CALL TRIMA(LINE,LLEN)
      MATCH=QALL
      MATCHID=' '
      LBLANK=.FALSE.
      LPAREN=.FALSE.
      DO IPT=1,LLEN
         IF(LINE(IPT:IPT).EQ.' ') THEN
            LBLANK=.NOT.LPAREN
         ELSE IF(LINE(IPT:IPT).EQ.'(') THEN
            LPAREN=.TRUE.
            LBLANK=.FALSE.
            JPT=IPT
         ELSE IF(LINE(IPT:IPT).EQ.')') THEN
            IF(.NOT.LPAREN)
     &            STOP 'ERROR: NO MATCH OF ID PARENTHESES'
            LPAREN=.FALSE.
            MATCHID=LINE(JPT:IPT)
            LBLANK=.FALSE.
         ELSE
            IF(LBLANK) THEN
               JPT=IPT+1
  40           CONTINUE
               IF(LINE(JPT:JPT).NE.' ') THEN
                  JPT=JPT+1
                  IF(JPT.LE.LLEN) GOTO 40
               ENDIF
               IF(LINE(IPT:IPT+4).EQ.'.NOT.' .OR.
     &            LINE(IPT:IPT+4).EQ.'.not.') THEN
                  QM=.FALSE.
                  DO I=1,NKEYS
                     IF(KEYS(I).EQ.LINE(IPT+5:JPT)) QM=.TRUE.
                  ENDDO
                  IF(.NOT.QALL .AND. .NOT.QM) MATCH=.TRUE.
                  IF(QALL .AND. QM) MATCH=.FALSE.
               ELSE
                  QM=.FALSE.
                  DO I=1,NKEYS
                     IF(KEYS(I).EQ.LINE(IPT:JPT)) QM=.TRUE.
                  ENDDO
                  IF(.NOT.QALL .AND. QM) MATCH=.TRUE.
                  IF(QALL .AND. .NOT.QM) MATCH=.FALSE.
               ENDIF
            ENDIF
            LBLANK=.FALSE.
         ENDIF
      ENDDO
      RETURN
      END
C=======================================================================
      SUBROUTINE TRIMA(LINE,LLEN)
C
C trims a character string
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      INTEGER LLEN,IPT,JPT
      CHARACTER*(*) LINE
      CHARACTER*120 LTEMP
      SAVE
C
      DO IPT=1,LLEN
         IF(LINE(IPT:IPT).NE.' ') GOTO 100
      ENDDO
      LLEN=0
      RETURN
 100  CONTINUE
C
      DO JPT=LLEN,1,-1
         IF(LINE(JPT:JPT).NE.' ') GOTO 200
      ENDDO
 200  CONTINUE
      LTEMP=LINE(IPT:JPT)
      LINE=LTEMP
      LLEN=JPT-IPT+1
      RETURN
      END
C=======================================================================
      INTEGER FUNCTION LINLEN(LINE)
C
C trims a character string
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      CHARACTER*(*) LINE
C
      INTEGER JPT,LLEN
      SAVE
C
      LLEN=LEN(LINE)
      DO JPT=LLEN,1,-1
         IF(LINE(JPT:JPT).NE.' ') GOTO 200
      ENDDO
      JPT=0
 200  CONTINUE
      LINLEN=JPT
      RETURN
      END
C=======================================================================
      SUBROUTINE OPENF(INU,OUTU,MSGU)
C
C  MACHINE SPECIFIC OPENING INITIAL FILES
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      INTEGER INU,OUTU,MSGU
      SAVE
C begin
C
C open FLECS input file on unit 1
C.##IF APOLLO
C.##ELSE
      OPEN(UNIT=MSGU,FILE='prefx.msg',FORM='FORMATTED',
     $     ACCESS='SEQUENTIAL',STATUS='UNKNOWN')
C.##ENDIF
      RETURN
      END
C=======================================================================
      SUBROUTINE INCLDE(INU,OUTU,LINE,LLEN,PUTFCM,QVMS,FCMDIR)
C
C MACHINE SPECIFIC OPEN INCLUDE FILE
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      INTEGER INU,OUTU,LLEN
      CHARACTER*(*) LINE
      LOGICAL PUTFCM,QVMS
      CHARACTER*60 FCMDIR
C
      INTEGER LNLINE,IPT,JPT,KPT,LPT
      CHARACTER*60 NLINE
      CHARACTER*1 DIRCHR
      SAVE
C begin
C
      DO IPT=1,LLEN
         IF(LINE(IPT:IPT).EQ.'''') GOTO 100
      ENDDO
      IF(IPT.GT.LLEN) STOP 'ERROR: cannot find include file.'
 100  CONTINUE
      DO JPT=LLEN,IPT,-1
         IF(LINE(JPT:JPT).EQ.'''') GOTO 200
      ENDDO
 200  CONTINUE
      IF(IPT.EQ.JPT) STOP 'ERROR: cannot find include file.'
      IPT=IPT+1
      JPT=JPT-1
C
      NLINE=LINE(IPT:JPT)
      LNLINE=JPT-IPT+1
C
C.##IF MACINTOSH
C.##ELSE
      IF (QVMS) THEN
         DIRCHR=':'
      ELSE
         DIRCHR='/'
      ENDIF
C.##ENDIF
C
      KPT=JPT
 50   CONTINUE
      KPT=KPT-1
      IF(KPT.EQ.IPT) GOTO 60
      IF(LINE(KPT:KPT).NE.'/'.AND.LINE(KPT:KPT).NE.':') GOTO 50
      KPT=KPT+1
 60   CONTINUE
C
C process name translation
C   (default is given name)
C
C use given name
      IF(FCMDIR.EQ.' ' .OR. FCMDIR.EQ.'CURRENT') THEN
         IF (QVMS) THEN
            DO KPT=IPT,JPT
               IF(LINE(KPT:KPT).EQ.'/') LINE(KPT:KPT)=':'
            ENDDO
         ENDIF
C use local directory
      ELSE IF(FCMDIR.EQ.'LOCAL') THEN
         NLINE=LINE(KPT:JPT)
         LNLINE=JPT-KPT+1
C use provided directory
      ELSE
         LPT=0
 70      CONTINUE
         LPT=LPT+1
         IF(FCMDIR(LPT:LPT).NE.' '.AND.LPT.LT.60) GOTO 70
         LPT=LPT-1
         NLINE = FCMDIR(1:LPT) // DIRCHR // LINE(KPT:JPT)
         LNLINE=LPT+1+JPT-KPT+1
      ENDIF
C
C open include file
C.##IF APOLLO
C.##ELSE
      IF (PUTFCM) THEN
C     open the include file
         OPEN(UNIT=INU,FORM='FORMATTED',FILE=NLINE(1:LNLINE),
     1        ACCESS='SEQUENTIAL',STATUS='OLD')
      ELSE
            WRITE (OUTU,'(A)') '      INCLUDE ''' // NLINE(1:LNLINE)
     1                         // ''''
      ENDIF
C
C.##ENDIF
C
      RETURN
      END
C=======================================================================
      SUBROUTINE SETKYS(NKEYS,MXKEYSX,KEYS,FCMDIR,MSGU)
C
C machine specific get of keynames
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      INTEGER NKEYS,MXKEYSX
      CHARACTER*(*) KEYS(MXKEYSX)
      CHARACTER*60 FCMDIR
      INTEGER MSGU
C
      INTEGER MXKEYS
      PARAMETER (MXKEYS=300)
      INTEGER LNKEYS(MXKEYS)
      CHARACTER*120 LINE
      INTEGER I,J,PFDU
      PARAMETER(PFDU=4)
      SAVE
C
      IF(MXKEYS.NE.MXKEYSX) STOP 'CODING ERROR: BAD MXKEYS'
C
C no more default keys... All explicit.
      NKEYS=0
      FCMDIR=' '
C
C open the local preflx.dat file (use defaults if it doesnt exist)
C.##IF APOLLO
C.##ELSE
      OPEN(UNIT=PFDU,FORM='FORMATTED',FILE='preflx.dat',
     1     ACCESS='SEQUENTIAL',STATUS='OLD',ERR=180)
      GOTO 190
 180  CONTINUE
      OPEN(UNIT=PFDU,FORM='FORMATTED',FILE='pref.dat',
     1     ACCESS='SEQUENTIAL',STATUS='OLD',ERR=200)
 190  CONTINUE
C.##ENDIF
C
      NKEYS=1
  50  CONTINUE
         READ(PFDU,20,END=100) LINE
  20     FORMAT(A)
         IF(LINE(1:7).EQ.'FCMDIR=') THEN
            FCMDIR=LINE(8:)
            GOTO 50
         ENDIF
         KEYS(NKEYS)=LINE(1:12)
         IF(KEYS(NKEYS).EQ.'END') GOTO 100
         IF(KEYS(NKEYS).NE.' ') NKEYS=NKEYS+1
         IF(NKEYS.GT.MXKEYS) STOP 'FATAL: TOO MANY KEYS'
      GOTO 50
 100  CONTINUE
      NKEYS=NKEYS-1
C
 200  CONTINUE
C
      DO I=1,NKEYS
         DO J=1,I-1
            IF(KEYS(I).EQ.KEYS(J)) THEN
               WRITE(MSGU,21) KEYS(I)
  21           FORMAT(' Duplicate key found: ',A)
               KEYS(I)='...'
            ENDIF
         ENDDO
      ENDDO
C
      J=0
      DO I=1,NKEYS
         IF(J.GT.0) KEYS(I-J)=KEYS(I)
         IF(KEYS(I).EQ.'...') J=J+1
      ENDDO
      NKEYS=NKEYS-J
      DO I=1,NKEYS
         LNKEYS(I)=12
         CALL TRIMA(KEYS(I),LNKEYS(I))
      ENDDO
C
      IF(NKEYS.GT.0) THEN
         WRITE(MSGU,22) NKEYS,(KEYS(I)(1:LNKEYS(I)),I=1,NKEYS)
  22     FORMAT(I4,' Conditional keys: ',12(A,1X)/,
     &        ('     Conditional keys: ',12(A,1X)))
      ELSE
         WRITE(MSGU,23)
  23     FORMAT(' No condition keys used.')
      ENDIF
C
      RETURN
      END
C
C=======================================================================
C=======================================================================
      SUBROUTINE ADDKYS(LINE,MSGU,NKEYS,MXKEYSX,KEYS,QINCOM)
C
C add or remove keys from the current list
C
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      CHARACTER*(*) LINE
      INTEGER MSGU,NKEYS,MXKEYSX
      CHARACTER*(*) KEYS(MXKEYSX)
      LOGICAL QINCOM
C
      INTEGER I,J,IPT,JPT,LLEN
C
      INTEGER MXKEYS
      PARAMETER (MXKEYS=300)
      INTEGER LNKEYS(MXKEYS)
      LOGICAL LBLANK,LPAREN,QM
      SAVE
C
      IF(MXKEYS.NE.MXKEYSX) STOP 'CODING ERROR: BAD MXKEYS'
C
      LLEN=LEN(LINE)
      CALL TRIMA(LINE,LLEN)
      LBLANK=.FALSE.
      LPAREN=.FALSE.
      DO IPT=1,LLEN
         IF(LINE(IPT:IPT).EQ.' ') THEN
            LBLANK=.NOT.LPAREN
         ELSE IF(LINE(IPT:IPT).EQ.'(') THEN
            LPAREN=.TRUE.
            LBLANK=.FALSE.
         ELSE IF(LINE(IPT:IPT).EQ.')') THEN
            IF(.NOT.LPAREN)
     &            STOP 'ERROR: NO MATCH OF ID PARENTHESES'
            LPAREN=.FALSE.
            LBLANK=.FALSE.
         ELSE
            IF(LBLANK) THEN
               JPT=IPT+1
  40           CONTINUE
               IF(LINE(JPT:JPT).NE.' ') THEN
                  JPT=JPT+1
                  IF(JPT.LE.LLEN) GOTO 40
               ENDIF
               IF(LINE(IPT:IPT+4).EQ.'.NOT.' .OR.
     &            LINE(IPT:IPT+4).EQ.'.not.') THEN
                  DO I=1,NKEYS
                     IF(KEYS(I).EQ.LINE(IPT+5:JPT)) KEYS(I)='...'
                  ENDDO
                  IF(LINE(IPT+5:JPT).EQ.'INCOMMENTS') QINCOM=.FALSE.
               ELSE
                  QM=.FALSE.
                  DO I=1,NKEYS
                     IF(KEYS(I).EQ.LINE(IPT:JPT)) QM=.TRUE.
                  ENDDO
                  IF(.NOT.QM) THEN
                     IF(LINE(IPT:JPT).NE.' ') NKEYS=NKEYS+1
                     IF(NKEYS.GT.MXKEYS) STOP 'FATAL: TOO MANY KEYS'
                     KEYS(NKEYS)=LINE(IPT:JPT)
                  ENDIF
                  IF(LINE(IPT:JPT).EQ.'INCOMMENTS') QINCOM=.TRUE.
               ENDIF
            ENDIF
            LBLANK=.FALSE.
         ENDIF
      ENDDO
C
C garbage collect old keys.
      J=0
      DO I=1,NKEYS
         IF(J.GT.0) KEYS(I-J)=KEYS(I)
         IF(KEYS(I).EQ.'...') J=J+1
      ENDDO
      NKEYS=NKEYS-J
C trim all keys to set lengths
      DO I=1,NKEYS
         LNKEYS(I)=12
         CALL TRIMA(KEYS(I),LNKEYS(I))
      ENDDO
C
      IF(NKEYS.GT.0) THEN
         WRITE(MSGU,22) NKEYS,(KEYS(I)(1:LNKEYS(I)),I=1,NKEYS)
  22     FORMAT(I4,' Conditional keys: ',12(A,1X)/,
     &        ('     Conditional keys: ',12(A,1X)))
      ELSE
         WRITE(MSGU,23)
  23     FORMAT(' No condition keys used.')
      ENDIF
      RETURN
      END
C
C=======================================================================
C=======================================================================
      SUBROUTINE SINGLES(LINE)
c-----------------------------------------------------------------------
c
c Replaces the following 32-bit machine specific features and
c converts everything into standard FORTRAN-77 single precision code
c
c 32-bit machine (VAX, Convex etc.) to 64-bit computers (CRAY)
c
c Double precision conversion:
c ====================================================================
c REAL*8            = REAL            real*8            = real
c COMPLEX*16        = COMPLEX         complex*16        = complex
c INTEGER*2         = INTEGER         integer*2         = integer
c *2(               = (
c *8(               = (
c CDABS(            = ABS(            cdabs(            = abs(
c DABS(             = ABS(            dabs(             = abs(
c CDSIN(            = SIN(            cdsin(            = sin(
c DSIN(             = SIN(            dsin(             = sin(
c CDCOS(            = COS(            cdcos(            = cos(
c DCOS(             = COS(            dcos(             = cos(
c CDTAN(            = TAN(            cdtan(            = tan(
c DTAN(             = TAN(            dtan(             = tan(
c DASIN(            = ASIN(           dasin(            = asin(
c DACOS(            = ACOS(           dacos(            = acos(
c DATAN(            = ATAN(           datan(            = atan(
c DMIN1(            = MIN(            dmin1(            = min(
c DMAX1(            = MAX(            dmax1(            = max(
c SNGL(             = (               sngl(             = (
c DBLE(             = REAL(           dble(             = real(
c DCMPLX(           = CMPLX(          dcmplx(           = cmplx(
c DCONJG(           = CONJG(          dconjg(           = conjg(
c DIMAG(            = AIMAG(          dimag(            = aimag(
c DLOG(             = LOG(            dlog(             = log(
c DMOD(             = AMOD(           dmod(             = amod(
c DSIGN(            = SIGN(           dsign(            = sign(
c DSQRT(            = SQRT(           dsqrt(            = sqrt(
c
c Youngdo Won 08-09-90
c ====================
c
C.##IFN APOLLO
      IMPLICIT NONE
C.##ENDIF
      CHARACTER*120 LINE
C
      INTEGER I
      LOGICAL DONE
      SAVE
C
C DO NOT REPLACE DOUBLE PRECISION  - BRB
CC      I=INDEX(LINE,'DOUBLE PRECISION')
CC      IF (I.GT.0) LINE(I:I+15)='REAL            '
CC      I=INDEX(LINE,'double precision')
CC      IF (I.GT.0) LINE(I:I+15)='real            '
c
      I=INDEX(LINE,'REAL*8')
      IF (I.GT.0) LINE(I:I+5)='REAL  '
      I=INDEX(LINE,'real*8')
      IF (I.GT.0) LINE(I:I+5)='real  '
c
      I=INDEX(LINE,'COMPLEX*16')
      IF (I.GT.0) LINE(I:I+9)='COMPLEX   '
      I=INDEX(LINE,'complex*16')
      IF (I.GT.0) LINE(I:I+9)='complex   '
c
      I=INDEX(LINE,'INTEGER*2')
      IF (I.GT.0) LINE(I:I+8)='INTEGER  '
      I=INDEX(LINE,'integer*2')
      IF (I.GT.0) LINE(I:I+8)='integer  '
c
      I=INDEX(LINE,'*2(')
      IF (I.GT.0) LINE(I:I+2)='(  '
C
      I=INDEX(LINE,'*8(')
      IF (I.GT.0) LINE(I:I+2)='(  '
c
   20 CONTINUE
      DONE=.TRUE.
c
      I=INDEX(LINE,'CDABS(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  ABS('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'cdabs(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  abs('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DABS(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' ABS('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dabs(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' abs('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'CDSIN(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  SIN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'cdsin(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  sin('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DSIN(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' SIN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dsin(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' sin('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'CDCOS(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  COS('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'cdcos(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  cos('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DCOS(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' COS('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dcos(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' cos('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'CDTAN(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' TAN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'cdtan(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' tan('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DTAN(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' TAN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dtan(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' tan('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DASIN(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' ASIN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dasin(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' asin('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DACOS(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' ACOS('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dacos(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' acos('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DATAN(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' ATAN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'datan(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' atan('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DMIN1(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  MIN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dmin1(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  min('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DMAX1(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  MAX('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dmax1(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  max('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'SNGL(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='    ('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'sngl(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='    ('
      DONE=.FALSE.
      ENDIF
C
C Conver DBLE to REAL - BRB
      I=INDEX(LINE,'DBLE(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='REAL('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dble(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='real('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DCMPLX(')
      IF (I.GT.0)  THEN
      LINE(I:I+6)=' CMPLX('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dcmplx(')
      IF (I.GT.0)  THEN
      LINE(I:I+6)=' cmplx('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DCONJG(')
      IF (I.GT.0)  THEN
      LINE(I:I+6)=' CONJG('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dconjg(')
      IF (I.GT.0)  THEN
      LINE(I:I+6)=' conjg('
      DONE=.FALSE.
      ENDIF
C
C There should be no DFLOATs (not part of F77) - BRB
c      I=INDEX(LINE,'DFLOAT(')
c      IF (I.GT.0)  THEN
c      LINE(I:I+6)=' FLOAT('
c      DONE=.FALSE.
c      ENDIF
c      I=INDEX(LINE,'dfloat(')
c      IF (I.GT.0)  THEN
c      LINE(I:I+6)=' float('
c      DONE=.FALSE.
c      ENDIF
C
      I=INDEX(LINE,'DIMAG(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='AIMAG('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dimag(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='aimag('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DLOG(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' LOG('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dlog(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' log('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DMOD(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='AMOD('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dmod(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)='amod('
      DONE=.FALSE.
      ENDIF
C
C There should be no DREALs (not part of F77) - BRB
c      I=INDEX(LINE,'DREAL(')
c      IF (I.GT.0)  THEN
c      LINE(I:I+5)=' REAL('
c      DONE=.FALSE.
c      ENDIF
c      I=INDEX(LINE,'dreal(')
c      IF (I.GT.0)  THEN
c      LINE(I:I+5)=' real('
c      DONE=.FALSE.
c      ENDIF
C
      I=INDEX(LINE,'DSIGN(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' SIGN('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dsign(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' sign('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'DSQRT(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' SQRT('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dsqrt(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)=' sqrt('
      DONE=.FALSE.
      ENDIF
C
Cln...Add the following 07-Jan-2005 for SINGLE versions
      I=INDEX(LINE,'DEXP(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' EXP('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dexp(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' exp('
      DONE=.FALSE.
      ENDIF
C
      I=INDEX(LINE,'IDINT(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  INT('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'idint(')
      IF (I.GT.0)  THEN
      LINE(I:I+5)='  int('
      DONE=.FALSE.
      ENDIF

C
      I=INDEX(LINE,'DINT(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' INT('
      DONE=.FALSE.
      ENDIF
      I=INDEX(LINE,'dint(')
      IF (I.GT.0)  THEN
      LINE(I:I+4)=' int('
      DONE=.FALSE.
      ENDIF
Cln...05-Jan-2005

      IF (.NOT.(DONE)) GOTO 20
c
c convert <CR> into C
c      IF (ICHAR(LINE(1:1)).EQ.12) LINE(1:1)='C'
c
c comment out 'IMPLICIT NONE'
c      I=INDEX(LINE,'IMPLICIT NONE')
c      IF (I.GT.0) LINE(1:1)='C'
c
      RETURN
      END
