#!/bin/csh -f
# setmk.com
#-----------------------------------------------------------------------
# 10-Feb-94 able to produce single module makefile
# 05-Feb-94 for c24 developments
#           c22 replaced by chm
# 01-Jan-92 generates CHARMM Module Makefiles
#           use Steve Fleischman's makemod shell script
# 
# Inquiries to Youngdo Won, chmgr@tammy.harvard.edu (617)495-1782
#-----------------------------------------------------------------------
#
if ( $#argv == 0 ) then
  echo " "
  echo " Usage: setmk.com host-machine-type"
  echo " "
  echo "        host-machine-type is one of the following."
  echo "             convex   for Convex computers"
  echo "             sgi      for SGI IRIS series"
  echo "             stardent for Stardent computers"
  echo "             ibmrs    for IBM RS/6000 series"
  echo "             sun      for SUN work stations"
  echo "             cray     for Cray X-MP and Y-MP"
  echo "             ibmsp    for IBM SP series"
  echo "             cspp     for Convex Exemplar SPP-1000 Series"
  echo "             terra    for TERRA Inc. parallel computer"
  echo "             hpux     for HP/7xx series"
  echo "             intel    for Intel iPSC/860 (gamma & delta & paragon)"
  echo " "
  exit
else
  set chm_host = $argv[1]
endif
#
if ( $#argv == 2 ) then
  set all = 0
  set mdlist = $argv[2]  
else
  set all = 1
endif
#
set main = charmm
alias pwd "pwd | sed -e 's;/tmp_mnt;;'"
#
# set up CHARMM environment variables
# if not previously set, the current working directory is assumed to
#   be $chmtool.
  setenv chmhost  $chm_host
  setenv chmroot  `cd ..;pwd | sed -e 's;/tmp_mnt;;'`
  setenv chmbuild $chmroot/build/$chm_host
  setenv chmsrc   $chmroot/source
  setenv chmtool  $chmroot/tool
  set path = ($path $chmtool)
if (! -e $chmbuild ) mkdir $chmbuild
#
if (! -e $chmtool/makemod ) then
  echo " "
  echo " setmk.com> $chmtool/makemod is not found."
  echo "            can not proceed to set up module makefiles."
  exit 1
endif
#
#
# process each module at a time
#
if ($all) set mdlist = `/bin/ls -1 $chmsrc | sed '/fcm/d' | sed -e /$main/d `
pushd $chmsrc >>! /dev/null
if ($all || $main == $mdlist[0] ) then
  pushd $main >>! /dev/null
  $chmtool/mainmake -v -n $main `pwd` `cd ..;pwd` $main.mk
  mv $main.mk $chmbuild/$main.mk
  popd >>! /dev/null
  if ($main == $mdlist[0] ) exit 0
endif
#
foreach module ( $mdlist )
  pushd $module >>! /dev/null
  $chmtool/makemod -v -n $module `pwd` `cd ..;pwd` $module.mk
  mv $module.mk $chmbuild/$module.mk
  if ( ( $module == "gamint" ) && ( -e gamess ) ) then
    pushd gamess >>! /dev/null
    $chmtool/makemod -v -n gamess `pwd` `cd ../..;pwd` gamess.mk
    mv gamess.mk $chmbuild/gamess.mk
    popd >>! /dev/null
  endif
  popd >>! /dev/null
end
popd >>! /dev/null
#
exit
