C $Header: /export/cvs/c34a1/source/moldyn/bhmult.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
c
c
      subroutine bhmult(isq,ihinge,iflag,B,C,nrb,krb,krc,ibhtype,
     &                  itopol,ibh_ptr)
c
C     PURPOSE:  This subroutine performs multiplication with the BH 
c     array. Descriptions of the BH associated variables used are in
c     BHBSRD.INC.
C
C     INPUTS:
c     isq     logical for q or p frame
c     ihinge  hinge number
c     iflag   type of multiplication (mult3,multad,mult3n)
c     B       matrix to multiply BH by
c     C       matrix to put result in
c     nrb     leading size to use of B
c     krb     declared leading size of B
c     krc     declared leading size of C
c
c
c     iflag = 1 if the multiplication is (mult3):
c
c         BH(leadxNRB) x B(NRBx1) = C(leadx1)
c
c     iflag = 0 if the multiplication is (multad):
c
c         BH x B + C = C
c
c     iflag = -1 if the multiplication is (mult3N):
c
c       - BH x B = C
c
c     (leading edge of BH is 3 or 6)
C
C
C     OUTPUTS:
C     C is returned with the multiplication product.
c
c

      implicit none
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: bhmult.f,v $$Revision: 1.1.1.1 $'/
c
#include "PARAM.FOR"
c
#include "BHBSRD.INC"
#include "HEAPMB.INC"
#include "SPECIF.INC"
#include "HANDS2.INC"
#include "PINRP.INC"
c
c     Passed Variables
c
      integer ibhtype(cL02)
      integer itopol(2,cL02)
      integer ibh_ptr(cL12+1)
      integer krb,krc,nrb            ! Sizes of B & C
      integer ihinge                 ! Hinge number
      integer iflag                  ! What type of multiplication
      double precision B(krb),C(krc)
      logical isq                    ! Which type of frame (Q/P)

c
c     local variables
c
      integer itype,iptbh
c
c
      itype = ibhtype(ihinge)       ! BH storage type

      if (isq) then
         iptbh = ibh_ptr(2*ihinge-2)
      else
         iptbh = ibh_ptr(2*ihinge-1)
      end if
c
c     Pass BH heap location
c
      call bhmult_1(BH(iptbh),isq,ihinge,itype,iflag,B,C,nrb,
     &              krb,krc,r0l(1,1,itopol(1,ihinge)),
     &              intLoopHeap(iihthptrptr),loopheap(ipinptr),
     &              LoopHeap(irhptr))

      return
      end
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     Beginning of bhmult_1
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine bhmult_1(bh_1,isq,ihinge,itype,iflag,B,C,
     &                    nrb,krb,krc,r0l,ihthptr,pin,rh)

#include "PARAM.FOR"
c
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "PINRP.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
c
c     Passed Variables
c
      integer ihthptr(2*cL02)
      integer krb,krc,nrb            ! Sizes of B & C
      integer ihinge,itype           ! Hinge number & type
      integer iflag                  ! What type of multiplication
      double precision bh_1(3,*)
      double precision B(krb),C(krc)
      double precision pin(3,3,cl02)
      double precision rh(3,3,cl15)
      double precision r0l(3,3)      ! passed because common in BHBSRD
      logical isq                    ! Which type of frame (Q/P)

c
c     local variables
c
      integer i
      integer lr4, lr5, ihptr
      integer nm

      data lr4/BADADDR/
c
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     This section supports a MULT3 call
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      IF (iflag .EQ. 1) THEN   
cccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 1 MULT3 CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccc
         IF (itype .EQ. 1) THEN
c        (3x3) +/- Identity Matrix
            if (isq) then
               do i = 1,3
                  C(i) = B(i)
               end do
            else  ! p-frame
               do i = 1,3
                  C(i) = -B(i)
               end do
            end if
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 2 MULT3 CALL
ccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 2) THEN
c        (3x3) +/- C2inv
            lr4 = 6*(ihinge-2) + 4      
            if (isq) then
CC@CC
CC@CC MULT3 rh(1,1,lr4), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = rh(1,1,lr4)*B(1) + rh(1,2,lr4)*B(2) + rh(1,3,lr4)*B(3)
      C(2) = rh(2,1,lr4)*B(1) + rh(2,2,lr4)*B(2) + rh(2,3,lr4)*B(3)
      C(3) = rh(3,1,lr4)*B(1) + rh(3,2,lr4)*B(2) + rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            else  ! p-frame
CC@CC
CC@CC MULT3N rh(1,1,lr4), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -rh(1,1,lr4)*B(1) - rh(1,2,lr4)*B(2) - rh(1,3,lr4)*B(3)
      C(2) = -rh(2,1,lr4)*B(1) - rh(2,2,lr4)*B(2) - rh(2,3,lr4)*B(3)
      C(3) = -rh(3,1,lr4)*B(1) - rh(3,2,lr4)*B(2) - rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            end if
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 3 MULT3 CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 3) THEN
            if (isq) then
               nm    = nrb - 6
c        multiply the (1,1) section

CC@CC
CC@CC MULT3 BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(2) = BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(3) = BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
C
c     multiply the (1,4) section
CC@CC
CC@CC MULTAD r0l(1,1), B(4), C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + r0l(1,1)*B(4) + r0l(1,2)*B(5) + r0l(1,3)*B(6)
      C(2) = C(2) + r0l(2,1)*B(4) + r0l(2,2)*B(5) + r0l(2,3)*B(6)
      C(3) = C(3) + r0l(3,1)*B(4) + r0l(3,2)*B(5) + r0l(3,3)*B(6)
CC@CC end of expanded call
CC@CC
               if (nm .GT. 0) then
c     multiply the (1,7) section
CC@CC
CC@CC MULTAD BH_1(1,4), B(7), C
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C,1)
CC@CC end of expanded call
CC@CC
               end if
            else  ! p-frame
               do i = 1,3
                  C(i) = -B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 4 MULT3 CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 4) THEN
            if (isq) then
               nm    = nrb - 6
c
c     multiply the (1,1) section
CC@CC
CC@CC MULT3 pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,3,ihing
     .e)*B(3)
      C(2) = pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,3,ihing
     .e)*B(3)
      C(3) = pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,3,ihing
     .e)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the (4,1) section
CC@CC
CC@CC MULT3 BH_1(1,1), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(5) = BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(6) = BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (4,4) section
CC@CC
CC@CC MULTAD r0l(1,1), B(4), C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + r0l(1,1)*B(4) + r0l(1,2)*B(5) + r0l(1,3)*B(6)
      C(5) = C(5) + r0l(2,1)*B(4) + r0l(2,2)*B(5) + r0l(2,3)*B(6)
      C(6) = C(6) + r0l(3,1)*B(4) + r0l(3,2)*B(5) + r0l(3,3)*B(6)
CC@CC end of expanded call
CC@CC
               if (nm .GT. 0) then
c     multiply the (4,7) section
CC@CC
CC@CC MULTAD BH_1(1,4), B(7), C(4)
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
               end if
            else  ! p-frame
               do i = 1,3
                  C(i)   = 0.0d0
                  C(i+3) = -B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 5 MULT3 CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 5) THEN
            if (isq) then
c        whole thing is stored
CC@CC
CC@CC MULT3 BH_1(1,1), B, C
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,1.0d0,BH_1(1,1),3,B,1,0.0d0,C,1)
CC@CC end of expanded call
CC@CC
            else  ! p-frame
               lr4 = 6*(ihinge-2) + 4      
CC@CC
CC@CC MULT3N rh(1,1,lr4), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -rh(1,1,lr4)*B(1) - rh(1,2,lr4)*B(2) - rh(1,3,lr4)*B(3)
      C(2) = -rh(2,1,lr4)*B(1) - rh(2,2,lr4)*B(2) - rh(2,3,lr4)*B(3)
      C(3) = -rh(3,1,lr4)*B(1) - rh(3,2,lr4)*B(2) - rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            end if
c               
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 6 MULT3 CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 6) THEN
            if (isq) then
c     multiply the (1,1) section
CC@CC
CC@CC MULT3 pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,3,ihing
     .e)*B(3)
      C(2) = pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,3,ihing
     .e)*B(3)
      C(3) = pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,3,ihing
     .e)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the bottom
CC@CC
CC@CC MULT3 BH_1(1,1), B, C(4)
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,1.0d0,BH_1(1,1),3,B,1,0.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
            else  ! p-frame
c        multiply the top (0's)
               do i = 1,3
                  C(i) = 0.0d0
               end do
c        multiply the bottom (-C2inv)
               lr4 = 6*(ihinge-2) + 4      
CC@CC
CC@CC MULT3N rh(1,1,lr4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = -rh(1,1,lr4)*B(1) - rh(1,2,lr4)*B(2) - rh(1,3,lr4)*B(3)
      C(5) = -rh(2,1,lr4)*B(1) - rh(2,2,lr4)*B(2) - rh(2,3,lr4)*B(3)
      C(6) = -rh(3,1,lr4)*B(1) - rh(3,2,lr4)*B(2) - rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 7 MULT3 CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 7) THEN
            nm    = nrb - 6
            if (isq) then
               lr5   = 6*(ihinge-2) + 5  
c
c     multiply the (1,1) section
CC@CC
CC@CC MULT3 pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,3,ihing
     .e)*B(3)
      C(2) = pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,3,ihing
     .e)*B(3)
      C(3) = pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,3,ihing
     .e)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the (4,1) section

CC@CC
CC@CC MULT3 BH_1(1,1), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(5) = BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(6) = BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (4,4) section
CC@CC
CC@CC MULTAD rh(1,1,lr5), B(4), C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + rh(1,1,lr5)*B(4) + rh(1,2,lr5)*B(5) + rh(1,3,lr5)*B(
     .6)
      C(5) = C(5) + rh(2,1,lr5)*B(4) + rh(2,2,lr5)*B(5) + rh(2,3,lr5)*B(
     .6)
      C(6) = C(6) + rh(3,1,lr5)*B(4) + rh(3,2,lr5)*B(5) + rh(3,3,lr5)*B(
     .6)
CC@CC end of expanded call
CC@CC
c
               if (nm.GT.0) then
c     multiply the (4,7) section
CC@CC
CC@CC MULTAD BH_1(1,4), B(7), C(4)
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
               end if
c
            else  ! p-frame
c     multiply the (1,1) section

CC@CC
CC@CC MULT3 BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(2) = BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(3) = BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (4,1) section

CC@CC
CC@CC MULT3 BH_1(1,4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = BH_1(1,4)*B(1) + BH_1(1,5)*B(2) + BH_1(1,6)*B(3)
      C(5) = BH_1(2,4)*B(1) + BH_1(2,5)*B(2) + BH_1(2,6)*B(3)
      C(6) = BH_1(3,4)*B(1) + BH_1(3,5)*B(2) + BH_1(3,6)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (4,4) section (-I)
               do i = 4,6
                  C(i) = C(i) - B(i)
               end do
c
               if (nm .GT. 0) then
c     multiply the (4,7) section
                  ihptr = ihthptr(2*ihinge-2) + 1
CC
CC    MULTAD -hats(ihptr), B(7), C(4), 3, nm, 1, 3, krb, krc
CC
      CALL DGEMV ('N',3,nm,-1.0d0,HATS(ihptr),3,B(7),1,1.0d0,C(4),1)
               end if
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 8 MULT3 CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 8) THEN
            if (isq) then
c
c     multiply the (1,1) section
CC@CC
CC@CC MULT3 pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,3,ihing
     .e)*B(3)
      C(2) = pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,3,ihing
     .e)*B(3)
      C(3) = pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,3,ihing
     .e)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the bottom
CC@CC
CC@CC MULT3 BH_1(1,1), B, C(4)
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,1.0d0,BH_1(1,1),3,B,1,0.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
c
            else  ! p-frame
               nm    = nrb - 6
c     multiply the (1,1) section
CC@CC
CC@CC MULT3 BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(2) = BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(3) = BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (4,1) section
CC@CC
CC@CC MULT3 BH_1(1,4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = BH_1(1,4)*B(1) + BH_1(1,5)*B(2) + BH_1(1,6)*B(3)
      C(5) = BH_1(2,4)*B(1) + BH_1(2,5)*B(2) + BH_1(2,6)*B(3)
      C(6) = BH_1(3,4)*B(1) + BH_1(3,5)*B(2) + BH_1(3,6)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (4,4) section (-rh)
               lr4 = 6*(ihinge-2) + 4  
CCC
CCC MULTAD -rh(1,1,lr4), B(4), C(4), 3, 3, 1, 3, krb, krc
CCC
      C(4) = C(4) - rh(1,1,lr4)*B(4) - rh(1,2,lr4)*B(5) - rh(1,3,lr4)*B(
     .6)
      C(5) = C(5) - rh(2,1,lr4)*B(4) - rh(2,2,lr4)*B(5) - rh(2,3,lr4)*B(
     .6)
      C(6) = C(6) - rh(3,1,lr4)*B(4) - rh(3,2,lr4)*B(5) - rh(3,3,lr4)*B(
     .6)
c
              if (nm .GT. 0) then
c     multiply the (4,7) section
CC@CC
CC@CC MULTAD BH_1(1,7), B(7), C(4)
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,7),3,B(7),1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
              end if
           end if
         ELSE   ! Unknown type
            GOTO 999
         END IF
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     This section supports a MULTAD call
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      ELSE IF (iflag .EQ. 0) THEN  ! MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 1 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         IF (itype .EQ. 1) THEN
c        (3x3) +/- Identity matrix
            if (isq) then
               do i = 1,3
                  C(i) = C(i) + B(i)
               end do
            else ! p-frame
               do i = 1,3
                  C(i) = C(i) - B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 2 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 2) THEN
            if (isq) then
c        (3x3) +/- C2inv
               lr4 = 6*(ihinge-2) + 4      
CC@CC
CC@CC MULTAD rh(1,1,lr4), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + rh(1,1,lr4)*B(1) + rh(1,2,lr4)*B(2) + rh(1,3,lr4)*B(
     .3)
      C(2) = C(2) + rh(2,1,lr4)*B(1) + rh(2,2,lr4)*B(2) + rh(2,3,lr4)*B(
     .3)
      C(3) = C(3) + rh(3,1,lr4)*B(1) + rh(3,2,lr4)*B(2) + rh(3,3,lr4)*B(
     .3)
CC@CC end of expanded call
CC@CC
            else  ! p-frame
CCC
CCC MULTAD -rh(1,1,lr4), B, C, 3, 3, 1, 3, krb, krc
CCC
      C(1) = C(1) - rh(1,1,lr4)*B(1) - rh(1,2,lr4)*B(2) - rh(1,3,lr4)*B(
     .3)
      C(2) = C(2) - rh(2,1,lr4)*B(1) - rh(2,2,lr4)*B(2) - rh(2,3,lr4)*B(
     .3)
      C(3) = C(3) - rh(3,1,lr4)*B(1) - rh(3,2,lr4)*B(2) - rh(3,3,lr4)*B(
     .3)
            end if
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 3 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 3) THEN
            if (isq) then
               nm    = nrb - 6
c
c     multiply the (1,1) section
CC@CC
CC@CC MULTAD BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(2) = C(2) + BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(3) = C(3) + BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (1,4) section
CC@CC
CC@CC MULTAD r0l(1,1), B(4), C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + r0l(1,1)*B(4) + r0l(1,2)*B(5) + r0l(1,3)*B(6)
      C(2) = C(2) + r0l(2,1)*B(4) + r0l(2,2)*B(5) + r0l(2,3)*B(6)
      C(3) = C(3) + r0l(3,1)*B(4) + r0l(3,2)*B(5) + r0l(3,3)*B(6)
CC@CC end of expanded call
CC@CC
               if (nm .GT. 0) then
c     multiply the (1,7) section
CC@CC
CC@CC MULTAD BH_1(1,4), B(7), C
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C,1)
CC@CC end of expanded call
CC@CC
               end if
c      
            else  ! p-frame
               do i = 1,3
                  C(i) = C(i) - B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 4 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 4) THEN
            if (isq) then
               nm    = nrb - 6
c     multiply the (1,1) section
CC@CC
CC@CC MULTAD pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,
     .3,ihinge)*B(3)
      C(2) = C(2) + pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,
     .3,ihinge)*B(3)
      C(3) = C(3) + pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,
     .3,ihinge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the (4,1) section
CC@CC
CC@CC MULTAD BH_1(1,1), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(5) = C(5) + BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(6) = C(6) + BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
CCC
c     multiply the (4,4) section
CC@CC
CC@CC MULTAD r0l(1,1), B(4), C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + r0l(1,1)*B(4) + r0l(1,2)*B(5) + r0l(1,3)*B(6)
      C(5) = C(5) + r0l(2,1)*B(4) + r0l(2,2)*B(5) + r0l(2,3)*B(6)
      C(6) = C(6) + r0l(3,1)*B(4) + r0l(3,2)*B(5) + r0l(3,3)*B(6)
CC@CC end of expanded call
CC@CC
               if (nm .GT. 0) then
c     multiply the (4,7) section
CC@CC
CC@CC MULTAD BH_1(1,4), B(7), C(4)
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
               end if
c      
            else  ! p-frame
               do i = 1,3
                  C(i+3) = C(i+3) - B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 5 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 5) THEN
            if (isq) then
c        whole thing is stored
CC@CC
CC@CC MULTAD BH_1(1,1), B, C
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,1.0d0,BH_1(1,1),3,B,1,1.0d0,C,1)
CC@CC end of expanded call
CC@CC
            else  ! p-frame
               lr4 = 6*(ihinge-2) + 4      
CCC
CCC MULTAD -rh(1,1,lr4), B, C, 3, 3, 1, 3, krb, krc
CCC
      C(1) = C(1) - rh(1,1,lr4)*B(1) - rh(1,2,lr4)*B(2) - rh(1,3,lr4)*B(
     .3)
      C(2) = C(2) - rh(2,1,lr4)*B(1) - rh(2,2,lr4)*B(2) - rh(2,3,lr4)*B(
     .3)
      C(3) = C(3) - rh(3,1,lr4)*B(1) - rh(3,2,lr4)*B(2) - rh(3,3,lr4)*B(
     .3)
            end if
c               
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 6 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 6) THEN
            if (isq) then
c     multiply the (1,1) section
CC@CC
CC@CC MULTAD pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,
     .3,ihinge)*B(3)
      C(2) = C(2) + pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,
     .3,ihinge)*B(3)
      C(3) = C(3) + pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,
     .3,ihinge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the bottom
CC@CC
CC@CC MULTAD BH_1(1,1), B, C(4)
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,1.0d0,BH_1(1,1),3,B,1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
            else ! p-frame
               lr4 = 6*(ihinge-2) + 4      
CCC
CCC MULTAD -rh(1,1,lr4), B, C(4), 3, 3, 1, 3, krb, krc
CCC
      C(4) = C(4) - rh(1,1,lr4)*B(1) - rh(1,2,lr4)*B(2) - rh(1,3,lr4)*B(
     .3)
      C(5) = C(5) - rh(2,1,lr4)*B(1) - rh(2,2,lr4)*B(2) - rh(2,3,lr4)*B(
     .3)
      C(6) = C(6) - rh(3,1,lr4)*B(1) - rh(3,2,lr4)*B(2) - rh(3,3,lr4)*B(
     .3)
            end if
c               
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 7 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 7) THEN
            nm    = nrb - 6
            if (isq) then
               lr5   = 6*(ihinge-2) + 5  
c
c     multiply the (1,1) section
CC@CC
CC@CC MULTAD pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,
     .3,ihinge)*B(3)
      C(2) = C(2) + pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,
     .3,ihinge)*B(3)
      C(3) = C(3) + pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,
     .3,ihinge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the (4,1) section
CC@CC
CC@CC MULTAD BH_1(1,1), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(5) = C(5) + BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(6) = C(6) + BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,4) section
CC@CC
CC@CC MULTAD rh(1,1,lr5), B(4), C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + rh(1,1,lr5)*B(4) + rh(1,2,lr5)*B(5) + rh(1,3,lr5)*B(
     .6)
      C(5) = C(5) + rh(2,1,lr5)*B(4) + rh(2,2,lr5)*B(5) + rh(2,3,lr5)*B(
     .6)
      C(6) = C(6) + rh(3,1,lr5)*B(4) + rh(3,2,lr5)*B(5) + rh(3,3,lr5)*B(
     .6)
CC@CC end of expanded call
CC@CC
c
               if (nm.GT.0) then
c     multiply the (4,7) section
CC@CC
CC@CC MULTAD BH_1(1,4), B(7), C(4)
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
               end if
c
            else  ! p-frame
c     multiply the (1,1) section
CC@CC
CC@CC MULTAD BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(2) = C(2) + BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(3) = C(3) + BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,1) section
CC@CC
CC@CC MULTAD BH_1(1,4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + BH_1(1,4)*B(1) + BH_1(1,5)*B(2) + BH_1(1,6)*B(3)
      C(5) = C(5) + BH_1(2,4)*B(1) + BH_1(2,5)*B(2) + BH_1(2,6)*B(3)
      C(6) = C(6) + BH_1(3,4)*B(1) + BH_1(3,5)*B(2) + BH_1(3,6)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,4) section (-I)
               do i = 4,6
                  C(i) = C(i) - B(i)
               end do
c
               if (nm .GT. 0) then
c     multiply the (4,7) section
                  ihptr = ihthptr(2*ihinge-2) + 1
CC
CC    MULTAD -hats(ihptr), B(7), C(4), 3, nm, 1, 3, krb, krc
CC
      CALL DGEMV ('N',3,nm,-1.0d0,HATS(ihptr),3,B(7),1,1.0d0,C(4),1)
CCC
               end if
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 8 MULTAD CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 8) THEN
            if (isq) then
c
c     multiply the (1,1) section
CC@CC
CC@CC MULTAD pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + pin(1,1,ihinge)*B(1) + pin(1,2,ihinge)*B(2) + pin(1,
     .3,ihinge)*B(3)
      C(2) = C(2) + pin(2,1,ihinge)*B(1) + pin(2,2,ihinge)*B(2) + pin(2,
     .3,ihinge)*B(3)
      C(3) = C(3) + pin(3,1,ihinge)*B(1) + pin(3,2,ihinge)*B(2) + pin(3,
     .3,ihinge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the bottom
CC@CC
CC@CC MULTAD BH_1(1,1), B, C(4)
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,1.0d0,BH_1(1,1),3,B,1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
c
            else  ! p-frame
               nm   = nrb - 6
c     multiply the (1,1) section
CC@CC
CC@CC MULTAD BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = C(1) + BH_1(1,1)*B(1) + BH_1(1,2)*B(2) + BH_1(1,3)*B(3)
      C(2) = C(2) + BH_1(2,1)*B(1) + BH_1(2,2)*B(2) + BH_1(2,3)*B(3)
      C(3) = C(3) + BH_1(3,1)*B(1) + BH_1(3,2)*B(2) + BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,1) section
CC@CC
CC@CC MULTAD BH_1(1,4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + BH_1(1,4)*B(1) + BH_1(1,5)*B(2) + BH_1(1,6)*B(3)
      C(5) = C(5) + BH_1(2,4)*B(1) + BH_1(2,5)*B(2) + BH_1(2,6)*B(3)
      C(6) = C(6) + BH_1(3,4)*B(1) + BH_1(3,5)*B(2) + BH_1(3,6)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,4) section (-rh)
               lr4 = 6*(ihinge-2) + 4      
CCC
CCC MULTAD -rh(1,1,lr4), B(4), C(4), 3, 3, 1, 3, krb, krc
CCC
      C(4) = C(4) - rh(1,1,lr4)*B(4) - rh(1,2,lr4)*B(5) - rh(1,3,lr4)*B(
     .6)
      C(5) = C(5) - rh(2,1,lr4)*B(4) - rh(2,2,lr4)*B(5) - rh(2,3,lr4)*B(
     .6)
      C(6) = C(6) - rh(3,1,lr4)*B(4) - rh(3,2,lr4)*B(5) - rh(3,3,lr4)*B(
     .6)
c
              if (nm .GT. 0) then
c     multiply the (4,7) section
CC@CC
CC@CC MULTAD BH_1(1,7), B(7), C(4)
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,BH_1(1,7),3,B(7),1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
              end if
           end if
         ELSE       ! Unrecognized type
            GOTO 999
         END IF
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     This section does a MULT3N call
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      ELSE IF (iflag .EQ. -1) THEN ! MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 1 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         IF (itype .EQ. 1) THEN
c        (3x3) +/- Identity matrix
            if (isq) then
               do i = 1,3
                  C(i) = -B(i)
               end do
            else
               do i = 1,3
                  C(i) =  B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 2 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 2) THEN
c        (3x3) +/- C2inv
            lr4 = 6*(ihinge-2) + 4      
            if (isq .OR. ihinge.EQ.1) then
CC@CC
CC@CC MULT3N rh(1,1,lr4), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -rh(1,1,lr4)*B(1) - rh(1,2,lr4)*B(2) - rh(1,3,lr4)*B(3)
      C(2) = -rh(2,1,lr4)*B(1) - rh(2,2,lr4)*B(2) - rh(2,3,lr4)*B(3)
      C(3) = -rh(3,1,lr4)*B(1) - rh(3,2,lr4)*B(2) - rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            else  ! p-frame
CC@CC
CC@CC MULT3 rh(1,1,lr4), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = rh(1,1,lr4)*B(1) + rh(1,2,lr4)*B(2) + rh(1,3,lr4)*B(3)
      C(2) = rh(2,1,lr4)*B(1) + rh(2,2,lr4)*B(2) + rh(2,3,lr4)*B(3)
      C(3) = rh(3,1,lr4)*B(1) + rh(3,2,lr4)*B(2) + rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            end if
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 3 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 3) THEN
            if (isq) then
               nm    = nrb - 6
c
c     multiply the (1,1) section
CC@CC
CC@CC MULT3N BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -BH_1(1,1)*B(1) - BH_1(1,2)*B(2) - BH_1(1,3)*B(3)
      C(2) = -BH_1(2,1)*B(1) - BH_1(2,2)*B(2) - BH_1(2,3)*B(3)
      C(3) = -BH_1(3,1)*B(1) - BH_1(3,2)*B(2) - BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (1,4) section
CCC
CCC MULTAD -r0l(1,1), B(4), C, 3, 3, 1, 3, krb, krc
CCC
      C(1) = C(1) - r0l(1,1)*B(4) - r0l(1,2)*B(5) - r0l(1,3)*B(6)
      C(2) = C(2) - r0l(2,1)*B(4) - r0l(2,2)*B(5) - r0l(2,3)*B(6)
      C(3) = C(3) - r0l(3,1)*B(4) - r0l(3,2)*B(5) - r0l(3,3)*B(6)
c
               if (nm .GT. 0) then
c     multiply the (1,7) section
CCC
CCC MULTAD -BH_1(1,4), B(7), C, 3, nm, 1 3, krb, krc
CCC
      CALL DGEMV ('N',3,nm,-1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C,1)
CCC
               end if
c      
            else    ! p-frame
               do i = 1,3
                  C(i) = B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 4 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 4) THEN
            if (isq) then
               nm    = nrb - 6
c
c     multiply the (1,1) section
CC@CC
CC@CC MULT3N pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -pin(1,1,ihinge)*B(1) - pin(1,2,ihinge)*B(2) - pin(1,3,ihin
     .ge)*B(3)
      C(2) = -pin(2,1,ihinge)*B(1) - pin(2,2,ihinge)*B(2) - pin(2,3,ihin
     .ge)*B(3)
      C(3) = -pin(3,1,ihinge)*B(1) - pin(3,2,ihinge)*B(2) - pin(3,3,ihin
     .ge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the (4,1) section
CC@CC
CC@CC MULT3N BH_1(1,1), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = -BH_1(1,1)*B(1) - BH_1(1,2)*B(2) - BH_1(1,3)*B(3)
      C(5) = -BH_1(2,1)*B(1) - BH_1(2,2)*B(2) - BH_1(2,3)*B(3)
      C(6) = -BH_1(3,1)*B(1) - BH_1(3,2)*B(2) - BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,4) section
CCC
CCC MULTAD -r0l(1,1), B(4), C(4), 3, 3, 1, 3, krb, krc
CCC
      C(1) = C(1) - r0l(1,1)*B(4) - r0l(1,2)*B(5) - r0l(1,3)*B(6)
      C(2) = C(2) - r0l(2,1)*B(4) - r0l(2,2)*B(5) - r0l(2,3)*B(6)
      C(3) = C(3) - r0l(3,1)*B(4) - r0l(3,2)*B(5) - r0l(3,3)*B(6)
c
               if (nm .GT. 0) then
c     multiply the (4,7) section
CCC
CCC MULTAD -BH_1(1,4), B(7), C(4), 3, nm, 1, 3, krb, krc
CCC
      CALL DGEMV ('N',3,nm,-1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C(4),1)
CCC
               end if
c      
            else  ! p-frame (-I)
               do i = 1,3
                  C(i)   = 0.0d0
                  C(i+3) = B(i)
               end do
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 5 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 5) THEN
            if (isq) then
c        whole thing is stored
CC@CC
CC@CC MULT3N BH_1(1,1), B, C
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,-1.0d0,BH_1(1,1),3,B,1,0.0d0,C,1)
CC@CC end of expanded call
CC@CC
            else ! p-frame
               lr4 = 6*(ihinge-2) + 4      
CC@CC
CC@CC MULT3 rh(1,1,lr4), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = rh(1,1,lr4)*B(1) + rh(1,2,lr4)*B(2) + rh(1,3,lr4)*B(3)
      C(2) = rh(2,1,lr4)*B(1) + rh(2,2,lr4)*B(2) + rh(2,3,lr4)*B(3)
      C(3) = rh(3,1,lr4)*B(1) + rh(3,2,lr4)*B(2) + rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 6 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 6) THEN
            if (isq) then
c     multiply the (1,1) section
CC@CC
CC@CC MULT3N pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -pin(1,1,ihinge)*B(1) - pin(1,2,ihinge)*B(2) - pin(1,3,ihin
     .ge)*B(3)
      C(2) = -pin(2,1,ihinge)*B(1) - pin(2,2,ihinge)*B(2) - pin(2,3,ihin
     .ge)*B(3)
      C(3) = -pin(3,1,ihinge)*B(1) - pin(3,2,ihinge)*B(2) - pin(3,3,ihin
     .ge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the bottom
CC@CC
CC@CC MULT3N BH_1(1,1), B, C(4)
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,-1.0d0,BH_1(1,1),3,B,1,0.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
            else  ! p-frame
               do i = 1,3
                  C(i) = 0.0d0
               end do
               lr4 = 6*(ihinge-2) + 4      
CC@CC
CC@CC MULT3 rh(1,1,lr4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = rh(1,1,lr4)*B(1) + rh(1,2,lr4)*B(2) + rh(1,3,lr4)*B(3)
      C(5) = rh(2,1,lr4)*B(1) + rh(2,2,lr4)*B(2) + rh(2,3,lr4)*B(3)
      C(6) = rh(3,1,lr4)*B(1) + rh(3,2,lr4)*B(2) + rh(3,3,lr4)*B(3)
CC@CC end of expanded call
CC@CC
            end if
c               
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 7 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 7) THEN
            nm    = nrb - 6
            if (isq) then
               lr5   = 6*(ihinge-2) + 5  
c
c     multiply the (1,1) section
CC@CC
CC@CC MULT3N pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -pin(1,1,ihinge)*B(1) - pin(1,2,ihinge)*B(2) - pin(1,3,ihin
     .ge)*B(3)
      C(2) = -pin(2,1,ihinge)*B(1) - pin(2,2,ihinge)*B(2) - pin(2,3,ihin
     .ge)*B(3)
      C(3) = -pin(3,1,ihinge)*B(1) - pin(3,2,ihinge)*B(2) - pin(3,3,ihin
     .ge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the (4,1) section
CC@CC
CC@CC MULT3N BH_1(1,1), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = -BH_1(1,1)*B(1) - BH_1(1,2)*B(2) - BH_1(1,3)*B(3)
      C(5) = -BH_1(2,1)*B(1) - BH_1(2,2)*B(2) - BH_1(2,3)*B(3)
      C(6) = -BH_1(3,1)*B(1) - BH_1(3,2)*B(2) - BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,4) section
CCC
CCC MULTAD -rh(1,1,lr5), B(4), C(4), 3, 3, 1, 3, krb, krc
CCC
      C(4) = C(4) - rh(1,1,lr5)*B(4) - rh(1,2,lr5)*B(5) - rh(1,3,lr5)*B(
     .6)
      C(5) = C(5) - rh(2,1,lr5)*B(4) - rh(2,2,lr5)*B(5) - rh(2,3,lr5)*B(
     .6)
      C(6) = C(6) - rh(3,1,lr5)*B(4) - rh(3,2,lr5)*B(5) - rh(3,3,lr5)*B(
     .6)
c
               if (nm.GT.0) then
c     multiply the (4,7) section
CCC
CCC MULTAD -BH_1(1,4), B(7), C(4), 3, nm, 1, 3, krb, krc
CCC
      CALL DGEMV ('N',3,nm,-1.0d0,BH_1(1,4),3,B(7),1,1.0d0,C(4),1)
CCC
               end if
c
            else  ! p-frame
c     multiply the (1,1) section
CC@CC
CC@CC MULT3N BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -BH_1(1,1)*B(1) - BH_1(1,2)*B(2) - BH_1(1,3)*B(3)
      C(2) = -BH_1(2,1)*B(1) - BH_1(2,2)*B(2) - BH_1(2,3)*B(3)
      C(3) = -BH_1(3,1)*B(1) - BH_1(3,2)*B(2) - BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,1) section
CC@CC
CC@CC MULT3N BH_1(1,4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = -BH_1(1,4)*B(1) - BH_1(1,5)*B(2) - BH_1(1,6)*B(3)
      C(5) = -BH_1(2,4)*B(1) - BH_1(2,5)*B(2) - BH_1(2,6)*B(3)
      C(6) = -BH_1(3,4)*B(1) - BH_1(3,5)*B(2) - BH_1(3,6)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,4) section (-I)
               do i = 4,6
                  C(i) = C(i) + B(i)
               end do
c
               if (nm .GT. 0) then
c     multiply the (4,7) section
               ihptr = ihthptr(2*ihinge-2) + 1
CC@CC
CC@CC MULTAD HATS(ihptr), B(7), C(4)
CC@CC 3, nm, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nm,1.0d0,HATS(ihptr),3,B(7),1,1.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
               end if
            end if
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c     ITYPE 8 MULT3N CALL
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         ELSE IF (itype .EQ. 8) THEN
            if (isq) then
c
c     multiply the (1,1) section
CC@CC
CC@CC MULT3N pin(1,1,ihinge), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -pin(1,1,ihinge)*B(1) - pin(1,2,ihinge)*B(2) - pin(1,3,ihin
     .ge)*B(3)
      C(2) = -pin(2,1,ihinge)*B(1) - pin(2,2,ihinge)*B(2) - pin(2,3,ihin
     .ge)*B(3)
      C(3) = -pin(3,1,ihinge)*B(1) - pin(3,2,ihinge)*B(2) - pin(3,3,ihin
     .ge)*B(3)
CC@CC end of expanded call
CC@CC
c     multiply the bottom
CC@CC
CC@CC MULT3N BH_1(1,1), B, C(4)
CC@CC 3, nrb, 1
CC@CC 3, krb, krc
      CALL DGEMV ('N',3,nrb,-1.0d0,BH_1(1,1),3,B,1,0.0d0,C(4),1)
CC@CC end of expanded call
CC@CC
c
            else   ! p-frame
               nm   = nrb - 6
c     multiply the (1,1) section
CC@CC
CC@CC MULT3N BH_1(1,1), B, C
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(1) = -BH_1(1,1)*B(1) - BH_1(1,2)*B(2) - BH_1(1,3)*B(3)
      C(2) = -BH_1(2,1)*B(1) - BH_1(2,2)*B(2) - BH_1(2,3)*B(3)
      C(3) = -BH_1(3,1)*B(1) - BH_1(3,2)*B(2) - BH_1(3,3)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,1) section
CC@CC
CC@CC MULT3N BH_1(1,4), B, C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = -BH_1(1,4)*B(1) - BH_1(1,5)*B(2) - BH_1(1,6)*B(3)
      C(5) = -BH_1(2,4)*B(1) - BH_1(2,5)*B(2) - BH_1(2,6)*B(3)
      C(6) = -BH_1(3,4)*B(1) - BH_1(3,5)*B(2) - BH_1(3,6)*B(3)
CC@CC end of expanded call
CC@CC
c
c     multiply the (4,4) section (-rh)
               lr4 = 6*(ihinge-2) + 4      
c     multad -(-rh).....
CC@CC
CC@CC MULTAD rh(1,1,lr4), B(4), C(4)
CC@CC 3, 3, 1
CC@CC 3, krb, krc
      C(4) = C(4) + rh(1,1,lr4)*B(4) + rh(1,2,lr4)*B(5) + rh(1,3,lr4)*B(
     .6)
      C(5) = C(5) + rh(2,1,lr4)*B(4) + rh(2,2,lr4)*B(5) + rh(2,3,lr4)*B(
     .6)
      C(6) = C(6) + rh(3,1,lr4)*B(4) + rh(3,2,lr4)*B(5) + rh(3,3,lr4)*B(
     .6)
CC@CC end of expanded call
CC@CC
              if (nm .GT. 0) then
c     multiply the (4,7) section
CCC
CCC MULTAD -BH_1(1,7), B(7), C(4), 3, nm, 1, 3, krb, krc
CCC
      CALL DGEMV ('N',3,nm,-1.0d0,BH_1(1,7),3,B(7),1,1.0d0,C(4),1)
CCC
              end if
           end if
         ELSE     ! Unrecognized type
            GOTO 999
         END IF
      ELSE        ! Unrecognized multiplication choice
         GOTO 999
      END IF

      RETURN
 999  write(MBOUT,*) 'Unrecognized option in BHMULT'
      RETURN
      END      ! End BHMULT_1





