C $Header: /export/cvs/c34a1/source/moldyn/delta_v.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine delta_v(T,Y,P,PMOM,ISCUT,PIN,BETAHD,ITOPOL,IHDATA)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     This subroutine calculates the correction velocity to add
c     to the Body 1 velocity to zero the system momentum.  This
c     velocity is returned to the calling program in the 6x1
c     vector vsys.
c
      IMPLICIT NONE

C     Local variables
C     External variables
c
c     Common blocks
c
#include "PARAM.FOR"
c
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "COMPOS.INC"
#include "CPATHS.INC"
#include "ERRHDL.INC"
#include "HEAPMB.INC"
#include "MOMENG.INC"
#include "NUMBRS.INC"
#include "PINRP.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
#ifdef CHARMM
#include "CHARMM.INC"
#endif
c
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: delta_v.f,v $$Revision: 1.1.1.1 $'/
c
c     Passed Variables
      double precision y(cL07)
      double precision p(cL10),pmom(cL10)
      double precision pin(3,3,cL02)
      double precision betahd(6,cL02)
      integer itopol(2,cl02)
      integer ihdata(7,cl02)
      logical iscut(cL02)
c
      double precision vsys(3), wsys(3)
      double precision tempm(3,3)
      double precision tempv1(3), tempv2(3), tempv3(6)
      double precision z(3), rcond
      double precision t, adt
      integer i, j, ibody, loc, ipvt(3)
      integer lo, le
      integer imb, inb, lp, lq, nmp, nmq
      integer idof1, ndof, ih, ihd, ibtdt
c
c     Calculate the total system mass and inertia matrix
c
      call compin_2
c
c     Calculate energy and momentum values
c
      call engmom(HEAP_MB(iyptr),HEAP_MB(ipptr),HEAP_MB(ipmomptr),
     &            LocalHeap(iwvptr_engmom))
c
c     Determine the linear velocity of the center 
c     of mass of the system from the linear momentum.
c     Expressed in inertial frame.
c
c     vsys = - totl/mass
c
      do i = 1, 3
	 vsys(i) = - totl(i)/amt
      end do
c
c     Store the system inertia matrix and angular momentum
c     vector in temporary locations
c 
      do i = 1, 3
	 tempv1(i) = htot(i)
	 do j = 1, 3
	    tempm(i,j) = ajc(i,j)
         end do
      end do
c
c     Determine the angular velocity for the center of
c     mass of the system from the vector-matrix equation.
c     Expressed in inertial frame.
c              -1
c     w = - [I]  * htot
c  
      call dsico (tempm, 3, 3, ipvt, rcond, z)
      if (1.0d0+rcond .eq. 1.0d0) then
         CALL CLEARERR()
	 WRITE (MBOUT,100) 'System inertia matrix is singular.',
     +                    'rcond = ',rcond
         WRITE(ERRMSG,'(1X,A)') 'Error in subroutine.'
         CALL ERRHDL(-5,'<DELTA_V>')
      end if
 100  FORMAT(A,/,A,E15.8)  
      call dsisl (tempm, 3, 3, ipvt, tempv1)
      do i = 1, 3
	 wsys(i) = - tempv1(i)
      end do
c
c     Loop over all bodies
c
      do 500 ibody = 1, nb
c
c        Define vector from system COM to body COM.
c        Expressed in inertial frame.
c
         do i = 1, 3
  	    tempv1(i) = d0l(i,ibody) - d0c(i)
         end do
c
c        Determine the linear velocity correction.
c        dv = vsys + wsys x p
c
         call vcross (wsys(1),tempv1,tempv2)
         do i = 1, 3
            tempv1(i) = vsys(i) + tempv2(i)
         end do
c
         if (ibody .gt. nbdy) then     ! particle
c
c           Add particle correction to appropriate location of
c           U (y) vector
c  
#ifdef CHARMM
c Only change mobile
c         print*,'delta_v ibody PFix PCvel'
c         print*,ibody,PrtFix(iBody),PrtCvel(iBody)
            if (PrtFix(iBody) .eq. 0 .and. PrtCvel(iBody) .eq. 0) then 
#endif
               loc = locu(ibody) - 1       ! particles
               do i = 1, 3
                  y(loc+i) = y(loc+i) + tempv1(i)
               end do
#ifdef CHARMM
            end if
#endif
c
         else        !  flex/rigid body
c
c           Convert correction of linear velocity to body frame.
CC@CC
CC@CC MULT3T r0l(1,1,ibody), tempv1(1), tempv3(1)
CC@CC 3, 3, 1
CC@CC 3, 3, 6
      tempv3(1) = r0l(1,1,ibody)*tempv1(1) + r0l(2,1,ibody)*tempv1(2) + 
     .r0l(3,1,ibody)*tempv1(3)
      tempv3(2) = r0l(1,2,ibody)*tempv1(1) + r0l(2,2,ibody)*tempv1(2) + 
     .r0l(3,2,ibody)*tempv1(3)
      tempv3(3) = r0l(1,3,ibody)*tempv1(1) + r0l(2,3,ibody)*tempv1(2) + 
     .r0l(3,3,ibody)*tempv1(3)
CC@CC end of expanded call
CC@CC
c
c           Add correction to appropriate location of U (y) vector
c  
            loc = locu(ibody) + 2
            do i = 1, 3
               y(loc+i) = y(loc+i) + tempv3(i)
            end do
c
c           For rigid/flex bodies, correct angular velocity
c
c
c           Express the angular velocity correction in the body frame.
c
CC@CC
CC@CC MULT3T r0l(1,1,ibody), wsys(1), tempv1(1)
CC@CC 3, 3, 1
CC@CC 3, 3, 6
      tempv1(1) = r0l(1,1,ibody)*wsys(1) + r0l(2,1,ibody)*wsys(2) + r0l(
     .3,1,ibody)*wsys(3)
      tempv1(2) = r0l(1,2,ibody)*wsys(1) + r0l(2,2,ibody)*wsys(2) + r0l(
     .3,2,ibody)*wsys(3)
      tempv1(3) = r0l(1,3,ibody)*wsys(1) + r0l(2,3,ibody)*wsys(2) + r0l(
     .3,3,ibody)*wsys(3)
CC@CC end of expanded call
CC@CC
c
c           Add correction to appropriate location of U (y) vector
c  
            loc = locu(ibody) - 1
            do i = 1, 3
               y(loc+i) = y(loc+i) + tempv1(i)
            end do
c
c        End of two sections:  for particles and rigid/flex bodies
c  
        end if
c
c     End loop for all bodies
c
 500  continue
c
c     Transform absolute velocities in body frame system
c     into relative velocities
c     (Note:  the velocity part of cmpic3.f is used here)
c
CCC
CCC MULT3 bh(1,1,1), y(1), betahd(4,1), 3, 3, 1, 6, 3, 6
CCC
      if (nbdy .EQ. 0) then  ! 1st body is a particle
         nmp = 3
c     BH is a 3x3 identity matrix
      betahd(4,1) = y(1)
      betahd(5,1) = y(2)
      betahd(6,1) = y(3)
c
      else
c
         nmp = 6 + IRGFLX(1)
c
c     The BH array for Hinge 1 has no flexible partitions! 
c     (BH is pin in the (1,1) section, and r0l in the (4,4) section)
c
CC@CC
CC@CC MULT3 pin(1,1,1), Y(1), BETAHD(1,1)
CC@CC 3, 3, 1
CC@CC 3, 3, 6
      BETAHD(1,1) = pin(1,1,1)*Y(1) + pin(1,2,1)*Y(2) + pin(1,3,1)*Y(3)
      BETAHD(2,1) = pin(2,1,1)*Y(1) + pin(2,2,1)*Y(2) + pin(2,3,1)*Y(3)
      BETAHD(3,1) = pin(3,1,1)*Y(1) + pin(3,2,1)*Y(2) + pin(3,3,1)*Y(3)
CC@CC end of expanded call
CC@CC
CC@CC
CC@CC MULT3 r0l(1,1,1), Y(4), BETAHD(4,1)
CC@CC 3, 3, 1
CC@CC 3, 3, 6
      BETAHD(4,1) = r0l(1,1,1)*Y(4) + r0l(1,2,1)*Y(5) + r0l(1,3,1)*Y(6)
      BETAHD(5,1) = r0l(2,1,1)*Y(4) + r0l(2,2,1)*Y(5) + r0l(2,3,1)*Y(6)
      BETAHD(6,1) = r0l(3,1,1)*Y(4) + r0l(3,2,1)*Y(5) + r0l(3,3,1)*Y(6)
CC@CC end of expanded call
CC@CC
c
      end if
c
      do ih = 2, nh
         lq = 2*ih - 2
         lp = 2*ih - 1
         inb = ITOPOL(1,ih)
         imb = ITOPOL(2,ih)
         nmq = lenu(inb)
         nmp = lenu(imb)
         if (ihdata(1,ih) .eq. 0) then
            idof1 = 4
            ndof  = 3
         else
            idof1 = 1
            ndof  = 6
         end if
CCC
CCC MULT3 bh(1,1,lp),y(locu(imb)),betahd(idof1,ih),ndof,nmp,1,6,nmp,6
CCC
         CALL BHMULT(.FALSE.,ih,1,y(locu(imb)),betahd(idof1,ih),
     &               nmp,nmp,6,intLoopHeap(iibhtypeptr),
     &               intLoopHeap(iitopolptr),intLoopHeap(iibh_ptrptr))
CCC
CCC MULTAD bh(1,1,lq),y(locu(inb)),betahd(idof1,ih),ndof,nmq,1,6,nmq,6
CCC
         CALL BHMULT(.TRUE.,ih,0,y(locu(inb)),betahd(idof1,ih),
     &               nmq,nmq,6,intLoopHeap(iibhtypeptr),
     &               intLoopHeap(iitopolptr),intLoopHeap(iibh_ptrptr))
c
      end do
c
c        Load the augmented part of the state vector Y with the
C        input beta dots.
c        (Note:  This part is taken from dyns20.f  
c
      ibtdt = locu(2*nb + 3)
      DO 215 J=1,NH
         if (ihdata(1,j) .eq. 0) then
            idof1 = 4
         else
            idof1 = 1
         end if
         if (.not. iscut(j)) then
            DO 214 I = idof1, 6
               ihd = ihdata(i+1,j)
               if (ihd .eq. 0) then
                  y(ibtdt) = betahd(i,j)
                  ibtdt    = ibtdt + 1
               end if
               IF (IHD .EQ. 1) BETAHD(I,J) = ZRO
               IF (IHD .EQ. 2) BETAHD(I,J) = ADT(6*(j-1)+i,T)
 214        continue
         end if
 215  CONTINUE

c
c     code from ydot.f
c  
c     Calculate ordinary momenta,
c     Reference:  Vol. 1, Equation II-49
c
      do 10 ibody = 1, nb
         lo   = locu(ibody)
         le   = lenu(ibody)

CC@CC
CC@CC ...MULT3 HEAP_MB(iamuptr(ibody)), Y(LO), P(LO)
CC@CC ...LE, LE, 1
CC@CC ...LE, LE, LE
      CALL DGEMV ('N',LE,LE,1.0d0,HEAP_MB(iamuptr(ibody)),LE,Y(LO),1,0.0
     .d0,P(LO),1)
CC@CC end of expanded call
CC@CC
c
 10   continue
c
      return
      end
