C $Header: /export/cvs/c34a1/source/moldyn/engmom.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      SUBROUTINE ENGMOM(Y,P,PMOM,WV)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
CSE
C                                                          DEBUG #  58
CPS        PURPOSE:
C
C                   TO COMPUTE LINEAR AND ANGULAR MOMENTUM ALONG WITH
C                   KINETIC AND POTENTIAL ENERGY FOR EACH BODY AND
C                   FOR THE TOTAL SYSTEM
C
CPE
CAS
C
C     SYSTEM AND BODY ENERGY AND MOMENTUM
C----------------------------------------
C
C     THE FOLLOWING PARAMETERS ARE CONTINUALLY MONITORED FOR THE USER
C        TO CHECK TO DETERMINE IF ALL ENERGY AND MOMENTUM CONSERVATION
C        LAWS ARE HOLDING TRUE. THESE PARAMETERS ARE WITH THE EXCEPTION
C        OF ORDINARY MOMENTA COMPUTED IN SUBROUTINE ENGMOM, THEY ARE
C        ALL STORED IN /MOMENG/ AND PRINTED AS A FUNCTION OF TIME
C
C     P      = ORDINARY MOMENTA ARRAY, SEE EQUATION II-49 VOL 1
C              DIMENSIONED (NBMAX*(6+NMDBOD)+NMWMAX)
C        (P(I),I=1,NU)  = ORDINARY MOMENTA ASSOCIATED WITH VELOCITY
C                         COORDINATE I, I=1,...,NU
C     PMOM   = BODY ANGULAR AND LINEAR MOMENTUM ARRAY
C              DIMENSIONED (6*NBMAX)
C        (PMOM(I),I=6*(N-1)+1,6*(N-1)+3) = 3 COMPONENTS OF THE INERTIAL
C                         ANGULAR MOMENTUM VECTOR ASSOCIATED WITH THE
C                         MOTION OF BODY N WRT INERTIAL COORDINATES
C        (PMOM(I),I=6*(N-1)+4,6*(N-1)+6) = 3 COMPONENTS OF THE INERTIAL
C                         LINEAR MOMENTUM VECTOR ASSOCIATED WITH THE
C                         MOTION OF BODY N WRT INERTIAL COORDINATES
C     HTOT   = TOTAL SYSTEM ANGULAR MOMENTUM ARRAY DIMENSIONED (3) THE 3
C              COMPONENTS ARE WRT INERTIAL COORDINATES
C     TOTL   = TOTAL SYSTEM LINEAR MOMENTUM ARRAY DIMENSIONED (3) THE 3
C              COMPONENTS ARE WRT INERTIAL COORDINATES
C     ENGKE  = KINETIC ENERGY ARRAY, DIMENSIONED (NBMAX)
C              ENGKE(N) =KINETIC ENERGY OF BODY N
C     ENGPE  = POTENTIAL ENERGY ARRAY, DIMENSIONED (NBMAX)
C              ENGPE(N) = POTENTIAL ENERGY OF BODY M
C     TOTKE  = TOTAL SYSTEM KINETIC ENERGY
C     TOTPE  = TOTAL SYSTEM POTENTIAL ENERGY
C     TOTENG = TOTKE +TOTPE
C     ATOTL  = TOTAL SYSTEM LINEAR MOMENTUM (VECTOR MAGNITUDE)
C     AHTOT  = TOTAL SYSTEM ANGULAR MOMENTUM (VECTOR MAGNITUDE)
C
CAE
      IMPLICIT NONE
 
C
#include "PARAM.FOR"
C
C        FOR THEORY OF EQUATIONS PROGRAMMED SEE VOL 1 APPENDIX D
c
#include "BHBSRD.INC"
#ifdef CHARMM
#include "CHARMM.INC"
#endif
#include "COMPOS.INC"
#include "HEAPMB.INC"
#include "INTGRL.INC"
#include "MAXMUM.INC"
#include "MOMENG.INC"
#include "MULTIPOLE.INC"
#include "NUMBRS.INC"
#include "SPECIF.INC"
#include "VECTOR.INC"
C
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: engmom.f,v $$Revision: 1.1.1.1 $'/
c
c     Passed variables
c
      DOUBLE PRECISION Y(cL07)
      DOUBLE PRECISION P(cL10),PMOM(cL10)
c
C     Local Variables
C
      INTEGER iloc     ! Local pointer to aintg1 vector
      INTEGER KM,LA,LL,LOL,LOA,LOU,LEU
      INTEGER I1,I2,LO,LE
      INTEGER I,J,N
      DOUBLE PRECISION WV(cL06),SUM
      DOUBLE PRECISION VEC(3)   ! R relative to body
      DOUBLE PRECISION DDOT     ! A Function
C
      KM = NMDBOD
      DO 5 I=1,3
      HTOT(I) = ZRO
    5 TOTL(I) = ZRO
      DO 10 N=1,NB
      LA = 6*N - 5
      LL = LA + 3
      if (n .GT. nbdy) then  ! Its a particle
         lol = locu(n)
         pmom(la)   = ZRO
         pmom(la+1) = ZRO
         pmom(la+2) = ZRO
#ifdef CHARMM
         IF (PrtCvel(n) .eq. 0) THEN ! Ignore dummy atoms
#endif

            pmom(ll)   = p(lol)
            pmom(ll+1) = p(lol+1)
            pmom(ll+2) = p(lol+2)
#ifdef CHARMM
         else
            pmom(ll)   = ZRO
            pmom(ll+1) = ZRO
            pmom(ll+2) = ZRO
            p(lol)   = ZRO
            p(lol+1) = ZRO
            p(lol+2) = ZRO

         ENDIF
#endif

      else                   ! Its a body
         LOA = LOCU(N)
         LOL = LOCU(N) + 3
c
c     Transform P vectors from body frames to inertial frame
c
CC@CC
CC@CC ...MULT3 R0L(1,1,N), P(LOA), PMOM(LA)
CC@CC ...3, 3, 1
CC@CC ...3, 3, 3
      PMOM(LA+0) = R0L(1,1,N)*P(LOA+0) + R0L(1,2,N)*P(LOA+1) + R0L(1,3,N
     .)*P(LOA+2)
      PMOM(LA+1) = R0L(2,1,N)*P(LOA+0) + R0L(2,2,N)*P(LOA+1) + R0L(2,3,N
     .)*P(LOA+2)
      PMOM(LA+2) = R0L(3,1,N)*P(LOA+0) + R0L(3,2,N)*P(LOA+1) + R0L(3,3,N
     .)*P(LOA+2)
CC@CC end of expanded call
CC@CC
CC@CC
CC@CC ...MULT3 R0L(1,1,N), P(LOL), PMOM(LL)
CC@CC ...3, 3, 1
CC@CC ...3, 3, 3
      PMOM(LL+0) = R0L(1,1,N)*P(LOL+0) + R0L(1,2,N)*P(LOL+1) + R0L(1,3,N
     .)*P(LOL+2)
      PMOM(LL+1) = R0L(2,1,N)*P(LOL+0) + R0L(2,2,N)*P(LOL+1) + R0L(2,3,N
     .)*P(LOL+2)
      PMOM(LL+2) = R0L(3,1,N)*P(LOL+0) + R0L(3,2,N)*P(LOL+1) + R0L(3,3,N
     .)*P(LOL+2)
CC@CC end of expanded call
CC@CC
      end if
c
c     Calculate vector from system COM to body reference origin
c
      DO J=1,3
         VEC(J) = D0L(J,N) - D0C(J)
      END DO
c
c     Add the R x P term to get angular momentum about system COM
c
      PMOM(LA  ) = PMOM(LA  )+VEC(2)*PMOM(LL+2)-VEC(3)*PMOM(LL+1)
      PMOM(LA+1) = PMOM(LA+1)+VEC(3)*PMOM(LL  )-VEC(1)*PMOM(LL+2)
      PMOM(LA+2) = PMOM(LA+2)+VEC(1)*PMOM(LL+1)-VEC(2)*PMOM(LL  )
c
      DO 15 I=1,3
      I1 = LA - 1 + I
      I2 = LL - 1 + I
      HTOT(I) = HTOT(I) + PMOM(I1)
   15 TOTL(I) = TOTL(I) + PMOM(I2)
   10 CONTINUE
C
      TOTKE = ZRO
      TOTPE = ZRO
      DO n = 1, nb
         lou = locu(n)
         lo  = locu(n+nb)
         le  = lenu(n+nb)
         leu = lenu(n)
         IF (le .EQ. 0) GO TO 22
         IF (.NOT. internal_atom) THEN
c
c        Add effects of modal stiffness
c
            iloc = ainptr(1)+masptr(6,n)-1
            call multsym (aintgl(iloc),y(lo),ZRO,wv,irgflx(n))
CC@CC
CC@CC ......MULT3 Y(LO), WV, sum
CC@CC ......1, LE, 1
CC@CC ......1, LE, 1
      sum = DDOT (LE,Y(LO),1,WV,1)
CC@CC end of expanded call
CC@CC
         engpm(n) = sum / two
      ELSE
         engpm(n) = ZRO
      END IF
      engpe(n) = engpe(n) + engpm(n)
   22 CONTINUE
CC@CC
CC@CC ...MULT3 Y(LOU), P(LOU), ENGKE(N)
CC@CC ...1, LEU, 1
CC@CC ...1, LEU, 1
      ENGKE(N) = DDOT (LEU,Y(LOU),1,P(LOU),1)
CC@CC end of expanded call
CC@CC
         engke(n) = engke(n)/TWO
         totke = totke + engke(n)
         totpe = totpe + engpe(n)
      END DO
c
#ifdef CHARMM
c     Add up the modal contributions to potential energy
c
      sum = ZRO
      DO n = 1, nb
         IF (irgflx(n) .GT. 0) sum = sum + engpm(n)
      END DO
      rparam(8) = sum
#endif
      toteng = totke + totpe
      atotl = dsqrt(totl(1)**2 + totl(2)**2 + totl(3)**2)
      ahtot = dsqrt(htot(1)**2 + htot(2)**2 + htot(3)**2)
C
      RETURN
      END
