C $Header: /export/cvs/c34a1/source/moldyn/eulseq.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      subroutine eulseq(noh,nbet,y,pimat,pin,rp2,rp3,ibetdt,betah,
     &                  betahd,ihdata)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      IMPLICIT NONE
c
#include "PARAM.FOR"
c
#include "ERRHDL.INC"
#include "MDTYPE.INC"
#include "MISCNO.INC"
#include "NUMBRS.INC"
#include "ORDERN.INC"
#include "PINRP.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "TIMESS.INC"
#include "VECTOR.INC"
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: eulseq.f,v $$Revision: 1.1.1.1 $'/

C     External Variables
C
c     Passed Variables
c
      DOUBLE PRECISION pimat(3,3,cl02)
      DOUBLE PRECISION pin(3,3,cL02)
      DOUBLE PRECISION rp2(3,3,cl02)
      DOUBLE PRECISION rp3(3,3,cl02)
      DOUBLE PRECISION betah(6,cl02)
      DOUBLE PRECISION betahd(6,cl02)
      INTEGER ibetdt(cL02)
      INTEGER ihdata(7,cL02)
c
c     Local Variables
c
      INTEGER I,J
      INTEGER IT,IBTDT,ICOUNT,NOH,NBET
      DOUBLE PRECISION DUM,TIME
      DOUBLE PRECISION ang(3),rw(3,3)
      DOUBLE PRECISION Y(cL07)
C
C     Maximum number of euler sequence changes permitted
      INTEGER MAXCHANGE
      PARAMETER (MAXCHANGE=12)
c
 2010 FORMAT(A,I3,A,I3,A,F10.5)
 2020 FORMAT(A,A)
 2030 FORMAT(A,I3,A,A)
 2040 FORMAT(1X,A,F10.5)
C
C     Compute the current time in picoseconds
C
#ifdef AMBER
      IF (ntu .EQ. 1) THEN
         time = t/20.455D0
      ELSE
         time = t
      END IF
#else
      time = t/20.455D0
#endif
C
c     Check for alterability
c
      if (ihdata(2,noh) .ne. 0 .and. ihdata(4,noh) .ne. 0) then
         WRITE (MBOUT,2010) ' Euler sequence for hinge', noh,
     +        ' is near singular, but cannot be altered'
         WRITE (MBOUT,2010) ' First and third angles are fixed'
         GO TO 999
      end if
c
      if (ihdata(2,noh) .eq. 2 .or. ihdata(4,noh) .eq. 2) then
          WRITE (MBOUT,2030) ' Euler sequence for hinge', noh,
     +        ' is near singular, but cannot be altered',
     +        ' because of rheonomic constraints.'
         GO TO 999
      end if
c
      it = ihdata(1,noh)
c
c        RW transforms from q frame to p frame coordinates
c
      CALL ROTTR(3,IT,BETAH(1,NOH),RW,DUM,DUM)
c
c        PIMAT transforms from Euler rates to body angular velocities
c
      CALL ROTTR (-1,IT,BETAH(1,NOH),PIMAT(1,1,NOH),DUM,DUM)
C
C     Extract the Euler angle rates from the Y vector
C
      ibtdt = ibetdt(noh)
      DO i = 1, 3
         IF (ihdata(i+1,noh) .eq. 0) THEN
            betahd(i,noh) = y(ibtdt)
            ibtdt = ibtdt + 1
         ELSE
            betahd(i,noh) = ZRO
         END IF
      END DO
c
c        Compute relative body angular velocities
c
CC@CC
CC@CC MULT3 pimat(1,1,noh), betahd(1,noh), ang
CC@CC 3, 3, 1
CC@CC 3, 6, 3
      ang(1) = pimat(1,1,noh)*betahd(1,noh) + pimat(1,2,noh)*betahd(2,no
     .h) + pimat(1,3,noh)*betahd(3,noh)
      ang(2) = pimat(2,1,noh)*betahd(1,noh) + pimat(2,2,noh)*betahd(2,no
     .h) + pimat(2,3,noh)*betahd(3,noh)
      ang(3) = pimat(3,1,noh)*betahd(1,noh) + pimat(3,2,noh)*betahd(2,no
     .h) + pimat(3,3,noh)*betahd(3,noh)
CC@CC end of expanded call
CC@CC
c
      icount = 0
      go to 105
c
 100  continue
      WRITE (MBOUT,2010) ' Euler sequence for hinge', noh,
     +     ' changed again from within EULSEQ'
 105  icount = icount + 1
      if (icount .gt. MAXCHANGE) then
         WRITE (MBOUT,2010) ' Euler sequence for hinge', noh,
     +   ' is near singular, but sequence is not successfully changed.'
         go to 999
      end if
c
c     Select new Euler sequence
c
      if (ihdata(2,noh) .eq. 1) then
         if (it.eq.1 .or. it.eq.11) then
            it = 5
         else if (it.eq.2 .or. it.eq.12) then
            it = 7
         else if (it.eq.3 .or. it.eq.5) then
            it = 9
         else if (it.eq.4 .or. it.eq.6) then
            it = 11
         else if (it.eq.7 .or. it.eq.9) then
            it = 1
         else if (it.eq.8 .or. it.eq.10) then
            it = 3
         end if
         ihdata(2,noh) = 0
         ihdata(4,noh) = 1
c
      else if (ihdata(4,noh) .eq. 1) then
         if (it.eq.1 .or. it.eq.2) then
            it = 7
         else if (it.eq.3 .or. it.eq.4) then
            it = 8
         else if (it.eq.5 .or. it.eq.6) then
            it = 11
         else if (it.eq.7 .or. it.eq.8) then
            it = 12
         else if (it.eq.9 .or. it.eq.10) then
            it = 3
         else if (it.eq.11 .or. it.eq.12) then
            it = 4
         end if
         ihdata(2,noh) = 1
         ihdata(4,noh) = 0
c
      else if (ihdata(2,noh) .eq. 0 .and. ihdata(4,noh) .eq. 0) then
         it = mod(it+1,12)
         if (it .eq. 0) it = 12
c
      end if
      ihdata(1,noh) = it
      WRITE (MBOUT,2010) ' Euler sequence for Hinge', noh,
     +     ' changed to Type', it, ' at time = ', time
c
c     Extract new Euler angles
c
      if (it .eq. 1) then         ! 1-2-3
         if (abs(rw(1,3)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,1)**2 + rw(1,2)**2)
         betah(1,noh) = atan2(-rw(2,3), rw(3,3))
         betah(2,noh) = atan2( rw(1,3), dum)
         betah(3,noh) = atan2(-rw(1,2), rw(1,1))
c
      else if (it .eq. 2) then    ! 1-2-1
         if (abs(rw(1,1)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,2)**2 + rw(1,3)**2)
         betah(1,noh) = atan2( rw(2,1),-rw(3,1))
         betah(2,noh) = atan2( dum,     rw(1,1))
         betah(3,noh) = atan2( rw(1,2), rw(1,3))
c
      else if (it .eq. 3) then    ! 1-3-1
         if (abs(rw(1,1)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,2)**2 + rw(1,3)**2)
         betah(1,noh) = atan2( rw(3,1), rw(2,1))
         betah(2,noh) = atan2( dum,     rw(1,1))
         betah(3,noh) = atan2( rw(1,3),-rw(1,2))
c
      else if (it .eq. 4) then    ! 1-3-2
         if (abs(rw(1,2)) .gt. EULER_THRESHOLD) go to 100
         betah(1,noh) = atan2( rw(3,2), rw(2,2))
         dum = DSQRT(rw(1,1)**2 + rw(1,3)**2)
         betah(2,noh) = atan2(-rw(1,2), dum)
         betah(3,noh) = atan2( rw(1,3), rw(1,1))
c
      else if (it .eq. 5) then    ! 2-3-1
         if (abs(rw(2,1)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,1)**2 + rw(3,1)**2)
         betah(1,noh) = atan2(-rw(3,1), rw(1,1))
         betah(2,noh) = atan2( rw(2,1), dum)
         betah(3,noh) = atan2(-rw(2,3), rw(2,2))
c
      else if (it .eq. 6) then    ! 2-3-2
         if (abs(rw(2,2)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,2)**2 + rw(3,2)**2)
         betah(1,noh) = atan2( rw(3,2),-rw(1,2))
         betah(2,noh) = atan2( dum,     rw(2,2))
         betah(3,noh) = atan2( rw(2,3), rw(2,1))
c
      else if (it .eq. 7) then    ! 2-1-2
         if (abs(rw(2,2)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,2)**2 + rw(3,2)**2)
         betah(1,noh) = atan2( rw(1,2), rw(3,2))
         betah(2,noh) = atan2( dum,     rw(2,2))
         betah(3,noh) = atan2( rw(2,1),-rw(2,3))
c
      else if (it .eq. 8) then    ! 2-1-3
         if (abs(rw(2,3)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,3)**2 + rw(3,3)**2)
         betah(1,noh) = atan2( rw(1,3), rw(3,3))
         betah(2,noh) = atan2(-rw(2,3), dum)
         betah(3,noh) = atan2( rw(2,1), rw(2,2))
c
      else if (it .eq. 9) then    ! 3-1-2
         if (abs(rw(3,2)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,2)**2 + rw(2,2)**2)
         betah(1,noh) = atan2(-rw(1,2), rw(2,2))
         betah(2,noh) = atan2( rw(3,2), dum)
         betah(3,noh) = atan2(-rw(3,1), rw(3,3))
c
      else if (it .eq. 10) then   ! 3-1-3
         if (abs(rw(3,3)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,3)**2 + rw(2,3)**2)
         betah(1,noh) = atan2( rw(1,3),-rw(2,3))
         betah(2,noh) = atan2( dum,     rw(3,3))
         betah(3,noh) = atan2( rw(3,1), rw(3,2))
c
      else if (it .eq. 11) then   ! 3-2-3
         if (abs(rw(3,3)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,3)**2 + rw(2,3)**2)
         betah(1,noh) = atan2( rw(2,3), rw(1,3))
         betah(2,noh) = atan2( dum,     rw(3,3))
         betah(3,noh) = atan2( rw(3,2),-rw(3,1))
c
      else if (it .eq. 12) then   ! 3-2-1
         if (abs(rw(3,1)) .gt. EULER_THRESHOLD) go to 100
         dum = DSQRT(rw(1,1)**2 + rw(2,1)**2)
         betah(1,noh) = atan2( rw(2,1), rw(1,1))
         betah(2,noh) = atan2(-rw(3,1), dum)
         betah(3,noh) = atan2( rw(3,2), rw(3,3))
c
      else
         WRITE (MBOUT,2010) ' Error in new Euler type for hinge', noh
         go to 999
      end if
c
c     Transform the relative body angular velocities to the new Euler rates
c
      CALL ROTTR (1,IT,betah(1,noh),PIN(1,1,noh),
     +     RP2(1,1,noh),RP3(1,1,noh))
C
C     Compute the new pimat for use in frwd1a
C
      CALL ROTTR (-1,IT,BETAH(1,NOH),PIMAT(1,1,NOH),DUM,DUM)
CC@CC
CC@CC MULT3 pin(1,1,noh), ang, betahd(1,noh)
CC@CC 3, 3, 1
CC@CC 3, 3, 6
      betahd(1,noh) = pin(1,1,noh)*ang(1) + pin(1,2,noh)*ang(2) + pin(1,
     .3,noh)*ang(3)
      betahd(2,noh) = pin(2,1,noh)*ang(1) + pin(2,2,noh)*ang(2) + pin(2,
     .3,noh)*ang(3)
      betahd(3,noh) = pin(3,1,noh)*ang(1) + pin(3,2,noh)*ang(2) + pin(3,
     .3,noh)*ang(3)
CC@CC end of expanded call
CC@CC
c
c     Load the Euler angles and Euler rates into the Y vector
c
      j = nbet + 1
      ibtdt = ibetdt(noh)
      do i = 1, 3
         if (ihdata(i+1,noh) .eq. 0) then
            y(j)     = betah(i,noh)
            y(ibtdt) = betahd(i,noh)
            j        = j + 1
            ibtdt    = ibtdt + 1
         else
            betahd(i,noh) = 0d0
         end if
      end do
      return
c
 999  continue
      CALL CLEARERR()
      WRITE (ERRMSG,2040) ' Program stopped at time = ', time
      CALL ERRHDL (-5,'<EULSEQ>')
      end
