C $Header: /export/cvs/c34a1/source/moldyn/frwd1a.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      SUBROUTINE FRWD1A(Y,RINV,IHTHPTR,PIMAT,IHPVT,IBETDT,ITOPOL,
     &                  IHDATA,ibh_ptr)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C                                                  DEBUG #  76
CSE
      IMPLICIT NONE
C
CPS   PURPOSE
C
C     THIS SUBROUTINE IS USED TO MAKE THE FIRST FORWARD PASS OF THE
C     ORDER N ALGORITHM.  IT RECURSIVELY COMPUTES THE FIRST 6 ELEMENTS
C     OF EACH U(I) VECTOR AS FUNCTIONS OF THE ABSOLUTE VELOCITIES OF THE
C     INBOARD BODY, RELATIVE HINGE VELOCITIES, AND BODY MODAL AMPLITUDE
C     RATES.  THE RECURSION PROGRESSES FROM THE BASE BODY FORWARD TO THE
C     END BODY OF EACH CHAIN.
C
CPE
CAS                       ARGUMENT LIST
C
C   FRWD1A - First forward sweep of the Order N recursion algorithm.  It
C            computes absolute rates.  These are needed to fill out the
C            system state vector "Y" in such a manner that a total rewrite
C            of old DISCOS can be avoided.  Data for backward and second
C            forward pass is also generated herein and stored in /F1BKF2/
C
C       THE FOLLOWING ARRAY NEEDED FOR FORWARD AND BACKWARD SWEEPING
C       "RINV" IS COMPUTED HEREIN, AND IS STORED IN COMMON BLOCK /F1BKF2/.
C
C     RINV   = Array of 6x6 transformation matrices used to map vectors of
C              length 6 relative coordinates to absolute coodinates, stored
C              in /F1BKF2/, dimensioned (6,6,NHMAX).  The elements of RINV
C              can be analytically determined from matrix Rsub(I,ihinge).
C              Used in the creation of the gamma sub i vector; that is,
C              the first six elements of the state velocity vector Ui.
C    ((RINV(I,J,ihinge),I=1,6),J=1,6)  =  R(inverse)sub(I,ihinge) for
C                                         hinge "ihinge"
c
CAE
C
#include "PARAM.FOR"
C
C     COMMON BLOCKS
C
#include "BHBSRD.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "IFIRST.INC"
#include "LDEBUG.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "TIMESS.INC"
#include "VECTOR.INC"
C
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: frwd1a.f,v $$Revision: 1.1.1.1 $'/

c     Passed variables
c
      DOUBLE PRECISION Y(cL07)
      DOUBLE PRECISION RINV(6,6,cL02)
      DOUBLE PRECISION PIMAT(3,3,cL02)
      INTEGER IHTHPTR(2*cL02)
      INTEGER IHPVT(cL02)
      INTEGER IBETDT(cL02)
      INTEGER ITOPOL(2,cL02)
      INTEGER IHDATA(7,cL02)
      INTEGER ibh_ptr(cL12+1)
c
C     Local Variables
C
      INTEGER i1st
      INTEGER IHINGE,IHINGA
      INTEGER IBODY,ICBODY
      INTEGER IABS
      INTEGER NM
      INTEGER LOCUIC
      INTEGER LP,LQ
      INTEGER IDOF,IHD,LOC
      INTEGER I,J
      INTEGER ibhin,ibhp    ! HEAP_MB bh pointers
      LOGICAL LEQU, qin, useHATS
      DOUBLE PRECISION TVEC2(6)
      DOUBLE PRECISION ADT
C
C     Equivalences
C
      EQUIVALENCE (LEQU,DEBUG(76))
      EQUIVALENCE (i1st,i1pass(3))
C
      IF (LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE FRWD1A ')
c
      if (i1st .eq. 0) then
         i1st = 1
         do ihinge = 1, nh
            do j = 1, 6
               do i = 1, 6
                  rinv(i,j,ihinge) = 0d0
               end do
            end do
         end do
c
         if (nbdy .EQ. 0) then  ! 1st body is a particle
C      -1     
C     R    =  [ I ]
C      I1     
            rinv(1,1,1) = 1D0
            rinv(2,2,1) = 1D0
            rinv(3,3,1) = 1D0
         end if
c
         do ibody = 2, nb
            IHINGE = ITOPW(1,IBODY)
            IHINGA = IABS(IHINGE)
            if (ihinge .lt. 0) then  ! P frame is on outboard body
               if (ihdata(1,ihinga) .eq. 0 .and.
     +             ihpvt(ihinga) .eq. 0) then
c      -1
c     R   = - I    for free bond-length
c      Ii
                  rinv(1,1,ihinga) = - 1d0
                  rinv(2,2,ihinga) = - 1d0
                  rinv(3,3,ihinga) = - 1d0
               else if (itopol(1,ihinga).LE.nbdy .and.
     +                  itopol(2,ihinga).LE.nbdy .and.
     +                  ihpvt(ihinga) .eq. 0) then
                  rinv(4,4,ihinga) = - 1d0
                  rinv(5,5,ihinga) = - 1d0
                  rinv(6,6,ihinga) = - 1d0
               end if
            else                     ! Q frame is on outboard body
               if (ihdata(1,ihinga) .eq. 0 .and.
     +             ihpvt(ihinga) .eq. 0) then
c      -1
c     R   =  I  for free bond-length.
c      Ii
                  rinv(1,1,ihinge) = 1d0
                  rinv(2,2,ihinge) = 1d0
                  rinv(3,3,ihinge) = 1d0
               end if
            end if
         end do
      end if                         ! End of test for I1ST
C
C     Compute R(inverse).  First do the reference body, body 1.
C        Remember the 2,1 and 1,2 blocks are zero.
c
      if (nbdy .GT. 0) then
C
C      -1     PIMAT |        0
C     R    =  ______|__________
C      I1           |        T
C               0   |     R0L
C
         DO 100 J = 1, 3
            DO 200 I = 1, 3
               RINV(I,J,1)     = PIMAT(I,J,1)
               RINV(I+3,J+3,1) = R0L(J,I,1)
 200        CONTINUE
 100     CONTINUE
      end if
C                                                              .  f
C     Determine the location in the state vector of the first beta
C
      LOC = IBETDT(1)
C
C   In the DO 400 store in TVEC2 all of the relative rate terms.
C
      if (nbdy .EQ. 0) then
         do idof = 1, 3
            ihd = ihdata(idof+4,1)
            if (ihd .ne. 0) then
               CALL CLEARERR()
               WRITE(ERRMSG,300) 'Wrong constraints for Hinge 1'
               CALL ERRHDL(-5,'<FRWD1A>') 
            end if
            tvec2(idof) = y(loc)
            loc = loc + 1
         end do
      else
         DO 400 IDOF = 1, 6
            IHD = IHDATA(IDOF+1,1)
            IF (IHD .EQ. 0) THEN
               TVEC2(IDOF) = Y(LOC)
               LOC = LOC + 1
            ELSE IF (IHD .EQ. 1) THEN
               TVEC2(IDOF) = 0.0
            ELSE IF (IHD .EQ. 2) THEN
               TVEC2(IDOF) = ADT(IDOF,T)
            END IF
 400     CONTINUE
      end if
 300  FORMAT(1X,A)
c                            -1             .
c     Multiply by pieces of R   to create gamma, first six elements of U
c       .                    I1
c     Gamma elements are loaded directly into the state vector Y
c
c       .          -1         .  f    ___    .          .
c     Gamma    =  R   ( PHI  Beta   + PHI  Alpha  -  F  Xi  )
c          1       I1      I1    I1      I1     I1    I1  1
c
      if (nbdy .EQ. 0) then
         y(1) = tvec2(1)
         y(2) = tvec2(2)
         y(3) = tvec2(3)
      else
CC@CC
CC@CC ...MULT3 PIMAT(1,1,1), TVEC2, Y
CC@CC ...3, 3, 1
CC@CC ...3, 3, 3
      Y(1) = PIMAT(1,1,1)*TVEC2(1) + PIMAT(1,2,1)*TVEC2(2) + PIMAT(1,3,1
     .)*TVEC2(3)
      Y(2) = PIMAT(2,1,1)*TVEC2(1) + PIMAT(2,2,1)*TVEC2(2) + PIMAT(2,3,1
     .)*TVEC2(3)
      Y(3) = PIMAT(3,1,1)*TVEC2(1) + PIMAT(3,2,1)*TVEC2(2) + PIMAT(3,3,1
     .)*TVEC2(3)
CC@CC end of expanded call
CC@CC
CC@CC
CC@CC ...MULT3T R0L(1,1,1), TVEC2(4), Y(4)
CC@CC ...3, 3, 1
CC@CC ...3, 3, 3
      Y(4) = R0L(1,1,1)*TVEC2(4) + R0L(2,1,1)*TVEC2(5) + R0L(3,1,1)*TVEC
     .2(6)
      Y(5) = R0L(1,2,1)*TVEC2(4) + R0L(2,2,1)*TVEC2(5) + R0L(3,2,1)*TVEC
     .2(6)
      Y(6) = R0L(1,3,1)*TVEC2(4) + R0L(2,3,1)*TVEC2(5) + R0L(3,3,1)*TVEC
     .2(6)
CC@CC end of expanded call
CC@CC
      end if
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c          Finished with hinge 1.
c
c-----------------------------------------------------------------------
c
c          Proceed outward along all forward
c          paths defined in the ITOPW array.  Local variable IBODY
c          in the 600 loop is a sequence locator.  Body number and
c          hinge numbers are taken for arrays ITOPW and ITOPOL
c
      DO 600 IBODY = 2, NB
C
         IHINGE = ITOPW(1,IBODY)
         ICBODY = ITOPW(2,IBODY)
         NM     = IRGFLX(ICBODY)
C
         if (ihinge .LT. 0) then  ! q-frame in, p-frame out
            ihinge = -ihinge
            qin = .TRUE.
         else                     ! p-frame in, q-frame out
            qin = .FALSE.
         end if
               
         lq   = 2*ihinge - 2    ! q-frame
         lp   = lq + 1          ! p-frame
         ibhp = ibh_ptr(lp)     ! pointer to bh p-frame
         useHATS = .FALSE.
c
c        get pointer to mode section of BH
c
         if (nm .GT. 0) then
            if (qin) then
               if (ihpvt(ihinge) .EQ. 0) then
c              BH (4,7) = - hats
                  ibhin = ihthptr(lq) + 1
                useHATS = .TRUE.  
               else
                  ibhin = ibh_ptr(lp) + 18
               end if
            else
               if (ihpvt(ihinge) .EQ. 0) then
                  ibhin = ibh_ptr(lq) + 9
               else
                  ibhin = ibh_ptr(lq) + 18
               end if
            end if
         else
            ibhin = BADADDR
         end if
c
c     call frwd1a_1 in the body loop to pass bh (on heap_mb) as an array
c
         if (useHATS) then
           call frwd1a_1(BH(ibhp),HATS(ibhin),ihinge,icbody,
     &                 nm,lp,lq,qin,HEAP_MB(iyptr),
     &                 LoopHeap(irinvptr),LoopHeap(ipimatptr),
     &                 intLoopHeap(iihpvtptr),intLoopheap(
     &                 iibetdtptr),intloopheap(iitopolptr),
     &                 intloopheap(iihdataptr),Loopheap(irhptr),
     &                 Loopheap(idhptr))
         else
           call frwd1a_1(Bh(ibhp),BH(ibhin),ihinge,icbody,
     &                 nm,lp,lq,qin,HEAP_MB(iyptr),
     &                 LoopHeap(irinvptr),LoopHeap(ipimatptr),
     &                 intLoopHeap(iihpvtptr),intLoopheap(
     &                 iibetdtptr),intLoopheap(iitopolptr),
     &                 intloopheap(iihdataptr),Loopheap(irhptr),
     &                 Loopheap(idhptr))
         end if 
 600  CONTINUE
c
      IF (LEQU) THEN
         DO IBODY=1,NBDY
            LOCUIC=LOCU(IBODY) 
            J=ABS(ITOPW(1,IBODY))
            WRITE (MBOUT,1020) J,(Y(LOCUIC-1+I),I=1,6)
 1020       FORMAT (1X,' GAMMADOT(',I3,') = ',1P6D11.3)
         END DO
         DO IBODY=1,NB
            J=ABS(ITOPW(1,IBODY))
            WRITE (MBOUT,1030) J
 1030       FORMAT (1X,' RINV ARRAY FOR HINGE ',I3)
            CALL WRITES(RINV(1,1,J),6,6,6)
         END DO
      END IF
C
      RETURN
      END
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C
C     This is the beginning of frwd1a_1
C
Ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
      subroutine frwd1a_1(bhp,bh17,ihinge,icbody,nm,lp,lq,qin,y,rinv,
     *                    pimat,ihpvt,ibetdt,itopol,ihdata,rh,dh)
C
#include "PARAM.FOR"
C
C     COMMON BLOCKS
C
#include "BHBSRD.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "IFIRST.INC"
#include "LDEBUG.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "TIMESS.INC"
#include "VECTOR.INC"
C
C     Passed Variables
C
      INTEGER ihinge,nm,lp,lq,icbody
      LOGICAL qin
      DOUBLE PRECISION bhp(3,*)   ! p-frame
      DOUBLE PRECISION bh17(3,*)  ! mode section
      DOUBLE PRECISION y(cL07)
      DOUBLE PRECISION rinv(6,6,cl02)
      DOUBLE PRECISION pimat(3,3,cl02)
      DOUBLE PRECISION rh(3,3,cl15)
      DOUBLE PRECISION dh(3,cl16)
      INTEGER ihpvt(cL02)
      INTEGER ibetdt(cL02)
      INTEGER itopol(2,cL02)
      INTEGER ihdata(7,cL02)
C
C     Local Variables
C
      INTEGER i1st
      INTEGER INBODY
      INTEGER ILEN,LEN
      INTEGER N,NMI,N0
      INTEGER LOCUIN,LOCUIC
      INTEGER LR1,LR2,LR5,LD3
      INTEGER IDOF,IHD,LOC,NDOF
      INTEGER I,J
      LOGICAL LEQU
      DOUBLE PRECISION TVEC2(6)
      DOUBLE PRECISION TEMP3(3,3)
      DOUBLE PRECISION TVEC1(6)
      DOUBLE PRECISION ADT
C
C     Equivalences
C
      EQUIVALENCE (LEQU,DEBUG(76))
      EQUIVALENCE (i1st,i1pass(3))
C
      IF (LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE FRWD1A ')
c

         len    = lenu(icbody)
C
         LR1 = 6 * (IHINGE - 2) + 1
         LR2 = LR1 + 1
         LR5 = LR1 + 4
         ld3 = 7 * (ihinge - 2) + 3
c
c-----------------------------------------------------------------------
c
C                          -1
C    Begin computation of R     Check if inboard hinge frame is pframe
C    or qframe.            Ii
C
         IF (qin) THEN
c           inboard hinge frame is a q-frame, p-frame is on outboard body
C
            INBODY = ITOPOL(1,IHINGE)
C
            if (ihdata(1,ihinge) .eq. 0 .and. ihpvt(ihinge) .ne. 0) then
c
c     Both objects are particles, or the particle is outboard of the body.
c
c      -1
c     R   = - C    for fixed bond-length
c      Ii      2
c
               ndof = 3
               rinv(1,1,ihinge) = - rh(1,1,lr2)
               rinv(2,1,ihinge) = - rh(2,1,lr2)
               rinv(3,1,ihinge) = - rh(3,1,lr2)
               rinv(1,2,ihinge) = - rh(1,2,lr2)
               rinv(2,2,ihinge) = - rh(2,2,lr2)
               rinv(3,2,ihinge) = - rh(3,2,lr2)
               rinv(1,3,ihinge) = - rh(1,3,lr2)
               rinv(2,3,ihinge) = - rh(2,3,lr2)
               rinv(3,3,ihinge) = - rh(3,3,lr2)
c
            else if (ihdata(1,ihinge) .eq. 0 .and.
     +               ihpvt(ihinge) .eq. 0) then
               ndof = 3
c
            else if (inbody.LE.nbdy .and. icbody.LE.nbdy) then
c
c     Both objects are rigid/flexible bodies.
c
c                    -1     - pRq pi                    |   0
c           compute R    = _____________________________|______
c                    Ii           (m)   (p)             |
c                           (pRm Smp + Spq  pRm) pRq pi | - I or - C2
C
               ndof = 6
CC@CC
CC@CC .........MULT3N rh(1,1,lr5), pimat(1,1,ihinge), rinv(1,1,ihinge)
CC@CC .........3, 3, 3
CC@CC .........3, 3, 6
      rinv(1,1,ihinge) = -rh(1,1,lr5)*pimat(1,1,ihinge) - rh(1,2,lr5)*pi
     .mat(2,1,ihinge) - rh(1,3,lr5)*pimat(3,1,ihinge)
      rinv(2,1,ihinge) = -rh(2,1,lr5)*pimat(1,1,ihinge) - rh(2,2,lr5)*pi
     .mat(2,1,ihinge) - rh(2,3,lr5)*pimat(3,1,ihinge)
      rinv(3,1,ihinge) = -rh(3,1,lr5)*pimat(1,1,ihinge) - rh(3,2,lr5)*pi
     .mat(2,1,ihinge) - rh(3,3,lr5)*pimat(3,1,ihinge)
      rinv(1,2,ihinge) = -rh(1,1,lr5)*pimat(1,2,ihinge) - rh(1,2,lr5)*pi
     .mat(2,2,ihinge) - rh(1,3,lr5)*pimat(3,2,ihinge)
      rinv(2,2,ihinge) = -rh(2,1,lr5)*pimat(1,2,ihinge) - rh(2,2,lr5)*pi
     .mat(2,2,ihinge) - rh(2,3,lr5)*pimat(3,2,ihinge)
      rinv(3,2,ihinge) = -rh(3,1,lr5)*pimat(1,2,ihinge) - rh(3,2,lr5)*pi
     .mat(2,2,ihinge) - rh(3,3,lr5)*pimat(3,2,ihinge)
      rinv(1,3,ihinge) = -rh(1,1,lr5)*pimat(1,3,ihinge) - rh(1,2,lr5)*pi
     .mat(2,3,ihinge) - rh(1,3,lr5)*pimat(3,3,ihinge)
      rinv(2,3,ihinge) = -rh(2,1,lr5)*pimat(1,3,ihinge) - rh(2,2,lr5)*pi
     .mat(2,3,ihinge) - rh(2,3,lr5)*pimat(3,3,ihinge)
      rinv(3,3,ihinge) = -rh(3,1,lr5)*pimat(1,3,ihinge) - rh(3,2,lr5)*pi
     .mat(2,3,ihinge) - rh(3,3,lr5)*pimat(3,3,ihinge)
CC@CC end of expanded call
CC@CC
               IF (ihpvt(ihinge) .EQ. 0) THEN
C     originally:  bh(4,1,ithisb)   p-frame is outboard body
CC@CC
CC@CC MULT3 bhp(1,4), rinv(1,1,ihinge), rinv(4,1,ihinge)
CC@CC 3, 3, 3
CC@CC 3, 6, 6
      rinv(4,1,ihinge) = bhp(1,4)*rinv(1,1,ihinge) + bhp(1,5)*rinv(2,1,i
     .hinge) + bhp(1,6)*rinv(3,1,ihinge)
      rinv(5,1,ihinge) = bhp(2,4)*rinv(1,1,ihinge) + bhp(2,5)*rinv(2,1,i
     .hinge) + bhp(2,6)*rinv(3,1,ihinge)
      rinv(6,1,ihinge) = bhp(3,4)*rinv(1,1,ihinge) + bhp(3,5)*rinv(2,1,i
     .hinge) + bhp(3,6)*rinv(3,1,ihinge)
      rinv(4,2,ihinge) = bhp(1,4)*rinv(1,2,ihinge) + bhp(1,5)*rinv(2,2,i
     .hinge) + bhp(1,6)*rinv(3,2,ihinge)
      rinv(5,2,ihinge) = bhp(2,4)*rinv(1,2,ihinge) + bhp(2,5)*rinv(2,2,i
     .hinge) + bhp(2,6)*rinv(3,2,ihinge)
      rinv(6,2,ihinge) = bhp(3,4)*rinv(1,2,ihinge) + bhp(3,5)*rinv(2,2,i
     .hinge) + bhp(3,6)*rinv(3,2,ihinge)
      rinv(4,3,ihinge) = bhp(1,4)*rinv(1,3,ihinge) + bhp(1,5)*rinv(2,3,i
     .hinge) + bhp(1,6)*rinv(3,3,ihinge)
      rinv(5,3,ihinge) = bhp(2,4)*rinv(1,3,ihinge) + bhp(2,5)*rinv(2,3,i
     .hinge) + bhp(2,6)*rinv(3,3,ihinge)
      rinv(6,3,ihinge) = bhp(3,4)*rinv(1,3,ihinge) + bhp(3,5)*rinv(2,3,i
     .hinge) + bhp(3,6)*rinv(3,3,ihinge)
CC@CC end of expanded call
CC@CC
C
               ELSE
C     originally:  bh(4,1,ithisb)  
CC@CC
CC@CC MULT3 bhp(1,4), rinv(1,1,ihinge), temp3
CC@CC 3, 3, 3
CC@CC 3, 6, 3
      temp3(1,1) = bhp(1,4)*rinv(1,1,ihinge) + bhp(1,5)*rinv(2,1,ihinge)
     . + bhp(1,6)*rinv(3,1,ihinge)
      temp3(2,1) = bhp(2,4)*rinv(1,1,ihinge) + bhp(2,5)*rinv(2,1,ihinge)
     . + bhp(2,6)*rinv(3,1,ihinge)
      temp3(3,1) = bhp(3,4)*rinv(1,1,ihinge) + bhp(3,5)*rinv(2,1,ihinge)
     . + bhp(3,6)*rinv(3,1,ihinge)
      temp3(1,2) = bhp(1,4)*rinv(1,2,ihinge) + bhp(1,5)*rinv(2,2,ihinge)
     . + bhp(1,6)*rinv(3,2,ihinge)
      temp3(2,2) = bhp(2,4)*rinv(1,2,ihinge) + bhp(2,5)*rinv(2,2,ihinge)
     . + bhp(2,6)*rinv(3,2,ihinge)
      temp3(3,2) = bhp(3,4)*rinv(1,2,ihinge) + bhp(3,5)*rinv(2,2,ihinge)
     . + bhp(3,6)*rinv(3,2,ihinge)
      temp3(1,3) = bhp(1,4)*rinv(1,3,ihinge) + bhp(1,5)*rinv(2,3,ihinge)
     . + bhp(1,6)*rinv(3,3,ihinge)
      temp3(2,3) = bhp(2,4)*rinv(1,3,ihinge) + bhp(2,5)*rinv(2,3,ihinge)
     . + bhp(2,6)*rinv(3,3,ihinge)
      temp3(3,3) = bhp(3,4)*rinv(1,3,ihinge) + bhp(3,5)*rinv(2,3,ihinge)
     . + bhp(3,6)*rinv(3,3,ihinge)
CC@CC end of expanded call
CC@CC
C
CC@CC
CC@CC ............MULT3 rh(1,1,lr2), temp3, rinv(4,1,ihinge)
CC@CC ............3, 3, 3
CC@CC ............3, 3, 6
      rinv(4,1,ihinge) = rh(1,1,lr2)*temp3(1,1) + rh(1,2,lr2)*temp3(2,1)
     . + rh(1,3,lr2)*temp3(3,1)
      rinv(5,1,ihinge) = rh(2,1,lr2)*temp3(1,1) + rh(2,2,lr2)*temp3(2,1)
     . + rh(2,3,lr2)*temp3(3,1)
      rinv(6,1,ihinge) = rh(3,1,lr2)*temp3(1,1) + rh(3,2,lr2)*temp3(2,1)
     . + rh(3,3,lr2)*temp3(3,1)
      rinv(4,2,ihinge) = rh(1,1,lr2)*temp3(1,2) + rh(1,2,lr2)*temp3(2,2)
     . + rh(1,3,lr2)*temp3(3,2)
      rinv(5,2,ihinge) = rh(2,1,lr2)*temp3(1,2) + rh(2,2,lr2)*temp3(2,2)
     . + rh(2,3,lr2)*temp3(3,2)
      rinv(6,2,ihinge) = rh(3,1,lr2)*temp3(1,2) + rh(3,2,lr2)*temp3(2,2)
     . + rh(3,3,lr2)*temp3(3,2)
      rinv(4,3,ihinge) = rh(1,1,lr2)*temp3(1,3) + rh(1,2,lr2)*temp3(2,3)
     . + rh(1,3,lr2)*temp3(3,3)
      rinv(5,3,ihinge) = rh(2,1,lr2)*temp3(1,3) + rh(2,2,lr2)*temp3(2,3)
     . + rh(2,3,lr2)*temp3(3,3)
      rinv(6,3,ihinge) = rh(3,1,lr2)*temp3(1,3) + rh(3,2,lr2)*temp3(2,3)
     . + rh(3,3,lr2)*temp3(3,3)
CC@CC end of expanded call
CC@CC
C
                  rinv(4,4,ihinge) = - rh(1,1,lr2)
                  rinv(5,4,ihinge) = - rh(2,1,lr2)
                  rinv(6,4,ihinge) = - rh(3,1,lr2)
                  rinv(4,5,ihinge) = - rh(1,2,lr2)
                  rinv(5,5,ihinge) = - rh(2,2,lr2)
                  rinv(6,5,ihinge) = - rh(3,2,lr2)
                  rinv(4,6,ihinge) = - rh(1,3,lr2)
                  rinv(5,6,ihinge) = - rh(2,3,lr2)
                  rinv(6,6,ihinge) = - rh(3,3,lr2)
               END IF
            END IF     ! End of tests for kinematic conditions
C
         ELSE          ! IHINGE > 0 (q-frame outboard)
c
c           inboard hinge frame is a p-frame, q frame is on outboard body
c           computation logic same as above
c        now do the same stuff for q frame on this body
c
            INBODY = ITOPOL(2,IHINGE)
c
            if (icbody.GT.nbdy .and. ihpvt(ihinge) .ne. 0) then
c                        Both objects are particles, and bond length is fixed
c      -1
c     R   =  C    for fixed bond-length.
c      Ii     2
c
               ndof = 3
               rinv(1,1,ihinge) = rh(1,1,lr2)
               rinv(2,1,ihinge) = rh(2,1,lr2)
               rinv(3,1,ihinge) = rh(3,1,lr2)
               rinv(1,2,ihinge) = rh(1,2,lr2)
               rinv(2,2,ihinge) = rh(2,2,lr2)
               rinv(3,2,ihinge) = rh(3,2,lr2)
               rinv(1,3,ihinge) = rh(1,3,lr2)
               rinv(2,3,ihinge) = rh(2,3,lr2)
               rinv(3,3,ihinge) = rh(3,3,lr2)
            else if (inbody.GT.nbdy .and. ihdata(1,ihinge) .ne. 0) then
c
c                   outboard object is a body ==> 6 DOF hinge
               ndof = 6
               rinv(1,1,ihinge) = pimat(1,1,ihinge)
               rinv(2,1,ihinge) = pimat(2,1,ihinge)
               rinv(3,1,ihinge) = pimat(3,1,ihinge)
               rinv(1,2,ihinge) = pimat(1,2,ihinge)
               rinv(2,2,ihinge) = pimat(2,2,ihinge)
               rinv(3,2,ihinge) = pimat(3,2,ihinge)
               rinv(1,3,ihinge) = pimat(1,3,ihinge)
               rinv(2,3,ihinge) = pimat(2,3,ihinge)
               rinv(3,3,ihinge) = pimat(3,3,ihinge)
               call skewv3(dh(1,ld3),temp3,1,3)
CC@CC
CC@CC .........MULT3N temp3, pimat(1,1,ihinge), rinv(4,1,ihinge)
CC@CC .........3, 3, 3
CC@CC .........3, 3, 6
      rinv(4,1,ihinge) = -temp3(1,1)*pimat(1,1,ihinge) - temp3(1,2)*pima
     .t(2,1,ihinge) - temp3(1,3)*pimat(3,1,ihinge)
      rinv(5,1,ihinge) = -temp3(2,1)*pimat(1,1,ihinge) - temp3(2,2)*pima
     .t(2,1,ihinge) - temp3(2,3)*pimat(3,1,ihinge)
      rinv(6,1,ihinge) = -temp3(3,1)*pimat(1,1,ihinge) - temp3(3,2)*pima
     .t(2,1,ihinge) - temp3(3,3)*pimat(3,1,ihinge)
      rinv(4,2,ihinge) = -temp3(1,1)*pimat(1,2,ihinge) - temp3(1,2)*pima
     .t(2,2,ihinge) - temp3(1,3)*pimat(3,2,ihinge)
      rinv(5,2,ihinge) = -temp3(2,1)*pimat(1,2,ihinge) - temp3(2,2)*pima
     .t(2,2,ihinge) - temp3(2,3)*pimat(3,2,ihinge)
      rinv(6,2,ihinge) = -temp3(3,1)*pimat(1,2,ihinge) - temp3(3,2)*pima
     .t(2,2,ihinge) - temp3(3,3)*pimat(3,2,ihinge)
      rinv(4,3,ihinge) = -temp3(1,1)*pimat(1,3,ihinge) - temp3(1,2)*pima
     .t(2,3,ihinge) - temp3(1,3)*pimat(3,3,ihinge)
      rinv(5,3,ihinge) = -temp3(2,1)*pimat(1,3,ihinge) - temp3(2,2)*pima
     .t(2,3,ihinge) - temp3(2,3)*pimat(3,3,ihinge)
      rinv(6,3,ihinge) = -temp3(3,1)*pimat(1,3,ihinge) - temp3(3,2)*pima
     .t(2,3,ihinge) - temp3(3,3)*pimat(3,3,ihinge)
CC@CC end of expanded call
CC@CC
               if (ihpvt(ihinge) .ne. 0) then
c
c      -1       pi    |      0
c     R   =    ---------------------
c      Ii      -S pi  |  R0L**T  C2
c                  
CC@CC
CC@CC ............MULT3T r0l(1,1,icbody), rh(1,1,lr2), rinv(4,4,ihinge)
CC@CC ............3, 3, 3
CC@CC ............3, 3, 6
      rinv(4,4,ihinge) = r0l(1,1,icbody)*rh(1,1,lr2) + r0l(2,1,icbody)*r
     .h(2,1,lr2) + r0l(3,1,icbody)*rh(3,1,lr2)
      rinv(5,4,ihinge) = r0l(1,2,icbody)*rh(1,1,lr2) + r0l(2,2,icbody)*r
     .h(2,1,lr2) + r0l(3,2,icbody)*rh(3,1,lr2)
      rinv(6,4,ihinge) = r0l(1,3,icbody)*rh(1,1,lr2) + r0l(2,3,icbody)*r
     .h(2,1,lr2) + r0l(3,3,icbody)*rh(3,1,lr2)
      rinv(4,5,ihinge) = r0l(1,1,icbody)*rh(1,2,lr2) + r0l(2,1,icbody)*r
     .h(2,2,lr2) + r0l(3,1,icbody)*rh(3,2,lr2)
      rinv(5,5,ihinge) = r0l(1,2,icbody)*rh(1,2,lr2) + r0l(2,2,icbody)*r
     .h(2,2,lr2) + r0l(3,2,icbody)*rh(3,2,lr2)
      rinv(6,5,ihinge) = r0l(1,3,icbody)*rh(1,2,lr2) + r0l(2,3,icbody)*r
     .h(2,2,lr2) + r0l(3,3,icbody)*rh(3,2,lr2)
      rinv(4,6,ihinge) = r0l(1,1,icbody)*rh(1,3,lr2) + r0l(2,1,icbody)*r
     .h(2,3,lr2) + r0l(3,1,icbody)*rh(3,3,lr2)
      rinv(5,6,ihinge) = r0l(1,2,icbody)*rh(1,3,lr2) + r0l(2,2,icbody)*r
     .h(2,3,lr2) + r0l(3,2,icbody)*rh(3,3,lr2)
      rinv(6,6,ihinge) = r0l(1,3,icbody)*rh(1,3,lr2) + r0l(2,3,icbody)*r
     .h(2,3,lr2) + r0l(3,3,icbody)*rh(3,3,lr2)
CC@CC end of expanded call
CC@CC
               else
c
c      -1       pi    |    0
c     R   =    -----------------
c      Ii      -S pi  |  R0L**T
c                  
                  rinv(4,4,ihinge) = r0l(1,1,icbody)
                  rinv(5,4,ihinge) = r0l(1,2,icbody)
                  rinv(6,4,ihinge) = r0l(1,3,icbody)
                  rinv(4,5,ihinge) = r0l(2,1,icbody)
                  rinv(5,5,ihinge) = r0l(2,2,icbody)
                  rinv(6,5,ihinge) = r0l(2,3,icbody)
                  rinv(4,6,ihinge) = r0l(3,1,icbody)
                  rinv(5,6,ihinge) = r0l(3,2,icbody)
                  rinv(6,6,ihinge) = r0l(3,3,icbody)
               end if      ! End of test for fixed or free bonds of
                           ! 6 DOF particle hinge
c
            else if (inbody.LE.nbdy .and. icbody.LE.nbdy) then
c
c     Both objects are rigid/flexible bodies.
c
c                    -1         pi     |    0
c           compute R    =  ___________|________________
c                    Ii       (n)      |
c                           -Snq   pi  |   qRp  or qRp C2
c
C
               ndof = 6
               CALL SKEWV3 (DH(1,LD3),TEMP3,1,3)
CC@CC
CC@CC .........MULT3N temp3, pimat(1,1,ihinge), rinv(4,1,ihinge)
CC@CC .........3, 3, 3
CC@CC .........3, 3, 6
      rinv(4,1,ihinge) = -temp3(1,1)*pimat(1,1,ihinge) - temp3(1,2)*pima
     .t(2,1,ihinge) - temp3(1,3)*pimat(3,1,ihinge)
      rinv(5,1,ihinge) = -temp3(2,1)*pimat(1,1,ihinge) - temp3(2,2)*pima
     .t(2,1,ihinge) - temp3(2,3)*pimat(3,1,ihinge)
      rinv(6,1,ihinge) = -temp3(3,1)*pimat(1,1,ihinge) - temp3(3,2)*pima
     .t(2,1,ihinge) - temp3(3,3)*pimat(3,1,ihinge)
      rinv(4,2,ihinge) = -temp3(1,1)*pimat(1,2,ihinge) - temp3(1,2)*pima
     .t(2,2,ihinge) - temp3(1,3)*pimat(3,2,ihinge)
      rinv(5,2,ihinge) = -temp3(2,1)*pimat(1,2,ihinge) - temp3(2,2)*pima
     .t(2,2,ihinge) - temp3(2,3)*pimat(3,2,ihinge)
      rinv(6,2,ihinge) = -temp3(3,1)*pimat(1,2,ihinge) - temp3(3,2)*pima
     .t(2,2,ihinge) - temp3(3,3)*pimat(3,2,ihinge)
      rinv(4,3,ihinge) = -temp3(1,1)*pimat(1,3,ihinge) - temp3(1,2)*pima
     .t(2,3,ihinge) - temp3(1,3)*pimat(3,3,ihinge)
      rinv(5,3,ihinge) = -temp3(2,1)*pimat(1,3,ihinge) - temp3(2,2)*pima
     .t(2,3,ihinge) - temp3(2,3)*pimat(3,3,ihinge)
      rinv(6,3,ihinge) = -temp3(3,1)*pimat(1,3,ihinge) - temp3(3,2)*pima
     .t(2,3,ihinge) - temp3(3,3)*pimat(3,3,ihinge)
CC@CC end of expanded call
CC@CC
C
c           assemble rinverse
c
               DO 605 J = 1, 3
                  DO 606 I = 1, 3
                     RINV(I,J,IHINGE) = PIMAT(I,J,IHINGE)
 606              CONTINUE
 605           CONTINUE
c
c     inline use of the (4,4) section of BH (q-frame)
               if (ihpvt(ihinge) .eq. 0) then 
                  DO J = 1, 3
                     DO I = 1, 3
                        RINV(i+3,j+3,IHINGE) =  rh(j,i,lr5)
                     END DO
                  END DO
               else
CC@CC
CC@CC ............MULT3T rh(1,1,lr5), rh(1,1,lr2), rinv(4,4,ihinge)
CC@CC ............3, 3, 3
CC@CC ............6, 3, 6
      rinv(4,4,ihinge) = rh(1,1,lr5)*rh(1,1,lr2) + rh(2,1,lr5)*rh(2,1,lr
     .2) + rh(3,1,lr5)*rh(3,1,lr2)
      rinv(5,4,ihinge) = rh(1,2,lr5)*rh(1,1,lr2) + rh(2,2,lr5)*rh(2,1,lr
     .2) + rh(3,2,lr5)*rh(3,1,lr2)
      rinv(6,4,ihinge) = rh(1,3,lr5)*rh(1,1,lr2) + rh(2,3,lr5)*rh(2,1,lr
     .2) + rh(3,3,lr5)*rh(3,1,lr2)
      rinv(4,5,ihinge) = rh(1,1,lr5)*rh(1,2,lr2) + rh(2,1,lr5)*rh(2,2,lr
     .2) + rh(3,1,lr5)*rh(3,2,lr2)
      rinv(5,5,ihinge) = rh(1,2,lr5)*rh(1,2,lr2) + rh(2,2,lr5)*rh(2,2,lr
     .2) + rh(3,2,lr5)*rh(3,2,lr2)
      rinv(6,5,ihinge) = rh(1,3,lr5)*rh(1,2,lr2) + rh(2,3,lr5)*rh(2,2,lr
     .2) + rh(3,3,lr5)*rh(3,2,lr2)
      rinv(4,6,ihinge) = rh(1,1,lr5)*rh(1,3,lr2) + rh(2,1,lr5)*rh(2,3,lr
     .2) + rh(3,1,lr5)*rh(3,3,lr2)
      rinv(5,6,ihinge) = rh(1,2,lr5)*rh(1,3,lr2) + rh(2,2,lr5)*rh(2,3,lr
     .2) + rh(3,2,lr5)*rh(3,3,lr2)
      rinv(6,6,ihinge) = rh(1,3,lr5)*rh(1,3,lr2) + rh(2,3,lr5)*rh(2,3,lr
     .2) + rh(3,3,lr5)*rh(3,3,lr2)
CC@CC end of expanded call
CC@CC
               end if
c
            else if (icbody.GT.nbdy .and. ihdata(5,ihinge) .eq. 0) then
               ndof = 3
c
            else
               CALL CLEARERR()
               WRITE(ERRMSG,300) 'Error in constraint/topology setup'
               CALL ERRHDL(-5,'<FRWD1A_1>') 
            end if     ! End of test for types of objects and constraints
 300     FORMAT(1X,A)
C
         END IF    ! End of test for p- or q-frame being inboard or outboard.
c
         NMI = IRGFLX(INBODY)
         ilen = lenu(inbody)
         n0 = 6 - ndof
c                                           -1
c        Finished with the computation of  R    for i=IHINGE
c                                           Ii
c
c-----------------------------------------------------------------------
c
C                          .
C   Begin computation of Gamma for outboard body at hinge IHINGE
C        ICBODY is the outboard body at hinge IHINGE
C        INBODY is the inboard body at hinge IHINGE
C     .
C   Gamma contains first 6 elements of rate vector for body ICBODY
C      Get contribution from inboard body INBODY, store in TVEC1
C
         LOCUIN = LOCU(INBODY)
         LOCUIC = LOCU(ICBODY)
C
c        Compute -b     U    term.  Store in TVEC1
c                  O,i-1 i-1
c
CCC
CCC ...MULT3N bh(1,1,inbody), y(locuin), tvec1, ndof, ilen, 1, 6, ilen, 6
CCC
         CALL BHMULT(qin,ihinge,-1,y(locuin),tvec1,ilen,ilen,6,
     .             intLoopHeap(iibhtypeptr),intLoopHeap(iitopolptr),
     .             intLoopHeap(iibh_ptrptr))
C
C
         IF (NM .GT. 0) THEN
C     .
C   Gamma contains first 6 elements of rate vector for body ICBODY
C      Get contribution from body ICBODY modal rates, store in TVEC2
C      Store accumulation in TVEC1
c                       .
c       compute the F   xi   term
c                    Ii   i
C
ccc ...MULT3 bh(1,7,icbody), y(locuic+6), tvec2, ndof, nm, 1, 6, nm, 6
c     (use the mode section of BH)
c
            if (ihinge .GT. 1) then
               if (ndof .EQ. 6) then
                  j = 4
                  do i = 1,3
                     tvec2(i) = 0.0d0
                  end do
               else
                  j = 1
               end if 
CC@CC
CC@CC MULT3 bh17, y(locuic+6), tvec2(j)
CC@CC 3, nm, 1
CC@CC 3, nm, 6
      CALL DGEMV ('N',3,nm,1.0d0,bh17,3,y(locuic+6),1,0.0d0,tvec2(j),1)
CC@CC end of expanded call
CC@CC
            else   ! 1st hinge mode section = 0
               do i = 1,ndof
                  tvec2(i) = 0.0d0
               end do
            end if
c                  
C
            CALL MSUB (TVEC1,TVEC2,TVEC1,ndof,1,6)
         END IF
c
C     .
C   Gamma contains the first 6 elements of the U vector for body ICBODY.
C      Get contribution from hinge IHINGE relative rate BETAHD.  Use
c      locator array IBETDT to find required BETAHD's.  Accumulate
c      results in TVEC1.
c             .  f                  .
c        Add beta   for free DOF, alpha  for rheonomically constrained DOF
c                Ii                    Ii
c
         LOC = IBETDT(IHINGE)
         DO 700 IDOF = 1, ndof
            n = n0 + idof
            IHD = IHDATA(n+1,IHINGE)
            IF (IHD .EQ. 0) THEN
               TVEC1(IDOF) = TVEC1(IDOF) + Y(LOC)
               LOC = LOC + 1
            ELSE IF (IHD .EQ. 2) THEN
               TVEC1(IDOF) = TVEC1(IDOF) + ADT(6*(IHINGE-1)+n,T)
            END IF
 700     CONTINUE
c
c      Finish off by multiplying results in TVEC1 by R_sub Ii inverse.
C      Load results directly into 6- or 3-long segment of Y starting at LOCUIC.
C
c          .         -1     .           .
c        gamma   =  R   ( beta   -  F   xi   - b     U     )
c             i      Ii       Ii     Ii   i     Oi-1  i-1
c
CC@CC
CC@CC ...MULT3 rinv(1,1,ihinge), tvec1, y(locuic)
CC@CC ...ndof, ndof, 1
CC@CC ...6, 6, ndof
      CALL DGEMV ('N',ndof,ndof,1.0d0,rinv(1,1,ihinge),6,tvec1,1,0.0d0,y
     .(locuic),1)
CC@CC end of expanded call
CC@CC
C
      RETURN
      END     ! End of frwrd1a_1






