C $Header: /export/cvs/c34a1/source/moldyn/frwd1b.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      SUBROUTINE FRWD1B(Y,BDOTIC,BDOTIN,ZETA,IHTHPTR,ISDOF,ITOPOL,
     &                  IHDATA,ibh_ptr)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C                                                  DEBUG #  76
CSE
      IMPLICIT NONE
C
C     THIS SUBROUTINE IS USED TO FINISH THE FIRST FORWARD PASS OF THE
C     ORDER N ALGORITHM.  IT RECURSIVELY COMPUTES THE ZETA VECTORS FOR EACH
C     BODY.  THE RECURSION PROGRESSES FROM THE BASE BODY FORWARD TO THE
C     END BODY OF EACH CHAIN.
C
CPE
CAS                       ARGUMENT LIST
C
C   FRWD1B - Data for backward and second
C            forward pass is generated herein and stored in /F1BKF2/
C
C       THE FOLLOWING ARRAY NEEDED FOR FORWARD AND BACKWARD SWEEPING
C       "ZETA" IS COMPUTED HEREIN, AND IS STORED IN COMMON BLOCK /F1BKF2/.
C
C     ZETA   = array of vectors labeled "{zeta}i" needed for backward and
C              second forward pass.
C     (ZETA(I,ihinge),I=1,7)  = Intermediate acceleration vector computed
C                               at the end of the first forward pass.
C                               Rows 1-6 are associated with relative motion
C                               between bodies, If hinge "ihinge" is geared
C                               the 7-th row is associated with the relative
C                               motion of the rotor and the driving body in
C                               which the rotor is embedded.
C
CAE
C
#include "PARAM.FOR"
C
C     COMMON BLOCKS
C
#include "BHBSRD.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "LDEBUG.INC"
#include "ORDERN.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "TIMESS.INC"
#include "VECTOR.INC"
C
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: frwd1b.f,v $$Revision: 1.1.1.1 $'/

c     Passed variables
c
      DOUBLE PRECISION Y(cL07)
      DOUBLE PRECISION ZETA(6,cL02)
      INTEGER IHTHPTR(2*cL02)
      INTEGER ISDOF(9,cL02)
      INTEGER ITOPOL(2,cL02)
      INTEGER IHDATA(7,cL02)
      INTEGER ibh_ptr(cL12+1)
c
C     Local Variables
C
      INTEGER IDOF,NDOF,IHD
      INTEGER ILEN,LEN
      INTEGER IBODY,ICBODY,INBODY
      INTEGER IHINGE,LOCUIN,LOCUIC
      INTEGER N,N0
      INTEGER I,J
      INTEGER l1,l2,l3                  ! pointer indexes
      INTEGER ibhin,ibhic,ihatin,ihatic ! heap_mb pointers
      LOGICAL LEQU
      DOUBLE PRECISION TVEC1(6)
      DOUBLE PRECISION BDOTIC(6,cl08), BDOTIN(6,cl08)
      DOUBLE PRECISION ADDT,dum
C
C     Equivalences
C
      EQUIVALENCE (LEQU,DEBUG(76))
C
C
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE FRWD1B ')
C
C     COMPUTE THE ZETA'S FOR HINGE 1
C
C   Initialize the Zeta vector with the hinge rheonomic relative acceleration,
C   Section 3.1.4.1
C
      if (nbdy .EQ. 0) then   ! Body 1 is a particle
         do idof = 1, 3
            ihd = ihdata(idof+4,1)
            if (ihd .ne. 0) then
               CALL CLEARERR()
               WRITE(ERRMSG,200) 'Wrong constraints for Hinge 1'
               CALL ERRHDL(-5,'<FRWD1B>')
            end if
            zeta(idof,1) = 0d0
         end do
      else                    ! Body 1 is a body
         DO 400 IDOF = 1, 6
            IF (IHDATA(IDOF+1,1) .EQ. 2) THEN
               ZETA(IDOF,1) = ADDT(IDOF,T)
            ELSE
               ZETA(IDOF,1) = 0.0
            END IF
 400     CONTINUE
      end if
 200  FORMAT(1X,A)
c              ___    ..        .
c     zeta  =  PHI   alpha   -  b   U
c         1       I1      I1     I1  1
C
      CALL BDOTQP (1,BDOTIC,BDOTIN,dum,dum,dum,dum)
C
      LEN = LENU(1)
      if (nbdy .EQ. 0) then  ! Body 1 is a particle
         ndof = 3
      else                   ! Body 1 is a body
         ndof = 6
      end if
CC@CC
CC@CC MULT3 bdotic, y, tvec1
CC@CC ndof, len, 1
CC@CC 6, len, 6
      CALL DGEMV ('N',ndof,len,1.0d0,bdotic,6,y,1,0.0d0,tvec1,1)
CC@CC end of expanded call
CC@CC
      CALL MSUB(ZETA,TVEC1,ZETA,ndof,1,6)
C
c          Finished with hinge 1.
c
c-----------------------------------------------------------------------
c
c          Proceed outward along all forward
c          paths defined in the ITOPW array.  Local variable IBODY
c          in the 600 loop is a sequence locator.  Body number and
c          hinge numbers are taken for arrays ITOPW and ITOPOL
c
      DO 600 IBODY = 2, NB
C
         IHINGE = ITOPW(1,IBODY)
         ICBODY = ITOPW(2,IBODY)
c
         IF (IHINGE .LT. 0) THEN     ! P frame is on this body
            IHINGE = -IHINGE
            INBODY = ITOPOL(1,IHINGE)
            l1     = 2*ihinge - 1    ! lp
            l2     = l1 - 1          ! lq/lhsp
            l3     = l1 - 2          ! lhsq
            ibhic  = ibh_ptr(l1)     ! pointer to bhout (P)
            ibhin  = ibh_ptr(l2)     ! pointer to bhin  (Q)
            ihatic = ihthptr(l2) + 1 ! pointer to hatsin
            ihatin = ihthptr(l3) + 1 ! pointer to hatsout
            CALL BDOTQP (IHINGE,BDOTIN,BDOTIC,BH(ibhin),
     &             BH(ibhic),HATS(ihatin),HATS(ihatic))
         ELSE                        ! Q frame is on this body
            INBODY = ITOPOL(2,IHINGE)
            l1     = 2*ihinge - 1    ! lp
            l2     = l1 - 1          ! lq/lhsp
            l3     = l1 - 2          ! lhsq
            ibhin  = ibh_ptr(l1)     ! pointer to bhin  (P)
            ibhic  = ibh_ptr(l2)     ! pointer to bhout (Q)
            ihatin = ihthptr(l2) + 1 ! pointer to hatsin
            ihatic = ihthptr(l3) + 1 ! pointer to hatsout
            CALL BDOTQP (IHINGE,BDOTIC,BDOTIN,BH(ibhic),
     &             BH(ibhin),HATS(ihatic),HATS(ihatin))
         END IF
c
         LOCUIN = LOCU(INBODY)
         LOCUIC = LOCU(ICBODY)
         len    = lenu(icbody)
         ilen   = lenu(inbody)
         ndof   = isdof(3,ihinge) + isdof(4,ihinge)
         n0     = 6 - ndof
c
c     Begin computation of zeta vector.  This will be required during
c     both the backward and second forward pass.
c
c                        ___     ..      .            .
c        Compute zeta  = PHI   alpha   - b     U    - b  U
c                    i      Ii      Ii    Oi-1  i-1    Ii i
c
c                          .
c        First compute the b      U    term
c                           O,i-1  i-1
c
CC@CC
CC@CC ...MULT3 bdotin, y(locuin), zeta(1,ihinge)
CC@CC ...ndof, ilen, 1
CC@CC ...6, ilen, 6
      CALL DGEMV ('N',ndof,ilen,1.0d0,bdotin,6,y(locuin),1,0.0d0,zeta(1,
     .ihinge),1)
CC@CC end of expanded call
CC@CC
c                       .
c        now add on the b   U  term
c                        Ii  i
c
CC@CC
CC@CC ...MULTAD bdotic, y(locuic), zeta(1,ihinge)
CC@CC ...ndof, len, 1
CC@CC ...6, len, 6
      CALL DGEMV ('N',ndof,len,1.0d0,bdotic,6,y(locuic),1,1.0d0,zeta(1,i
     .hinge),1)
CC@CC end of expanded call
CC@CC
C                                      ..
c        compute the selection terms (alpha for selected DOF)
c
         DO 800 IDOF = 1, ndof
            n = n0 + idof
            IF (IHDATA(n+1,IHINGE) .EQ. 2) THEN
               ZETA(IDOF,IHINGE) = ADDT(6*(IHINGE-1)+n,T) -
     +                             ZETA(IDOF,IHINGE)
            ELSE
               ZETA(IDOF,IHINGE) = -ZETA(IDOF,IHINGE)
            END IF
 800     CONTINUE
C
 600  CONTINUE
c
      IF (LEQU) THEN
         DO IBODY=1,NB
            J=ABS(ITOPW(1,IBODY))
            WRITE (MBOUT,1010) J,(ZETA(I,J),I=1,6)
 1010       FORMAT (1X,'  ZETA(',I3,') = ',1P7D11.3)
         END DO
      END IF
c
      RETURN
      END



