C $Header: /export/cvs/c34a1/source/moldyn/frwrd2.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      SUBROUTINE FRWRD2 (GTILDE,PSI,YDT,KPVT,TEMP,TEMP1,ZWK,RINV,
     *                   ZETA,IHTHPTR,IV,IHPVT,ISEL,ISDOF,IBETDT,
     *                   ITOPOL,ALAM,ibh_ptr)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

C
CPS   PURPOSE
C
C     THIS SUBROUTINE IS USED TO MAKE THE SECOND FORWARD PASS FOR
C     THE ORDER N ALGORITHM.  DURING THIS PASS THE RELATIVE HINGE
C     ACCELERATIONS AND MODAL ACCELERATIONS ARE COMPUTED.  THE
C     RECURSION BEGINS WITH THE BASE BODY AND TERMINATES WITH THE
C     END BODY OR BODIES.  COMPUTED INFORMATION IS LOADED DIRECTLY
C     INTO THE YDT ARRAY AT LOCATIONS MARKED BY THE CONTENTS OF THE
C     IBTDT ARRAY.
C
C       .. f
C      Beta   = Computed herein, acceleration of the unconstrained hinge
C          Ii   degrees of freedom for the inboard hinge of body i.
C
C        ..                                                .
C      Gamma  = Computed herein, the first six elements of U(i).  These
C           i   are the components of the rigid body absolute acceleration
C               vector.
C
C        ..                                                           .
C        Xi   = Computed herein, the modal accelerations contained in U(i)
C          i
C        ..                                                           .
C        Th  = Computed herein, the wheel accelerations contained in U(i)
C          i
C
C      Kappa   =  Wheel constraints acting on body i
C           i
C
CPE
CAS                   ARGUMENT LIST
C
C      GTILDE = Input from BCKWRD through YDOT.  The vector which contains
C               all G(tilda)'s.  These are the equivalent generalized
C               forces acting on each body.
C                                            .
C      Within the Order N recusive algorithm U(i) vectors are not integrated
C      to obtain U(i), they are computed as intermediate variables which are
C      used in the recursion for Beta(double dot) and Xi(double dot).
C
C      YDT = First derivative of the system state vector.  The derivatives
C            of the U(i) vectors are computed and stored in partitions of YDT.
C            The inconstrained relative hinge accelerations are also computed,
C            these are stored in the augmented portion of the YDT array.
C
CAE
C
#include "PARAM.FOR"
C
C     COMMON BLOCKS FROM DISCOS
C
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "CONSTR.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "IVCONS.INC"
#include "JILFLG.INC"
#include "LAMBDA.INC"
#include "MAXMUM.INC"
#include "MISCNO.INC"
#include "MISCN2.INC"
#include "NUMBRS.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
C
 
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: frwrd2.f,v $$Revision: 1.1.1.1 $'/

C     Passed Variables
c
      DOUBLE PRECISION PSI(6,cL06,MAXBOD) 
      DOUBLE PRECISION YDT(cL07)
      DOUBLE PRECISION RINV(6,6,cL02)
      DOUBLE PRECISION ZETA(6,cL02)
      DOUBLE PRECISION ALAM(cL09)
      INTEGER IHTHPTR(2*cL02)
      INTEGER IV(6,cL02)
      INTEGER IHPVT(cL02)
      INTEGER ISEL(6*cL02)
      INTEGER ISDOF(9,cL02)
      INTEGER IBETDT(cL02)
      INTEGER ITOPOL(2,cL02)
      INTEGER ibh_ptr(cL12+1)
c
C     Local Variables
C
      INTEGER KPVT(cL08)
      INTEGER LOCF, NFREE,LEN
      INTEGER IBTDT,LOCC
      INTEGER ISROW,IBODY,IHINGE,ICBODY,INBODY
      INTEGER N0,IC
      INTEGER NM,NCONS,NDOF,MTSIZE
      INTEGER I,J
      INTEGER icrm,icamu,icbh     ! pointers to heap_mb
      DOUBLE PRECISION DUMMY,RCOND
      DOUBLE PRECISION  GTILDE(cL10)
      DOUBLE PRECISION  ZMBOIU(6)
      DOUBLE PRECISION  TEMP (cL08)
      DOUBLE PRECISION  TEMP1(cL08)
      DOUBLE PRECISION  zwk(cL08)
      LOGICAL QIN, useHATS
C
#include "LDEBUG.INC"
      LOGICAL LEQU
      EQUIVALENCE (LEQU,DEBUG(77))
c
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE FRWRD2 ')
C
C     FIRST DO THE REFERENCE BODY, BODY 1
      NM     = IRGFLX(1)
      LOCF   = ISDOF(1,1)
      NFREE  = ISDOF(3,1)
      NCONS  = ISDOF(4,1)
      ndof   = nfree + ncons
      N0     = 6 - ndof
      MTSIZE = LENU(1)
      len    = mtsize
      icrm   = irm_ptr(1)
      icamu  = iamuptr(1)
C
C     +-  -+
C     | ..f|
C     | B  |                           +-             -+
C     |  I1|       _-1    T  ~     ~   |  -1           |
C     |    |   =   M   PSI  (G  -  M   | R     zeta    |
C     | .. |        1     1   1     1  |  I1       1   |
C     | Xi |                           |______________ |
C     |   1|                           |      0        |
C     |    |                           |       nm      |
C     +-  -+                           +-             -+
C
C  If body 1 has any independent dof's then compute acceleration of relative
C   joint motion coordinates (1,..,NFREE), and modal (NFREE+1,...NFREE+NM)
C   SEE SECTION 4.5, EQN. (33).
C
      IF (NFREE+NM .GT. 0) THEN
C
CC@CC
CC@CC ...MULT3 rinv(1,1,1), zeta(1,1), temp
CC@CC ...ndof, ndof, 1
CC@CC ...6, 6, 6
      CALL DGEMV ('N',ndof,ndof,1.0d0,rinv(1,1,1),6,zeta(1,1),1,0.0d0,te
     .mp,1)
CC@CC end of expanded call
CC@CC
CC@CC
CC@CC ...MULT3 HEAP_MB(icamu), temp, temp1
CC@CC ...mtsize, ndof, 1
CC@CC ...mtsize, 6, 6
      CALL DGEMV ('N',mtsize,ndof,1.0d0,HEAP_MB(icamu),mtsize,temp,1,0.0
     .d0,temp1,1)
CC@CC end of expanded call
CC@CC
C
         CALL MSUB(GTILDE,TEMP1,TEMP1,MTSIZE,1,LEN)

         IF (nbdy .gt. 0) THEN
            CALL PSITX(PSI,RINV,ISEL(LOCF),NDOF,NFREE,NM,TEMP1,TEMP)
         ELSE                   ! If only particles
            CALL PSITX(DUMMY,RINV,ISEL(LOCF),NDOF,NFREE,NM,TEMP1,TEMP)
         END IF
C
C  Multiply TEMP by M_sub_1_bar inverse to get all accelerations
c
         CALL DSICO (HEAP_MB(icrm),MTSIZE,NFREE+NM,KPVT,RCOND,ZWK)
         if (rcond+1d0 .eq. 1d0) then
           CALL CLEARERR()
           WRITE(ERRMSG,100) '1>> RCOND = ',rcond
           CALL ERRHDL(-5,'<FRWRD2>')  
         end if
 100     FORMAT(1X,A20,E15.8E2)
         CALL DSISL (HEAP_MB(icrm),MTSIZE,NFREE+NM,KPVT,TEMP)
C
C  Temporary storage summary, body 1:
C                  ~    ~   -1
C         TEMP1 =  G  - M  R   Zeta               size  MTSIZE x 1
C                   1    1  I1     1
C
C                  _-1    T  ~    ~   -1
C          TEMP =  M   Psi ( G  - M  R   Zeta )
C                   1     1   1    1  I1     1
C
C                     +-   T    -+
C                     | PHI  P   |
C                 _-1 |    I1 I1 |
C               + M   |          | Kappa          size NFREE+NM X 1
C                  1  |    0     |      I1
C                     +-        -+
C
C   Proceed to compute 6 components of rigid body inertial acceleration
C
C          ..       -1          .. f         
C        gamma  =  R   ( PHI   Beta   + zeta )
C             1     I1      I1     I1       1
C
C    Use above data to compute gamma_double_dot, the ndofx1 rigid body
C    acceleration elements.  Elements of ZETA were computed in FRWRD1
C    First NFREE elements of TEMP are the Beta_double_dots.
c
         DO J = 1, ndof
            ZMBOIU(J) = ZETA(J,1)
         END DO
         DO J = 1, NFREE
            ZMBOIU(ISEL(LOCF+J-1)) = ZETA(ISEL(LOCF+J-1),1)
     *           + TEMP(J)
         END DO

         IF(LEQU) THEN
            WRITE (MBOUT,1040) 1
 1040       FORMAT (' ZMBOIU FOR BODY ',I3)
            CALL WRITES(ZMBOIU,1,ndof,1)
         END IF
C
C  Load ndof elements of body_1 gamma_double_dot into body_1 locations of YDT
CC@CC
CC@CC ...MULT3 rinv(1,1,1), zmboiu, ydt
CC@CC ...ndof, ndof, 1
CC@CC ...6, 6, 6
      CALL DGEMV ('N',ndof,ndof,1.0d0,rinv(1,1,1),6,zmboiu,1,0.0d0,ydt,1
     .)
CC@CC end of expanded call
CC@CC
C
C  Load NFREE elements of TEMP into IHINGE=1 BETA_double_dot locations of YDT
         IBTDT = IBETDT(1)
         DO 300 I = 1, NFREE
            YDT(IBTDT) = TEMP(I)
            IBTDT      = IBTDT + 1
 300     CONTINUE
C
C  Load next NM elements of TEMP into body_1 Xi_double_dot locations of YDT
C        AND NOW THE XIS
         DO 400 I = 1, NM
            YDT(6 + I) = TEMP(NFREE + I)
 400     CONTINUE
C
C  If there are no independent dof's just include possible rheonomic conditions
C  that were included within ZETA in FRWRD1.
      ELSE
CC@CC
CC@CC ...MULT3 rinv(1,1,1), zeta(1,1), ydt
CC@CC ...ndof, ndof, 1
CC@CC ...6, 6, 6
      CALL DGEMV ('N',ndof,ndof,1.0d0,rinv(1,1,1),6,zeta(1,1),1,0.0d0,yd
     .t,1)
CC@CC end of expanded call
CC@CC
      END IF
C
C  COMPUTE LAMDA CONSTRAINTS FOR BODY 1.
C    ITS CONSTRAINT VECTOR IS OF LENGTH NCONS.  USE IV ARRAY, COMPUTED IN
C    COMPIV, TO LOAD ALAM ARRAY
C    ALL ELEMENTS OF RJUPPR, JPVT, AND VLAM CAME FROM BCKWRD
C
      IF ((NCONS .GT. 0) .AND. (IPRNT .EQ. NOPRNT)
     1     .AND. JIL .EQ. 4) THEN
         LOCC = ISDOF(2,1) - 1
         CALL DSISL (RJUPPR,6,NCONS,JPVT,VLAM)
         DO I = 1, NCONS
            ISROW = ISEL(LOCC+I)
            IC = IV(ISROW+N0,1)
            ALAM(IC) = VLAM(I,1)
         END DO
         IF(LEQU) THEN
            WRITE (MBOUT,1005) 1
 1005       FORMAT (' ALAM AFTER CONSTRAINTS ADDED FOR HINGE ',I3)
            CALL WRITES(ALAM,1,NCONS,1)
         END IF
      END IF
C
      IF(LEQU) THEN
         WRITE (MBOUT,1010) 1,MTSIZE,1
1010     FORMAT (' YDT(',I3,'...',I3,'), U(dot) FOR BODY ',I3)
         WRITE (MBOUT,1030) (YDT(I),I=1,MTSIZE)
         IF(NFREE.GT.0) THEN
            WRITE (MBOUT,1020) IBETDT(1),IBETDT(1)-1+NFREE,1
1020        FORMAT (' YDT(',I3,'...',I3,
     *                    '), Beta(double dot) FOR HINGE ',I3)
            WRITE (MBOUT,1030) (YDT(IBETDT(1)-1+I),I=1,NFREE)
1030        FORMAT (20X,1P6E15.5)
         END IF
      ENDIF
C
C     NOW FOR THE REST OF THE BODIES PROCEED SEQUENTIALLY,
C     FROM BODY 1, OUTBOARD BASED UPON ITOPW ARRAY DATA
C         IBODY  -  SEQUENCE NUMBER
C        ICBODY  - OUTBOARD BODY NUMBER
C        INBODY  - INBOARD BODY NUMBER
C        IHINGE  - ASSOCIATED HINGE NUMBER
C
      DO 600 IBODY = 2, NB
C
         IHINGE = ITOPW(1,IBODY)
         ICBODY = ITOPW(2,IBODY)
         NM     = IRGFLX(ICBODY)
         len    = lenu(ICBODY)
C
         IF (IHINGE .LT. 0) THEN
C           PFRAME IS ON THIS BODY (OUTBOARD BODY)
C
            IHINGE = -IHINGE
            INBODY = ITOPOL(1,IHINGE)
            qin = .TRUE.
C
         ELSE     ! QFRAME IS ON THIS BODY (OUTBOARD BODY)
C
            INBODY = ITOPOL(2,IHINGE)
            qin = .FALSE.
         END IF
C
         icrm    = irm_ptr(icbody)
         icamu   = iamuptr(icbody)
         useHATS = .FALSE.
         if (nm .GT. 0) then
            if (qin) then       ! p is outboard
               if (ihpvt(ihinge) .EQ. 0) then
                  icbh = ihthptr(2*ihinge-2) + 1
                  useHATS = .TRUE.
               else
                  icbh = ibh_ptr(2*ihinge-1) + 18
               end if
            else                ! q is outboard
               if (ihpvt(ihinge) .EQ. 0) then
                  icbh = ibh_ptr(2*ihinge-2) + 9
               else
                  icbh = ibh_ptr(2*ihinge-2) + 18
               end if
            end if
         end if
C
C     Pass heap_mb pointers to amu,rmbar, and bh to frwrd2_1
C
         if (useHATS) then
           Call frwrd2_1(GTILDE,HEAP_MB(icamu),HEAP_MB(icrm),
     &              HATS(icbh),icbody,inbody,ihinge,nm,len,qin,
     &		    HEAP_MB(ipsiptr),HEAP_MB(iydtptr),
     &              intLocalHeap(ikpvtptr_frwrd2_1),
     &              LocalHeap(itempptr_frwrd2_1),
     &              LocalHeap(itemp1ptr_frwrd2_1),
     &              LocalHeap(izwkptr_frwrd2_1),
     &              LoopHeap(irinvptr),LoopHeap(izetaptr),
     &              intLoopHeap(iivptr),intLoopHeap(iiselptr),
     &              intLoopHeap(iisdofptr),intLoopHeap(
     &              iibetdtptr),Loopheap(ialamptr))
         else
           Call frwrd2_1(GTILDE,HEAP_MB(icamu),HEAP_MB(icrm),
     &              BH(icbh),icbody,inbody,ihinge,nm,len,qin,
     &		    HEAP_MB(ipsiptr),HEAP_MB(iydtptr),
     &              intLocalHeap(ikpvtptr_frwrd2_1),
     &              LocalHeap(itempptr_frwrd2_1),
     &              LocalHeap(itemp1ptr_frwrd2_1),
     &              LocalHeap(izwkptr_frwrd2_1),
     &              LoopHeap(irinvptr),LoopHeap(izetaptr),
     &              intLoopHeap(iivptr),intLoopheap(iiselptr),
     &              intLoopHeap(iisdofptr),intLoopHeap(
     &              iibetdtptr),Loopheap(ialamptr))
         end if
 600  CONTINUE
c
      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     Beginning of frwrd2_1
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine frwrd2_1(GTILDE,amu,rmbar,bh17,icbody,inbody,ihinge,nm,
     &                    len,qin,psi,ydt,kpvt,temp,temp1,zwk,rinv,
     &                    zeta,iv,isel,isdof,ibetdt,alam)
c
c
#include "PARAM.FOR"
C
C     COMMON BLOCKS FROM DISCOS
C
#include "AMUBW.INC"
#include "BHBSRD.INC"
#include "CONSTR.INC"
#include "ERRHDL.INC"
#include "F1BKF2.INC"
#include "HANDS2.INC"
#include "IVCONS.INC"
#include "JILFLG.INC"
#include "LAMBDA.INC"
#include "MAXMUM.INC"
#include "HEAPMB.INC"
#include "MISCNO.INC"
#include "MISCN2.INC"
#include "ORDERN.INC"
#include "PARTCL.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
C
C     Passed Variables
C
      INTEGER icbody,inbody,ihinge
      INTEGER nm, len
      LOGICAL qin
      DOUBLE PRECISION amu(len,len)   ! for outboard frame
      DOUBLE PRECISION rmbar(len,len) ! for outboard frame
      DOUBLE PRECISION bh17(3,*)      ! 1,7 section only
      DOUBLE PRECISION psi(6,cL06,MAXBOD)
      DOUBLE PRECISION ydt(cL07)
      DOUBLE PRECISION rinv(6,6,cL02)
      DOUBLE PRECISION zeta(6,cL02)
      DOUBLE PRECISION alam(cl09)
      INTEGER IV(6,cL02)
      INTEGER ISEL(6*cL02)
      INTEGER ISDOF(9,cL02)
      INTEGER IBETDT(cL02)
C
C     Local Variables C
      INTEGER KPVT(cL08)
      INTEGER LOCF, NFREE,ILEN
      INTEGER LOCUIN,LOCUIC,LQ,LP
      INTEGER IBTDT,LOCC
      INTEGER ISROW
      INTEGER N0,IC,ISELR
      INTEGER NCONS,NDOF,MTSIZE,MTISIZE
      INTEGER I,J
      DOUBLE PRECISION DUMMY,RCOND
      DOUBLE PRECISION  GTILDE(*)
      DOUBLE PRECISION  ZMBOIU(6), BOU(6)
      DOUBLE PRECISION  TEMP (cL08)
      DOUBLE PRECISION  TEMP1(cL08)
      DOUBLE PRECISION  zwk(cL08)
C
#include "LDEBUG.INC"
      LOGICAL LEQU
      EQUIVALENCE (LEQU,DEBUG(77))
c
      IF(LEQU) WRITE (MBOUT,1000)
 1000 FORMAT (' SUBROUTINE FRWRD2 ')
C
C
         LQ  = 2 * IHINGE - 2
         LP  = LQ + 1
C
         LOCF  = ISDOF(1,IHINGE)
         NFREE = ISDOF(3,IHINGE)
         NCONS = ISDOF(4,IHINGE)
         ndof  = nfree + ncons
         N0    = 6 - ndof
C
         LOCUIN  = LOCU(INBODY)
         LOCUIC  = LOCU(ICBODY)
         ilen    = LENU(INBODY)
         MTSIZE  = len    ! nm+6 or 3 for icbody
         MTISIZE = ilen   ! nm+6 or 3 for inbody
C
C   Compute:
C                                    .
C            ZMBOIU  =  zeta  - b    U
C                           i    Oi-1 i-1
C
C                       -1               .
C        YDT(LOCUIC) = R   (zeta  - b    U   )
C                       Ii      i    Oi-1 i-1
C
C   This equals gamma_double_dot for body ICBODY if nm+nfree=0, if not
C     more terms are required.  Results stored in YDT(LOCUIC) will be
C     over-written.
C
CCC
CCC MULT3 bh(1,1,inbord), ydt(locuin), bou, ndof, ilen, 1, 6, ilen, 6
CCC
         CALL BHMULT(qin,ihinge,1,ydt(locuin),bou,ilen,ilen,6,
     .          intLoopHeap(iibhtypeptr),intLoopheap(iitopolptr),
     .          intLoopHeap(iibh_ptrptr))
C
         CALL MSUB(ZETA(1,IHINGE),BOU,ZMBOIU,ndof,1,6)
C
CC@CC
CC@CC ...MULT3 rinv(1,1,ihinge), zmboiu, ydt(locuic)
CC@CC ...ndof, ndof, 1
CC@CC ...6, 6, 6
      CALL DGEMV ('N',ndof,ndof,1.0d0,rinv(1,1,ihinge),6,zmboiu,1,0.0d0,
     .ydt(locuic),1)
CC@CC end of expanded call
CC@CC
c
         IF(LEQU) THEN
            WRITE (MBOUT,1040) ICBODY
            CALL WRITES(ZMBOIU,1,ndof,1)
         END IF
C
         IF (NFREE+NM .GT. 0) THEN
C
C        +-  -+
C        | ..f|
C        | B  |                        +-                            -+
C        |  I |    _-1    T  ~     ~   |  -1                  .       |
C        |    | =  M   PSI  (G  -  M   | R   ( zeta  -  b     U    )  | ) +
C        | .. |     i     i   i     i  |  Ii       i     Oi-1  i-1    |
C        | xi |                        |_____________________________ |
C        |   i|                        |             0                |
C        |    |                        |              nm              |
C        +-  -+                        +-                            -+
C
C                          +-   T    -+ +-       -+
C                      _-1 | Phi  P   | | Kappa   |
C                      M   |    Ii Ii | |      Ii |
C                          |          | +-       -+
C                          |    0     |
C                          |     nm   |
C                          +-        -+
C
CC@CC
CC@CC ......MULT3 amu, ydt(locuic), temp1
CC@CC ......mtsize, ndof, 1
CC@CC ......mtsize, 6, l08
      CALL DGEMV ('N',mtsize,ndof,1.0d0,amu,mtsize,ydt(locuic),1,0.0d0,t
     .emp1,1)
CC@CC end of expanded call
CC@CC
C
            CALL MSUB(GTILDE(LOCUIC),TEMP1,TEMP1,MTSIZE,1,MTSIZE)
C
C
            IF (icbody .le. nbdy) THEN
               CALL PSITX(PSI(1,1,ICBODY),RINV(1,1,IHINGE),
     +           ISEL(LOCF),NDOF,NFREE,NM,TEMP1,TEMP)
            ELSE
               CALL PSITX(DUMMY,RINV(1,1,IHINGE),
     +           ISEL(LOCF),NDOF,NFREE,NM,TEMP1,TEMP)
            END IF
            CALL DSICO (rmbar,MTSIZE,NFREE+NM,KPVT,
     *           RCOND,ZWK)
            if (rcond+1d0 .eq. 1d0) then
               CALL CLEARERR()
               WRITE(ERRMSG,100) '2>> RCOND = ',rcond
               CALL ERRHDL(-5,'<FRWRD2_1>')
            end if
 100        FORMAT(1X,A20,E15.8E2)
            CALL DSISL (rmbar,MTSIZE,NFREE+NM,KPVT,TEMP)
C
            IF (LEQU) THEN
               WRITE (MBOUT,1050) ICBODY
 1050          FORMAT (' TEMP1 FOR BODY ',I3)
               CALL WRITES(TEMP1,1,MTSIZE,1)
               WRITE (MBOUT,1060) ICBODY
 1060          FORMAT (' TEMP FOR BODY ',I3)
               CALL WRITES(TEMP,1,NFREE+NM,1)
            END IF
C
C  Temporary storage summary, body ICBODY:
C
C                  ~    ~   -1              .
C         TEMP1 =  G  - M  R   (zeta - b    U   )        size  MTSIZE x 1
C                   i    i  Ii      i   0i-1 i-1
C
C                  _-1    T
C          TEMP =  M   Psi  TEMP1                   size NFREE+NM x 1
C                   i     i
C
C    Use above data to compute gamma_double_dot, the ndofx1 rigid body
C    acceleration elements.  Elements of ZETA were computed in FRWRD1
C    First NFREE elements of TEMP are the Beta_double_dots.
C
C         ..       -1          .. f                 .          ..
C       gamma  =  R   ( PHI   Beta   + zeta  - b    U    - F   xi  )
C            i     Ii      Ii     Ii       i    Oi-1 i-1    Ii   i
C
C    Add beta_double_dot term
            DO 201 J = 1, NFREE
               ZMBOIU(ISEL(LOCF+J-1)) = ZMBOIU(ISEL(LOCF+J-1))
     *              + TEMP(J)
 201        CONTINUE
C
C    Add Xi_double_dot term
            IF (NM .GT. 0) THEN
CCC
CCC MULT3 BHOUT(1,7), TEMP(NFREE + 1), TEMP1, 6, NM, 1, 6, NM, 6
c     (use the mode section of BH)
c
               if (ndof .NE. 6) then
                  CALL CLEARERR()
                  WRITE(ERRMSG,200)    
     +                'ndof does not equal 6, bad size'
                  CALL ERRHDL(-5,'<FRWRD2_1>') 
               end if
 200           FORMAT(1X,A)
               if (ihinge .GT. 1) then
                  do i = 1,3
                     temp1(i) = 0.0d0
                  end do
                  
CC@CC
CC@CC MULT3 bh17, temp(nfree+1), temp1(4)
CC@CC 3, nm, 1
CC@CC 3, nm, 6
      CALL DGEMV ('N',3,nm,1.0d0,bh17,3,temp(nfree+1),1,0.0d0,temp1(4),1
     .)
CC@CC end of expanded call
CC@CC
               else  ! 1st hinge mode section = 0
                  do i = 1,6
                     temp1(i) = 0.0d0
                  end do
               end if
c
c
               CALL MSUB  (ZMBOIU,TEMP1,ZMBOIU,6,1,6)
            END IF
C
C     Finish computing gamma_double_dot, and store results in first ndof
C       locations of body ICBODY partition in YDT.
CC@CC
CC@CC ......MULT3 rinv(1,1,ihinge), zmboiu, ydt(locuic)
CC@CC ......ndof, ndof, 1
CC@CC ......6, 6, 6
      CALL DGEMV ('N',ndof,ndof,1.0d0,rinv(1,1,ihinge),6,zmboiu,1,0.0d0,
     .ydt(locuic),1)
CC@CC end of expanded call
CC@CC
C
C  Load NFREE elements of TEMP into IHINGE BETA_double_dot locations of YDT
            IBTDT = IBETDT(IHINGE)
            DO 301 I = 1, NFREE
               YDT(IBTDT) = TEMP(I)
               IBTDT      = IBTDT + 1
 301        CONTINUE
C
C  Load next NM elements of TEMP into ICBODY Xi_double_dot locations of YDT
            DO 401 I = 1, NM
               YDT(LOCUIC+5+I) = TEMP(NFREE+I)
 401        CONTINUE
C     
         END IF
         IF (LEQU) THEN
            WRITE (MBOUT,1010) LOCUIC,LOCUIC+MTSIZE-1,ICBODY
            WRITE (MBOUT,1030) (YDT(LOCUIC+I-1),I=1,MTSIZE)
            IF (NFREE.GT.0) THEN
               WRITE (MBOUT,1020) IBTDT,IBTDT+NFREE-1,IHINGE
               WRITE (MBOUT,1030) (YDT(IBETDT(IHINGE)-1+I),I=1,NFREE)
            END IF
         ENDIF
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     Compute the Lagrange multipliers only when needed
C
         IF ((NCONS .GT. 0) .AND. (JIL .EQ. 4) .AND.
     +       (IPRNT .EQ. NOPRNT)) THEN
            LOCC = ISDOF(2,IHINGE) - 1
            DO I = 1, NCONS
               ISELR = ISEL(LOCC + I)
               VLAM(I,ICBODY) = VLAM(I,ICBODY) - BOU(ISELR)
            END DO
C
            CALL DSISL (RJUPPR(1,1,ICBODY),6,NCONS,
     *                         JPVT(1,ICBODY),VLAM(1,ICBODY))
            IF(LEQU) THEN
               WRITE (MBOUT,1080) ICBODY
 1080          FORMAT (' VLAM FOR BODY ',I3)
               CALL WRITES(VLAM,1,NCONS,1)
            END IF
            DO I = 1, NCONS
               ISROW = ISEL(LOCC+I)
               IC = IV(ISROW+N0,IHINGE)
               ALAM(IC) = VLAM(I,ICBODY)
            END DO
            IF(LEQU) THEN
               WRITE (MBOUT,1005) IHINGE
               CALL WRITES(ALAM,1,NCONS,1)
            END IF
         END IF
 1005    FORMAT (' ALAM AFTER CONSTRAINTS ADDED FOR HINGE ',I3)
 1010    FORMAT (' YDT(',I3,'...',I3,'), U(dot) FOR BODY ',I3)
 1020    FORMAT (' YDT(',I3,'...',I3,
     *        '), Beta(double dot) FOR HINGE ',I3)
 1030    FORMAT (20X,1P6E15.5)
 1040    FORMAT (' ZMBOIU FOR BODY ',I3)
C
c
      RETURN
      END    ! End of frwrd2_1
