C $Header: /export/cvs/c34a1/source/moldyn/getbh.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     The parts of BH that are constant or zero are not stored.  GETBH restores
c     the whole BH array for one or both frames of a given hinge.
c     (see bhgenr and BHBSRD.INC for a complete description of what is 
c      stored in bh)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      SUBROUTINE getbh(ihinge,ipick,isq,bhq,nobq,nmq,bhp,nobp,nmp,
     &                 ibhtype,ihthptr,ibh_ptr)
c
c     ihinge is the hinge number
c     ipick  choses the task, partial/full/multiple frame, of filling BH
c            =  2   fill both frames fully
c            =  1   fill bh(q) or bh(p) only
c     isq    is the logical for the type of frame (Q/P) if only one
c     bhq    is the fully restored Q-frame to be passed back (if requested)
c     nobq   is the q body number
c     nmq    is the number of modes for q
c     bhp    is the fully restored Q-frame to be passed back (if requested)
c     nobp   is the p body number
c     nmp    is the number of modes for p
c
c
c
      IMPLICIT NONE
C
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: getbh.f,v $$Revision: 1.1.1.1 $'/
c
#include "PARAM.FOR"
C
#include "BHBSRD.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "PINRP.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
c
c     Passes Variables
c
      INTEGER ibhtype(cL02)
      INTEGER ihinge,ipick
      INTEGER nobq,nobp,nmq,nmp
      INTEGER ihthptr(2*cl02)
      integer ibh_ptr(cL12+1)
      LOGICAL isq
      DOUBLE PRECISION bhq(6,*),bhp(6,*)
c
c     Local Variables
c
      INTEGER itype
      INTEGER ibhq,ibhp,ihat   ! pointers to heap_mb
C
C
      itype= ibhtype(ihinge)   ! to determine what type to assemble
      ibhq = ibh_ptr(2*ihinge-2)     ! pointer to q-frame
      ibhp = ibh_ptr(2*ihinge-1)     ! pointer to p-frame
      ihat = ihthptr(2*ihinge-2) + 1 ! pointer to hats (p)
c
c     Pass heap_mb locations of stored bh to getbh_1
c     
      call getbh_1(ihinge,ipick,isq,itype,BH(ibhq),BH(ibhp),
     &             bhq,bhp,HATS(ihat),nobq,nobp,nmq,nmp,
     &             loopheap(ipinptr),Loopheap(irhptr))

      RETURN
      END
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C
C     Beginning of getbh_1
C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
      subroutine getbh_1(ihinge,ipick,isq,itype,bhq,bhp,frameq,framep,
     &                   hats,nobq,nobp,nmq,nmp,pin,rh)
c
      IMPLICIT NONE
C
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: getbh.f,v $$Revision: 1.1.1.1 $'/
c
#include "PARAM.FOR"
C
#include "BHBSRD.INC"
#include "ERRHDL.INC"
#include "PINRP.INC"
#include "SELECT.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
c
c     Passes Variables
c
      INTEGER ihinge,ipick, itype
      INTEGER nobq,nobp,nmq,nmp
      LOGICAL isq
      DOUBLE PRECISION bhq(3,*),bhp(3,*)       ! Stored bh
      DOUBLE PRECISION frameq(6,*),framep(6,*) ! Filled frames to
                                               ! pass back
      DOUBLE PRECISION hats(3,*)
      DOUBLE PRECISION pin(3,3,cl02)
      DOUBLE PRECISION rh(3,3,cl15)
c
c     Local Variables
c
      INTEGER i,j
      INTEGER lr4,lr5
C
C
c
      IF (ipick.EQ.1 .AND. .NOT.ISQ) GO TO 100  ! Assemble P-frame only
c
      
      if (itype .EQ. 0) then    ! First hinge/body
         DO j = 1,3             ! Only (1,1) & (4,4) sections are
            DO i = 1,3          ! filled.
               frameq(i,j)     = pin(i,j,1)
               frameq(i+3,j+3) = r0l(i,j,1)
               frameq(i+3,j)   = 0.0d0
               frameq(i,j+3)   = 0.0d0
            END DO
         END DO
c
      else if (itype .EQ. 1) then ! (3x3) Identity
         do i = 1,3
            do j = 1,3
               frameq(j,i) = 0.0d0
            end do
            frameq(i,i) = 1.0d0
         end do
c
      else if (itype .EQ. 2) then ! (3x3) C2inv
         LR4  = 6*(ihinge-2) + 4
         do j = 1,3
            do i = 1,3
               frameq(i,j) = rh(i,j,lr4)
            end do
         end do
c
      else if (itype .EQ. 3) then 
         do j = 1,3               ! Fill (1,1) & (1,4) part
            do i = 1,3
               frameq(i,j)   = bhq(i,j)
               frameq(i,j+3) = r0l(i,j,nobq)
            end do
         end do
         do j = 1,nmq             ! Fill mode section
            do i = 1,3
               frameq(i,j+6) = bhq(i,j+3)
            end do
         end do
c
      else if (itype .EQ. 4) then
         do j = 1,3             ! Fill front part
            do i = 1,3
               frameq(i,j)     = pin(i,j,ihinge)
               frameq(i+3,j)   = bhq(i,j)
               frameq(i+3,j+3) = r0l(i,j,nobq)
               frameq(i,j+3)   = 0.0d0
            end do
         end do
         do j = 1, nmq          ! Fill mode section
            do i = 1,3
               frameq(i+3,j+6) = bhq(i,j+3)
               frameq(i,j+6)   = 0.0d0
            end do
         end do
c
      else if (itype .EQ. 5) then
         do j = 1,nmq+6         ! Whole thing is stored
            do i = 1,3
               frameq(i,j) = bhq(i,j)
            end do
         end do
c
      else if (itype .EQ. 6) then
         do j = 1,nmq+6         ! Whole bottom is stored
            do i = 1,3
               frameq(i+3,j) = bhq(i,j)
               if (j .LE. 3) then !Fill (1,1) and rest = 0
                  frameq(i,j) = pin(i,j,ihinge)
               else
                  frameq(i,j) = 0.0d0
               end if
            end do
         end do
c
      else if (itype .EQ. 7) then
         lr5 = 6*(ihinge-2) + 5
         do j = 1,3             ! Fill front part
            do i = 1,3
               frameq(i,j)     = pin(i,j,ihinge)
               frameq(i+3,j)   = bhq(i,j)
               frameq(i,j+3)   = 0.0d0
               frameq(i+3,j+3) = rh(i,j,lr5)
            end do
         end do
         do j = 1, nmq          ! Fill mode part
            do i = 1,3
               frameq(i+3,j+6) = bhq(i,j+3)
               frameq(i,j+6)   = 0.0d0
            end do
         end do
c     
      else if (itype .EQ. 8) then
         do j = 1,nmq+6         ! Whole bottom is stored
            do i = 1,3
               frameq(i+3,j) = bhq(i,j)
               if (j .LE. 3) then ! Fill (1,1) and rest = 0
                  frameq(i,j) = pin(i,j,ihinge)
               else
                  frameq(i,j) = 0.0d0
               end if
            end do
         end do
      else
         go to 999
      end if
      
      if (ipick .EQ. 1) GO TO 200    ! Assemble Q-frame only
c
c     Come here to fill the P-frame
c
   
 100  if (itype .EQ. 1) then      ! (3x3) -Identity
         do i = 1,3
            do j = 1,3
               framep(j,i) = 0.0d0
            end do
            framep(i,i) = -1.0d0
         end do
c
      else if (itype .EQ. 2) then ! (3x3) -C2inv
         LR4  = 6*(ihinge-2) + 4
         do j = 1,3
            do i = 1,3
               framep(i,j) = -rh(i,j,lr4)
            end do
         end do
c     
      else if (itype .EQ. 3) then ! (3x3) -Identity
         do j = 1,3
            do i = 1,3
               framep(i,i) = 0.0d0
            end do
            framep(j,j) = -1.0d0
         end do
c     
      else if (itype .EQ. 4) then ! Zero on top, -I on bottom
         do i = 1,3
            do j = 1,3
               framep(j,i)   = 0.0d0
               framep(j+3,i) = 0.0d0
            end do
            framep(i+3,i) = -1.0d0
         end do
c
      else if (itype .EQ. 5) then ! (3x3) -C2inv
         LR4  = 6*(ihinge-2) + 4
         do j = 1,3
            do i = 1,3
               framep(i,j) = -rh(i,j,lr4)
            end do
         end do
c
      else if (itype .EQ. 6) then ! Zero on top, -C2inv on bottom
         LR4  = 6*(ihinge-2) + 4
         do j = 1,3
            do i = 1,3
               framep(i+3,j) = -rh(i,j,lr4)
               framep(i,j)   = 0.0d0
            end do
         end do
c
      else if (itype .EQ. 7) then
         do j = 1, 3              ! Fill front part
            do i = 1,3
               framep(i,j)     = bhp(i,j)
               framep(i+3,j)   = bhp(i,j+3)
               framep(i+3,j+3) = 0.0d0
            end do
            framep(j+3,j+3) = -1.0d0
         end do
         do j = 1,nmp             ! Fill mode part
            do i = 1,3
               framep(i+3,j+6) = -hats(i,j)
               framep(i,j+6)   = 0.0d0
            end do
         end do
c     
      else if (itype .EQ. 8) then
         LR4  = 6*(ihinge-2) + 4
         do j = 1, 3              ! Fill front part
            do i = 1,3
               framep(i,j)     = bhp(i,j)
               framep(i+3,j)   = bhp(i,j+3)
               framep(i+3,j+3) = -rh(i,j,lr4)
               framep(i,j+3)   = 0.0d0
            end do
         end do
         do j = 1,nmp             ! Fill mode part
            do i = 1,3
               framep(i+3,j+6) = bhp(i,j+6)
               framep(i,j+6)   = 0.0d0
            end do
         end do
      else
         go to 999
      end if
      
 200  return
c
 999  CALL CLEARERR()
      WRITE(ERRMSG,300) 'Unrecognized type in subroutine.'
      CALL ERRHDL(-5,'<GETBH_1>') 
 300  FORMAT(1X,A)
      END


