C $Header: /export/cvs/c34a1/source/moldyn/init_vcd.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      SUBROUTINE init_vcd(nz,t0,z,zdt,y,ydt,ysav,dysav,x_tmp)
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
C     PURPOSE: Use Runge-Kutta to initialize the VCD integrator.
C              This is also used when a sequence is changed, either the 
C              Euler sequence in IHDATA(1,ih) or pivot type in IHPVT(ih).
C
      IMPLICIT NONE
c
c     MBO(N)D COMMON Blocks
c
#include "PARAM.FOR"
#include "HEAPMB.INC"
#include "MISCN2.INC"
#include "MISCNO.INC"
#include "PARTCL.INC"
#include "SPECIF.INC"
#include "TIMESS.INC"
#include "VECTOR.INC"
c
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: init_vcd.f,v $$Revision: 1.1.1.1 $'/
c
c     Argument List
c
      INTEGER nz                    !  Length of the z vector
      DOUBLE PRECISION t0            !  Time of (re)initialization
      DOUBLE PRECISION z(cL07), zdt(cL07)  !  Compacted states.
      DOUBLE PRECISION y(cL07), ydt(cL07) 
c
c     Local Variables
c
      INTEGER i, j, k, l            !  DO loop indices
      INTEGER lo                    !  Offset for modal velocities
      INTEGER lp                    !  Pointer into the y vector for positions
      INTEGER iextor                !  Indicator of call to external torque
      DOUBLE PRECISION dt           !  Integration step size
      DOUBLE PRECISION ysav(cL07)    !  Saved array for y
      DOUBLE PRECISION dysav(cL07)   !  Saved array for ydt
      DOUBLE PRECISION x_tmp(cL07/3) !  Saved array for prior positions
c
c     External Functions
c
      EXTERNAL ydot
c
c     Temporarily set the RK flag on and VCD flag off
c
      verl_cd = .FALSE.
      verlet  = .FALSE.
      runge_kutta = .TRUE.
c
c     Save current states and propagate backwards using RK
c
c         nbytes = 8 * neq
c         nysav  = MALLOC(nbytes)
c         ndysav = MALLOC(nbytes)
c
      DO i = 1, neq
         ysav(i)  = y(i)
         dysav(i) = ydt(i)
      END DO
c
C     The first call to ydot is assumed to be done prior to this subroutine
c     Hence we must pack zdt as well as z from the y and ydt arrays

      j = 0
      do i = 1, nb
         lo = locu(i) + 5       !  This section is used only if body is flex
         do k = 1, irgflx(i)
            j = j + 1
            z(j)   = y(lo+k)
            zdt(j) = ydt(lo+k)
         end do
      end do
      do i = locu(nb+1), neq
         j = j + 1
         z(j)   = y(i)
         zdt(j) = ydt(i)
      end do
      nz = j
      dt = - deltat
      CALL rk4(z,zdt,nz,t0,dt,z,YDOT,iextor,LocalHeap(iytptr_rk4),
     *         LocalHeap(idytptr_rk4),LocalHeap(idymptr_rk4))
c     
c     Save (t0-deltat) modal coordinates
c
c         nbytes = 8 * (locu(2*nb+2)-locu(nb+1))
c         nx_tmp = MALLOC(nbytes)
c
      i = 0
      DO j = 1, nb
         i = i + irgflx(j)
      END DO
      lp = i
         
      i = 0
      DO j = 1, nb
         DO k = 1, irgflx(j)
            i = i + 1
            x_tmp(i) = z(lp+i)
         END DO
      END DO
c
c     Save the (t0-deltat) relative hinge coordinates (beta's)
c
      DO k = 1, nbeta
         i = i + 1
         x_tmp(i) = z(lp+i)
      END DO
c
c     Restore the (t0) states and propagate forwards using RK
c
      j = 0
      do i = 1, nb
         lo = locu(i) + 5       !  This section is used only if body is flex
         do k = 1, irgflx(i)
            j = j + 1
            z(j)   = ysav(lo+k)
            zdt(j) = dysav(lo+k)
         end do
      end do
      do i = locu(nb+1), neq
         j = j + 1
         z(j)   = ysav(i)
         zdt(j) = dysav(i)
      end do
c
      dt = deltat
      CALL rk4(z,zdt,nz,t0,dt,z,YDOT,iextor,LocalHeap(iytptr_rk4),
     *         LocalHeap(idytptr_rk4),LocalHeap(idymptr_rk4))
c
c     Now pack the z vector to initialize the VCD integrator
c     Process the modal coordinates
c
c     Load the old positions
c
      DO j = 1, lp+nbeta
         zdt(j) = x_tmp(j)
      END DO
c
c     Copy the new positions into temp. storage
c
      i = 0
      DO j = 1, nb
         DO k = 1, irgflx(j)
            i = i + 1
            x_tmp(i) = z(lp+i)
         END DO
      END DO
      DO k = 1, nbeta
         i = i + 1
         x_tmp(i) = z(lp+i)
      END DO
c
c     Load the new positions from temp. storage
c
      DO j = 1, lp+nbeta
         z(j) = x_tmp(j)
      END DO
c
c     Load the current positions
c
      i  = 0
      nz = locu(2*nb+2) - locu(nb+1)
      DO j = 1, nb
         l = locu(nb+j) - 1
         DO k = 1, irgflx(j)
            i = i + 1
            z(nz+i) = ysav(l+k)
         END DO
      END DO
      l = locu(2*nb+1) - 1
      DO k = 1, nbeta
         i = i + 1
         z(nz+i) = ysav(l+k)
      END DO
c
c     Set the RK flag back off and VCD flag on
c
      verl_cd = .TRUE.
      verlet  = .TRUE.
      runge_kutta = .FALSE.

      RETURN
      END
