C $Header: /export/cvs/c34a1/source/moldyn/mdloop_CHARMM.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1994 MOLDYN, INC.  ALL RIGHTS RESERVED
C ========================================================================
C
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER 
C STATE AND FEDERAL LAWS.  USE, DISCLOSURE, REPRODUCTION OR DUPLICATION 
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS 
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT 
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
#ifdef ANALYZE_FORCE
      SUBROUTINE mdloop_CHARMM(Y,YDT,G_chm,G_gyro,G_deform,p,z,zdt)
#else
      SUBROUTINE mdloop_CHARMM(Y,YDT,p,z,zdt)
#endif
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     This performs the integration loop for CHARMM/MBO(N)D
c
      IMPLICIT NONE
 
#include "PARAM.FOR"      
c
c     MBO(N)D COMMON blocks
c
#include "CONPAR.INC"
#include "CPATHS.INC"
#include "ERRHDL.INC"
#include "EXTRA.INC"
#include "HEAPMB.INC"
#include "IVCONS.INC"
#include "MISCN2.INC"
#include "MISCNO.INC"
#include "MOMENG.INC"
#include "NUMBRS.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "TIMESS.INC"
#include "TIMING.INC"
#include "VECTOR.INC"
#include "GCOMP.INC"
#include "SELECT.INC"
c
c
      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: mdloop_CHARMM.f,v $$Revision: 1.1.1.1 $'/
c
c     MBO(N)D/CHARMM COMMON blocks
c
#include "CHARMM.INC"
c
c     Other COMMON blocks
c
      INTEGER ntx, ntcx, ntxo, ntt, ntp, ntr, init, ntcm, nscm, ntu
      DOUBLE PRECISION tx, dtx
      COMMON/MDTYPE/NTX,NTCX,NTXO,NTT,NTP,NTR,INIT,NTCM,NSCM,NTU,TX,DTX
c
c     Passed variables
c
      DOUBLE PRECISION Y(cL07), YDT(cL07)
      DOUBLE PRECISION P(cL10)

#ifdef ANALYZE_FORCE
      DOUBLE PRECISION G_chm(cL10)
      DOUBLE PRECISION G_gyro(6*MAXBOD)
      DOUBLE PRECISION G_deform(6*MAXBOD)
#endif
c
c     Local Variables
c
      INTEGER l                     
      INTEGER i                     !  Counter
      INTEGER iter                  !  Iterate flag
      INTEGER iextor                !  Indicator of call to external torque
      INTEGER lo, ifx               !  Pointers
      INTEGER nstp, nstpold, nstep  !  Integration counters
      INTEGER istpb, istpv, istpe, istps  
      INTEGER neqz                  
      LOGICAL nbflag, prtflg
      DOUBLE PRECISION tim1, tim2   
      DOUBLE PRECISION z(cL07), zdt(cL07)  
      DOUBLE PRECISION t0, tf  
      DOUBLE PRECISION tx0     
      DOUBLE PRECISION h1, hmin 
      DOUBLE PRECISION eps      
      DOUBLE PRECISION dum          !  Dummy variable
c
c     DATA and EQUIVALENCE Statements
c
      DATA adpstp /.FALSE./
      EQUIVALENCE (cntdta(1),eps)
c
c     External Functions
c
      EXTERNAL ydot
C
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     COMMENTS on definition of z and zdt
c
c     For Velocity Verlet integrator:
c         z(1:n) is used to store positions
c         z(n+1:2*n) is used to store velocities
c         zdt(1:n) is used to store accelerations
c         n is number of position variables
c
c     For Verlet Central Difference integrator:
c         z(1:n) is used to store positions at t+dt
c         z(n+1:2*n) is used to store positions at t
c         zdt(1:n) is used to store positions at t-dt
c         zdt(n+1:2*n) is used to store velocities at t
c         n is number of position variables
c
c     For Runge-Kutta and Bulirsch Stoer integrators:
c         z(1:m) is used to store system states (i.e. positions & velocities)
c         zdt(1:m) is used to store state derivatives (i.e. velocities
c                    and accelerations)   
c         m is number of states = 2*n
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C
#ifdef ANALYZE_FORCE
      OPEN(UNIT=20,STATUS='NEW',ERR=100)
      WRITE(20,216) nbdy
      WRITE(20,216) (irgflx(i),i=1,nbdy)
      GO TO 150
 100  OPEN(UNIT=20,STATUS='OLD',ACCESS='APPEND')
 150  CONTINUE
#endif
      
C        BRANCH TO YDOT TO GET TIME DERIVATIVE OF STATE VECTOR, RESULTS
C        STORED IN YDT ARRAY EQUATIONS II-1 THRU 6 ALL EVALUATED IN YDOT
c
      seqflg  = .FALSE.
      nstp    = 0
      nstpold = 0
      noprnt  = noprnt * 4
      noplot  = noplot * 4
C
      IPRNT = NOPRNT
      enflag = .TRUE.
C 
C     First Call to YDOT 
C
C     Set indicator that this is initialization of integration 
      icur_step = 0
C
      CALL msecond(tim1)

#ifdef CHARMM
C     If using Langevin dynamics call GASRND to populate RF, a 1xNU 
C     vector, with random values (forces)
      IF (LANGEVIN) CALL GASRND(mbdynaHeap(ilangevinptr),
     &                          mbdynaHeap(irfptr),
     &                          HEAP_MB(ipptr))
#endif

      timer(imbond0) = tim1 - timer(imbond0)
      iter = 0
      iextor = 1
      IF (v_verlet .OR. lobatto) THEN
         CALL YDOT(tx,dum,dum,zdt,iter,iextor,HEAP_MB(iyptr),
     &             HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &             LocalHeap(iggvptr_ydot),
     &             logicLoopHeap(iiscutptr),
     &             intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &             LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &             intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      ELSE IF (verl_cd) THEN
         CALL YDOT(tx,dum,dum,zdt,iter,iextor,HEAP_MB(iyptr),
     &             HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &             LocalHeap(iggvptr_ydot),
     &             logicLoopHeap(iiscutptr),
     &             intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &             LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &             intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      ELSE   !  RK or BS
         CALL YDOT(tx,dum,zdt,dum,iter,iextor,HEAP_MB(iyptr),
     &             HEAP_MB(iydtptr),HEAP_MB(ipptr),
     &             LocalHeap(iggvptr_ydot),
     &             logicLoopHeap(iiscutptr),
     &             intLoopHeap(iivptr),intLoopHeap(iibetdtptr),
     &             LoopHeap(ibetahdptr),intloopheap(iitopolptr),
     &             intLoopheap(iihdataptr),LocalHeap(iirhptr_ydot))
      END IF
      CALL msecond(tim2)
      timer(irk4) = tim2 - tim1
C
C     Indicate that initialization of integration is over 
C     and initialize the counter for MTSL steps
      icur_step = 1
C
      tx0 = tx
      istpb = nbdfrq
      istpv = nvlfrq
      istpe = nenfrq
      istps = nsmfrq
C
      IF (nenfrq .LT. nstlim) CALL prnten(HEAP_MB(ipptr))
      IF (nbdfrq .LT. nstlim) CALL prntbd
      IF (nsmfrq .LT. nstlim) CALL prntsm
C
C      INITIALIZE COUNTER CONTROL FLAGS
C
      IPRNT = 0
C
C        START INTEGRATION OF EQUATIONS OF MOTION
C            NEQZ = TOTAL NUMBER OF EQUATIONS TO INTEGRATE
C        RETURN TO THIS POINT AFTER EACH INTEGRATION CYCLE
C
      tf = startt
      h1 = deltat
      hmin = deltat
c
c     PACK THE Z VECTOR
c
      IF (v_verlet .OR. lobatto) THEN
c
c     Note that we do not have any delta control variables here since
c     we are limited to 2nd order form.  We do have rheonomic constraint
c     variables.
c
c     Copy the modal coordinates and beta's
c
         i = 0
         DO l = locu(nb+1), locu(2*nb+2)-1
            i = i + 1
            z(i) = y(l)
         END DO
         neqz = i
c
c     Copy the velocity variables
c
         i = 0
         DO l = 1, nb        !  Copy the modal velocities
            lo = locu(l) + 5
            DO ifx = 1, irgflx(l)
               i = i + 1
               z(neqz+i) = y(lo+ifx)
            END DO
         END DO
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
cblink   Test the following two sections of code
c     Remember to skip the cut joints.  Not properly done yet.
c
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c         lva = locu(2*nb+2) - 1   !  Copy the alpha dots
c         lvb = locu(2*nb+3) - 1   !  Copy the beta dots
c         DO l = 1, nh
c            IF (l .GT. nbdy) THEN ! Its a particle
c               idof1 = 5
c            ELSE                  ! Its a body
c               idof1 = 2
c            END IF
c            DO ip1 = idof1, 7
c               IF (ihdata(ip1,l) .EQ. 0) THEN
c                  i   = i + 1
c                  lvb = lvb + 1
c                  z(neqz+i) = y(lvb)
c               ELSE IF (ihdata(ip1,l) .EQ. 2) THEN
c                  i   = i + 1
c                  lva = lva + 1
c                  z(neqz+i) = y(lva)
c               END IF
c            END DO
c         END DO
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
         lo = locu(2*nb+3)   !  Copy the beta dots
         DO l = lo, lo+nbeta-1
            i = i + 1
            z(neqz+i) = y(l)
         END DO
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      ELSE IF (verl_cd) THEN  !  Verlet central difference
c
         CALL init_vcd(neqz,startt,z,zdt,HEAP_MB(iyptr),
     *                 HEAP_MB(iydtptr),LocalHeap(iysavptr_init_vcd),
     *                 LocalHeap(idysavptr_init_vcd),
     *                 LocalHeap(ix_tmpptr_init_vcd))
c
      ELSE                      !  Runge-Kutta or Bulirsch-Stoer
c
         neqz = 0
         do l = 1, nb
            lo = locu(l) + 5    !  This section is used only if body is flex
            do ifx = 1, irgflx(l)
               neqz = neqz + 1
               z(neqz) = y(lo+ifx)
            end do
         end do
         do l = locu(nb+1), neq
            neqz = neqz + 1
            z(neqz) = y(l)
         end do
c
      END IF
c
      CALL msecond(tim1)
      timer(imbond0) = timer(imbond0) + tim1 - tim2
C
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     BEGINNING OF THE INTEGRATION LOOP
C
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
C
 11   continue
      nstp = min(istpb,istpv,istpe,istps,nstlim)
c
      if (nstp .le. nstpold) then
         CALL CLEARERR()
         WRITE (MBOUT,200) ' NSTP has not incremented.',
     +                    ' NSTP    = ', nstp,
     +                    ' NSTPOLD = ', nstpold,
     +                    ' istpb   = ', istpb,
     +                    ' istpv   = ', istpv,
     +                    ' istpe   = ', istpe,
     +                    ' istps   = ', istps,
     +                    ' nstlim  = ', nstlim
         WRITE(ERRMSG,210) 'Error in subroutine.'
         CALL ERRHDL(-5,'<MDLOOP_CHARMM>')
      end if
 200  FORMAT(A26,/,A12,I6,/,A12,I6,/,A12,I6,/,A12,I6,/,A12,I6,/,
     +       A12,I6,/,A12,I6)
c
      enflag = nstp .EQ. istpe
      vlflag = (nstp.EQ.istpv) .OR. (nstp.EQ.nstlim)
      prtflg = nstp .EQ. istpb
c
      nstpold = nstp
      if (nstp .eq. nstlim) then
         istpb = nstlim
      end if
      t0 = tf
      tf = startt + nstp * deltat
      nstep = MAX(1,NINT((tf-t0)/deltat))
c
c     Call to the specified integrator
c     Note:  The n-bin MTSL integrator is realized within a regular (non-MTS)
c            Lobatto integrator
c 
      IF (v_verlet) THEN
         CALL vvint(z,z(neqz+1),zdt,neqz,t0,tf,nstep,YDOT,
     *        nbflag,prtflg)
      ELSE IF (lobatto) THEN
         CALL lobint(z,z(neqz+1),zdt,neqz,t0,tf,nstep,YDOT,
     *        nbflag,prtflg)
      ELSE IF (verl_cd) THEN
c
c     Make sure stepsize hasn't changed
c
         IF ((ABS(tf-t0)/DBLE(nstep)-deltat)/deltat .GT. 1D-2)
     *        THEN
               CALL CLEARERR()
               WRITE(ERRMSG,210) 
     *         ' Verlet Central Difference assumes fixed steps.'
               CALL ERRHDL(-5,'<MDLOOP_CHARMM>')
         END IF
         CALL vcdint(z,zdt,neqz,t0,tf,nstep,YDOT,nbflag,prtflg)
      ELSE
         CALL rkdumb(z,zdt,neqz,t0,tf,nstep,YDOT,nbflag,prtflg)
      END IF                  
 210  FORMAT(1X,A)
      nrk4 = nrk4 + nstep
      tx = tx0 + dtx*DBLE(nstp)
      CALL msecond(tim2)
      timer(irk4) = timer(irk4) + tim2 - tim1
C
C       OUTPUT PRINT DATA
C
      write(noutCH, *) 'MBDYN> Integration Step  ', iparam(16)
      if (enflag) then
         istpe = istpe + nenfrq
         CALL prnten(HEAP_MB(ipptr))
      end if
      if (prtflg) then
         istpb = istpb + nbdfrq
         call prntbd
#ifdef ANALYZE_FORCE
         WRITE(20,216) nstp
         WRITE(20,215) (G_chm(i),i=1,locu(nbdy+1)-1)
         WRITE(20,215) (G_gyro(i),i=1,6*nbdy)
         WRITE(20,215) (G_deform(i),i=1,6*nbdy)
 215     FORMAT(5D16.8)
 216     FORMAT(8I10)
#endif
      end if
      if (nstp .eq. istps) then
         istps = istps + nsmfrq
         CALL prntsm
      end if
      IF (vlflag) istpv = istpv + nvlfrq
C
      CALL msecond(tim1)
      timer(ioutput) = timer(ioutput) + tim1 - tim2
      IF (nstp .lt. nstlim) GO TO 11
C
#ifdef ANALYZE_FORCE
      CLOSE (20)
#endif
C
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
c     END OF THE INTEGRATION LOOP
c
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c
      RETURN
      END








