C $Header: /export/cvs/c34a1/source/moldyn/rotds.f,v 1.1.1.1 2006/08/17 02:29:56 brooks Exp $
C
C COPYRIGHT (C) 1991-1998 MOLDYN, INC.  ALL RIGHTS RESERVED
C =====================================================================
C 
C THIS IS UNPUBLISHED PROPRIETARY SOURCE CODE OF MOLDYN, INC.
C 
C THIS SOFTWARE CONTAINS PROPRIETARY AND CONFIDENTIAL INFORMATION OF
C MOLDYN, INC., AND IS PROTECTED BY COPYRIGHT, TRADE SECRET AND OTHER
C STATE AND FEDERAL LAWS. USE, DISCLOSURE, REPRODUCTION OR DUPLICATION
C IN WHOLE OR IN PART IS STRICTLY PROHIBITED WITHOUT THE PRIOR EXPRESS
C WRITTEN CONSENT OF MOLDYN, INC. THE ABOVE COPYRIGHT NOTICE DOES NOT
C EVIDENCE ANY ACTUAL OR INTENDED PUBLICATION OF SUCH SOURCE CODE.
C 
C 

      SUBROUTINE ROTDS

      IMPLICIT NONE

      CHARACTER*128  RCSID
      DATA           RCSID
     &/'$RCSfile: rotds.f,v $$Revision: 1.1.1.1 $'/

#include "PARAM.FOR"
c
#include "HEAPMB.INC"
#include "VECTOR.INC"

      CALL ROTDS_1(HEAP_MB(iyptr))

      RETURN
      END

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc

      SUBROUTINE ROTDS_1(Y)

cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
CSE
C                                                          DEBUG #  46
CPS        PURPOSE:
C
C                   TO COMPUTE SENSOR POINT FRAME TRANSFORMATION MATRIX
C                   THE ELEMENTS OF THE ARRAYS RS AND DS ARE COMPUTED.
C
C        COMPUTE RS ROTATION TRANSFORMATION BETWEEN SENSOR AXIS SYSTEM
C        AND BODY FIXED AXIS SYSTEM VOL II,PAGE 19
C        NOTHING DONE IF SENSOR ON RIGID BODY
C
C                   WHEN ACCOUNTING FOR FLEXIBILITY IN THE DETERMINATION
C                   OF ORIENTATION OF SENSOR FRAMES WRT THE BODY FIXED
C                   FRAMES USE ROTTRQ
C        REASON
C                   QUATERNION TECHNIQUE IS MORE EFFICIENT THAN EULER
C                   TECHNIQUE DUE TO SMALL ANGLE ELASTIC DEFORMATION
C                   ALSO EULER METHOD CAN LEAD TO ERRORS
C
CPE
CAS
C
C         ROTDS - SENSOR FRAME LOCATION AND ORIENTATION
C         ---------------------------------------------
C     RS     = TRANSFORMATION MATRICES FROM SENSOR POINT TO BODY FIXED
C              REFERENCE FRAME, DIMENSIONED (3,3,2*NSPMAX), STORED
C              IN /SPECIF/. SET DEBUG(46) = .TRUE. TO PRINT
C     DS     = COORDINATES OF POSITION VECTOR FROM BODY FIXED REFERENCE
C              TO SENSOR POINT WRT TO BODY FRAME, STORED IN /SPECIF/
C              DIMENSIONED (3,2*NSPMAX). SET DEBUG(46) = .TRUE. TO PRINT
C                FOR SENSOR POINT NS, NS=1,2,...,NSPT LET
C                     N = IFTSMW(NS)
C                   LR1 = 2*NS-1
C                   LR2 = 2*NS
C         ((RS(I,J,LR1),I=1,3),J=1,3) = TRANSFORMATION MATRIX WHICH
C                             TAKES VECTORS FROM SENSOR POINT NS FIXED
C                             COORDINATES TO BODY N FIXED COORDINATES
C                             IN THE UNDEFORMED STATE OF BODY N
C         ((RS(I,J,LR2),I=1,3),J=1,3) = TRANSFORMATION MATRIX SENSOR
C                             POINT NS FRAME TO BODY N FRAME IN DEFORMED
C                             STATE OF BODY N
C         (DS(I,LR1),I=1,3)           = POSITION VECTOR FROM BODY N
C                             REFERENCE POINT TO SENSOR POINT NS WRT THE
C                             BODY N FIXED REFERENCE IN UNDEFORMED STATE
C         (DS(I,LR2),I=1,3)           = POSITION VECTOR FROM BODY N
C                             REFERENCE POINT TO SENSOR POINT MS WRT THE
C                             BODY N FIXED REFERENCE IN DEFORMED STATE
C
CAE
      IMPLICIT NONE
 
C
#include "PARAM.FOR"
C
#include "BHBSRD.INC"
#include "COREA.INC"
#include "HANDS2.INC"
#include "HEAPMB.INC"
#include "LDEBUG.INC"
#include "MEMLB.INC"
#include "SPECIF.INC"
#include "TAPENO.INC"
#include "VECTOR.INC"
c
C     Passed variables
c
      DOUBLE PRECISION Y(cL07)
c
c     Local Variables
c
      LOGICAL LEQU
      INTEGER i, j, l             ! DO loop indices
      INTEGER nob, ibod           ! Body indices
      INTEGER lr1, lr2            ! Indices
      INTEGER lo, le              ! Indices into y
      INTEGER ipoint              ! Index into X array
      DOUBLE PRECISION DF(3)
C
C     DATA and EQUIVALENCE statements
C
      EQUIVALENCE (LEQU,DEBUG(46))
C
      IF(LEQU) WRITE (MBOUT,1000)
      DO 10 L=1,NSPT
         NOB = IFTSMW(L)
         IF (IRGFLX(NOB) .EQ. 0) GO TO 10
         LR1 = 2*L - 1
         LR2 = LR1 + 1
         LO = LOCU(NB+NOB)
         LE = LENU(NB+NOB)
C Original: MULT3 HATS(htsptr(L)+1),Y(LO),DF,3,LE,1,3,LE,3
CC@CC
CC@CC ...MULT3 HATS(ihtsptr(L)+1), Y(LO), DF
CC@CC ...3, LE, 1
CC@CC ...3, LE, 3
      CALL DGEMV ('N',3,LE,1.0d0,HATS(ihtsptr(L)+1),3,Y(LO),1,0.0d0,DF,1
     .)
CC@CC end of expanded call
CC@CC
         ds(1,lr2) = ds(1,lr1) + df(1)
         ds(2,lr2) = ds(2,lr1) + df(2)
         ds(3,lr2) = ds(3,lr1) + df(3)
C
 10   CONTINUE
C
C     COMPUTE THE COORDINATES OF THE SENSOR POINTS (DEFORMED STATE)
C
      IPOINT = M30
      DO I = 1, NSPT
         IBOD = IFTSMW(I)
         do j = 1, 3
            x(ipoint+j-1) = d0l(j,ibod)
         end do
         if (ibod .LE. nbdy) then  ! ibod is a body
CC@CC
CC@CC ......MULTAD R0L(1,1,IBOD), DS(1,2*I), x(ipoint)
CC@CC ......3, 3, 1
CC@CC ......3, 3, 3
      x(ipoint+0) = x(ipoint+0) + R0L(1,1,IBOD)*DS(1,2*I) + R0L(1,2,IBOD
     .)*DS(2,2*I) + R0L(1,3,IBOD)*DS(3,2*I)
      x(ipoint+1) = x(ipoint+1) + R0L(2,1,IBOD)*DS(1,2*I) + R0L(2,2,IBOD
     .)*DS(2,2*I) + R0L(2,3,IBOD)*DS(3,2*I)
      x(ipoint+2) = x(ipoint+2) + R0L(3,1,IBOD)*DS(1,2*I) + R0L(3,2,IBOD
     .)*DS(2,2*I) + R0L(3,3,IBOD)*DS(3,2*I)
CC@CC end of expanded call
CC@CC
         end if
         IPOINT = IPOINT + 3
      END DO
C
      IF(.NOT.LEQU) RETURN
      WRITE (MBOUT,1003)  (I,(DS(J,I),J=1,3),I=1,2*nspt)
 1000 FORMAT (' SUBROUTINE ROTDS ENTERED')
 1003 FORMAT (1X,'  DS(',I2,') =',1P3D11.3)
C
      RETURN
      END
